/*---------------------------------------------------------------------------*
*  Copyright (c) 2005        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.agents;

import rca.content.abs.AbsContentElementList;
import rca.content.abs.AbsPredicate;
import rca.content.lang.Codec;
import rca.content.onto.OntologyException;
import rca.domain.FIPANames;
import rca.lang.acl.ACLMessage;
import rca.proto.SimpleAchieveREInitiator;

/**
 * Agent request initiator behaviour.
 * This class incapsulates agent communication FIPA_REQUEST protocol.
 *
 * @author Vardan Gyurjyan
 *         Date: Jan 13, 2007
 *         Time: 3:27:17 PM
 */
public class GenRequestBehaviour extends SimpleAchieveREInitiator {

    /** Object copy of the CAgent class, passed to the constructor*/
    private CAgent myCAgent;

    /**
     * Constructor
     *
     * @param ag is the reference to the Agent object where from this behaviour class is instansiated.
     * @param requestMsg the ACLMessage object.
     * @param vrb int indicating the verbosity level.
     */
    public GenRequestBehaviour(CAgent ag, ACLMessage requestMsg, int vrb){

// Call the constructor of the parent class
        super(ag, requestMsg);

//  We define here the interaction protocol of the request message object
        requestMsg.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
        myCAgent = ag;
//        System.out.println(myCAgent.getLocalName()+" ................ initiates an ACL message request");

    }

    /**
     * Handls the agree messages from the responder.
     *
     * @param msg agree ACLMessage
     */
    protected void handleAgree(ACLMessage msg) {
    }

    /**
     * Responder sends you back inform message. Deal with it here.
     *
     * @param msg inform ACLMessage
     */
    protected void handleInform(ACLMessage msg) {

// Inform is handled in the agent
        myCAgent.processInform(msg);
    }

    /**
     * Responder sends back "not understood" message. Deal with it here.
     *
     * @param msg ACLMessage
     */
    protected void handleNotUnderstood(ACLMessage msg) {
    }

    /**
     *  Responder sends back "failure" message. Deal with it here.
     *
     * @param msg ACLMessage
     */
    protected void handleFailure(ACLMessage msg) {

        System.err.println(myCAgent.getLocalName()+" : - "+msg+" - Failed");
        try{
            AbsPredicate absPred =(AbsPredicate)myCAgent.getContentManager().extractContent(msg);
            System.out.println(myCAgent.getLocalName()+" : - "+"The reason is: " + absPred.getTypeName());
        }
        catch (Codec.CodecException fe){
            System.err.println(myCAgent.getLocalName()+" : FIPAException reading failure reason: " + fe.getMessage());
        }
        catch (OntologyException oe){
            oe.printStackTrace();
        }
    }

    /**
     * Responder sends the "refuse" message. Handle it here.
     *
     * @param msg ACLMessage
     */
    protected void handleRefuse(ACLMessage msg) {

        System.out.println(myCAgent.getLocalName()+" : - "+msg+" - Refused");
        try{
            AbsContentElementList list =(AbsContentElementList)myCAgent.getContentManager().extractAbsContent(msg);
            AbsPredicate absPred = (AbsPredicate)list.get(1);
            System.out.println(myCAgent.getLocalName()+" : - "+"The reason is: " + absPred.getTypeName());
        }
        catch (Codec.CodecException fe){
            System.err.println(myCAgent.getLocalName()+" : FIPAException reading refusal reason: " + fe.getMessage());
        }
        catch (OntologyException oe){
            System.err.println(myCAgent.getLocalName()+" : OntologyException reading refusal reason: " + oe.getMessage());
        }
    }
}
