/*---------------------------------------------------------------------------*
*  Copyright (c) 2007        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.gui;

import jas.hist.*;
import jas.hist.util.ObserverAdapter;

import javax.swing.*;
import java.awt.*;

import org.jlab.coda.runcontrol.util.rcConstants;

/**
 * @author Vardan Gyurjyan
 *         Date: Oct 31, 2005
 *         Time: 10:56:24 AM
 */

public class BarGraphLite
{
    public volatile double[] values;
    public TimeLineDataSource datasource;
    public JASHist	plot;
    public int numberOfBins;

    BarGraphLite(final String title, final int nb, double ymax, final JScrollPane pp)
    {
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                plot = new JASHist();
                numberOfBins = nb;
                values = new double[nb];

                datasource =new TimeLineDataSource();
                plot.addData(datasource).show(true);
//        plot.setTitle(title);
                plot.setDataAreaBorderType(JASHist.ETCHED);
                plot.getYAxis().setLabel(title);
                plot.getYAxis().setRangeAutomatic(true);
                plot.getYAxis().setAllowSuppressedZero(false);
//        plot.getYAxis().setRange(0.0, ymax);
                plot.getXAxis().setLabel(" ");
                plot.setAllowUserInteraction(false);
                plot.setDoubleBuffered(true);
//        plot.setBounds(845,693,403,255);
                pp.setDoubleBuffered(true);
//        pp.setFont(new java.awt.Font("Dialog", 0, 10));
                pp.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
                pp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
                pp.setViewportView(plot);
            }
        });
    }

    public void remove(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                plot.destroy();
            }
        });
    }

    // Note	that this class	extends	Observable so that it can notify
    // the plot	(JASHist) when the data	changes.
    public class TimeLineDataSource extends ObserverAdapter implements
            Rebinnable1DHistogramData, HasStyle
    {

        // Create the HistogramUpdate object that will be sent
        // to all our Observers	(specifically the JASHist).	This
        // HistogramUpdate will	specify	that both the bin contents
        // (data) and the axis min/max (range) have	changed. The
        // boolean argument	to the HistogramUpdate constructor
        // indicates that this is a	"final"	update,	which the
        // JASHist should resond to	immediately.
        int	flags =	HistogramUpdate.DATA_UPDATE	|
                HistogramUpdate.RANGE_UPDATE;
        HistogramUpdate	hu = new HistogramUpdate(flags,true);


        public void update () {
            setChanged(); // notifyObservers needs this
            notifyObservers(hu);
        }

        public double[][] rebin(int	rBins, double rMin,	double rMax,
                                boolean	wantErrors,	boolean	hurry)
        {
            return new double[][]{ values };
        }

        public String[]	getAxisLabels()	{ return null; }
        public double getMin() { return	0; }
        public double getMax() { return	values.length; }
        public boolean isRebinnable() {	return false; }
        public int getBins() { return values.length; }
        public int getAxisType() { return DOUBLE; }
        public String getTitle() { return ""; }
        public JASHistStyle getStyle()
        {
            JASHist1DHistogramStyle style = new JASHist1DHistogramStyle();
            style.setShowErrorBars(false);
            style.setShowLinesBetweenPoints(false);
            if(numberOfBins >= rcConstants.BAR_OR_POINT){
                style.setShowHistogramBars(false);
                style.setShowDataPoints(true);
                style.setDataPointStyle(1);
                style.setDataPointSize(3);
                style.setDataPointColor(Color.BLUE);
            }else{
                style.setShowDataPoints(false);
                style.setShowHistogramBars(true);
            }
//            style.setHistogramFill(false);
//            style.setHistogramBarLineColor(Color.BLUE);
//            style.setHistogramBarLineWidth(3f);
//        style.setDataPointSize(1);
//        style.setLinesBetweenPointsWidth(2);
            return style;
        }
    }
}
