/*---------------------------------------------------------------------------*
*  Copyright (c) 2005        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                          *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.gui;

import org.jlab.coda.runcontrol.util.rcSubscription;
import org.jlab.coda.runcontrol.util.rcConstants;
import org.jlab.coda.runcontrol.util.RcSort;
import org.jlab.coda.runcontrol.util.db.*;
import org.jlab.coda.runcontrol.util.datatypes.CompMonitor;
import org.jlab.coda.runcontrol.gui.dalogbrowser.DalogBrowserApp;
import org.jlab.coda.cMsg.*;

import javax.swing.*;
import javax.swing.JScrollPane;
import javax.swing.border.BevelBorder;
import javax.swing.text.StyledDocument;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.BadLocationException;
import java.io.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.*;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.text.DecimalFormat;


/**
 * @author Vardan Gyurjyan
 *         Date: Jul 13, 2006
 *         Time: 4:38:33 PM
 */
public class RcGui extends JFrame {

    /** Flag indicating that the run has bin configured*/
    private volatile boolean configed = false;

    /** Flag indicating that startrun has bin enabled */
    private volatile boolean startrun = false;

    private volatile boolean startbuttonclicked = false;

    /** Flag indicating that download button has bin clicked*/
    private volatile boolean downloadclicked = false;

    /** Flag indicating that prestart button has bin clicked*/
    private volatile boolean prestartclicked = false;

    /** Flag indicating that go button has bin clicked*/
    private volatile boolean goclicked = false;

    /** default timegraph averaging interval ( smooting factor) is 1 */
    private int AIV = 1;

    /** As usual rendering of the bar graphs take a lot of resources,
     * so in case we have many components we would like to render/show
     * the data rate bargraphs not for every reporting.
     * This variable will control that.*/
    private volatile int _graphPlotId;

    /** Color settings*/
    public static final Color VERY_DARK_GREEN = new Color(0x00, 0x80, 0x00);
    public static final Color DARK_GREEN = new Color(0x80, 0xFF, 0x80);
    public static final Color VERY_DARK_YELLOW = new Color(0x80, 0x80, 0x00);
    public static final Color DARK_YELLOW = new Color(0xC0, 0xC0, 0x00);
    public static final Color LIGHT_MAGENTA = new Color(0xFF, 0x40, 0xFF);

    /** THis variable indicates that error condition had hapened*/
    public boolean ERROR_HAPPENED = false;

    public JScrollPane verticalPane;
    public JLabel InfoLabel;
    public JLabel ClockLabel;
    public JLabel EmptyLabel;
    public DefaultListModel listModel;
    public JList ComponentList;
    public JScrollPane ComponentsPane;
    public JTextField ConfigurationTextField;
    public JButton ConfigureButton;
    public JLayeredPane ControlTransitionPane;
    public JTextField DataRateDifTextField;
    public JTextField DataRateIntTextField;
    public JTextField DatabaseTextField;
    public JButton DownloadButton;
    public JButton EndButton;
    public JTextField EndTimeTextField;
    public JTextField EventRateDifTextField;
    public JTextField EventRateIntTextField;
    public JTextField EventThisRunTextField;
    public JButton GoButton;
    public JTextField LimitEventsTextField;
    public JTextField DataLimitTextField;
    public JScrollPane MessagesScrollPane;
    public JTextPane MessagesTextPane;
    public StyledDocument doc;
    public Style style;
    public JButton PauseButton;
    public JButton PrestartButton;
    public JButton StartRunButton;
    public JButton AbortButton;
    public JTextField RunNumberTextField;
    public JLayeredPane RunProgressPane;
    public JLayeredPane RunStatusPane;
    public JTextField RunStatusTextField;
    public JTextField ERFileTextField;
    public JTextField SessionTextField;
    public JTextField StartTimeTextField;
    public JLayeredPane StaticPArametersPane;
    public JMenuItem aboutMenuItem;
    public JMenuItem contentsMenuItem;
    public JMenuItem copyMenuItem;
    public JRadioButtonMenuItem enableEasyStart;
    public JRadioButtonMenuItem disableEasyStart;
    public JRadioButtonMenuItem enableEvLimMenuItem;
    public JRadioButtonMenuItem disableEvLimMenuItem;
    public JRadioButtonMenuItem enableAutoStartMenuItem;
    public JRadioButtonMenuItem disableAutoStartMenuItem;
    public JMenuItem deleteMenuItem;
    public JRadioButtonMenuItem moreGraphsMenuItem;
    public JRadioButtonMenuItem lessGraphsMenuItem;
    public JMenuItem dalogBrowserMenuItem;
    public JRadioButtonMenuItem hideEBERBarsMenuItem;
    public JRadioButtonMenuItem showEBERBarsMenuItem;
    public JMenu editMenu;
    public JMenuItem exitMenuItem;
    public JMenuItem db2coolMenuItem;
    public JMenu fileMenu;
    public JMenu helpMenu;
    public JMenuBar menuBar;
    public JMenuItem setReportIntervalMenuItem;
    public JMenuItem openMenuItem;
    public JMenuItem ResetMenuItem;
    public JMenuItem CExitMenuItem;
    public JMenuItem setAiMenuItem;
    public JMenuItem pasteMenuItem;
    public JMenuItem saveAsMenuItem;
    public JMenuItem saveMenuItem;

    public JComponent MainPanel ;
    public JComponent ControlPanel ;
    public JComponent TransitionPanel ;
    public JComponent DatabasePanel ;
    public JComponent SessionPanel ;
    public JComponent ConfigurationPanel ;
    public JComponent RunNumberPanel ;
    public JComponent EndTimePanel ;
    public JComponent StartTimePanel ;
    public JComponent EventsPanel ;
    public JComponent DataLimitPanel ;
    public JComponent RunStatePanel ;
    public JComponent ERFilePanel ;
    public JComponent EventsThisRunPanel ;
    public JComponent EventRateDifPanel ;
    public JComponent EventRateIntPanel ;
    public JComponent DataRateDifPanel ;
    public JComponent DataRateIntPanel ;

    public ImageScrollPane TimeGraphPane ;
    public ImageScrollPane dataRateBGPane ;
    public ImageScrollPane liveTimeTGPane ;
    public ImageScrollPane DataRateTGPane ;
    public ImageScrollPane EventRateTGPane ;

    /** cMsg server*/
    private cMsg cMsgSrv;

    /** The name of this gui */
    private String myName;

    /** Data rate array*/
    private double[] _DataRate;

    /** Hashtabel keeps the names of the compoents as a key and value its index in the datarate array*/
    private  ConcurrentHashMap<String,Integer>  _Components = new ConcurrentHashMap<String,Integer>();

    /** Hashtabel keeps the names of the components as a key and value its type*/
    private  ConcurrentHashMap<String,String>  _CompTypes = new ConcurrentHashMap<String,String>();

    /** Hashtabel keeps the names of the components as a key and the CompMonitor object*/
    private ConcurrentHashMap<String, CompMonitor> _CompStatistics = new ConcurrentHashMap<String, CompMonitor>();

    /** HashMap contains the name and corresponding reporting intervals for configured components*/
    private ConcurrentHashMap<String,String> _CompReportInterval = new ConcurrentHashMap<String,String>();

    /** Nomber of components for the particular run*/
    private int _ComponentCount;

    /** Flag indicates that the system is reseted*/
    public volatile boolean _Reseted = true;

    /** This is in case a component was rebooted and the system will be forced/auto reseted*/
    public volatile boolean _AutoReseted = false;

    /** Hide/show eb/er bars*/
    public volatile boolean _hideEbErBars = true;

    /** File Chooser for COOL configuration file. */
    JFileChooser coolFileChooser;

    /** Selected cool configuration filename. */
    String coolFileName;

    /** Selected cool configuration file object. */
    File coolFile;

    /** State of the run*/
    private String _RunState = "undefined";
    private String _State = "undefined";

    /** AFECS platform UDL*/
    private String _PlatformcMsgServerUDL4Gui;

    /** AFECS platform cMsg host*/
    public static String  PCMSG_HOST;

    /** AFECS platform cMsg Tcp port*/
    public static String CMSG_PORT;

    /** AFECS platform cMsg Udp port*/
    public static String CMSG_BROADCAST_PORT;

    /** Current run/configuration name*/
    private String _RunType = "undefined";

    /** Current session of the run*/
    private String _Session = "undefined";

    /** The name of this UI*/
    public static String NAME;

    /** The session of the control run*/
    public static String SESSION;

    /** Defines the platform (cMsg server)*/
    public static String EXPID;

    /** Graphics style ( command line option -style)*/
    public static String GSTYLE;

    /** Image of the clock*/
    public static String CLOCK;

    /** Background style (plain for eclipse style)*/
    public static String BACKGROUND;

    /** Title color of the UI*/
    public static Color TITLE_COLOR;

    /** CODA basic transition image icons*/
    public static ImageIcon Configure;
    public static ImageIcon ConfigureOver;
    public static ImageIcon Reset;
    public static ImageIcon ResetOver;
    public static ImageIcon Pause;
    public static ImageIcon PauseOver;
    public static ImageIcon Resume;
    public static ImageIcon ResumeOver;
    public static ImageIcon Download;
    public static ImageIcon DownloadOver;
    public static ImageIcon Prestart;
    public static ImageIcon PrestartOver;
    public static ImageIcon Go;
    public static ImageIcon GoOver;
    public static ImageIcon End;
    public static ImageIcon EndOver;
    public static ImageIcon StartRun;
    public static ImageIcon StartRunOver;

    /** UI toolbox image icons*/
    public static ImageIcon JoinSession;
    public static ImageIcon JoinSessionOver;
    public static ImageIcon SetGraphSmoothingFactor;
    public static ImageIcon SetGraphSmoothingFactorOver;
    public static ImageIcon SetReportingInterval;
    public static ImageIcon SetReportingIntervalOver;
    public static ImageIcon TranslateDb2Cool;
    public static ImageIcon TranslateDb2CoolOver;
    public static ImageIcon Disconnect;
    public static ImageIcon DisconnectOver;
    public static ImageIcon HideEbEr;
    public static ImageIcon HideEbErOver;
    public static ImageIcon NoHideEbEr;
    public static ImageIcon NoHideEbErOver;
    public static ImageIcon AutoStart;
    public static ImageIcon AutoStartOver;
    public static ImageIcon NoAutoStart;
    public static ImageIcon NoAutoStartOver;
    public static ImageIcon EnableStartRun;
    public static ImageIcon EnableStartRunOver;
    public static ImageIcon DisableStartRun;
    public static ImageIcon DisableStartRunOver;
    public static ImageIcon EventLimit;
    public static ImageIcon EventLimitOver;
    public static ImageIcon NoEventLimit;
    public static ImageIcon NoEventLimitOver;
    public static ImageIcon MoreGraphics;
    public static ImageIcon MoreGraphicsOver;
    public static ImageIcon LessGraphics;
    public static ImageIcon LessGraphicsOver;

    /** Toolbox buttons*/
    public JButton BJoinSession;
    public JButton BSetGraphSmoothingFactor;
    public JButton BSetReportingInterval;
    public JButton BTranslateDb2Cool;
    public JButton BDisconnect;
    public JButton BHideEbEr;
    public JButton BNoHideEbEr;
    public JButton BAutoStart;
    public JButton BNoAutoStart;
    public JButton BEnableStartRun;
    public JButton BDisableStartRun;
    public JButton BEventLimit;
    public JButton BNoEventLimit;
    public JButton BMoreGraphics;
    public JButton BLessGraphics;

    /** Selected component*/
    public String selComp;

    private TimeGraphLite myGraph;
    private TimeGraphLite myEventRateTG;
    private TimeGraphLite myDataRateTG;
    private TimeGraphLite myLiveTimeTG;
    private BarGraphLite myDataRateBG;
    private BarGraphLite myTriggerBitsBG;

    /** Callback objects */
    private cMsgCallbackInterface infocb, warningcb, errorcb, tscb, hepaticcb, cortexcb, stemcellcb;

    /** cMsg subscription tmp objects*/
    private Object ob1, ob2, ob3, ob4, ob5, ob6, ob7;

    /**
     * main method
     *
     * @param args          Input arguments 
     */
    public static void main(String[] args){

        boolean help = false;

        if(args.length==0 || args.length == 1 ){
            System.out.println("Synopsis: rcgui -<option> <value>" +
                    "\n-h or -help" +
                    "\n-name <UI_component name> " +
                    "\n-session <session name> " +
                    "\n-expid <experiment id> " +
                    "\n-cMsgHost <host name> " +
                    "\n-cMsgTcpPort <port number> " +
                    "\n-cMsgUdpPort <port number>" +
                    "\n-style <gui style [green(default),blue,purple,yellow,lightblue]>" +
                    "\n\nN.B. -name is mandatory.");
            System.exit(-1);
        }
        else {

            if(args.length == 2) {
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;
            }

            if (args.length == 4){
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;

                if ( args[2].equals( "-cMsgTcpPort" ))
                    CMSG_PORT = args[3];
                else if ( args[2].equals( "-cMsgUdpPort" ))
                    CMSG_BROADCAST_PORT = args[3];
                else if ( args[2].equals( "-session" ))
                    SESSION = args[3];
                else if ( args[2].equals( "-cMsgHost" ))
                    PCMSG_HOST = args[3];
                else if ( args[2].equals( "-expid" ))
                    EXPID = args[3];
                else if (args[2].equals( "-style"))
                    GSTYLE = args[3];
                else help = true;
            }

            else if (args.length == 6){
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;

                if (args[2].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[3];
                else if (args[2].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[3];
                else if (args[2].equals( "-session"))
                    SESSION = args[3];
                else if (args[2].equals( "-cMsgHost"))
                    PCMSG_HOST = args[3];
                else if (args[2].equals( "-expid"))
                    EXPID = args[3];
                else if (args[2].equals( "-style"))
                    GSTYLE = args[3];
                else help = true;

                if (args[4].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[5];
                else if (args[4].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[5];
                else if (args[4].equals( "-session"))
                    SESSION = args[5];
                else if (args[4].equals( "-cMsgHost"))
                    PCMSG_HOST = args[5];
                else if (args[4].equals( "-expid"))
                    EXPID = args[5];
                else if (args[4].equals( "-style"))
                    GSTYLE = args[5];
                else help = true;
            }

            else if (args.length == 8) {
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;

                if (args[2].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[3];
                else if (args[2].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[3];
                else if (args[2].equals( "-session"))
                    SESSION = args[3];
                else if (args[2].equals( "-cMsgHost"))
                    PCMSG_HOST = args[3];
                else if (args[2].equals( "-expid"))
                    EXPID = args[3];
                else if (args[2].equals( "-style"))
                    GSTYLE = args[3];
                else help = true;

                if (args[4].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[5];
                else if (args[4].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[5];
                else if (args[4].equals( "-session"))
                    SESSION = args[5];
                else if (args[4].equals( "-cMsgHost"))
                    PCMSG_HOST = args[5];
                else if (args[4].equals( "-expid"))
                    EXPID = args[5];
                else if (args[4].equals( "-style"))
                    GSTYLE = args[5];
                else help = true;

                if (args[6].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[7];
                else if (args[6].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[7];
                else if (args[6].equals( "-session"))
                    SESSION = args[7];
                else if (args[6].equals( "-cMsgHost"))
                    PCMSG_HOST = args[7];
                else if (args[6].equals( "-expid"))
                    EXPID = args[7];
                else if (args[6].equals( "-style"))
                    GSTYLE = args[7];
                else help = true;
            }


            else if (args.length == 10){
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;

                if (args[2].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[3];
                else if (args[2].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[3];
                else if (args[2].equals( "-session"))
                    SESSION = args[3];
                else if (args[2].equals( "-cMsgHost"))
                    PCMSG_HOST = args[3];
                else if (args[2].equals( "-expid"))
                    EXPID = args[3];
                else if (args[2].equals( "-style"))
                    GSTYLE = args[3];
                else help = true;

                if (args[4].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[5];
                else if (args[4].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[5];
                else if (args[4].equals( "-session"))
                    SESSION = args[5];
                else if (args[4].equals( "-cMsgHost"))
                    PCMSG_HOST = args[5];
                else if (args[4].equals( "-expid"))
                    EXPID = args[5];
                else if (args[4].equals( "-style"))
                    GSTYLE = args[5];
                else help = true;

                if (args[6].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[7];
                else if (args[6].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[7];
                else if (args[6].equals( "-session"))
                    SESSION = args[7];
                else if (args[6].equals( "-cMsgHost"))
                    PCMSG_HOST = args[7];
                else if (args[6].equals( "-expid"))
                    EXPID = args[7];
                else if (args[6].equals( "-style"))
                    GSTYLE = args[7];
                else help = true;

                if (args[8].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[9];
                else if (args[8].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[9];
                else if (args[8].equals( "-session"))
                    SESSION = args[9];
                else if (args[8].equals( "-cMsgHost"))
                    PCMSG_HOST = args[9];
                else if (args[8].equals( "-expid"))
                    EXPID = args[9];
                else if (args[8].equals( "-style"))
                    GSTYLE = args[9];
                else help = true;
            }


            else if (args.length == 12){
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;

                if (args[2].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[3];
                else if (args[2].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[3];
                else if (args[2].equals( "-session"))
                    SESSION = args[3];
                else if (args[2].equals( "-cMsgHost"))
                    PCMSG_HOST = args[3];
                else if (args[2].equals( "-expid"))
                    EXPID = args[3];
                else if (args[2].equals( "-style"))
                    GSTYLE = args[3];
                else help = true;

                if (args[4].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[5];
                else if (args[4].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[5];
                else if (args[4].equals( "-session"))
                    SESSION = args[5];
                else if (args[4].equals( "-cMsgHost"))
                    PCMSG_HOST = args[5];
                else if (args[4].equals( "-expid"))
                    EXPID = args[5];
                else if (args[4].equals( "-style"))
                    GSTYLE = args[5];
                else help = true;

                if (args[6].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[7];
                else if (args[6].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[7];
                else if (args[6].equals( "-session"))
                    SESSION = args[7];
                else if (args[6].equals( "-cMsgHost"))
                    PCMSG_HOST = args[7];
                else if (args[6].equals( "-expid"))
                    EXPID = args[7];
                else if (args[6].equals( "-style"))
                    GSTYLE = args[7];
                else help = true;

                if (args[8].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[9];
                else if (args[8].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[9];
                else if (args[8].equals( "-session"))
                    SESSION = args[9];
                else if (args[8].equals( "-cMsgHost"))
                    PCMSG_HOST = args[9];
                else if (args[8].equals( "-expid"))
                    EXPID = args[9];
                else if (args[8].equals( "-style"))
                    GSTYLE = args[9];
                else help = true;

                if (args[10].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[11];
                else if (args[10].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[11];
                else if (args[10].equals( "-session"))
                    SESSION = args[11];
                else if (args[10].equals( "-cMsgHost"))
                    PCMSG_HOST = args[11];
                else if (args[10].equals( "-expid"))
                    EXPID = args[11];
                else if (args[10].equals( "-style"))
                    GSTYLE = args[11];
                else help = true;
            }


            else if (args.length == 14) {
                if(args[0].equals("-name"))
                    NAME = args[1];
                else help = true;

                if (args[2].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[3];
                else if (args[2].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[3];
                else if (args[2].equals( "-session"))
                    SESSION = args[3];
                else if (args[2].equals( "-cMsgHost"))
                    PCMSG_HOST = args[3];
                else if (args[2].equals( "-expid"))
                    EXPID = args[3];
                else if (args[2].equals( "-style"))
                    GSTYLE = args[3];
                else help = true;

                if (args[4].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[5];
                else if (args[4].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[5];
                else if (args[4].equals( "-session"))
                    SESSION = args[5];
                else if (args[4].equals( "-cMsgHost"))
                    PCMSG_HOST = args[5];
                else if (args[4].equals( "-expid"))
                    EXPID = args[5];
                else if (args[4].equals( "-style"))
                    GSTYLE = args[5];
                else help = true;

                if (args[6].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[7];
                else if (args[6].equals( "-cMsgUdpPort") )
                    CMSG_BROADCAST_PORT =  args[7];
                else if (args[6].equals( "-session"))
                    SESSION = args[7];
                else if (args[6].equals( "-cMsgHost"))
                    PCMSG_HOST = args[7];
                else if (args[6].equals( "-expid"))
                    EXPID = args[7];
                else if (args[6].equals( "-style"))
                    GSTYLE = args[7];
                else help = true;

                if (args[8].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[9];
                else if (args[8].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[9];
                else if (args[8].equals( "-session"))
                    SESSION = args[9];
                else if (args[8].equals( "-cMsgHost"))
                    PCMSG_HOST = args[9];
                else if (args[8].equals( "-expid"))
                    EXPID = args[9];
                else if (args[8].equals( "-style"))
                    GSTYLE = args[9];
                else help = true;

                if (args[10].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[11];
                else if (args[10].equals( "-cMsgUdpPort") )
                    CMSG_BROADCAST_PORT =  args[11];
                else if (args[10].equals( "-session"))
                    SESSION = args[11];
                else if (args[10].equals( "-cMsgHost"))
                    PCMSG_HOST = args[11];
                else if (args[10].equals( "-expid"))
                    EXPID = args[11];
                else if (args[10].equals( "-style"))
                    GSTYLE = args[11];
                else help = true;

                if (args[12].equals( "-cMsgTcpPort"))
                    CMSG_PORT = args[13];
                else if (args[12].equals( "-cMsgUdpPort"))
                    CMSG_BROADCAST_PORT =  args[13];
                else if (args[12].equals( "-session"))
                    SESSION = args[13];
                else if (args[12].equals( "-cMsgHost"))
                    PCMSG_HOST = args[13];
                else if (args[12].equals( "-expid"))
                    EXPID = args[13];
                else if (args[12].equals( "-style"))
                    GSTYLE = args[13];
                else help = true;
            }
            if(help){
                System.out.println("Synopsis: rcgui -<option> <value>" +
                        "\n-h or -help" +
                        "\n-name <UI_component name> " +
                        "\n-session <session name> " +
                        "\n-expid <experiment id> " +
                        "\n-cMsgHost <host name> " +
                        "\n-cMsgTcpPort <port number> " +
                        "\n-cMsgUdpPort <port number>" +
                        "\n-style <gui style [green(default),blue,purple,yellow,lightblue]>" +
                        "\n\nN.B. -name is mandatory.");
                System.exit(-1);
            }
        }
        new RcGui();
    }

    /** Constructor
     * Here we complete three subscriptions, and create the callback objects for them:
     * info, warning, errror and tsdata
     *
     *
     * */
    public RcGui() {

        super();
        myName = NAME;
        if(SESSION!=null)_Session = SESSION;
        else
            _Session = System.getenv("SESSION");
        if(_Session == null) _Session = "undefined";

        if(EXPID==null) EXPID = System.getenv("EXPID");
        if(EXPID ==null) {
            System.out.println("Fatal-error: EXPID is not defined. " +
                    "\nDefine it as an env variable or through the command line interface.\n");
            System.out.println("Synopsis: rcgui -<option> <value>" +
                    "\n-h or -help" +
                    "\n-name <UI_component name> " +
                    "\n-session <session name> " +
                    "\n-expid <experiment id> " +
                    "\n-cMsgHost <host name> " +
                    "\n-cMsgTcpPort <port number> " +
                    "\n-cMsgUdpPort <port number>" +
                    "\n-style <gui style [green(default),blue,purple,yellow,lightblue]>" +
                    "\n\nN.B. -name is mandatory.");

            System.exit(-1);
        }

        if(CMSG_PORT==null)CMSG_PORT = Integer.toString(cMsgNetworkConstants.tcpServerPort);
        if(CMSG_BROADCAST_PORT==null)CMSG_BROADCAST_PORT = Integer.toString(cMsgNetworkConstants.nameServerBroadcastPort);
        if(PCMSG_HOST == null) {
            PCMSG_HOST = "broadcast";
            _PlatformcMsgServerUDL4Gui = "cMsg:cMsg://broadcast:"+CMSG_BROADCAST_PORT+"/cMsg/"+ EXPID +"/?broadcastTO=5";
            System.out.println("Starting RcGui..."+
                    "\nExperiment ID   = "+ EXPID +
                    "\ncMsgHost        = "+PCMSG_HOST+
                    "\ncMsgUdpPort     = "+CMSG_BROADCAST_PORT);
        }
        else{
            _PlatformcMsgServerUDL4Gui = "cMsg:cMsg://"+PCMSG_HOST+":"+CMSG_PORT+"/cMsg/"+ EXPID;

            System.out.println("Starting RcGui..."+
                    "\nExperiment ID   = "+ EXPID +
                    "\ncMsgHost        = "+PCMSG_HOST+
                    "\ncMsgTcpPort     = "+CMSG_PORT);
        }

// GUI style
// Unspecified/default = plain (eclipse style)
        if(GSTYLE==null){
            GSTYLE = "plain";
            BACKGROUND = "plain";
            TITLE_COLOR = new java.awt.Color(140, 180, 200);
            Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/configure.png","Configure button");
            ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/configureOver.png","Configure button");
            Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/reset.png","Reset button");
            ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/resetOver.png","Reset button");
            Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/pause.png","Pause button");
            PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/pauseOver.png","Pause button");
            Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/resume.png","Resume button");
            ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/resumeOver.png","Resume button");
            Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/download.png","Download button");
            DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/downloadOver.png","Download button");
            Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/prestart.png","Prestart button");
            PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/prestartOver.png","Prestart button");
            Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/go.png","Go button");
            GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/goOver.png","Go button");
            End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/end.png","End button");
            EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/endOver.png","End button");
            StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/startrun.png","StartRun button");
            StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/startRunOver.png","StartRun button");

// Toolbox icons
            JoinSession = new ImageIcon(System.getenv("AFECS_HOME")+"/images/JoinSession.png","");
            JoinSessionOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/JoinSessionOver.png","");
            SetGraphSmoothingFactor = new ImageIcon(System.getenv("AFECS_HOME")+"/images/SetGraphSmoothingFactor.png","");
            SetGraphSmoothingFactorOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/SetGraphSmoothingFactorOver.png","");
            SetReportingInterval = new ImageIcon(System.getenv("AFECS_HOME")+"/images/SetReportingInterval.png","");
            SetReportingIntervalOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/SetReportingIntervalOver.png","");
            TranslateDb2Cool = new ImageIcon(System.getenv("AFECS_HOME")+"/images/TranslateDb2Cool.png","");
            TranslateDb2CoolOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/TranslateDb2CoolOver.png","");
            Disconnect = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Disconnect.png","");
            DisconnectOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/DisconnectOver.png","");
            HideEbEr = new ImageIcon(System.getenv("AFECS_HOME")+"/images/HideEbEr.png","");
            HideEbErOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/HideEbErOver.png","");
            NoHideEbEr = new ImageIcon(System.getenv("AFECS_HOME")+"/images/NoHideEbEr.png","");
            NoHideEbErOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/NoHideEbErOver.png","");
            AutoStart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/AutoStart.png","");
            AutoStartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/AutoStartOver.png","");
            NoAutoStart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/NoAutoStart.png","");
            NoAutoStartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/NoAutoStartOver.png","");
            EnableStartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/EnableStartRun.png","");
            EnableStartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/EnableStartRunOver.png","");
            DisableStartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/DisableStartRun.png","");
            DisableStartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/DisableStartRunOver.png","");
            EventLimit = new ImageIcon(System.getenv("AFECS_HOME")+"/images/EventLimit.png","");
            EventLimitOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/EventLimitOver.png","");
            NoEventLimit = new ImageIcon(System.getenv("AFECS_HOME")+"/images/NoEventLimit.png","");
            NoEventLimitOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/NoEventLimitOver.png","");
            MoreGraphics = new ImageIcon(System.getenv("AFECS_HOME")+"/images/MoreGraphics.png","");
            MoreGraphicsOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/MoreGraphicsOver.png","");
            LessGraphics = new ImageIcon(System.getenv("AFECS_HOME")+"/images/LessGraphics.png","");
            LessGraphicsOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/LessGraphicsOver.png","");

        }else{
// Blue
            if(GSTYLE.equalsIgnoreCase("blue")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/blue.jpg";
                TITLE_COLOR = new java.awt.Color(0x66, 0xFF, 0xFF);
                Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BConfigure.jpg","Configure button");
                ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BConfigureOver.jpg","Configure button");
                Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BReset.jpg","Reset button");
                ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BResetOver.jpg","Reset button");
                Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPause.jpg","Pause button");
                PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPauseOver.jpg","Pause button");
                Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BResume.jpg","Resume button");
                ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BResumeOver.jpg","Resume button");
                Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BDownload.jpg","Download button");
                DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BDownloadOver.jpg","Download button");
                Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPrestart.jpg","Prestart button");
                PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPrestartOver.jpg","Prestart button");
                Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BGo.jpg","Go button");
                GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BGoOver.jpg","Go button");
                End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BEnd.jpg","End button");
                EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BEndOver.jpg","End button");
                StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BStartRun.jpg","StartRun button");
                StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BStartRunOver.jpg","StartRun button");
// Lightblue
            }else if(GSTYLE.equalsIgnoreCase("lightblue")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/lightblue.jpg";
                TITLE_COLOR = new java.awt.Color(0x00, 0x33, 0x66);
                Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BConfigure.jpg","Configure button");
                ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BConfigureOver.jpg","Configure button");
                Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BReset.jpg","Reset button");
                ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BResetOver.jpg","Reset button");
                Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPause.jpg","Pause button");
                PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPauseOver.jpg","Pause button");
                Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BResume.jpg","Resume button");
                ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BResumeOver.jpg","Resume button");
                Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BDownload.jpg","Download button");
                DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BDownloadOver.jpg","Download button");
                Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPrestart.jpg","Prestart button");
                PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BPrestartOver.jpg","Prestart button");
                Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BGo.jpg","Go button");
                GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BGoOver.jpg","Go button");
                End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BEnd.jpg","End button");
                EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BEndOver.jpg","End button");
                StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BStartRun.jpg","StartRun button");
                StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/BStartRunOver.jpg","StartRun button");
// Green
            }else if(GSTYLE. equalsIgnoreCase("green")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/Green-Growth.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
                Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Configure.jpg","Configure button");
                ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/ConfigureOver.jpg","Configure button");
                Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Reset.jpg","Reset button");
                ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/ResetOver.jpg","Reset button");
                Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Pause.jpg","Pause button");
                PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PauseOver.jpg","Pause button");
                Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Resume.jpg","Resume button");
                ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/ResumeOver.jpg","Resume button");
                Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Download.jpg","Download button");
                DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/DownloadOver.jpg","Download button");
                Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Prestart.jpg","Prestart button");
                PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PrestartOver.jpg","Prestart button");
                Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Go.jpg","Go button");
                GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/GoOver.jpg","Go button");
                End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/End.jpg","End button");
                EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/EndOver.jpg","End button");
                StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/StarRun.jpg","StartRun button");
                StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/StarRunOver.jpg","StartRun button");
// Purple
            }else if(GSTYLE.equalsIgnoreCase("purple")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/purple.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
                Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PConfigure.jpg","Configure button");
                ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PConfigureOver.jpg","Configure button");
                Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PReset.jpg","Reset button");
                ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PResetOver.jpg","Reset button");
                Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PPause.jpg","Pause button");
                PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PPauseOver.jpg","Pause button");
                Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PResume.jpg","Resume button");
                ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PResumeOver.jpg","Resume button");
                Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PDownload.jpg","Download button");
                DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PDownloadOver.jpg","Download button");
                Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PPrestart.jpg","Prestart button");
                PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PPrestartOver.jpg","Prestart button");
                Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PGo.jpg","Go button");
                GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PGoOver.jpg","Go button");
                End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PEnd.jpg","End button");
                EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PEndOver.jpg","End button");
                StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PStartRun.jpg","StartRun button");
                StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PStartRunOver.jpg","StartRun button");
// Yellow
            }else if(GSTYLE.equalsIgnoreCase("yellow")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/yellow.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
                Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YConfigure.jpg","Configure button");
                ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YConfigureOver.jpg","Configure button");
                Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YReset.jpg","Reset button");
                ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YResetOver.jpg","Reset button");
                Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YPause.jpg","Pause button");
                PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YPauseOver.jpg","Pause button");
                Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YResume.jpg","Resume button");
                ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YResumeOver.jpg","Resume button");
                Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YDownload.jpg","Download button");
                DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YDownloadOver.jpg","Download button");
                Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YPrestart.jpg","Prestart button");
                PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YPrestartOver.jpg","Prestart button");
                Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YGo.jpg","Go button");
                GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YGoOver.jpg","Go button");
                End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YEnd.jpg","End button");
                EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YEndOver.jpg","End button");
                StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YStartRun.jpg","StartRun button");
                StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/YStartRunOver.jpg","StartRun button");
// If you typed something but not from the list - Green
            }else {
                GSTYLE = "green";
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/Green-Growth.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
                Configure = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Configure.jpg","Configure button");
                ConfigureOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/ConfigureOver.jpg","Configure button");
                Reset = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Reset.jpg","Reset button");
                ResetOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/ResetOver.jpg","Reset button");
                Pause = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Pause.jpg","Pause button");
                PauseOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PauseOver.jpg","Pause button");
                Resume = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Resume.jpg","Resume button");
                ResumeOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/ResumeOver.jpg","Resume button");
                Download = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Download.jpg","Download button");
                DownloadOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/DownloadOver.jpg","Download button");
                Prestart = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Prestart.jpg","Prestart button");
                PrestartOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/PrestartOver.jpg","Prestart button");
                Go = new ImageIcon(System.getenv("AFECS_HOME")+"/images/Go.jpg","Go button");
                GoOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/GoOver.jpg","Go button");
                End = new ImageIcon(System.getenv("AFECS_HOME")+"/images/End.jpg","End button");
                EndOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/EndOver.jpg","End button");
                StartRun = new ImageIcon(System.getenv("AFECS_HOME")+"/images/StarRun.jpg","StartRun button");
                StartRunOver = new ImageIcon(System.getenv("AFECS_HOME")+"/images/StarRunOver.jpg","StartRun button");
            }
        }

        CLOCK = System.getenv("AFECS_HOME")+"/images/clockT.gif";
        initComponents();

// Connect to cMsg server for getting messages from Medulla
        try {
            cMsgSrv = new cMsg(_PlatformcMsgServerUDL4Gui, myName, rcSubscription.PlatformcMsgServerDescription);
            cMsgSrv.connect();
            addConsoleLine("Connected to the server.","info");

// Enable message reception
            cMsgSrv.start();

// Subscribe to RcGuiSubjInfo and type=*
            infocb = new InfoCallback();
            ob1 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjInfo+"_"+_Session, "*", infocb, null);

// Subscribe to RcGuiSubjWarning and type=*
            warningcb = new WarningCallback();
            ob2 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjWarning+"_"+_Session, "*", warningcb, null);

// Subscribe to RcGuiSubjError and type=*
            errorcb = new ErrorCallback();
            ob3 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjError+"_"+_Session, "*", errorcb, null);

// Subscribe to RcGuiSubjTsData and type=*
            tscb = new TsDataCallback();
            ob4 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjTsData+"_"+_Session, "*", tscb, null);

// Subscribe to Hepatic messages and type=*
            hepaticcb = new HepaticCallback();
            ob5 = cMsgSrv.subscribe(rcSubscription.Hepatic, "*", hepaticcb, null);

// Subscribe to Cortex messages and type=*
            cortexcb = new CortexCallback();
            ob6 = cMsgSrv.subscribe(rcSubscription.CrtSubject, "*", cortexcb, null);

// Subscribe to Stemcell messages and type=*
            stemcellcb = new StemcellCallback();
            ob7 = cMsgSrv.subscribe(rcSubscription.Stemcell, "*", stemcellcb, null);
        } catch (cMsgException e) {
            e.printStackTrace();
            System.exit(0);
        }
        if(BACKGROUND.equalsIgnoreCase("plain"))setSize(566,883);
        else  setSize(570,970);
        setVisible(true);
        setTextEditable(false);
        setDatabase(EXPID);
        setSession(_Session);
        reportRuntype();

// Start cMsg server connection testing thread.
        HeartBeat myheart = new HeartBeat();
        myheart.start();
    }


    /**
     * Asks Cortex to report the runtype
     *
     */
    public void reportRuntype(){

// Send cMsg to Cortex
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(rcSubscription.CrtSubject);
        msg.setType(rcSubscription.CrtType);
        msg.setText(rcConstants.CMD_ReportActiveRunType+" "+_Session);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
    }


    /**
     * Sends kill message to the supervisor agent, which will
     * broadcst through ACL the same message to all agents.
     * Agents will  pass kill command to their physical components.
     */
    public void killComponents(){

// Reset first by sending message to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_reset);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Sending messgae to Medulla
        msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_componentKill);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Modiy the gui
        stopClock();
        setRunState("Killed");
        addConsoleLine("Components receive kill command.","warning");
        try {
            startInfo("Killed", "Info");} catch (Exception e){e.printStackTrace();}
        AbortButton.setEnabled(false);
        ConfigureButton.setEnabled(true);
        PauseButton.setEnabled(false);
        DownloadButton.setEnabled(false);
        PrestartButton.setEnabled(false);
        GoButton.setEnabled(false);
        EndButton.setEnabled(false);
        setStartTime("");
        setEndTime("");
        setEventNumber("");
        setGEvtRate(0.0);
        setGDataRate(0.0);
        setGAverEvtRate(0.0);
        setGAverDataRate(0.0);
    }


    /**
     * Asks Cortex to disconnect the runtype
     *
     * @param medullaname                the name of the supervisor agent for that SESSION and runtype
     *                                   N.B. constructed like "SESSION_runtype"
     *
     */
    public void disconnectRuntype(String medullaname){

// Reset first by sending message to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_reset);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Send disconnect cMsg to Cortex
        msg = new cMsgMessage();
        msg.setSubject(rcSubscription.CrtSubject);
        msg.setType(rcSubscription.CrtType);
        msg.setText(rcConstants.CMD_DisconnectRunType+" "+medullaname+" "+_Session);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
        stopClock();
        setRunState("Disconnected");
        addConsoleLine("Disconnected "+_RunType+" from the "+_Session,"warning");
        try {
            startInfo("Disconnected", "Info");} catch (Exception e){e.printStackTrace();}
        AbortButton.setEnabled(false);
        ConfigureButton.setEnabled(true);
        PauseButton.setEnabled(false);
        DownloadButton.setEnabled(false);
        PrestartButton.setEnabled(false);
        StartRunButton.setEnabled(false);
        startrun = false;
        startbuttonclicked = false;
        GoButton.setEnabled(false);
        EndButton.setEnabled(false);
        setStartTime("");
        setEndTime("");
        setEventNumber("");
        setGEvtRate(0.0);
        setGDataRate(0.0);
        setGAverEvtRate(0.0);
        setGAverDataRate(0.0);
    }


    /**
     * Ask Medulla to report component names, component count, runnumber , etc.
     *
     * @param runt              Runtype of the experiment
     */
    public void reportConfiguration(String runt){

        cMsgMessage msg;

// Ask to send the runnumber
        msg = new cMsgMessage();
        msg.setSubject(runt);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_ReportRunNumber);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Ask to send the run status
        msg = new cMsgMessage();
        msg.setSubject(runt);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_ReportRunStatus);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Ask to send the names of the configured components
        msg = new cMsgMessage();
        msg.setSubject(runt);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_ReportComponents);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

    }


    /**
     * This method is called from within the constructor to initialize the form.
     *
     * */
    private void initComponents() {

        if(BACKGROUND.equalsIgnoreCase("plain")){
            MainPanel = new JPanel();
            ControlPanel = new JPanel();
            TransitionPanel = new JPanel();
            DatabasePanel = new JPanel();
            SessionPanel = new JPanel();
            ConfigurationPanel = new JPanel();
            RunNumberPanel = new JPanel();
            EndTimePanel = new JPanel();
            StartTimePanel = new JPanel();
            EventsPanel = new JPanel();
            DataLimitPanel = new JPanel();
            RunStatePanel = new JPanel();
            ERFilePanel = new JPanel();
            EventsThisRunPanel = new JPanel();
            EventRateDifPanel = new JPanel();
            EventRateIntPanel = new JPanel();
            DataRateDifPanel = new JPanel();
            DataRateIntPanel = new JPanel();
            TimeGraphPane = new ImageScrollPane();
            dataRateBGPane = new ImageScrollPane();
            liveTimeTGPane = new ImageScrollPane();
            DataRateTGPane = new ImageScrollPane();
            EventRateTGPane = new ImageScrollPane();
            BJoinSession = new JButton();
            BSetGraphSmoothingFactor = new JButton();
            BSetReportingInterval = new JButton();
            BTranslateDb2Cool = new JButton();
            BDisconnect = new JButton();
            BHideEbEr = new JButton();
            BNoHideEbEr = new JButton();
            BAutoStart = new JButton();
            BNoAutoStart = new JButton();
            BEnableStartRun = new JButton();
            BDisableStartRun = new JButton();
            BEventLimit = new JButton();
            BNoEventLimit = new JButton();
            BMoreGraphics = new JButton();
            BLessGraphics = new JButton();
        } else{
            MainPanel = new ImagePanel(BACKGROUND);
            ControlPanel = new ImagePanel(BACKGROUND);
            TransitionPanel = new ImagePanel(BACKGROUND);
            DatabasePanel = new ImagePanel(BACKGROUND);
            SessionPanel = new ImagePanel(BACKGROUND);
            ConfigurationPanel = new ImagePanel(BACKGROUND);
            RunNumberPanel = new ImagePanel(BACKGROUND);
            EndTimePanel = new ImagePanel(BACKGROUND);
            StartTimePanel = new ImagePanel(BACKGROUND);
            EventsPanel = new ImagePanel(BACKGROUND);
            DataLimitPanel = new ImagePanel(BACKGROUND);
            RunStatePanel = new ImagePanel(BACKGROUND);
            ERFilePanel = new ImagePanel(BACKGROUND);
            EventsThisRunPanel = new ImagePanel(BACKGROUND);
            EventRateDifPanel = new ImagePanel(BACKGROUND);
            EventRateIntPanel = new ImagePanel(BACKGROUND);
            DataRateDifPanel = new ImagePanel(BACKGROUND);
            DataRateIntPanel = new ImagePanel(BACKGROUND);
            TimeGraphPane = new ImageScrollPane(BACKGROUND);
            dataRateBGPane = new ImageScrollPane(BACKGROUND);
            liveTimeTGPane = new ImageScrollPane(BACKGROUND);
            DataRateTGPane = new ImageScrollPane(BACKGROUND);
            EventRateTGPane = new ImageScrollPane(BACKGROUND);
        }
        ImageIcon clock = new ImageIcon(CLOCK);
        EmptyLabel = new javax.swing.JLabel();
        listModel = new DefaultListModel();
        ControlTransitionPane = new javax.swing.JLayeredPane();
        ConfigureButton = new javax.swing.JButton();
        AbortButton = new javax.swing.JButton();
        PauseButton = new javax.swing.JButton();
        DownloadButton = new javax.swing.JButton();
        PrestartButton = new javax.swing.JButton();
        StartRunButton = new javax.swing.JButton();
        GoButton = new javax.swing.JButton();
        EndButton = new javax.swing.JButton();
        StaticPArametersPane = new javax.swing.JLayeredPane();
        DatabaseTextField = new javax.swing.JTextField();
        SessionTextField = new javax.swing.JTextField();
        ConfigurationTextField = new javax.swing.JTextField();
        RunStatusPane = new javax.swing.JLayeredPane();
        RunNumberTextField = new javax.swing.JTextField();
        EndTimeTextField = new javax.swing.JTextField();
        StartTimeTextField = new javax.swing.JTextField();
        LimitEventsTextField = new javax.swing.JTextField();
        DataLimitTextField = new javax.swing.JTextField();
        RunStatusTextField = new javax.swing.JTextField();
        ERFileTextField = new javax.swing.JTextField();
        RunProgressPane = new javax.swing.JLayeredPane();
        EventThisRunTextField = new javax.swing.JTextField();
        EventRateDifTextField = new javax.swing.JTextField();
        EventRateIntTextField = new javax.swing.JTextField();
        DataRateDifTextField = new javax.swing.JTextField();
        DataRateIntTextField = new javax.swing.JTextField();
        MessagesScrollPane = new javax.swing.JScrollPane();
        MessagesTextPane = new javax.swing.JTextPane();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        openMenuItem = new javax.swing.JMenuItem();
        setReportIntervalMenuItem = new javax.swing.JMenuItem();
        ResetMenuItem = new javax.swing.JMenuItem();
        CExitMenuItem = new javax.swing.JMenuItem();
        setAiMenuItem = new javax.swing.JMenuItem();
        saveMenuItem = new javax.swing.JMenuItem();
        saveAsMenuItem = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        db2coolMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        enableEasyStart = new javax.swing.JRadioButtonMenuItem();
        disableEasyStart = new javax.swing.JRadioButtonMenuItem();
        enableEvLimMenuItem = new javax.swing.JRadioButtonMenuItem();
        disableEvLimMenuItem = new javax.swing.JRadioButtonMenuItem();
        enableAutoStartMenuItem = new javax.swing.JRadioButtonMenuItem();
        disableAutoStartMenuItem = new javax.swing.JRadioButtonMenuItem();
        moreGraphsMenuItem = new javax.swing.JRadioButtonMenuItem();
        lessGraphsMenuItem = new javax.swing.JRadioButtonMenuItem();
        dalogBrowserMenuItem = new javax.swing.JMenuItem();
        hideEBERBarsMenuItem = new javax.swing.JRadioButtonMenuItem();
        showEBERBarsMenuItem = new javax.swing.JRadioButtonMenuItem();
        copyMenuItem = new javax.swing.JMenuItem();
        pasteMenuItem = new javax.swing.JMenuItem();
        deleteMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        contentsMenuItem = new javax.swing.JMenuItem();
        aboutMenuItem = new javax.swing.JMenuItem();

        ClockLabel = new javax.swing.JLabel(" ",clock,SwingConstants.CENTER);
        ClockLabel.setBounds(210, 110, 45, 15);

        InfoLabel = new javax.swing.JLabel(" ",SwingConstants.CENTER);
        InfoLabel.setBounds(210, 110, 45, 15);

        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints m = new GridBagConstraints();
        m.weighty = 1.0;
        m.weightx = 1.0;
        m.fill=GridBagConstraints.BOTH;

// Mainpanel holds all gui components except info terminal
        MainPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setForeground(java.awt.Color.white);

        if(!BACKGROUND.equalsIgnoreCase("plain")){
            javax.swing.border.TitledBorder bd = new javax.swing.border.TitledBorder(
                    new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED), "Run Control Buttons");
            bd.setTitleColor(TITLE_COLOR);

// Control transition pane
            ControlTransitionPane.setBorder(bd);

//Controlpanel includes 3 bottons configure,reset, pause/resume
            ControlPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
            bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Control");

            bd.setTitleColor(TITLE_COLOR);
            ControlPanel.setBorder(new javax.swing.border.TitledBorder(bd));

// Configure button
            ConfigureButton.setFont(new java.awt.Font("Dialog", 1, 12));
            ConfigureButton.setBorderPainted(false);
            ConfigureButton.setMargin(new Insets(-3, -3, -3, -3));
            ConfigureButton.setFocusPainted(false);
            ConfigureButton.setContentAreaFilled(false);
            ConfigureButton.setIcon(Configure);
            ConfigureButton.setRolloverIcon(ConfigureOver);
            ConfigureButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    ConfigureButtonClicked(evt);
                }
            });
            ControlPanel.add(ConfigureButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 15, 100, -1));

// Reset button
            AbortButton.setFont(new java.awt.Font("Dialog", 1, 12));
            AbortButton.setBorderPainted(false);
            AbortButton.setMargin(new Insets(-3, -3, -3, -3));
            AbortButton.setFocusPainted(false);
            AbortButton.setContentAreaFilled(false);
            AbortButton.setIcon(Reset);
            AbortButton.setRolloverIcon(ResetOver);
            AbortButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    AbortButtonClicked(evt);
                }
            });
            ControlPanel.add(AbortButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 45, 100, -1));

// Pause button
            PauseButton.setFont(new java.awt.Font("Dialog", 1, 12));
            PauseButton.setBorderPainted(false);
            PauseButton.setMargin(new Insets(-3, -3, -3, -3));
            PauseButton.setFocusPainted(false);
            PauseButton.setContentAreaFilled(false);
            PauseButton.setIcon(Pause);
            PauseButton.setRolloverIcon(PauseOver);
            PauseButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    PauseButtonClicked(evt);
                }
            });
            ControlPanel.add(PauseButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 75, 100, -1));
            ControlPanel.setBounds(10, 20, 120, 110);

            TransitionPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

            bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Transition");
            bd.setTitleColor(TITLE_COLOR);
            TransitionPanel.setBorder(new javax.swing.border.TitledBorder(bd));

// Download button
            DownloadButton.setFont(new java.awt.Font("Dialog", 1, 12));
            DownloadButton.setBorderPainted(false);
            DownloadButton.setMargin(new Insets(-3, -3, -3, -3));
            DownloadButton.setFocusPainted(false);
            DownloadButton.setContentAreaFilled(false);
            DownloadButton.setIcon(Download);
            DownloadButton.setRolloverIcon(DownloadOver);
            DownloadButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    DownloadButtonClicked(evt);
//                    PrestartButton.setIcon(Prestart);
//                    PrestartButton.setRolloverIcon(PrestartOver);
                    PrestartButton.setEnabled(false);
                }
            });
            TransitionPanel.add(DownloadButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 90, 80));

// Prestart button
            PrestartButton.setFont(new java.awt.Font("Dialog", 1, 12));
            PrestartButton.setBorderPainted(false);
            PrestartButton.setMargin(new Insets(-3, -3, -3, -3));
            PrestartButton.setFocusPainted(false);
            PrestartButton.setContentAreaFilled(false);
            PrestartButton.setIcon(Prestart);
            PrestartButton.setRolloverIcon(PrestartOver);
            PrestartButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    if(startrun){
                        startbuttonclicked = true;
                    if((_State.equalsIgnoreCase(rcConstants.CMD_postend_ok)) ||
                            (_State.equalsIgnoreCase(rcConstants.CMD_postdownload_ok))){
                        DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        startPrestart();
                    }
                    else if((_State.equalsIgnoreCase("Configured")) ||
                            (_State.equalsIgnoreCase(rcConstants.CMD_reset_ok))){
                        DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        startDownload();
                    }
                    }else {
                        PrestartButton.setIcon(Prestart);
                        PrestartButton.setRolloverIcon(PrestartOver);
                        PrestartButtonClicked(evt);
                    }
                }
            });
            TransitionPanel.add(PrestartButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(110, 20, 90, 80));

// Go button
            GoButton.setFont(new java.awt.Font("Dialog", 1, 12));
            GoButton.setBorderPainted(false);
            GoButton.setMargin(new Insets(-3, -3, -3, -3));
            GoButton.setFocusPainted(false);
            GoButton.setContentAreaFilled(false);
            GoButton.setIcon(Go);
            GoButton.setRolloverIcon(GoOver);
            GoButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    GoButtonClicked(evt);
//                    PrestartButton.setIcon(Prestart);
//                    PrestartButton.setRolloverIcon(PrestartOver);
//                    PrestartButton.setEnabled(false);
                }
            });
            TransitionPanel.add(GoButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(210, 20, 90, 80));

            EndButton.setFont(new java.awt.Font("Dialog", 1, 12));
            EndButton.setBorderPainted(false);
            EndButton.setMargin(new Insets(-3, -3, -3, -3));
            EndButton.setFocusPainted(false);
            EndButton.setContentAreaFilled(false);
            EndButton.setIcon(End);
            EndButton.setRolloverIcon(EndOver);
            EndButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    EndButtonClicked(evt);
                }
            });
            TransitionPanel.add(EndButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 20, 90, 80));

            TransitionPanel.setBounds(130, 20, 410, 110);
            ControlTransitionPane.add(TransitionPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);
            ControlTransitionPane.add(ControlPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);
            MainPanel.add(ControlTransitionPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 550, 140));


// todo ... Eclipse style ...
        } else {
// Control transition pane
            ControlTransitionPane.setBorder(new BevelBorder(BevelBorder.RAISED));

//Controlpanel includes all the runcontrol releated  bottons
            ControlPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
            ControlPanel.setBorder(new BevelBorder(BevelBorder.RAISED));

// Configure button
            ConfigureButton.setSize(48,48);
            ConfigureButton.setBorderPainted(false);
            ConfigureButton.setFocusPainted(false);
            ConfigureButton.setContentAreaFilled(false);
            ConfigureButton.setIcon(Configure);
            ConfigureButton.setRolloverIcon(ConfigureOver);
            ConfigureButton.setToolTipText("Configure");
            ConfigureButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    ConfigureButtonClicked(evt);
                }
            });
            ControlPanel.add(ConfigureButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(5, 5, 50, 50));

// Reset button
            AbortButton.setSize(48,48);
            AbortButton.setBorderPainted(false);
            AbortButton.setFocusPainted(false);
            AbortButton.setContentAreaFilled(false);
            AbortButton.setIcon(Reset);
            AbortButton.setRolloverIcon(ResetOver);
            AbortButton.setToolTipText("Abort");
            AbortButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    AbortButtonClicked(evt);
                }
            });
            ControlPanel.add(AbortButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(65, 5, 50, 50));

// Pause button
            PauseButton.setSize(48,48);
            PauseButton.setBorderPainted(false);
            PauseButton.setFocusPainted(false);
            PauseButton.setContentAreaFilled(false);
            PauseButton.setIcon(Pause);
            PauseButton.setRolloverIcon(PauseOver);
            PauseButton.setToolTipText("Pause");
            PauseButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    PauseButtonClicked(evt);
                }
            });
            ControlPanel.add(PauseButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(125, 5, 50, 50));

// Download button
            DownloadButton.setSize(48,48);
            DownloadButton.setBorderPainted(false);
            DownloadButton.setFocusPainted(false);
            DownloadButton.setContentAreaFilled(false);
            DownloadButton.setIcon(Download);
            DownloadButton.setRolloverIcon(DownloadOver);
            DownloadButton.setToolTipText("Download");
            DownloadButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    DownloadButtonClicked(evt);
                }
            });
            ControlPanel.add(DownloadButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(225, 5, 50, 50));

// Prestart button
            PrestartButton.setSize(48,48);
            PrestartButton.setBorderPainted(false);
            PrestartButton.setFocusPainted(false);
            PrestartButton.setContentAreaFilled(false);
            PrestartButton.setIcon(Prestart);
            PrestartButton.setRolloverIcon(PrestartOver);
            PrestartButton.setToolTipText("Prestart");
            PrestartButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
//                    if(startrun && ((_State.equalsIgnoreCase(rcConstants.CMD_postend_ok)) ||
//                            (_State.equalsIgnoreCase(rcConstants.CMD_postdownload_ok)))){
//                        DownloadButton.setEnabled(false);
//                        PrestartButton.setEnabled(false);
//                        GoButton.setEnabled(false);
//                        startPrestart();
//                    }
//                    else if(startrun && ((_State.equalsIgnoreCase("Configured")) ||
//                            (_State.equalsIgnoreCase(rcConstants.CMD_reset_ok)))){
//                        DownloadButton.setEnabled(false);
//                        PrestartButton.setEnabled(false);
//                        GoButton.setEnabled(false);
//                        startDownload();
//                    }
//                    else {
                    startbuttonclicked = false;
                        PrestartButtonClicked(evt);
//                    }
                }
            });
            ControlPanel.add(PrestartButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(285, 5, 50, 50));

// Go button
            GoButton.setSize(48,48);
            GoButton.setBorderPainted(false);
            GoButton.setFocusPainted(false);
            GoButton.setContentAreaFilled(false);
            GoButton.setIcon(Go);
            GoButton.setRolloverIcon(GoOver);
            GoButton.setToolTipText("Go");
            GoButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    GoButtonClicked(evt);
                }
            });
            ControlPanel.add(GoButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(345, 5, 50, 50));

            EndButton.setSize(48,48);
            EndButton.setBorderPainted(false);
            EndButton.setFocusPainted(false);
            EndButton.setContentAreaFilled(false);
            EndButton.setIcon(End);
            EndButton.setRolloverIcon(EndOver);
            EndButton.setToolTipText("End");
            EndButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    _AutoReseted = false;
                    EndButtonClicked(evt);
                }
            });
            ControlPanel.add(EndButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(405, 5, 50, 50));

// Startrun  button
            StartRunButton.setSize(48,48);
            StartRunButton.setBorderPainted(false);
            StartRunButton.setFocusPainted(false);
            StartRunButton.setContentAreaFilled(false);
            StartRunButton.setIcon(StartRun);
            StartRunButton.setRolloverIcon(StartRunOver);
            StartRunButton.setToolTipText("StartRun");
            StartRunButton.setEnabled(false);
            startrun = false;
            StartRunButton.addActionListener(new java.awt.event.ActionListener() {

                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    ERROR_HAPPENED = false;
                    startrun = true;
                    startbuttonclicked = true;
                    if((_State.equalsIgnoreCase(rcConstants.CMD_postend_ok)) ||
                            (_State.equalsIgnoreCase(rcConstants.CMD_postdownload_ok))){
                        DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        startPrestart();
                    }
                    else if((_State.equalsIgnoreCase("Configured")) ||
                            (_State.equalsIgnoreCase(rcConstants.CMD_reset_ok))){
                        DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        startDownload();
                    }
                }
            });
            ControlPanel.add(StartRunButton, new org.netbeans.lib.awtextra.AbsoluteConstraints(490, 5, 50, 50));
            ControlPanel.setBounds(5, 5, 540, 60);

// ToolBox buttons and panel
// Transitionpanel includes ToolBox bottons
            TransitionPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
            TransitionPanel.setBorder(new BevelBorder(BevelBorder.RAISED));

            BJoinSession.setSize(16,16);
            BJoinSession.setBorderPainted(false);
            BJoinSession.setFocusPainted(false);
            BJoinSession.setContentAreaFilled(false);
            BJoinSession.setIcon(JoinSession);
            BJoinSession.setRolloverIcon(JoinSessionOver);
            BJoinSession.setToolTipText("Join the session");
            BJoinSession.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {

// Ask Cortex to report the registered Sessions
                    cMsgMessage msg = new cMsgMessage();
                    msg.setSubject(rcSubscription.CrtSubject);

// We do not want to get SESSIONs report on all guis in the SESSION. Thus myname in the
// type will distiguish what gui was requested.
                    msg.setType("ReportSessions_"+myName);
                    msg.setText(_Session);
                    try {
                        cMsgSrv.send(msg);
                        cMsgSrv.flush(1);
                    } catch (cMsgException e) {
                        e.printStackTrace();
                    }
                }
            });
            TransitionPanel.add(BJoinSession, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 2, 20, 20));

            BSetGraphSmoothingFactor.setSize(16,16);
            BSetGraphSmoothingFactor.setBorderPainted(false);
            BSetGraphSmoothingFactor.setFocusPainted(false);
            BSetGraphSmoothingFactor.setContentAreaFilled(false);
            BSetGraphSmoothingFactor.setIcon(SetGraphSmoothingFactor);
            BSetGraphSmoothingFactor.setRolloverIcon(SetGraphSmoothingFactorOver);
            BSetGraphSmoothingFactor.setToolTipText("Graph smoothing factor");
            BSetGraphSmoothingFactor.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {

// Start dialog with the slider (AIV: Averaging Interval Value)
                    RcSliderDialog dialog = new RcSliderDialog(GSTYLE);
                    dialog.showDialog(RcGui.this, Integer.toString(AIV),"Smoothing","RCWarning");
                    if(dialog.getReturnStatus() == RcSliderDialog.RET_OK){
                        AIV = Integer.parseInt(dialog.getSliderValue());
                    }
                }
            });
            TransitionPanel.add(BSetGraphSmoothingFactor, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 2, 20, 20));

            BSetReportingInterval.setSize(16,16);
            BSetReportingInterval.setBorderPainted(false);
            BSetReportingInterval.setFocusPainted(false);
            BSetReportingInterval.setContentAreaFilled(false);
            BSetReportingInterval.setIcon(SetReportingInterval);
            BSetReportingInterval.setRolloverIcon(SetReportingIntervalOver);
            BSetReportingInterval.setToolTipText("Reporting intervals");
            BSetReportingInterval.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    cMsgMessage msg = new cMsgMessage();
                    msg.setSubject(_Session+"_"+_RunType);
                    msg.setType(rcSubscription.SupervisorControlType+"/"+rcConstants.CMD_reportInterval);

// We send the name of this gui in the text to allow communication ONLY between initiator gui and Medulla.
                    msg.setText(myName);
                    try {
                        cMsgSrv.send(msg);
                        cMsgSrv.flush(1);
                    } catch (cMsgException e) {
                        e.printStackTrace();
                    }
                }
            });
            TransitionPanel.add(BSetReportingInterval, new org.netbeans.lib.awtextra.AbsoluteConstraints(60, 2, 20, 20));

            BTranslateDb2Cool.setSize(16,16);
            BTranslateDb2Cool.setBorderPainted(false);
            BTranslateDb2Cool.setFocusPainted(false);
            BTranslateDb2Cool.setContentAreaFilled(false);
            BTranslateDb2Cool.setIcon(TranslateDb2Cool);
            BTranslateDb2Cool.setRolloverIcon(TranslateDb2CoolOver);
            BTranslateDb2Cool.setToolTipText("Db2Cool");
            BTranslateDb2Cool.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    db2coolMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BTranslateDb2Cool, new org.netbeans.lib.awtextra.AbsoluteConstraints(90, 2, 20, 20));

            BDisconnect.setSize(16,16);
            BDisconnect.setBorderPainted(false);
            BDisconnect.setFocusPainted(false);
            BDisconnect.setContentAreaFilled(false);
            BDisconnect.setIcon(Disconnect);
            BDisconnect.setRolloverIcon(DisconnectOver);
            BDisconnect.setToolTipText("Disconnect");
            BDisconnect.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {

// Start confirmation dialog
                    RcDialog dialog = new RcDialog(GSTYLE);
                    String ms = "N.B.\nThis will release the " +
                            "current session ownership of the agents in the current run " +
                            "control and will make them available for the other control " +
                            "systems, running in different sessions, to gain ownership. ";
                    dialog.showDialog(RcGui.this,ms,"Are You Sure?","RCWarning");
                    if(dialog.getReturnStatus() == RcDialog.RET_OK) {
                        disconnectRuntype(_Session+"_"+_RunType);
                        resetGui();
                    }
                }
            });
            TransitionPanel.add(BDisconnect, new org.netbeans.lib.awtextra.AbsoluteConstraints(120, 2, 20, 20));

            BHideEbEr.setSize(16,16);
            BHideEbEr.setBorderPainted(false);
            BHideEbEr.setFocusPainted(false);
            BHideEbEr.setContentAreaFilled(false);
            BHideEbEr.setIcon(HideEbEr);
            BHideEbEr.setRolloverIcon(HideEbErOver);
            BHideEbEr.setToolTipText("Show EB/ER graphs");
            BHideEbEr.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    _hideEbErBars=false;
                    showEBERBarsMenuItem.setSelected(true);
                }
            });
            TransitionPanel.add(BHideEbEr, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 2, 20, 20));

            BNoHideEbEr.setSize(16,16);
            BNoHideEbEr.setBorderPainted(false);
            BNoHideEbEr.setFocusPainted(false);
            BNoHideEbEr.setContentAreaFilled(false);
            BNoHideEbEr.setIcon(NoHideEbEr);
            BNoHideEbEr.setRolloverIcon(NoHideEbErOver);
            BNoHideEbEr.setToolTipText("Hide EB/ER graphs");
            BNoHideEbEr.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    _hideEbErBars=true;
                    hideEBERBarsMenuItem.setSelected(true);
                }
            });
            TransitionPanel.add(BNoHideEbEr, new org.netbeans.lib.awtextra.AbsoluteConstraints(180, 2, 20, 20));

            BAutoStart.setSize(16,16);
            BAutoStart.setBorderPainted(false);
            BAutoStart.setFocusPainted(false);
            BAutoStart.setContentAreaFilled(false);
            BAutoStart.setIcon(AutoStart);
            BAutoStart.setRolloverIcon(AutoStartOver);
            BAutoStart.setToolTipText("Enable autostart");
            BAutoStart.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    enableASMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BAutoStart, new org.netbeans.lib.awtextra.AbsoluteConstraints(210, 2, 20, 20));

            BNoAutoStart.setSize(16,16);
            BNoAutoStart.setBorderPainted(false);
            BNoAutoStart.setFocusPainted(false);
            BNoAutoStart.setContentAreaFilled(false);
            BNoAutoStart.setIcon(NoAutoStart);
            BNoAutoStart.setRolloverIcon(NoAutoStartOver);
            BNoAutoStart.setToolTipText("Disable autostart");
            BNoAutoStart.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    disableASMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BNoAutoStart, new org.netbeans.lib.awtextra.AbsoluteConstraints(240, 2, 20, 20));

            BEnableStartRun.setSize(16,16);
            BEnableStartRun.setBorderPainted(false);
            BEnableStartRun.setFocusPainted(false);
            BEnableStartRun.setContentAreaFilled(false);
            BEnableStartRun.setIcon(EnableStartRun);
            BEnableStartRun.setRolloverIcon(EnableStartRunOver);
            BEnableStartRun.setToolTipText("Enable startrun");
            BEnableStartRun.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    enableESMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BEnableStartRun, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 2, 20, 20));

            BDisableStartRun.setSize(16,16);
            BDisableStartRun.setBorderPainted(false);
            BDisableStartRun.setFocusPainted(false);
            BDisableStartRun.setContentAreaFilled(false);
            BDisableStartRun.setIcon(DisableStartRun);
            BDisableStartRun.setRolloverIcon(DisableStartRunOver);
            BDisableStartRun.setToolTipText("Disable startrun");
            BDisableStartRun.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    disableESMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BDisableStartRun, new org.netbeans.lib.awtextra.AbsoluteConstraints(300, 2, 20, 20));

            BEventLimit.setSize(16,16);
            BEventLimit.setBorderPainted(false);
            BEventLimit.setFocusPainted(false);
            BEventLimit.setContentAreaFilled(false);
            BEventLimit.setIcon(EventLimit);
            BEventLimit.setRolloverIcon(EventLimitOver);
            BEventLimit.setToolTipText("Enable event limit");
            BEventLimit.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    enableELMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BEventLimit, new org.netbeans.lib.awtextra.AbsoluteConstraints(330, 2, 20, 20));

            BNoEventLimit.setSize(16,16);
            BNoEventLimit.setBorderPainted(false);
            BNoEventLimit.setFocusPainted(false);
            BNoEventLimit.setContentAreaFilled(false);
            BNoEventLimit.setIcon(NoEventLimit);
            BNoEventLimit.setRolloverIcon(NoEventLimitOver);
            BNoEventLimit.setToolTipText("Disable event limit");
            BNoEventLimit.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    disableELMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BNoEventLimit, new org.netbeans.lib.awtextra.AbsoluteConstraints(360, 2, 20, 20));

            BMoreGraphics.setSize(16,16);
            BMoreGraphics.setBorderPainted(false);
            BMoreGraphics.setFocusPainted(false);
            BMoreGraphics.setContentAreaFilled(false);
            BMoreGraphics.setIcon(MoreGraphics);
            BMoreGraphics.setRolloverIcon(MoreGraphicsOver);
            BMoreGraphics.setToolTipText("Expanded view");
            BMoreGraphics.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    moreGraphsMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BMoreGraphics, new org.netbeans.lib.awtextra.AbsoluteConstraints(390, 2, 20, 20));

            BLessGraphics.setSize(16,16);
            BLessGraphics.setBorderPainted(false);
            BLessGraphics.setFocusPainted(false);
            BLessGraphics.setContentAreaFilled(false);
            BLessGraphics.setIcon(LessGraphics);
            BLessGraphics.setRolloverIcon(LessGraphicsOver);
            BLessGraphics.setToolTipText("Condenced view");
            BLessGraphics.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    lessGraphsMenuItemActionPerformed(evt);
                }
            });
            TransitionPanel.add(BLessGraphics, new org.netbeans.lib.awtextra.AbsoluteConstraints(420, 2, 20, 20));
            TransitionPanel.setBounds(5, 65, 540, 25);

// Add panels to the main panel
            MainPanel.add(TransitionPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 60, 550, 25));
            MainPanel.add(ControlPanel, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 0, 550, 60));
        }

// ... Static parameters. N.B. eclipse like look and feel does not use title boarders
        javax.swing.border.TitledBorder bd = new javax.swing.border.TitledBorder(
                new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Static Parameters");
        bd.setTitleColor(TITLE_COLOR);
        if(BACKGROUND.equalsIgnoreCase("plain"))
            StaticPArametersPane.setBorder(new BevelBorder(BevelBorder.RAISED));
        else StaticPArametersPane.setBorder(new javax.swing.border.TitledBorder(bd));
        DatabasePanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Experiment ID");
        bd.setTitleColor(TITLE_COLOR);
        DatabasePanel.setBorder(new javax.swing.border.TitledBorder(bd));
        DatabaseTextField.setFont(new java.awt.Font("Dialog", 1, 12));
        DatabaseTextField.setText("");
        DatabaseTextField.setForeground(new java.awt.Color(139, 52, 108));
        DatabasePanel.add(DatabaseTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) DatabasePanel.setBounds(10, 10, 170, 50);
        else  DatabasePanel.setBounds(10, 20, 170, 50);

        StaticPArametersPane.add(DatabasePanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        SessionPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Session");
        bd.setTitleColor(TITLE_COLOR);
        SessionPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        SessionTextField.setFont(new java.awt.Font("Dialog", 1, 12));
        SessionTextField.setText("");
        SessionTextField.setForeground(new java.awt.Color(139, 52, 108));
        SessionPanel.add(SessionTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) SessionPanel.setBounds(190, 10, 170, 50);
        else SessionPanel.setBounds(190, 20, 170, 50);

        StaticPArametersPane.add(SessionPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        ConfigurationPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Configuration");
        bd.setTitleColor(TITLE_COLOR);
        ConfigurationPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        ConfigurationTextField.setFont(new java.awt.Font("Dialog", 1, 12));
        ConfigurationTextField.setText("");
        ConfigurationTextField.setForeground(new java.awt.Color(139, 52, 108));
        ConfigurationPanel.add(ConfigurationTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        ConfigurationPanel.setBounds(370, 10, 170, 50);
        StaticPArametersPane.add(ConfigurationPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);
        if(BACKGROUND.equalsIgnoreCase("plain")) MainPanel.add(StaticPArametersPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 90, 550, 75));
        else MainPanel.add(StaticPArametersPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 140, 550, 80));
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Run Status");
        bd.setTitleColor(TITLE_COLOR);
        if(BACKGROUND.equalsIgnoreCase("plain")) RunStatusPane.setBorder(new BevelBorder(BevelBorder.RAISED));
        else  RunStatusPane.setBorder(new javax.swing.border.TitledBorder(bd));
        RunNumberPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Run Number");
        bd.setTitleColor(TITLE_COLOR);

        RunNumberPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        RunNumberTextField.setFont(new java.awt.Font("Dialog", 1, 14));
        RunNumberTextField.setForeground(new java.awt.Color(139, 52, 108));
        RunNumberTextField.setText("");
        RunNumberPanel.add(RunNumberTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) RunNumberPanel.setBounds(10, 10, 170, 50);
        else RunNumberPanel.setBounds(10, 20, 170, 50);

        RunStatusPane.add(RunNumberPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        EndTimePanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"End Time");
        bd.setTitleColor(TITLE_COLOR);
        EndTimePanel.setBorder(new javax.swing.border.TitledBorder(bd));
        EndTimeTextField.setText("");
        EndTimeTextField.setForeground(new java.awt.Color(139, 52, 108));
        EndTimePanel.add(EndTimeTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) EndTimePanel.setBounds(190, 60, 170, 50);
        else EndTimePanel.setBounds(190, 70, 170, 50);

        RunStatusPane.add(EndTimePanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        StartTimePanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Start Time");
        bd.setTitleColor(TITLE_COLOR);
        StartTimePanel.setBorder(new javax.swing.border.TitledBorder(bd));
        StartTimeTextField.setText("");
        StartTimeTextField.setForeground(new java.awt.Color(139, 52, 108));
        StartTimePanel.add(StartTimeTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) StartTimePanel.setBounds(10, 60, 170, 50);
        else StartTimePanel.setBounds(10, 70, 170, 50);

        RunStatusPane.add(StartTimePanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        EventsPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Event Limit");
        bd.setTitleColor(TITLE_COLOR);
        EventsPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        LimitEventsTextField.setFont(new java.awt.Font("Dialog", 1, 11));
        LimitEventsTextField.setText("");
        LimitEventsTextField.setForeground(new java.awt.Color(139, 52, 108));
        EventsPanel.add(LimitEventsTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) EventsPanel.setBounds(370, 10, 170, 50);
        else EventsPanel.setBounds(370, 20, 170, 50);

        RunStatusPane.add(EventsPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        DataLimitPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Data Limit");
        bd.setTitleColor(TITLE_COLOR);
        DataLimitPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        DataLimitTextField.setFont(new java.awt.Font("Dialog", 1, 11));
        DataLimitTextField.setText("");
        DataLimitTextField.setForeground(new java.awt.Color(139, 52, 108));
        DataLimitPanel.add(DataLimitTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain"))  DataLimitPanel.setBounds(370, 60, 170, 50);
        else DataLimitPanel.setBounds(370, 70, 170, 50);

        RunStatusPane.add(DataLimitPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        RunStatePanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Run State");
        bd.setTitleColor(TITLE_COLOR);
        RunStatePanel.setBorder(new javax.swing.border.TitledBorder(bd));
        RunStatusTextField.setFont(new java.awt.Font("Dialog", 1, 12));
        RunStatusTextField.setText("");
        RunStatusTextField.setForeground(new java.awt.Color(139, 52, 108));
        RunStatePanel.add(RunStatusTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) RunStatePanel.setBounds(190, 10, 170, 50);
        else RunStatePanel.setBounds(190, 20, 170, 50);

        RunStatusPane.add(RunStatePanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        ERFilePanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Output File");
        bd.setTitleColor(TITLE_COLOR);
        ERFilePanel.setBorder(new javax.swing.border.TitledBorder(bd));
        ERFileTextField.setFont(new java.awt.Font("Dialog", 1, 12));
        ERFileTextField.setText("");
        ERFileTextField.setForeground(new java.awt.Color(139, 52, 108));
        ERFilePanel.add(ERFileTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 510, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) ERFilePanel.setBounds(10, 110, 530, 50);
        else ERFilePanel.setBounds(10, 120, 530, 50);

        RunStatusPane.add(ERFilePanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        if(BACKGROUND.equalsIgnoreCase("plain")) MainPanel.add(RunStatusPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 170, 550, 170));
        else  MainPanel.add(RunStatusPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 220, 550, 180));
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Run Progress");
        bd.setTitleColor(TITLE_COLOR);
        if(BACKGROUND.equalsIgnoreCase("plain"))  RunProgressPane.setBorder(new BevelBorder(BevelBorder.RAISED));
        else RunProgressPane.setBorder(new javax.swing.border.TitledBorder(bd));
        EventsThisRunPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Events This Run");
        bd.setTitleColor(TITLE_COLOR);
        EventsThisRunPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        EventThisRunTextField.setFont(new java.awt.Font("Dialog", 1, 18));
        EventThisRunTextField.setForeground(new java.awt.Color(139, 52, 108));
        EventThisRunTextField.setText("");
        EventsThisRunPanel.add(EventThisRunTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 35, 150, 40));
        if(BACKGROUND.equalsIgnoreCase("plain"))  EventsThisRunPanel.setBounds(10, 10, 170, 100);
        else EventsThisRunPanel.setBounds(10, 20, 170, 100);

        RunProgressPane.add(EventsThisRunPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        EventRateDifPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Event Rate Hz");
        bd.setTitleColor(TITLE_COLOR);
        EventRateDifPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        EventRateDifTextField.setFont(new java.awt.Font("Dialog", 1, 14));
        EventRateDifTextField.setText("");
        EventRateDifTextField.setForeground(new java.awt.Color(139, 52, 108));
        EventRateDifPanel.add(EventRateDifTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) EventRateDifPanel.setBounds(190, 10, 170, 50);
        else EventRateDifPanel.setBounds(190, 20, 170, 50);

        RunProgressPane.add(EventRateDifPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        DataRateDifPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Data Rate KByte/s");
        bd.setTitleColor(TITLE_COLOR);
        DataRateDifPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        DataRateDifTextField.setFont(new java.awt.Font("Dialog", 1, 14));
        DataRateDifTextField.setText("");
        DataRateDifTextField.setForeground(new java.awt.Color(139, 52, 108));
        DataRateDifPanel.add(DataRateDifTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) DataRateDifPanel.setBounds(370, 10, 170, 50);
        else  DataRateDifPanel.setBounds(370, 20, 170, 50);

        RunProgressPane.add(DataRateDifPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        EventRateIntPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Integrated");
        bd.setTitleColor(TITLE_COLOR);
        EventRateIntPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        EventRateIntTextField.setFont(new java.awt.Font("Dialog", 1, 14));
        EventRateIntTextField.setText("");
        EventRateIntTextField.setForeground(new java.awt.Color(139, 52, 108));
        EventRateIntPanel.add(EventRateIntTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) EventRateIntPanel.setBounds(190, 60, 170, 50);
        else EventRateIntPanel.setBounds(190, 70, 170, 50);

        RunProgressPane.add(EventRateIntPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        DataRateIntPanel.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.EtchedBorder(javax.swing.border.EtchedBorder.RAISED),"Integrated");
        bd.setTitleColor(TITLE_COLOR);
        DataRateIntPanel.setBorder(new javax.swing.border.TitledBorder(bd));
        DataRateIntTextField.setFont(new java.awt.Font("Dialog", 1, 14));
        DataRateIntTextField.setText("");
        DataRateIntTextField.setForeground(new java.awt.Color(139, 52, 108));
        DataRateIntPanel.add(DataRateIntTextField, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 20, 150, 20));
        if(BACKGROUND.equalsIgnoreCase("plain")) DataRateIntPanel.setBounds(370, 60, 170, 50);
        else DataRateIntPanel.setBounds(370, 70, 170, 50);

        RunProgressPane.add(DataRateIntPanel, javax.swing.JLayeredPane.DEFAULT_LAYER);

// Add to the main panel
        if(BACKGROUND.equalsIgnoreCase("plain")) MainPanel.add(RunProgressPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 345, 550, 120));
        else MainPanel.add(RunProgressPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 400, 550, 128));
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Event Rate Hz");
        bd.setTitleColor(TITLE_COLOR);
        TimeGraphPane.setBorder(new javax.swing.border.TitledBorder(bd));
        if(BACKGROUND.equalsIgnoreCase("plain")) MainPanel.add(TimeGraphPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(1, 465, 440, 220));
        else MainPanel.add(TimeGraphPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(1, 528, 440, 220));

// Create the list and put it in a scroll pane.
        ComponentList = new JList(listModel);
        ComponentList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        ComponentList.setSelectedIndex(0);
        ComponentsPane = new javax.swing.JScrollPane();
        ComponentsPane = new JScrollPane(ComponentList);
        ComponentList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                ComponentListMouseClicked(evt);
            }
        });
        ComponentsPane.setViewportView(ComponentList);
        if(BACKGROUND.equalsIgnoreCase("plain")) MainPanel.add(ComponentsPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(443, 473, 110, 210));
        else MainPanel.add(ComponentsPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(443, 538, 110, 210));

// Text pane component and style settings.
        doc = (StyledDocument)MessagesTextPane.getDocument();
        style = doc.addStyle("StyleName",null);
        MessagesScrollPane.setViewportView(MessagesTextPane);
        if(BACKGROUND.equalsIgnoreCase("plain")) MainPanel.add(MessagesScrollPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(3, 685, 550, 137));
        else MainPanel.add(MessagesScrollPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(3, 750, 550, 160));

// ... EVENT RATE TIMELINE
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Event Rate Hz");
        bd.setTitleColor(TITLE_COLOR);
        EventRateTGPane.setBorder(new javax.swing.border.TitledBorder(bd));
        MainPanel.add(EventRateTGPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(550, 0, 440, 200));
        myEventRateTG = new org.jlab.coda.runcontrol.gui.TimeGraphLite("Hz",50,2000.0,EventRateTGPane, false);

// ... DATA RATE TIMELINE
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Data Rate KByte/s");
        bd.setTitleColor(TITLE_COLOR);
        DataRateTGPane.setBorder(new javax.swing.border.TitledBorder(bd));
        MainPanel.add(DataRateTGPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(550, 200, 440, 200));
        myDataRateTG = new org.jlab.coda.runcontrol.gui.TimeGraphLite("KBytes",50,2000.0,DataRateTGPane, false);

// ... DATA RATE BARGRAPH
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Data Rate All Components");
        bd.setTitleColor(TITLE_COLOR);
        dataRateBGPane.setBorder(new javax.swing.border.TitledBorder(bd));
        MainPanel.add(dataRateBGPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(550, 400, 440, 200));
        myDataRateBG = new org.jlab.coda.runcontrol.gui.BarGraphLite("KByte",21,500.0, dataRateBGPane);

// ... LIVETIME TIMELINE
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Live Time");
        bd.setTitleColor(TITLE_COLOR);
        liveTimeTGPane.setBorder(new javax.swing.border.TitledBorder(bd));
        MainPanel.add(liveTimeTGPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(550, 600, 440, 220));
        myLiveTimeTG = new TimeGraphLite("%",50,100.0, liveTimeTGPane, true);

//@todo remove level1 trigger bits bargraph for now.
/*
// ... TS TRIGGER BITS BARGRAPH
        bd = new javax.swing.border.TitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),"Level1 Trigger");
        bd.setTitleColor(TITLE_COLOR);
        level1TriggerBGPane.setBorder(new javax.swing.border.TitledBorder(bd));
//        getContentPane().add(level1TriggerBGPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(550, 600, 440, 220));
        MainPanel.add(level1TriggerBGPane, new org.netbeans.lib.awtextra.AbsoluteConstraints(550, 600, 440, 220));
        myTriggerBitsBG = new BarGraphLite("Hz",12,500.0, level1TriggerBGPane);
*/

// Put everything together
// First put everything into the scrollpane. This is necessary because we are using absoluteLayout and for some
// displays we might need to scrole entire gui to get an access to the desired information.
        verticalPane = new JScrollPane(MainPanel, // component
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, // vertical bar
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER); // assuming we only need vertical scrolling
        m.gridx=0;
        m.gridy=0;
        m.insets = new Insets(0,0,0,0);
        getContentPane().add(verticalPane,m);


// ... Menu section ..........
        menuBar.setForeground(new java.awt.Color(0, 0, 0));
        fileMenu.setText("Control");

// Klick the "set reporting interval" will send a message to the Medulla asking to
// send the list of all components and their reporting intervals.
        setReportIntervalMenuItem.setText("Set Reporting Intervals");
        setReportIntervalMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cMsgMessage msg = new cMsgMessage();
                msg.setSubject(_Session+"_"+_RunType);
                msg.setType(rcSubscription.SupervisorControlType+"/"+rcConstants.CMD_reportInterval);

// We send the name of this gui in the text to allow communication ONLY between initiator gui and Medulla.
                msg.setText(myName);
                try {
                    cMsgSrv.send(msg);
                    cMsgSrv.flush(1);
                } catch (cMsgException e) {
                    e.printStackTrace();
                }
            }
        });
        fileMenu.add(setReportIntervalMenuItem);
        fileMenu.add(new JSeparator());

// Reset menu item
        ResetMenuItem.setText("Reset");
        ResetMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {

// Start confirmation dialog
                RcDialog dialog = new RcDialog(GSTYLE);
                String ms = "N.B.\n Reset will force the system to go into the 'booted' state.";
                dialog.showDialog(RcGui.this,ms,"Are You Sure?","RCWarning");
                if(dialog.getReturnStatus() == RcDialog.RET_OK) ResetButtonClicked(evt);

            }
        });

        fileMenu.add(ResetMenuItem);
        fileMenu.add(new JSeparator());

// CExit Component Exit menu item
        CExitMenuItem.setText("Exit Components");
        CExitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {

// Start confirmation dialog
                RcDialog dialog = new RcDialog(GSTYLE);
                String ms = "N.B.\n Exit will stop all acive control threads of the component";
                dialog.showDialog(RcGui.this,ms,"Are You Sure?","RCWarning");
                if(dialog.getReturnStatus() == RcDialog.RET_OK) CExitButtonClicked(evt);

            }
        });

        fileMenu.add(CExitMenuItem);
        fileMenu.add(new JSeparator());


// Set averaging intervals menue item
        setAiMenuItem.setText("Graph smoothing factor");
        setAiMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {

// Start dialog with the slider (AIV: Averaging Interval Value)
                RcSliderDialog dialog = new RcSliderDialog(GSTYLE);
                dialog.showDialog(RcGui.this, Integer.toString(AIV),"Smoothing","RCWarning");
                if(dialog.getReturnStatus() == RcSliderDialog.RET_OK){
                    AIV = Integer.parseInt(dialog.getSliderValue());
                }
            }
        });
        fileMenu.add(setAiMenuItem);
        fileMenu.add(new JSeparator());

// Disconnect menu item
        openMenuItem.setText("Disconnect");
        openMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {

// Start confirmation dialog
                RcDialog dialog = new RcDialog(GSTYLE);
                String ms = "N.B.\nThis will release the " +
                        "current session ownership of the agents in the current run " +
                        "control and will make them available for the other control " +
                        "systems, running in different sessions, to gain ownership. ";
                dialog.showDialog(RcGui.this,ms,"Are You Sure?","RCWarning");
                if(dialog.getReturnStatus() == RcDialog.RET_OK) disconnectRuntype(_Session+"_"+_RunType);
                resetGui();
            }
        });
        fileMenu.add(openMenuItem);
        fileMenu.add(new JSeparator());

// Join the SESSION menu item
        saveMenuItem.setText("Join the session");
        saveMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {

// Ask Cortex to report the registered Sessions
// Send cMsg to Cortex
                cMsgMessage msg = new cMsgMessage();
                msg.setSubject(rcSubscription.CrtSubject);

// We do not want to get SESSIONs report on all guis in the SESSION. Thus myname in the
// type will distiguish what gui was requesting.
                msg.setType("ReportSessions_"+myName);
                msg.setText(_Session);
                try {
                    cMsgSrv.send(msg);
                    cMsgSrv.flush(1);
                } catch (cMsgException e) {
                    e.printStackTrace();
                }
            }
        });
        fileMenu.add(saveMenuItem);
        fileMenu.add(new JSeparator());

// Add db2cool menu item
        db2coolMenuItem.setText("Translate DB to COOL");
        db2coolMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                db2coolMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(db2coolMenuItem);
        fileMenu.add(new JSeparator());

// Exit menu item
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);
        menuBar.add(fileMenu);
        editMenu.setText("Preference");

// Enable start run menu item
        ButtonGroup easyStartGroup = new ButtonGroup();
        enableEasyStart.setText("Enable StartRun");
        enableEasyStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                enableESMenuItemActionPerformed(evt);
            }
        });
        easyStartGroup.add(enableEasyStart);
        editMenu.add(enableEasyStart);

// Dissable start run menu item
        disableEasyStart.setText("Disable StartRun");
        disableEasyStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                disableESMenuItemActionPerformed(evt);
            }
        });
        disableEasyStart.setSelected(true);
        easyStartGroup.add(disableEasyStart);
        editMenu.add(disableEasyStart);
        editMenu.add(new JSeparator());

// Enable event limit menu item
        ButtonGroup eventLimitGroup = new ButtonGroup();
        enableEvLimMenuItem.setText("Enable Event Limit");
        enableEvLimMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                enableELMenuItemActionPerformed(evt);
            }
        });
        eventLimitGroup.add(enableEvLimMenuItem);
        editMenu.add(enableEvLimMenuItem);

// Dissable event limit menu item
        disableEvLimMenuItem.setText("Disable Event Limit");
        disableEvLimMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                disableELMenuItemActionPerformed(evt);
            }
        });
        disableEvLimMenuItem.setSelected(true);
        eventLimitGroup.add(disableEvLimMenuItem);
        editMenu.add(disableEvLimMenuItem);
        editMenu.add(new JSeparator());

// Enable auto start menu item. N.B> Scheduler must be defined in the cool for this
        ButtonGroup autoStartGroup = new ButtonGroup();
        enableAutoStartMenuItem.setText("Enable Auto Start");
        enableAutoStartMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                enableASMenuItemActionPerformed(evt);
            }
        });
        autoStartGroup.add(enableAutoStartMenuItem);
        editMenu.add(enableAutoStartMenuItem);

// Dissable auto start menu item
        disableAutoStartMenuItem.setText("Disable Auto Start ");
        disableAutoStartMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                disableASMenuItemActionPerformed(evt);
            }
        });
        disableAutoStartMenuItem.setSelected(true);
        autoStartGroup.add(disableAutoStartMenuItem);
        editMenu.add(disableAutoStartMenuItem);

        editMenu.add(new JSeparator());

// Hide EB/ER bars menu item
        ButtonGroup eberBarGroup = new ButtonGroup();
        hideEBERBarsMenuItem.setText("Hide EB/ER bar graphs");
        hideEBERBarsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                _hideEbErBars=true;
            }
        });
        hideEBERBarsMenuItem.setSelected(true);
        eberBarGroup.add(hideEBERBarsMenuItem);
        editMenu.add(hideEBERBarsMenuItem);

// Show EB/ER bars menu item
        showEBERBarsMenuItem.setText("Show EB/ER bar graphs");
        showEBERBarsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                _hideEbErBars=false;
            }
        });
        eberBarGroup.add(showEBERBarsMenuItem);
        editMenu.add(showEBERBarsMenuItem);
        editMenu.add(new JSeparator());

// Expend the gui menu item
        ButtonGroup moreGraphGroup = new ButtonGroup();
        moreGraphsMenuItem.setText("More Graphics");
        moreGraphsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moreGraphsMenuItemActionPerformed(evt);
            }
        });
        moreGraphGroup.add(moreGraphsMenuItem);
        editMenu.add(moreGraphsMenuItem);

// Shrink the gui menu item
        lessGraphsMenuItem.setText("Less Graphics");
        lessGraphsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lessGraphsMenuItemActionPerformed(evt);
            }
        });
        lessGraphsMenuItem.setSelected(true);
        moreGraphGroup.add(lessGraphsMenuItem);
        editMenu.add(lessGraphsMenuItem);
        editMenu.add(new JSeparator());

// DalogBrowser menue item
        dalogBrowserMenuItem.setText("Launch DalogMessageBrowser");
        dalogBrowserMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dalogBrowserMenuItemActionPerformed(evt);
            }
        });
        editMenu.add(dalogBrowserMenuItem);

        menuBar.add(editMenu);

// Help menu
        helpMenu.setText("Help");
        contentsMenuItem.setText("User's Guide");

// Show the README file
        contentsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                BufferedReader br=null;
                StringBuffer stb = new StringBuffer();
                String line;
                RcDialog dialog = new RcDialog(GSTYLE);
                try {
                    br = new BufferedReader(new FileReader(System.getenv("AFECS_HOME")+"/README"));
                } catch (FileNotFoundException e) {
                    e.printStackTrace();
                }
                try {
                    assert br != null;
                    while((line = br.readLine())!=null)stb.append(line).append("\n");
                    br.close();

                } catch (IOException e) {
                    e.printStackTrace();
                }
                dialog.showDialog(RcGui.this,stb.toString(),"Concise User's Guide","RCWarning",750,300);
            }
        });

        helpMenu.add(contentsMenuItem);
        aboutMenuItem.setText("About");
        helpMenu.add(aboutMenuItem);
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);
        pack();
    }


    /**
     * Method is called when we clicked the component name in the components list
     * @param evt            Swing action event
     */
    private synchronized void ComponentListMouseClicked(MouseEvent evt) {

        CompMonitor cm;

        if(_RunState.equalsIgnoreCase(rcConstants.CMD_postgo_ok)){
            cleanWhiteBoard();
            selComp =  (String)ComponentList.getSelectedValue();
            if(selComp !=null) {

// Here we show the event rate timegraph on the info borad.
                if(myGraph!=null) myGraph.remove();
                myGraph = new TimeGraphLite(selComp+": Hz",50,2000.0, TimeGraphPane, false);

// Here we duplicate it on the event rate timegraph pane.
                if(myEventRateTG!=null)myEventRateTG.remove();
                myEventRateTG = new TimeGraphLite(selComp+": Hz",50,2000.0, EventRateTGPane, false);
                if(_CompTypes.get(selComp).equalsIgnoreCase(rcConstants.ebType) || _CompTypes.get(selComp).equalsIgnoreCase(rcConstants.erType)){
                    if(myDataRateTG!=null)myDataRateTG.remove();
                    myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,3000.0,DataRateTGPane, false);
                } else {
                    if(myDataRateTG!=null)myDataRateTG.remove();
                    myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,500.0,DataRateTGPane, false);
                }
            }
        }

// If we are not in post go  show previosly monitored information. This is important after ending the run
// to see the average statistics information from the components.        
        else {
            cleanWhiteBoard();
            selComp =  (String)ComponentList.getSelectedValue();
            // set the title for the border
            if(selComp !=null && _CompStatistics!=null) {
                cm = _CompStatistics.get(selComp);
                if(cm!=null) {
                    setEventNumber(Integer.toString(cm.getEvtNumber()));
                    setGEvtRate((double)cm.getEvtRate());
                    setGDataRate(cm.getDataRate());
                    setGAverEvtRate((double)cm.getAverEvtRate());
                    setGAverDataRate(cm.getAverDataRate());
                }
            }
        }
    }


    /**
     * Sets texfields to non editable mode
     *
     * @param b                   defines the text editability
     */
    public void setTextEditable(final boolean b){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                SessionTextField.setEditable(b);
                DatabaseTextField.setEditable(b);
                ConfigurationTextField.setEditable(b);
                RunNumberTextField.setEditable(b);
                StartTimeTextField.setEditable(b);
                EndTimeTextField.setEditable(b);
                RunStatusTextField.setEditable(b);
                LimitEventsTextField.setEditable(b);
                DataLimitTextField.setEditable(b);
                EventThisRunTextField.setEditable(b);
                EventRateDifTextField.setEditable(b);
                DataRateDifTextField.setEditable(b);
                EventRateIntTextField.setEditable(b);
                DataRateIntTextField.setEditable(b);
                ERFileTextField.setEditable(b);

                AbortButton.setEnabled(false);
                ConfigureButton.setEnabled(true);
                PauseButton.setEnabled(false);
                DownloadButton.setEnabled(false);
                PrestartButton.setEnabled(false);
                GoButton.setEnabled(false);
                EndButton.setEnabled(false);
            }
        });
    }


    /** Method to get the cool configuration file name.
     *
     * @return coolFileName                  returns the global var indicating the name
     *                                       of the cool config file
     * */
    public String getCoolFileName()   {return coolFileName;}


    /**
     * Resets the gui to its initial state, i.e. only configure button is active
     */
    public void resetGui(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                PauseButton.setEnabled(false);
                AbortButton.setEnabled(false);
                ConfigureButton.setEnabled(true);
                PauseButton.setEnabled(false);
                DownloadButton.setEnabled(false);
                PrestartButton.setEnabled(false);
                GoButton.setEnabled(false);
                EndButton.setEnabled(false);
                LimitEventsTextField.setText("");
                LimitEventsTextField.setEditable(false);
                setRunState("Disconnected");
                setStartTime("");
                setEndTime("");
                setEventNumber("");
                setGEvtRate(0.0);
                setGDataRate(0.0);
                setGAverEvtRate(0.0);
                setGAverDataRate(0.0);
                setRunNumber("");
                listModel.clear();
            }
        });
    }


    /**
     * Stops animated clock
     */
    public void stopClock(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                TimeGraphPane.getViewport().remove(ClockLabel);
                TimeGraphPane.getViewport().remove(InfoLabel);
            }
        });
    }


    /**
     * Starts animated clock with the text
     * @param tt    Text to show with the anumation
     */
    public void startClock(String tt){
        final String t = tt;
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                ClockLabel.setText(t);
                TimeGraphPane.getViewport().add(ClockLabel);
            }
        });
    }


    /**
     * Puts the text inside the information board( shared with the timegraph, i.e. TimeGraphPanel)
     * @param tt            Information text
     * @param c             Severity
     */
    public void startInfo(String tt, String c){
        final String t,ceverity;
        t=tt;
        ceverity = c;
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                if(ceverity.equalsIgnoreCase("Info")) InfoLabel.setForeground(Color.GREEN);
                else if(ceverity.equalsIgnoreCase("Warning")) InfoLabel.setForeground(Color.MAGENTA);
                else if(ceverity.equalsIgnoreCase("Error")) InfoLabel.setForeground(Color.RED);
                InfoLabel.setText(t);
                TimeGraphPane.getViewport().add(InfoLabel);
            }
        });
    }


    /**
     * Clears the information board
     */
    public void cleanWhiteBoard(){
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                TimeGraphPane.getViewport().removeAll();
                DataRateTGPane.getViewport().removeAll();
            }
        });
    }


    /**
     * Set the text of the Session display in the swing thread.
     * @param txt text to display
     */
    public void setSession (final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                SessionTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the runType display in the swing thread.
     * @param txt text to display
     */
    public void setRunType(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                ConfigurationTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the ER output file.
     * @param txt text to display
     */
    public void setOutputFile(final String txt) {
        if(!ERFileTextField.getText().equals(txt)){
            ERFileTextField.setText("" + txt);
        }
    }


    /**
     * Set the text of the event rate display in the swing thread.
     * @param value double to display
     */
    public void setGEvtRate(final double value) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {

                DecimalFormat myFormatter = new DecimalFormat("###.##");
                String output = myFormatter.format(value);
                EventRateDifTextField.setText("" + output);
            }
        });
    }


    /**
     * Set the text of the average event rate display in the swing thread.
     * @param value double to display
     */
    public void setGAverEvtRate(final double value) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                DecimalFormat myFormatter = new DecimalFormat("###.##");
                String output = myFormatter.format(value);
                EventRateIntTextField.setText("" + output);
            }
        });
    }


    /**
     * Set the text of the data rate display in the swing thread.
     * @param value text to display
     */
    public void setGDataRate(final double value) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                DecimalFormat myFormatter = new DecimalFormat("###.##");
                String output = myFormatter.format(value);
                DataRateDifTextField.setText("" + output);
            }
        });
    }


    /**
     * Set the text of the average data rate display in the swing thread.
     * @param value text to display
     */
    public void setGAverDataRate(final double value) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                DecimalFormat myFormatter = new DecimalFormat("###.##");
                String output = myFormatter.format(value);
                DataRateIntTextField.setText("" + output);
            }
        });
    }


    /**
     * Set the text of the event number display in the swing thread.
     * @param txt text to display
     */
    public void setEventNumber(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                EventThisRunTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the runNumber display in the swing thread.
     * @param txt text to display
     */
    public void setRunNumber(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                RunNumberTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the run state display in the swing thread.
     * @param txt text to display
     */
    public void setRunState(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                RunStatusTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the database display in the swing thread.
     * @param txt text to display
     */
    public void setDatabase(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                DatabaseTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the starttime display in the swing thread.
     * @param txt text to display
     */
    public void setStartTime(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                StartTimeTextField.setText("" + txt);
            }
        });
    }


    /**
     * Set the text of the endtime display in the swing thread.
     * @param txt text to display
     */
    public void setEndTime(final String txt) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                EndTimeTextField.setText("" + txt);
            }
        });
    }


    /**
     *
     * @param tt  actual text to be printed
     * @param tp defines the color of the text
     */
    public void addConsoleLine(String tt, String tp){
        final String txt, type;
        txt = tt;
        type =tp;
        String t = new Date().toString();
        StringTokenizer tk = new StringTokenizer(t);


// reset color to black
        StyleConstants.setForeground(style, Color.black);

// get the time string only
        for(int i=0;i<3;i++) if(tk.hasMoreTokens())tk.nextToken();
        final String time = tk.nextToken();
        if(type.equalsIgnoreCase("info")){
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    StyleConstants.setForeground(style, Color.black);
                    try {
                        doc.insertString(doc.getLength()," "+time+" "+txt+"\n\n",style);
                    } catch (BadLocationException e) {
                        e.printStackTrace();
                    }
                    MessagesTextPane.setCaretPosition(MessagesTextPane.getDocument().getLength());
                }
            });
        }
        else if (type.equalsIgnoreCase("error")){
            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    StyleConstants.setForeground(style, Color.red);
                    try {
                        doc.insertString(doc.getLength()," "+time+" "+txt+"\n\n",style);
                    } catch (BadLocationException e) {
                        e.printStackTrace();
                    }
                    MessagesTextPane.setCaretPosition(MessagesTextPane.getDocument().getLength());
                }
            });
        }
        else if (type.equalsIgnoreCase("warning")){

            SwingUtilities.invokeLater(new Runnable(){
                public void run(){
                    StyleConstants.setForeground(style, LIGHT_MAGENTA);
                    try {
                        doc.insertString(doc.getLength()," "+time+" "+txt+"\n\n",style);
                    } catch (BadLocationException e) {
                        e.printStackTrace();
                    }
                    MessagesTextPane.setCaretPosition(MessagesTextPane.getDocument().getLength());
                }
            });
        }
    }


    /**
     *  When Thalamus starts auto end it tells Plexus to call this method to reset the gui
     */
    public void endThalamusStart() {

        AbortButton.setEnabled(true);
        ConfigureButton.setEnabled(false);
        PauseButton.setEnabled(false);
        DownloadButton.setEnabled(false);
        PrestartButton.setEnabled(true);
        GoButton.setEnabled(false);
        EndButton.setEnabled(false);
        String tmpt = new Date().toString();
        if((tmpt!=null) && (tmpt.lastIndexOf("EST")>0))setEndTime(tmpt.substring(0,tmpt.lastIndexOf("EST")));
        else if((tmpt!=null) && (tmpt.lastIndexOf("EDT")>0))setEndTime(tmpt.substring(0,tmpt.lastIndexOf("EDT")));
        zeroCharts();
        selComp = null;
        listModel.clear();
        if(!ERROR_HAPPENED) startInfo(" Run End succeeded.", "Info");

    }

    /**
     * Configure button has been clicked
     * @param evt         Swing action event
     */
    private void ConfigureButtonClicked(java.awt.event.ActionEvent evt) {

        AbortButton.setEnabled(false);
        ConfigureButton.setEnabled(false);
        DownloadButton.setEnabled(false);

// Send a message to cortex asking to send a list of all configurations in the $COOL dir
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(rcSubscription.CrtSubject);

// We do not want to get runtypes report on all guis in the SESSION. Thus myname in the
// type will distiguish what gui was requested.
        msg.setType("ReportRunTypes_"+myName);
        msg.setText(_Session);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
        startrun = false;

    }


    /**
     * Method is called when download button is clicked
     */
    public void startDownload(){
// Send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_download);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Start clock animation
        startClock(" Downloading...");
        zeroCharts();

    }

    /**
     * Download button has been clicked
     * @param evt         Swing action event
     */
    private void DownloadButtonClicked(java.awt.event.ActionEvent evt) {
        downloadclicked = true;
//        StartRunButton.setEnabled(false);
//        startrun = false;
        startbuttonclicked = false;
        DownloadButton.setEnabled(false);
        startDownload();
    }

    /**
     * This mehod is called when prestart button is clicked
     */
    public void startPrestart(){
// Send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_prestart);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
// Start clock animation
        startClock(" Prestarting...");
        zeroCharts();

    }

    /**
     * Prestart button has been clicked
     * @param evt         Swing action event
     */
    private void PrestartButtonClicked(java.awt.event.ActionEvent evt) {
        downloadclicked = false;
        startPrestart();
        PrestartButton.setEnabled(false);
//        StartRunButton.setEnabled(false);
//        startrun = false;
        startbuttonclicked = false;
        prestartclicked = true;
    }

    /**
     * This mehod is called when go button is clicked
     */
    public void startGo(){
// Send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_go);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Start clock animation
        startClock(" Activating...");
    }

    /**
     * Go button has been clicked
     * @param evt         Swing action event
     */
    private void GoButtonClicked(java.awt.event.ActionEvent evt) {
        startGo();
        GoButton.setEnabled(false);
        goclicked = true;
        startbuttonclicked = false;
//        startrun = false;

    }

    /**
     *  End button has been clicked
     * @param evt         Swing action event
     */
    private void EndButtonClicked(java.awt.event.ActionEvent evt) {
// Send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_end);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
// Start clock animation
        startClock(" Ending...");
    }

    /**
     *  Pause button has been clicked
     * @param evt         Swing action event
     */
    private void PauseButtonClicked(java.awt.event.ActionEvent evt) {

// ... In case we are in active state
        if(PauseButton.getIcon().equals(Pause)){
// Send cMsg to Medulla
            cMsgMessage msg = new cMsgMessage();
            msg.setSubject(_Session+"_"+_RunType);
            msg.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_pause);
            try {
                cMsgSrv.send(msg);
                cMsgSrv.flush(1);
            } catch (cMsgException e) {
                e.printStackTrace();
            }
// Start clock animation
            startClock(" Pausing...");

// ... In case we are in pause state
        } else if(PauseButton.getIcon().equals(Resume)){

// Send cMsg to Medulla
            cMsgMessage msg = new cMsgMessage();
            msg.setSubject(_Session+"_"+_RunType);
            msg.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_resume);
            try {
                cMsgSrv.send(msg);
                cMsgSrv.flush(1);
            } catch (cMsgException e) {
                e.printStackTrace();
            }

// Start clock animation
            startClock(" Activating...");
        }
        update(getGraphics());
    }

    /**
     * Reset menu item has been selected
     * @param evt         Swing action event
     */
    private void ResetButtonClicked(java.awt.event.ActionEvent evt) {
// send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_reset);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
        configureReseted();
    }


    /**
     * Exit Components menu item has been selected
     * @param evt         Swing action event
     */
    private void CExitButtonClicked(java.awt.event.ActionEvent evt) {
// send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_Exit);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
        configureReseted();
    }


    /**
     * Abort button has been clecked
     * @param evt         Swing action event
     */
    private void AbortButtonClicked(java.awt.event.ActionEvent evt) {

// send cMsg to Medulla
        cMsgMessage msg = new cMsgMessage();
        msg.setSubject(_Session+"_"+_RunType);
        msg.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_reset);
        try {
            cMsgSrv.send(msg);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }

// Configure gui for abort
        configureAborted(true);
    }

    /**
     * Exit menu item has been selected
     * @param evt         Swing action event
     */
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        System.exit(0);
    }

    /**
     * Db2Cool action performed
     * @param evt         Swing action event
     */
    private void db2coolMenuItemActionPerformed(java.awt.event.ActionEvent evt) {

// Read the database runtypes table. If exception is thrown pop up the dialog, informing that
// connection with the database is failed due to probably missing env variables.
// Get the evironmental variables for msql
        String _CH = System.getenv("COOL_HOME");
        String _URL = System.getenv("CODA_DB_URL");
        String _DRIVER = System.getenv("CODA_DB_DRIVER");
        String _USER = System.getenv("CODA_DB_USER");
        String _PASSWORD = System.getenv("CODA_DB_PASSWORD");
        if(_USER==null)_USER=" ";
        if(_PASSWORD==null)_PASSWORD=" ";
        String hst = "";
        String db = "";
        if((_URL == null) || (_DRIVER == null)){
            addConsoleLine("Database specific environmental variables are not set","error");
            return;
        }
        StringTokenizer ust = new StringTokenizer(_URL,"/");
        if(ust.hasMoreTokens()) ust.nextToken();
        if(ust.hasMoreTokens()) {
            hst = ust.nextToken();
            if(hst.contains("."))hst = hst.substring(0,hst.indexOf("."));
            else hst = hst.substring(0,hst.indexOf(":"));
        }
        if(ust.hasMoreTokens()) db = ust.nextToken();

// Connect to the database.
        try {
            DatabaseConnection.createConnection(_URL,_DRIVER,_USER,_PASSWORD);
        } catch (DBException e) {
            addConsoleLine("DB-Error:\n  Can not create DB connection","error");
            e.printStackTrace();
            return;
        }

// Get the list of configuations and create a jlist component object.
        List l = DatabaseCommunication.getRuntypes();
        String[] ss = new String[l.size()];
        Iterator it = l.iterator();
        int i = 0;
        while (it.hasNext()){
            ss[i]=(String)it.next();
            i++;
        }

// Pop up dp runtypes dialog
        RcListDialog rd = new RcListDialog(GSTYLE);
        rd.showDialog(RcGui.this,ss,"Configurations","DB "+hst+":"+db,null);
        if(rd.status==1){

// Execute shell command db2cool and show the dialog box showing stdout of the execution
            String runtype = rd.getSelectedSession();
            Db2Cool mydbcool = new Db2Cool();

// Remember that if COOL_HOME is not set  the generated cool file will be created on the ./
            mydbcool.generateCool(runtype);
            if(_CH==null) addConsoleLine("Info:\n  COOL file for the "+runtype+" is generated in the current dir. ","warning");
            else addConsoleLine("Info:\n  COOL file for the "+runtype+" is generated in "+_CH+" . ","info");
        }
    }


    /**
     * Enable autostart action performed
     * @param evt         Swing action event
     */
    private void enableASMenuItemActionPerformed(java.awt.event.ActionEvent evt) {

// send cMsg to Medulla
        cMsgMessage msgo = new cMsgMessage();
        msgo.setSubject(_Session+"_"+_RunType);
        msgo.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_AutoStartTrue);
        try {
            cMsgSrv.send(msgo);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
        enableAutoStartMenuItem.setSelected(true);
    }


    /**
     * Dissable autostart action performed
     * @param evt         Swing action event
     */
    private void disableASMenuItemActionPerformed(java.awt.event.ActionEvent evt) {

// Send cMsg to Medulla
        cMsgMessage msgo = new cMsgMessage();
        msgo.setSubject(_Session+"_"+_RunType);
        msgo.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_AutoStartFalse);
        try {
            cMsgSrv.send(msgo);
            cMsgSrv.flush(1);
        } catch (cMsgException e) {
            e.printStackTrace();
        }
        disableAutoStartMenuItem.setSelected(true);
    }


    /**
     * Enable start run action performed
     * @param evt         Swing action event
     */
    public void enableESMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        if (BACKGROUND.equalsIgnoreCase("plain")) {
            if(configed && !_State.equalsIgnoreCase(rcConstants.CMD_postgo_ok)) {
                StartRunButton.setEnabled(true);
                startrun = true;
            }
        }
        else{
            PrestartButton.setIcon(StartRun);
            PrestartButton.setRolloverIcon(StartRunOver);
            startrun = true;
            PrestartButton.setEnabled(true);
        }
        enableEasyStart.setSelected(true);
    }

    /**
     * Dissable start run action performed
     * @param evt         Swing action event
     */
    public void disableESMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        if (BACKGROUND.equalsIgnoreCase("plain")) {
            StartRunButton.setEnabled(false);
            startrun = false;
        }
        else {
            PrestartButton.setIcon(Prestart);
            PrestartButton.setRolloverIcon(PrestartOver);
            startrun = false;
        }
        disableEasyStart.setSelected(true);
    }


    /**
     * Enable event limit action performed
     * @param evt         Swing action event
     */
    public void enableELMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        if(_State!=null &&
                !(_State.equalsIgnoreCase(rcConstants.CMD_postgo_ok))){
            LimitEventsTextField.setEditable(true);
            enableEvLimMenuItem.setSelected(true);
        }
    }

    /**
     * Dissable event limit action performed
     * @param evt         Swing action event
     */
    public void disableELMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        LimitEventsTextField.setText("");
        LimitEventsTextField.setEditable(false);
        disableEvLimMenuItem.setSelected(true);
    }

    /**
     * Expend graphics action performed
     * @param evt         Swing action event
     */
    private void moreGraphsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        if(BACKGROUND.equalsIgnoreCase("plain")) setSize(1005,883);
        else setSize(1010,970);
        moreGraphsMenuItem.setSelected(true);

    }

    /**
     * Shrink graphics action performed
     * @param evt         Swing action event
     */
    private void lessGraphsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        if(BACKGROUND.equalsIgnoreCase("plain")) setSize(566,883);
        else setSize(570,970);
        lessGraphsMenuItem.setSelected(true);
    }

    /**
     * Start Dalog message browser in the separate thread
     * @param evt         Swing action event
     */
    private void dalogBrowserMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        DalogBrowserApp dabro = new DalogBrowserApp();
        dabro.name = myName+"_dabro";
        new Thread(dabro).start();
    }


    /**
     *  0s the histograms for rcGui and for flash
     */
    public void zeroCharts(){

        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                double [] l1r  = new double[12];
                double[] test = new double[_ComponentCount];

                if(myGraph!=null){
                    myGraph.value = 0;
                    myGraph.datasource.update();
                }

// 0 data rate global arrray
                for (int i= 0; i<_ComponentCount; i++) _DataRate[i] = 0;
                if (myDataRateBG != null) {
                    for (int i = 0; i < test.length; i++) test[i] = 0;
                    myDataRateBG.values = test;
                    if(myDataRateBG.datasource!=null)myDataRateBG.datasource.update();
                }

                if (myTriggerBitsBG != null) {
                    for (int i = 0; i < l1r.length; i++) l1r[i] = 0;
                    myTriggerBitsBG.values = l1r;
                    myTriggerBitsBG.datasource.update();
                }

                if (myLiveTimeTG != null) {
                    myLiveTimeTG.value = 0;
                    myLiveTimeTG.datasource.update();
                }

                if (myDataRateTG != null) {
                    myDataRateTG.value = 0;
                    myDataRateTG.datasource.update();
                }

                if (myEventRateTG != null) {
                    myEventRateTG.value = 0;
                    myEventRateTG.datasource.update();
                }
            }
        });
    }

    /**
     * Configures accordingly the UI elements based on the input parameter
     * @param rs _runState comming from Medulla
     */
    public void configureUI(final String rs){
// React accordingly to the state of the run
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){

// ... Configured
                if(rs.equalsIgnoreCase("configured")) {
                    _State = "Configured";
                    setRunState("Configured");
                    if(!ERROR_HAPPENED) {
                        addConsoleLine("Configure succeeded","info");
                        stopClock();
                        try {startInfo("Configure succeeded", "Info");} catch (Exception e){e.printStackTrace();}
                    }
                    configed = true;
                    AbortButton.setEnabled(true);
                    ConfigureButton.setEnabled(false);
                    PauseButton.setEnabled(false);
                    DownloadButton.setEnabled(true);
                    if(startrun ) {
                        if (BACKGROUND.equalsIgnoreCase("plain")) {
                            StartRunButton.setEnabled(true);
                        } else {
                            PrestartButton.setEnabled(true);
                        }
                    } else {
                        StartRunButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                    }
                    GoButton.setEnabled(false);
                    EndButton.setEnabled(false);
                    setStartTime("");
                    setEndTime("");
                    setEventNumber("");
                    setGEvtRate(0.0);
                    setGDataRate(0.0);
                    setGAverEvtRate(0.0);
                    setGAverDataRate(0.0);

// ... PostDownload
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_postdownload_ok)) {
                    _State = rcConstants.CMD_postdownload_ok;
                    configed = true;

// Stop the clock and print the run sate on the timegraphpane
                    setRunState("Downloaded");
                    if(!ERROR_HAPPENED) {
                        stopClock();
                        addConsoleLine("Download succeeded","info");
                        try {  startInfo("Download succeeded","Info");} catch (Exception e){e.printStackTrace();}
                    }
                    AbortButton.setEnabled(true);
                    ConfigureButton.setEnabled(false);
                    PauseButton.setEnabled(false);
                    DownloadButton.setEnabled(false);
                    PrestartButton.setEnabled(true);
                    GoButton.setEnabled(false);
                    EndButton.setEnabled(false);
                    if(startrun){
                        if (BACKGROUND.equalsIgnoreCase("plain")) {
                        DownloadButton.setEnabled(false);
                        StartRunButton.setEnabled(true);
                        GoButton.setEnabled(false);
                        } else {
                            DownloadButton.setEnabled(false);
                            PrestartButton.setEnabled(true);
                            GoButton.setEnabled(false);
                        }
                        if(startbuttonclicked)startPrestart();
                    }

// ... PostPrestart
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_postprestart_ok)) {
                    prestartclicked = false;
                    _State = rcConstants.CMD_postprestart_ok;
                    configed = true;
                    stopClock();
                    setRunState("Prestarted");
                    if(!ERROR_HAPPENED) {
                        stopClock();
                        addConsoleLine("Prestart succeeded","info");
                        try {  startInfo("Prestart succeeded", "Info");} catch (Exception e){e.printStackTrace();}
                    }
                    AbortButton.setEnabled(true);
                    ConfigureButton.setEnabled(false);
                    PauseButton.setEnabled(false);
                    DownloadButton.setEnabled(false);
                    PrestartButton.setEnabled(false);
                    GoButton.setEnabled(true);
                    EndButton.setEnabled(true);
                    setStartTime("");
                    setEndTime("");
                    setEventNumber("");
                    setGEvtRate(0.0);
                    setGDataRate(0.0);
                    setGAverEvtRate(0.0);
                    setGAverDataRate(0.0);
                    if(startrun){
                        if (BACKGROUND.equalsIgnoreCase("plain")) {
                            DownloadButton.setEnabled(false);
                            StartRunButton.setEnabled(true);
                            GoButton.setEnabled(true);
                        } else {
                        DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        }
                        if(startbuttonclicked)startGo();
                    }


// ... PostGo
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_postgo_ok)) {
                    goclicked = false;
                    _State = rcConstants.CMD_postgo_ok;
                    configed = true;
                    setRunState("Active");
                    if(!ERROR_HAPPENED) {
                        stopClock();
                        addConsoleLine("Go succeeded","info");
                        try {  startInfo("Run is active", "Info");} catch (Exception e){e.printStackTrace();}
                    }
                    AbortButton.setEnabled(true);
                    ConfigureButton.setEnabled(false);
                    PauseButton.setEnabled(true);
                    DownloadButton.setEnabled(false);
                    PrestartButton.setEnabled(false);
                    GoButton.setEnabled(false);
                    EndButton.setEnabled(true);
                    StartRunButton.setEnabled(false);
                    PauseButton.setIcon(Pause);
                    PauseButton.setRolloverIcon(PauseOver);
                    PauseButton.setForeground(Color.BLACK);
                    String tmpt = new Date().toString();
                    if((tmpt!=null) && (tmpt.lastIndexOf("EST")>0))setStartTime(tmpt.substring(0,tmpt.lastIndexOf("EST")));
                    else if((tmpt!=null) && (tmpt.lastIndexOf("EDT")>0))setStartTime(tmpt.substring(0,tmpt.lastIndexOf("EDT")));
                    if(myLiveTimeTG!=null)myLiveTimeTG.remove();
                    myLiveTimeTG = new TimeGraphLite("%",50,100.0, liveTimeTGPane, true);

// Start graphics for the selected copmponent (previously selected from the last run).
                    if(selComp !=null) {
                        if(myGraph!=null)myGraph.remove();
                        myGraph = new TimeGraphLite(selComp+": Hz",50,2000.0, TimeGraphPane, false);

// Here we duplicate it on the event rate timegraph pane.
                        if(myEventRateTG!=null)myEventRateTG.remove();
                        myEventRateTG = new TimeGraphLite(selComp+": Hz",50,2000.0, EventRateTGPane, false);

                        if(_CompTypes.get(selComp).equalsIgnoreCase(rcConstants.ebType) || _CompTypes.get(selComp).equalsIgnoreCase(rcConstants.erType)){
                            if(myDataRateTG!=null)myDataRateTG.remove();
                            myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,3000.0,DataRateTGPane, false);
                            if(_Components.get(selComp)!=null)ComponentList.setSelectedIndex(_Components.get(selComp));
                        }
                        else{
                            if(myDataRateTG!=null)myDataRateTG.remove();
                            myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,500.0,DataRateTGPane, false);
                            if(_Components.get(selComp)!=null)ComponentList.setSelectedIndex(_Components.get(selComp));
                        }
                    }

// ... PostEnd
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_postend_ok)) {
                    _State = rcConstants.CMD_postend_ok;
                    configed = true;
                    addConsoleLine("End succeeded","info");
                    if(!ERROR_HAPPENED){
                        stopClock();
                        setRunState("Ended");
                        try { startInfo("End succeeded", "Info");} catch (Exception e){e.printStackTrace();}
                    }
                    AbortButton.setEnabled(true);
                    ConfigureButton.setEnabled(false);
                    PauseButton.setEnabled(false);
                    DownloadButton.setEnabled(false);
                    if(startrun){
                        if (BACKGROUND.equalsIgnoreCase("plain")) {
                        StartRunButton.setEnabled(true);
                        }
                    }
                    PrestartButton.setEnabled(true);
                    GoButton.setEnabled(false);
                    EndButton.setEnabled(false);
                    String tmpt = new Date().toString();
                    if((tmpt!=null) && (tmpt.lastIndexOf("EST")>0))setEndTime(tmpt.substring(0,tmpt.lastIndexOf("EST")));
                    else if((tmpt!=null) && (tmpt.lastIndexOf("EDT")>0))setEndTime(tmpt.substring(0,tmpt.lastIndexOf("EDT")));

// Reset the charts
                    zeroCharts();

// Force garbage collection
                    System.runFinalization();
                    System.gc();

// ... Reseted
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_reset_ok)) {
//                    configureReseted();

// ...Paused
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_pause_ok)) {
                    if(!prestartclicked){
                        _State = rcConstants.CMD_pause_ok;
                        configed = true;
                        addConsoleLine("Pause succeeded","info");
                        setRunState("Paused");
                        try { startInfo("Run is paused", "Info");} catch (Exception e){e.printStackTrace();}
                        PauseButton.setIcon(Resume);
                        PauseButton.setRolloverIcon(ResumeOver);
                        PauseButton.setToolTipText("Resume");
                        if(!BACKGROUND.equalsIgnoreCase("plain")) PauseButton.setText("Resume");
                        PauseButton.setForeground(Color.RED);
                        RunStatusTextField.setText("Paused");
                        EndButton.setEnabled(true);
                    }

// ... Resumed
                } else if (rs.equalsIgnoreCase(rcConstants.CMD_resume_ok)) {
                    if(!goclicked){
                        _State = rcConstants.CMD_resume_ok;
                        configed = true;
                        addConsoleLine("Resume succeeded","info");
                        setRunState("Active");
                        try { startInfo("Activated", "Info");} catch (Exception e){e.printStackTrace();}
                        AbortButton.setEnabled(true);
                        ConfigureButton.setEnabled(false);
                        PauseButton.setEnabled(true);
                        DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        EndButton.setEnabled(true);
                        PauseButton.setIcon(Pause);
                        PauseButton.setRolloverIcon(PauseOver);
                        PauseButton.setToolTipText("Pause");
                        if(!BACKGROUND.equalsIgnoreCase("plain")) PauseButton.setText("Pause");
                        PauseButton.setForeground(Color.BLACK);

// Start graphics for the selected copmponent (previously selected from the last run).
                        if(selComp !=null) {
                            if(myGraph!=null)myGraph.remove();
                            myGraph = new TimeGraphLite(selComp+": Hz",50,2000.0, TimeGraphPane, false);

// Here we duplicate it on the event rate timegraph pane.
                            if(myEventRateTG!=null)myEventRateTG.remove();
                            myEventRateTG = new TimeGraphLite(selComp+": Hz",50,2000.0, EventRateTGPane, false);
                            if(_CompTypes.get(selComp).equalsIgnoreCase(rcConstants.ebType) || _CompTypes.get(selComp).equalsIgnoreCase(rcConstants.erType)){
                                if(myDataRateTG!=null)myDataRateTG.remove();
                                myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,3000.0,DataRateTGPane, false);
                                if(_Components.get(selComp)!=null) ComponentList.setSelectedIndex(_Components.get(selComp));
                            }
                            else {
                                if(myDataRateTG!=null)myDataRateTG.remove();
                                myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,500.0,DataRateTGPane, false);
                                if(_Components.get(selComp)!=null) ComponentList.setSelectedIndex(_Components.get(selComp));
                            }
                        }
                        _RunState = rcConstants.CMD_postgo_ok;
                    }
                }
            }
        });
    }


    /**
     * Configure gui for the case of reset succeeeded
     */
    public void configureReseted(){
        _AutoReseted = false;
        // Configure gui for reset
        _State = rcConstants.CMD_reset_ok;
        addConsoleLine("Reset succeeded","info");
        setRunState("Resetted");
        try { startInfo("Resetted", "Info");} catch (Exception e){e.printStackTrace();}
        PauseButton.setIcon(Pause);
        PauseButton.setRolloverIcon(PauseOver);
        AbortButton.setEnabled(false);
        ConfigureButton.setEnabled(true);
        ConfigureButton.setIcon(Configure);
        PauseButton.setEnabled(false);
        DownloadButton.setEnabled(false);
        PrestartButton.setEnabled(false);
        GoButton.setEnabled(false);
        EndButton.setEnabled(false);
        StartRunButton.setEnabled(false);
        startrun = false;
        startbuttonclicked = false;
        LimitEventsTextField.setText("");
        LimitEventsTextField.setEditable(false);
        ComponentList.setSelectionForeground(Color.black);
        _Reseted = true;
        configed = false;
// Clear the components list
        listModel.clear();
        setStartTime("");
        setEndTime("");
        setEventNumber("");
        setGEvtRate(0.0);
        setGDataRate(0.0);
        setGAverEvtRate(0.0);
        setGAverDataRate(0.0);
        zeroCharts();
    }

    /**
     * Configure gui for the case of abort succeeeded
     * @param resetclicked                     boolean indicates that abort button has been clecked
     */

    public void configureAborted(boolean resetclicked){

        _State = rcConstants.CMD_reset_ok;
        addConsoleLine("Abort succeeded","info");
        setRunState("Aborted");
        if(resetclicked) try { startInfo("Aborted", "Info");} catch (Exception e){e.printStackTrace();}
        PauseButton.setIcon(Pause);
        PauseButton.setRolloverIcon(PauseOver);
        AbortButton.setEnabled(true);
        ConfigureButton.setEnabled(true);
        ConfigureButton.setIcon(Configure);
        PauseButton.setEnabled(false);
        DownloadButton.setEnabled(true);
        PrestartButton.setEnabled(false);
        StartRunButton.setEnabled(false);
        GoButton.setEnabled(false);
        EndButton.setEnabled(false);
        if(startrun)StartRunButton.setEnabled(true);
        LimitEventsTextField.setText("");
        LimitEventsTextField.setEditable(false);
        ComponentList.setSelectionForeground(Color.black);
        _Reseted = true;
        configed = true;
// Clear the components list
        listModel.clear();
        setStartTime("");
        setEndTime("");
        setEventNumber("");
        setGEvtRate(0.0);
        setGDataRate(0.0);
        setGAverEvtRate(0.0);
        setGAverDataRate(0.0);
        zeroCharts();
    }


    /**
     * Method extracts the data (passed as a String ) and updateds the RcGui grahics
     * @param msg                 String of the format:
     *                           SESSION  runtype component_name:evt_number:evt_rate:data_rate ...
     */
    public void extractTextMonData(final String msg){
        final CompMonitor tcr;

        SwingBackgroundThread parseMonData = new SwingBackgroundThread(msg);
        parseMonData.execute();
        while(parseMonData.getState() != SwingWorker.StateValue.DONE){
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        parseMonData.done();

// Report runtype if _RunType is null. N.B. This will happen if gui starts asynchronly.
        if((_RunType==null) || (_Components.size()==0)) {
            reportRuntype();
            return;
        }

// Components are reporting, so turn them black in the component list.
// N.B.if only one component reports they all will turn black
        if(_Components.size() == _ComponentCount)
            ComponentList.setForeground(Color.black);

// Update gui if this particular component is selected
        if(selComp!=null && _CompStatistics!=null){
            tcr = _CompStatistics.get(selComp);
            if(tcr!=null){
                setEventNumber(Integer.toString(tcr.getEvtNumber()));
                setGAverEvtRate(tcr.getAverEvtRate());
                setGAverDataRate(tcr.getAverDataRate());
                setGEvtRate(tcr.getEvtRate());
                if(myGraph!=null){
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            myGraph.value = tcr.getPAEvtRate();
                            myGraph.datasource.update();
                        }
                    });
                }

                if(myEventRateTG!=null){
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            myEventRateTG.value = tcr.getPAEvtRate();
                            myEventRateTG.datasource.update();
                        }
                    });
                }

                setGDataRate(tcr.getDataRate());
                if(myDataRateTG !=null){
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            myDataRateTG.value = tcr.getPADataRate();
                            myDataRateTG.datasource.update();
                        }
                    });
                }

// End the run if the event limit is reached
                if(LimitEventsTextField.isEditable() && !(LimitEventsTextField.getText().equals(""))){
                    try{
                        if(tcr.getEvtNumber() >= Integer.parseInt(LimitEventsTextField.getText())){

// This is neccessary to prevent multiple end request to the Medulla, since
// this is initiated from the Medulla reportings
                            LimitEventsTextField.setEditable(false);

// Send end cMsg to Medulla
                            cMsgMessage m = new cMsgMessage();
                            m.setSubject(_Session+"_"+_RunType);
                            m.setType(rcSubscription.RunTransitionType +"/"+rcConstants.CMD_end);
                            try {
                                cMsgSrv.send(m);
                                cMsgSrv.flush(1);
                            } catch (cMsgException e) {
                                e.printStackTrace();
                            }
// Start clock animation
                            startClock(" Ending...");
                            return;
                        }
                    } catch(NumberFormatException e){
                        LimitEventsTextField.setText("");
                        LimitEventsTextField.setEditable(false);
                    }
                }
            }
        }

// Update the components list only once
        if(_Reseted){

// Components are reporting, so turn them black in the component list.
// N.B.if only one component reports they all will turn black
            if(_Components.size() == _ComponentCount)
                ComponentList.setForeground(Color.black);

// Start graphics for the EB component.
            if(selComp !=null) {
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        if(myGraph!=null)myGraph.remove();
                        myGraph = new TimeGraphLite(selComp+": Hz",50,2000.0, TimeGraphPane, false);
                    }
                });

// Here we duplicate it on the event rate timegraph pane.
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){
                        if(myEventRateTG!=null)myEventRateTG.remove();
                        myEventRateTG = new TimeGraphLite(selComp+": Hz",50,2000.0, EventRateTGPane, false);
                    }
                });
                if(_CompTypes.get(selComp).equalsIgnoreCase(rcConstants.ebType) || _CompTypes.get(selComp).equalsIgnoreCase(rcConstants.erType)){
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            if(myDataRateTG!=null)myDataRateTG.remove();
                            myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,3000.0,DataRateTGPane, false);
                        }
                    });
                    if(_Components.get(selComp)!=null)ComponentList.setSelectedIndex(_Components.get(selComp));
                }
                else{
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){
                            if(myDataRateTG!=null)myDataRateTG.remove();
                            myDataRateTG = new TimeGraphLite(selComp+": KBytes",50,500.0,DataRateTGPane, false);
                        }
                    });
                    if(_Components.get(selComp)!=null)ComponentList.setSelectedIndex(_Components.get(selComp));
                }
            }
            _Reseted = false;
        }

// Update the bargraph for data rates
        if(_graphPlotId > _ComponentCount/50){
            if(myDataRateBG!=null){
                SwingUtilities.invokeLater(new Runnable(){
                    public void run(){

// Fill the data rate array for data_rate bargraph and components hashtable
                        for (String s : _CompStatistics.keySet()) {

// Put 0 for EB and ER datarates, so that scaling of the barchart will not be intimidated by EB/ER datarates.
// N.B. data rates in the components are usually smaller  the EB and ER.
                            if( (_CompTypes.get(s).equalsIgnoreCase(rcConstants.ebType)
                                    || _CompTypes.get(s).equalsIgnoreCase(rcConstants.erType)) && _hideEbErBars)
                                _DataRate[_Components.get(s)]=0.0;
                            else
                                _DataRate[_Components.get(s)] = _CompStatistics.get(s).getDataRate();
                        }
                        myDataRateBG.values = _DataRate;
                        myDataRateBG.datasource.update();
                    }
                });
            }
            _graphPlotId = 0;
        } else {
            _graphPlotId++;
        }
    }




// .... Inner class

    /**
     * Swing worker class
     */
    class SwingBackgroundThread extends SwingWorker<ConcurrentHashMap<String, CompMonitor>, Void> {

        private final String msg;

        public SwingBackgroundThread(String ms){
            msg = ms;
        }

        public ConcurrentHashMap<String, CompMonitor> doInBackground() {
            String componentName, session, runtype, runstate_current;
            int evt_num;
            float evt_rate, average_evt_rate;
            double data_rate, average_data_rate;
            String componentInfo;
            CompMonitor tc;
            ConcurrentHashMap<String, CompMonitor> CurCompStatistics = new ConcurrentHashMap<String, CompMonitor>();

            StringTokenizer stt = new StringTokenizer(msg);
            if(stt.hasMoreTokens()) session = stt.nextToken();
            if(stt.hasMoreTokens()) runtype = stt.nextToken();
            if(stt.hasMoreTokens()) runstate_current = stt.nextToken();

            String tmp;

// Extracting all reported components info
            while(stt.hasMoreTokens()){
                componentInfo = stt.nextToken();

// Tokanaize the component info of the form component_name:evt_num:evt_rate:data_rate
                StringTokenizer ct = new StringTokenizer(componentInfo,":");
                if(ct.hasMoreTokens()) componentName = ct.nextToken(); else break;
                if(ct.hasMoreTokens()) evt_num = Integer.parseInt(ct.nextToken()); else break;

//The values for evt-rate and data_rate comming from component agent might be NotAvailable in case
// agent is averaging it for smooting purposes.
                if(ct.hasMoreTokens()){
                    tmp = ct.nextToken();
                    evt_rate = Float.parseFloat(tmp);
                }else break;
                if(ct.hasMoreTokens()){
                    tmp = ct.nextToken();
                    data_rate = Double.parseDouble(tmp);
                }else break;
                if(ct.hasMoreTokens()) average_evt_rate = Float.parseFloat(ct.nextToken()); else break;
                if(ct.hasMoreTokens()) average_data_rate = Double.parseDouble(ct.nextToken());else break;
                tc = new CompMonitor();
                tc.evtR[0] = evt_rate;
                tc.dataR[0] = data_rate;
                tc.setPAEvtRate(evt_rate);
                tc.setPADataRate(data_rate);

// Fill actual CompMonitor object
                tc.setEvtNumber(evt_num);
                tc.setEvtRate(evt_rate);
                tc.setDataRate(data_rate);
                tc.setAverEvtRate(average_evt_rate);
                tc.setAverDataRate(average_data_rate);
                CurCompStatistics.put(componentName,tc);
            }
            return CurCompStatistics;
        }

        public void done() {
            ConcurrentHashMap<String, CompMonitor> tmpCompStatistics;
            String tmpcomponentName;
            CompMonitor tmptc;
            try {

                tmpCompStatistics = get(10l, TimeUnit.MILLISECONDS);
                for (String s : tmpCompStatistics.keySet()) {
                    tmpcomponentName = s;

// See if component was reporting before. If not  create the CompMonitor object for storing the statistics info.
                    if (!_CompStatistics.containsKey(tmpcomponentName)) tmptc = tmpCompStatistics.get(tmpcomponentName);
                    else {
                        tmptc = _CompStatistics.get(tmpcomponentName);
                        tmptc.arrayShift();
                        tmptc.evtR[0] = tmpCompStatistics.get(tmpcomponentName).getEvtRate();
                        tmptc.dataR[0] = tmpCompStatistics.get(tmpcomponentName).getDataRate();

// Calculate sliding average values
                        float tmpevtr = 0f;
                        double tmpdatar = 0l;
                        for (int i = 0; i < AIV; i++) {
                            tmpevtr = tmpevtr + tmptc.evtR[i];
                            tmpdatar += tmptc.dataR[i];
                        }
                        tmptc.setPAEvtRate(tmpevtr / AIV);
                        tmptc.setPADataRate(tmpdatar / AIV);

// Fill actual CompMonitor object
                        tmptc.setEvtNumber(tmpCompStatistics.get(tmpcomponentName).getEvtNumber());
                        tmptc.setEvtRate(tmpCompStatistics.get(tmpcomponentName).getEvtRate());
                        tmptc.setDataRate(tmpCompStatistics.get(tmpcomponentName).getDataRate());
                        tmptc.setAverEvtRate(tmpCompStatistics.get(tmpcomponentName).getAverEvtRate());
                        tmptc.setAverDataRate(tmpCompStatistics.get(tmpcomponentName).getAverDataRate());
                        _CompStatistics.remove(tmpcomponentName);

//                        System.out.println("------------------");
//                        System.out.println(tmpcomponentName);
//                        System.out.println("------------------");
//                        System.out.println(tmpCompStatistics.get(tmpcomponentName).getEvtNumber());
//                        System.out.println(tmpCompStatistics.get(tmpcomponentName).getEvtRate());
//                        System.out.println(tmpCompStatistics.get(tmpcomponentName).getDataRate());
//                        System.out.println(tmpCompStatistics.get(tmpcomponentName).getAverEvtRate());
//                        System.out.println(tmpCompStatistics.get(tmpcomponentName).getAverDataRate());

                    }
                    _CompStatistics.put(tmpcomponentName, tmptc);
                }
            }
            catch (java.util.concurrent.ExecutionException e) {
                e.printStackTrace();
            } catch (InterruptedException e1) {
                e1.printStackTrace();
            } catch (TimeoutException e) {
                e.printStackTrace();
            }
        }
    }


    // .... Inner class
    /**
     *
     * cMsg callback class to handle INFO messages from the Medulla
     */
    class InfoCallback extends cMsgCallbackAdapter {

        public void callback(cMsgMessage msg, Object userObject){

            boolean itsmine = false;
            String messageType = msg.getType();
            String message = msg.getText();

// Coda transition related Info messages
// if text starts with GUI:, as usually it will follow the state of the run (e.g. GUI:ended)
            if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeTransition)) {
                if (message.startsWith("GUI")) {
                    if (message.lastIndexOf(":") > 0)
                        _RunState = message.substring(message.lastIndexOf(":") + 1);
                    SwingUtilities.invokeLater(new Runnable(){
                        public void run(){

// Cnfigure the UI based on the _RunState comming from Medulla
                            configureUI(_RunState);
                        }
                    });
                }
                else {
                    addConsoleLine(message, "info");
                }
            }

// Rn status info message
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeRunStatus)){
                _RunState = message;

// Configure the UI based on the _RunState coming from Medulla
                configureUI(_RunState);
            }

// Autostart option setting from cortex
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeAutoStart)){
                if(message.equalsIgnoreCase("true")){
                    enableAutoStartMenuItem.setSelected(true);
                } else {
                    disableAutoStartMenuItem.setSelected(false);
                }
            }

// StartRun option setting from cortex
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeStartRun)){
                if(message.equalsIgnoreCase("true")){
                    startrun=true;
                    enableESMenuItemActionPerformed(null);
                } else {
                    startrun=false;
                    disableESMenuItemActionPerformed(null);
                }
            }

// EventLimit option setting from cortex
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeEventLimit)){
                try {
                    if(Integer.parseInt(message)>0){
                        enableELMenuItemActionPerformed(null);
                        LimitEventsTextField.setText(message);
                    } else {
                        disableELMenuItemActionPerformed(null);
                    }
                } catch (NumberFormatException e) {
                    e.printStackTrace();
                }
            }

// Run type
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeRunType)){
                if(message!=null) {
                    _RunType = message;
                    setRunType(_RunType);
                    reportConfiguration(_Session+"_"+_RunType);
                }
            }

// filename from ER
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeFileName)){
                if(message!=null) {
                    setOutputFile(message);
                }
            }

// Run SESSION  info message obtained from the Medulla
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeSession)){
                if(message.equalsIgnoreCase(_Session))
                    addConsoleLine("rcGui-Error: Wrong session. \n" +
                            "Exit the GUI and set your SESSION environmental variable accordingly.","info");
            }

// Data file info message
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeDataFile)){
                setOutputFile(message);
            }

// Run number info message
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeRunNumber)){
                setRunNumber(message);
            }

// Run component count info message
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeComponentCount)){
                _ComponentCount = Integer.parseInt(message);
                _DataRate = new double[_ComponentCount];
                myDataRateBG = new org.jlab.coda.runcontrol.gui.BarGraphLite("KByte",_ComponentCount,500.0, dataRateBGPane);
            }

// Medulla reports the componentName:reportingInterval paires
            else if(messageType.contains(rcSubscription.RcGuiTypeCompRepInterval)){

// Get the name of the gui that requested SESSIONs report
                StringTokenizer st = new StringTokenizer(messageType,"_");
                st.nextToken();
                String gn = null;
                if(st.hasMoreTokens())gn = st.nextToken();

// Ok this was me sending the report registered SESSIONs request
                if(gn.equals(myName)){
                    _CompReportInterval.clear();
                    StringTokenizer stk = new StringTokenizer(message);
                    StringTokenizer nameintervaltk;
                    String compname = "-", interval = "0.5";
                    while(stk.hasMoreTokens()){

// The message is space separated name:interval contructs
                        nameintervaltk = new StringTokenizer(stk.nextToken(),":");

// Get the component name
                        if(nameintervaltk.hasMoreTokens()) compname = nameintervaltk.nextToken();

// Get the component type
                        if(nameintervaltk.hasMoreTokens())interval = nameintervaltk.nextToken();
                        _CompReportInterval.put(compname,interval);
                    }

// Open the sliderlistdialog gui.
                    RcSliderListDialog dialog = new RcSliderListDialog(GSTYLE);
                    String rs[] = new String[_CompReportInterval.keySet().size()];
                    int i=0;
                    for (String s : _CompReportInterval.keySet()) {
                        rs[i++] = s+" "+_CompReportInterval.get(s);
                    }
                    dialog.showDialog(RcGui.this, rs,"Reporting Intervals","RC Reporting",null);

// Get the user selected component and its reporting interval
                    if(dialog.getReturnStatus() == RcSliderListDialog.RET_OK){
                        for(String s:dialog.compNameValue.keySet()){
                            if(_CompReportInterval.containsKey(s)){
                                String vv = String.valueOf(dialog.compNameValue.get(s));
                                if(!_CompReportInterval.get(s).equals(vv)){
                                    _CompReportInterval.remove(s);
                                    _CompReportInterval.put(s,vv);

// Here we tell RCS agent to change the reporting interval of the particular component.
// Since the request is for the single component only (no multiple selections from the gui list)
// The format is: [name interval]
                                    cMsgMessage msgo = new cMsgMessage();
                                    msgo.setSubject(_Session+"_"+_RunType);
                                    msgo.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_SetInterval);
                                    msgo.setText(s+" "+vv);
                                    try {
                                        cMsgSrv.send(msgo);
                                        cMsgSrv.flush(1);
                                    } catch (cMsgException e) {
                                        e.printStackTrace();
                                    }
                                }
                            }
                        }
                    }
                }
            }

// Run component names info message
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeComponentNames)){
                ComponentList.setSelectionForeground(Color.black);
                int index = 0;
                _Components.clear();
                selComp = null;

// Component name is red for now, until it will report data
                ComponentList.setForeground(VERY_DARK_GREEN);
                StringTokenizer stk = new StringTokenizer(message);
                StringTokenizer nametypetk;
                String compname = "-", comptype = "-";
                while(stk.hasMoreTokens()){

// The message is space separated name:type contructs
                    nametypetk = new StringTokenizer(stk.nextToken(),":");

// Get the component name
                    if(nametypetk.hasMoreTokens()) compname = nametypetk.nextToken();

// Get the component type
                    if(nametypetk.hasMoreTokens())comptype = nametypetk.nextToken();

// Assign this index to the component and fill the components hashmap
                    _Components.put(compname,index);

// Fill types hashmap
                    _CompTypes.put(compname,comptype);

// Increment the index
                    index++;
                }

// Populate the components list in the gui
// Clear the components list
                listModel.clear();
                for(int i=0;i<_Components.size();i++) listModel.add(i," ");

// Get iterator on _Components hash table and fill the components list
                selComp = null;

// First select ER component if there is one
                for (String s : _Components.keySet()) {
                    try{
                        try {
                            Thread.sleep(10);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                        listModel.set(_Components.get(s),s);
                    } catch(ArrayIndexOutOfBoundsException ex){
                        ex.printStackTrace();
                    }
                    if(_CompTypes.get(s).equalsIgnoreCase(rcConstants.erType)){
                        selComp = s;
                    }
                }

// If there is no ER  select EB
                if(selComp == null){
                    for (String s : _CompTypes.keySet()) {
                        if(_CompTypes.get(s).equalsIgnoreCase(rcConstants.ebType) ){
                            selComp = s;
                            break;
                        }
                    }
                }

// Otherwise select the first component from the component list
                if(selComp == null){

// If we do not have EB component select the first component from the cool file.
                    for (String s : _CompTypes.keySet()) {
                        if(_CompTypes.get(s).equalsIgnoreCase(rcConstants.rocType) ){
                            selComp = s;
                            break;
                        }
                    }
                }
            }

// Run statistics info message
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeRunStatistics)){

// Get the statistics data formated into the string with the following format:
// N.B. following is the internal format used:
// SESSION runtype component_name:evt_number:evt_rate:data_rate ...
                extractTextMonData(message);
            }

// Got the registered SESSIONs reported by cortex
            else if(messageType.contains(rcSubscription.RcGuiTypeRegisteredSessions)){

// Get the name of the gui that requested SESSIONs report
                StringTokenizer stt = new StringTokenizer(messageType,"_");
                stt.nextToken();
                String gn = null;
                if(stt.hasMoreTokens())gn = stt.nextToken();

// Ok this was me sending the report registered SESSIONs request
                if(gn.equals(myName)){

// Start SESSION dialog
                    StringTokenizer st = new StringTokenizer(message);
                    int ii=0;
                    String [] ss = new String[st.countTokens()];
                    while (st.hasMoreTokens()) {
                        ss[ii] = st.nextToken();
                        ii++;
                    }

// Pop up SESSION dialog
                    RcListDialog sd = new RcListDialog(GSTYLE);
                    sd.showDialog(RcGui.this,ss,"Platform session s","RCSession s",null);

// Get the user selected SESSION
                    if(sd.getReturnStatus() == RcListDialog.RET_OK){
                        if(!_Session.equals(sd.getSelectedSession())) {
                            _Session = sd.getSelectedSession();

// Show the new session
                            setSession(_Session);

// Reset the gui to its initial state
                            resetGui();

// Connect to cMsg server for getting messages from Medulla
                            try {

// Unsubscribe
                                if(cMsgSrv!=null) {
                                    if(ob1!=null)cMsgSrv.unsubscribe(ob1);
                                    if(ob2!=null)cMsgSrv.unsubscribe(ob2);
                                    if(ob3!=null)cMsgSrv.unsubscribe(ob3);
                                    if(ob4!=null)cMsgSrv.unsubscribe(ob4);
                                    if(ob5!=null)cMsgSrv.unsubscribe(ob5);
                                    if(ob6!=null)cMsgSrv.unsubscribe(ob6);
                                    if(ob7!=null)cMsgSrv.unsubscribe(ob7);
                                }

// Subscribe to RcGuiSubjInfo and type=*
                                ob1 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjInfo+"_"+_Session, "*", infocb, null);

// Subscribe to RcGuiSubjWarning and type=*
                                ob2 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjWarning+"_"+_Session, "*", warningcb, null);

// Subscribe to RcGuiSubjError and type=*
                                ob3 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjError+"_"+_Session, "*", errorcb, null);

// Subscribe to RcGuiSubjTsData and type=*
                                ob4 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjTsData+"_"+_Session, "*", tscb, null);

// Subscribe to Hepatic messages and type=*
                                ob5 = cMsgSrv.subscribe(rcSubscription.Hepatic, "*", hepaticcb, null);

// Subscribe to Cortex messages and type=*
                                ob6 = cMsgSrv.subscribe(rcSubscription.CrtSubject, "*", cortexcb, null);

// Subscribe to Stemcell messages and type=*
                                ob7 = cMsgSrv.subscribe(rcSubscription.Stemcell, "*", stemcellcb, null);
                            } catch (cMsgException e) {
                                e.printStackTrace();
                                System.exit(0);
                            }

// Ask Cortex to report the runtype active in the new selected SESSION.
                            reportRuntype();
                        }
                    }
                }
            }

// Got the runtypes reported by cortex
            else if(messageType.contains(rcSubscription.RcGuiTypeConfigurations)){

// Get the name of the gui that requested configurations report
                StringTokenizer stt = new StringTokenizer(messageType,"_");
                stt.nextToken();
                String gn = null;
                if(stt.hasMoreTokens())gn = stt.nextToken();

// Ok this was me sending the report configurations request
                if(gn.equals(myName)){

// Start configurations dialog
                    StringTokenizer st = new StringTokenizer(message);
                    int ii=0;
                    String [] s = new String[st.countTokens()];
                    while (st.hasMoreTokens()) {
                        s[ii] = st.nextToken();
                        ii++;
                    }

                    // sort alphabetically runtype names coming from cortex
                    RcSort mySort = new RcSort();
                    String[] ss = mySort.sortAlpha(s);

// Pop up the RcListDialog and show the configurations
// If we have previous runtype selected show the list with already selected runtype from the previous selection
                    RcListDialog sd = new RcListDialog(GSTYLE);
                    sd.showDialog(RcGui.this,ss,"Run Configurations","RCConfigurations",_RunType);

// Get the user selected SESSION
                    if(sd.getReturnStatus() == RcListDialog.RET_OK){

// So this is the same SESSION. Send cortex configure message
// Runtype is the same as configuration file name
                        _RunType = sd.getSelectedSession();
                        setRunType(_RunType);
                        coolFileName = _RunType;

// Send cMsg to Cortex
                        msg = new cMsgMessage();
                        msg.setSubject(rcSubscription.CrtSubject);
                        msg.setType(rcSubscription.CrtType);
                        msg.setText("Configure runtype:"+_RunType+" "+"session:"+_Session+" "+"cool:"+coolFileName);
                        try {
                            cMsgSrv.send(msg);
                            cMsgSrv.flush(1);
                        } catch (cMsgException e) {
                            e.printStackTrace();
                        }

// Start clock animation
                        startClock(" Configuring...");
                        configed = false;
                    }
                    else {
                        AbortButton.setEnabled(true);
                        ConfigureButton.setEnabled(true);
                        PauseButton.setEnabled(false);
                        if(configed)DownloadButton.setEnabled(true);
                        else DownloadButton.setEnabled(false);
                        PrestartButton.setEnabled(false);
                        GoButton.setEnabled(false);
                        EndButton.setEnabled(false);
                    }
                }
            }
        }
    }


    // ..... Inner class
    /**
     * cMsg callback class to handle  ERROR messages from the Medulla
     */
    class ErrorCallback extends cMsgCallbackAdapter {

        public void callback(cMsgMessage msg, Object userObject){
            boolean itsmine = false;
            String messageType = msg.getType();
            String message = msg.getText();
            String s = "";


// Coda transition related Info messages
            if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeTransition)){
                if (message.startsWith("GUI")) {
                    if (message.lastIndexOf(":") > 0) {
                        s =message.substring(message.lastIndexOf(":") + 1);
                        startInfo(s,"Error");
                        addConsoleLine(s,"error");
                        startInfo(message,"Error");
                        ERROR_HAPPENED = true;

                    }
                } else if(message.contains("Severe error")){
                    stopClock();
                    setRunState("Refused");
                    addConsoleLine("Configure Refused","error");
                    startInfo(message,"Error");
                    ERROR_HAPPENED = true;
                    try {
                        startInfo("Configure Failed", "Error");} catch (Exception e){e.printStackTrace();}
                    AbortButton.setEnabled(false);
                    ConfigureButton.setEnabled(true);
                    PauseButton.setEnabled(false);
                    DownloadButton.setEnabled(false);
                    PrestartButton.setEnabled(false);
                    GoButton.setEnabled(false);
                    EndButton.setEnabled(false);
                    setStartTime("");
                    setEndTime("");
                    setEventNumber("");
                    setGEvtRate(0.0);
                    setGDataRate(0.0);
                    setGAverEvtRate(0.0);
                    setGAverDataRate(0.0);
                } else  {
// Check to see if this message is for currently configured components
                    for (String so : _Components.keySet()) {
                        if(message.contains(so)) itsmine = true;
                    }
                    if(itsmine){
                        addConsoleLine(message, "error");
                        startInfo(message,"Error");
                        ERROR_HAPPENED = true;

                    }

                    StringTokenizer st = new StringTokenizer(message);
                    if(st.hasMoreTokens()) s = st.nextToken();
                    if(s.contains("SevereError")){

// Get the string "component"
                        if(st.hasMoreTokens()) s = st.nextToken();

// Get the component name
                        if(st.hasMoreTokens()) s = st.nextToken();
                        for (String ss : _Components.keySet()) {

// Get index of that component in _Components hash table.
                            if(ss.equals(s)) {

// Select that component in the componentList
                                ComponentList.setSelectedIndex(_Components.get(s));
                                ComponentList.setSelectionForeground(Color.red);
                            }
                        }
                    }
                }
            }

// Error messages sent from StemCells
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeStemCell)){
                addConsoleLine(message,"error");
                startInfo(message,"Error");
                ERROR_HAPPENED = true;
            }
        }
    }


    // ...... Inner class
    /**
     * cMsg callback class to handle WARNING messages from the Medulla
     */
    class WarningCallback extends cMsgCallbackAdapter {

        public void callback(cMsgMessage msg, Object userObject){
            String messageType = msg.getType();
            String message = msg.getText();

// Coda transition related Info messages
            if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeTransition)){
                addConsoleLine(message,"warning");
            }

// Error messages sent from StemCells.
// Attention consloline in this case is info
            else if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeStemCell)){
                addConsoleLine(message,"info");
            }
        }
    }


    // ...... Inner class
    /**
     * cMsg callback class to handle messages from the Hepatic
     */
    class HepaticCallback extends cMsgCallbackAdapter {

        public void callback(cMsgMessage msg, Object userObject){
            String message = msg.getText();

// Coda transition related Info messages
            addConsoleLine(message,"warning");
        }
    }



    // ...... Inner class
    /**
     * cMsg callback class to handle messages from the Cortex
     */
    class CortexCallback extends cMsgCallbackAdapter {

        public void callback(cMsgMessage msg, Object userObject){
            String messageType = msg.getType();
            String message = msg.getText();

// When crashed component comes back it registers with the cortex, which will send message to all
// plexuses with the name of the crashed component in the message text. Here plexuses will check
// if the name is the part of their component list and will send autoreset message to the supervisor
// agent of this runtype.           
            if(messageType.equals("CortexReset") && (!_AutoReseted)){

// Message is the name of the component that rebooted
                if((message!=null) && (_Components.containsKey(message))){

//// Send autoreset cMsg to Medulla
//                    cMsgMessage msgo = new cMsgMessage();
//                    msgo.setSubject(_Session+"_"+_RunType);
//                    msgo.setType(rcSubscription.SupervisorControlType +"/"+rcConstants.CMD_autoreset);
//                    try {
//                        cMsgSrv.send(msgo);
//                        cMsgSrv.flush(1);
//                    } catch (cMsgException e) {
//                        e.printStackTrace();
//                    }
//
// Start clock animation
                    startInfo(" Forced Reset. Component "+message+" was rebooted.", "Error");
                    ERROR_HAPPENED=true;
                    addConsoleLine("Forced Reset. Component "+message+" was rebooted. " +
                            "\nTo start the new run configure first.","error");

// Enabling reset and configure buttons in case there is no cool or some
// how configure faild at very top level, i.e. it did not even send cMsg to Cortex.
                    PauseButton.setIcon(Pause);
                    PauseButton.setRolloverIcon(PauseOver);
                    AbortButton.setEnabled(false);
                    ConfigureButton.setEnabled(true);
                    PauseButton.setEnabled(false);
                    DownloadButton.setEnabled(false);
                    PrestartButton.setEnabled(false);
                    GoButton.setEnabled(false);
                    EndButton.setEnabled(false);
                    StartRunButton.setEnabled(false);
                    startrun = false;
                    LimitEventsTextField.setText("");
                    LimitEventsTextField.setEditable(false);
                    ComponentList.setSelectionForeground(Color.black);
                }
                _AutoReseted = true;
            }
        }
    }


    // ..... Inner class
    /**
     * cMsg callback class to handle messages from the non differentiated Stemcells
     */
    class StemcellCallback extends cMsgCallbackAdapter {

        public void callback(cMsgMessage msg, Object userObject){
            String messageType = msg.getType();
            String message = msg.getText();

// Coda transition related Info messages
            addConsoleLine(message,"warning");
        }
    }



    // ....... Inner class
    /**
     * cMsg callback class to handle TsDataCallback messages from the Medulla
     */
    class TsDataCallback extends cMsgCallbackAdapter {

        /** String holds livetime value*/
        double LiveTime;

        /** Array holds l1 input rates*/
        final double[] l1rates = new double[12];

        public void callback(cMsgMessage msg, Object userObject){
            String messageType = msg.getType();
            String message = msg.getText();

// Coda transition related Info messages
            if(messageType.equalsIgnoreCase(rcSubscription.RcGuiTypeTransition)){

// Get the " " separated message
                StringTokenizer stk = new StringTokenizer(message);
                try{
                    if(stk.hasMoreTokens())LiveTime = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[0] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[1] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[2] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[3] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[4] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[5] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[6] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[7] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[8] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[9] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[10] = Double.parseDouble(stk.nextToken());
                    if(stk.hasMoreTokens())l1rates[11] = Double.parseDouble(stk.nextToken());
                } catch ( NumberFormatException e) {e.printStackTrace();}

//Update livetime timegraph
                if (myLiveTimeTG != null) {
                    myLiveTimeTG.value =LiveTime;
                    myLiveTimeTG.datasource.update();
                }

//Update the l1 bargraph
//                myTriggerBitsBG.values = l1rates;
//                myTriggerBitsBG.datasource.update();
            }
        }
    }


    /**
     * Class checks continuosly if we are connected to the cMsg server
     */
    class HeartBeat extends Thread {
        private boolean consoleprint = true;
        public void run(){
            while (true){
                if(cMsgSrv!=null){
                    if(!cMsgSrv.isConnected()){
                        try{
                            if(consoleprint){
                                addConsoleLine("Lost connection with the server","error");
                                resetGui();
                                ConfigureButton.setEnabled(false);
                            }
                            cMsgSrv.disconnect();
                            cMsgSrv = new cMsg(_PlatformcMsgServerUDL4Gui, myName, rcSubscription.PlatformcMsgServerDescription);
                            cMsgSrv.connect();
                            addConsoleLine("Connected to the server.","info");
                            ConfigureButton.setEnabled(true);
                            cMsgSrv.start();

// Unsubscribe
                            if(cMsgSrv!=null) {
                                if(ob1!=null)cMsgSrv.unsubscribe(ob1);
                                if(ob2!=null)cMsgSrv.unsubscribe(ob2);
                                if(ob3!=null)cMsgSrv.unsubscribe(ob3);
                                if(ob4!=null)cMsgSrv.unsubscribe(ob4);
                                if(ob5!=null)cMsgSrv.unsubscribe(ob5);
                                if(ob6!=null)cMsgSrv.unsubscribe(ob6);
                                if(ob7!=null)cMsgSrv.unsubscribe(ob7);
                            }

// Subscribe to RcGuiSubjInfo and type=*

                            ob1 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjInfo+"_"+_Session, "*", infocb, null);
// Subscribe to RcGuiSubjWarning and type=*

                            ob2 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjWarning+"_"+_Session, "*", warningcb, null);
// Subscribe to RcGuiSubjError and type=*

                            ob3 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjError+"_"+_Session, "*", errorcb, null);
// Subscribe to RcGuiSubjTsData and type=*

                            ob4 = cMsgSrv.subscribe(rcSubscription.RcGuiSubjTsData+"_"+_Session, "*", tscb, null);
// Subscribe to Hepatic messages and type=*

                            ob5 = cMsgSrv.subscribe(rcSubscription.Hepatic, "*", hepaticcb, null);
// Subscribe to Cortex messages and type=*

                            ob6 = cMsgSrv.subscribe(rcSubscription.CrtSubject, "*", cortexcb, null);
// Subscribe to Stemcell messages and type=*
                            ob7 = cMsgSrv.subscribe(rcSubscription.Stemcell, "*", stemcellcb, null);
                            consoleprint = true;
                        } catch (cMsgException e) {
                            consoleprint = false;
                            e.printStackTrace();
                        }
                        catch (NullPointerException j){j.printStackTrace();}
                    }
                }
                try {
                    Thread.sleep(5000);
                } catch (InterruptedException e){e.printStackTrace();}
            }
        }
    }
}



