package org.jlab.coda.runcontrol.gui;

import javax.swing.*;
import javax.accessibility.Accessible;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.*;

/**
 * Copyright (c)            Southeastern Universities Research Association,
 * Thomas Jefferson National Accelerator Facility
 * <p/>
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 * <p/>
 * @author: gurjyan
 * Date: Dec 18, 2006
 * Time: 2:05:00 PM
 */

@SuppressWarnings({"deprecation"})
public class RcListDialog extends JComponent implements Accessible {
    public JDialog dialog;
    private String InfoTitle;
    private String GuiTitle;

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /** list of objects used to furnish the jlist*/
    public String[] myList;

    /** selected session*/
    private String selectedSession;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private JComponent jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JButton okButton;
    private javax.swing.JList sessionsList;

    private int returnStatus = RET_CANCEL;

    public int status = 0;
    public static String BACKGROUND;
    public static Color TITLE_COLOR;

    public RcListDialog(String style){
        super();
        if(style.equalsIgnoreCase("plain")){
            BACKGROUND =  "plain";
            TITLE_COLOR = new java.awt.Color(140, 180, 200);
        }else{
// blue
            if(style.equalsIgnoreCase("blue")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/blue.jpg";
                TITLE_COLOR = new java.awt.Color(0x66, 0xFF, 0xFF);
// lightblue
            }else if(style.equalsIgnoreCase("lightblue")){
               BACKGROUND = System.getenv("AFECS_HOME")+"/images/lightblue.jpg";
               TITLE_COLOR = new java.awt.Color(0x00, 0x33, 0x66);
// green
            }else if(style. equalsIgnoreCase("green")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/Green-Growth.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
//purple
            }else if(style.equalsIgnoreCase("purple")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/purple.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
//yellow
            }else if(style.equalsIgnoreCase("yellow")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/yellow.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
// default green
            }else {
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/Green-Growth.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
            }
        }

    }

    /**
     *
     * @param  parent               gui container
     * @return                      Jdialog object
     * @throws HeadlessException
     */
    protected JDialog createDialog(Component parent) throws HeadlessException {

        dialog = new JDialog((Frame)parent, GuiTitle, true);
        dialog.setComponentOrientation(this.getComponentOrientation());

        Container contentPane = dialog.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(this, BorderLayout.CENTER);

        if (JDialog.isDefaultLookAndFeelDecorated()) {
            boolean supportsWindowDecorations =
                    UIManager.getLookAndFeel().getSupportsWindowDecorations();
            if (supportsWindowDecorations) {
                dialog.getRootPane().setWindowDecorationStyle(JRootPane.FILE_CHOOSER_DIALOG);
            }
        }

        initComponents();

        dialog.pack();

        dialog.setLocationRelativeTo(parent);

        return dialog;
    }

    /**
     *
     *@param parent                   gui container
     * @param ss                       array of strings to be shown in the list
     * @param title                    title of the list
     * @param guititle                 title of the dialog gui
     * @param selectedItem             preselect the jlist element name
     * @throws HeadlessException
     */
    public void showDialog(Component parent, String[] ss, String title, String guititle, String selectedItem) throws HeadlessException {

        myList = ss;
        InfoTitle = title;
        GuiTitle = guititle;

        dialog = createDialog(parent);


        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                returnStatus = RET_CANCEL;
            }
        });
        if(selectedItem!=null)selectRunType(selectedItem);
        dialog.show();
    }

    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
        return returnStatus;
    }

    /** @return the string of the selected session */
    public String getSelectedSession() {
        return selectedSession;
    }

    /** select the Jlist element
     * @param runtype to be selected, actually this is the previously selected runtype*/
    public void selectRunType(String runtype) {

        for(int i=0;i<myList.length;i++){
            if(myList[i].equalsIgnoreCase(runtype)) {
                sessionsList.setSelectedIndex(i);
                return;
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        if(BACKGROUND.equalsIgnoreCase("plain")) jPanel1 = new JPanel();
        else jPanel1 = new ImagePanel(BACKGROUND);

        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        sessionsList = new javax.swing.JList();
        jSeparator1 = new javax.swing.JSeparator();

        dialog.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        jLabel1.setFont(new java.awt.Font("Lucida Bright", 1, 14));
        jLabel1.setForeground(TITLE_COLOR);
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText(InfoTitle);

        jLabel2.setFont(new java.awt.Font("Lucida Bright", 1, 14));
        jLabel2.setForeground(TITLE_COLOR);
        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel2.setText("AMMMAN");

        sessionsList.setBackground(new java.awt.Color(238, 238, 238));
        sessionsList.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.LOWERED));
        sessionsList.setFont(new java.awt.Font("Lucida Bright", 1, 12));
        sessionsList.setForeground(new java.awt.Color(153, 51, 0));
        sessionsList.setModel(new javax.swing.AbstractListModel() {
            String[] strings = myList;
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });

        sessionsList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        sessionsList.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                SessionsListMouseClicked(evt);
            }
        });

        jScrollPane1.setViewportView(sessionsList);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);

        jPanel1Layout.setHorizontalGroup(
                jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                                .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addContainerGap())
                        .add(jPanel1Layout.createSequentialGroup()
                                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140 , org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                                .addContainerGap())
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
//                        .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cancelButton)
                        .addContainerGap())

                ))
        );

        jPanel1Layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanel1Layout.setVerticalGroup(
                jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanel1Layout.createSequentialGroup()
                        .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 179, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
//                        .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                .add(cancelButton)
                                .add(okButton))
                        .addContainerGap(23, Short.MAX_VALUE))
        );







        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(dialog.getContentPane());
        dialog.getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                )
        );

//        layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
                layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
//                        .addContainerGap()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
//                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
//                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
//                                .add(cancelButton)
//                                .add(okButton))
//                        .addContainerGap()
                )
        );
    }

    private void SessionsListMouseClicked(MouseEvent evt) {
        selectedSession = (String)sessionsList.getSelectedValue();
    }

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {
        selectedSession = (String)sessionsList.getSelectedValue();
        if(selectedSession!=null) {
            status = 1;
            doClose(RET_OK);
        }
        else  {
            status = 0;
            doClose(RET_CANCEL);
        }

    }

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {
        status = 0;
        doClose(RET_CANCEL);
    }

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {
        doClose(RET_CANCEL);
    }

    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dialog.dispose();
    }

}
