package org.jlab.coda.runcontrol.gui;

import org.jlab.coda.runcontrol.util.rcConstants;

import javax.swing.*;
import javax.accessibility.Accessible;
import java.awt.event.*;
import java.awt.*;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Copyright (c)            Southeastern Universities Research Association,
 * Thomas Jefferson National Accelerator Facility
 * <p/>
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 * <p/>
 * @author: gurjyan
 * Date: June 22, 2007
 * Time: 2:05:00 PM
 */
@SuppressWarnings({"deprecation"})
public class RcSliderDialog extends JComponent implements Accessible {
    public JDialog dialog;
    private String InfoTitle;
    private String GuiTitle;
    private int SliderValue;

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;
    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /** selected session*/
    private int RepInterval;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel jLabel1;
    private JComponent jPanel1;
    private javax.swing.JButton okButton;
    public ScrollSlider imSlider;
    private int returnStatus = RET_CANCEL;
    private JLabel sliderCurrentValue = new JLabel();

    public int status = 0;

    public static String BACKGROUND;
    public static Color TITLE_COLOR;

    public RcSliderDialog(String style){
        super();
        if(style.equalsIgnoreCase("plain")){
            BACKGROUND =  "plain";
            TITLE_COLOR = new java.awt.Color(140, 180, 200);
        }else{
// blue
            if(style.equalsIgnoreCase("blue")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/blue.jpg";
                TITLE_COLOR = new java.awt.Color(0x66, 0xFF, 0xFF);
// lightblue
            }else if(style.equalsIgnoreCase("lightblue")){
               BACKGROUND = System.getenv("AFECS_HOME")+"/images/lightblue.jpg";
               TITLE_COLOR = new java.awt.Color(0x00, 0x33, 0x66);
// green
            }else if(style. equalsIgnoreCase("green")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/Green-Growth.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
//purple
            }else if(style.equalsIgnoreCase("purple")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/purple.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
//yellow
            }else if(style.equalsIgnoreCase("yellow")){
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/yellow.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
// default green
            }else {
                BACKGROUND = System.getenv("AFECS_HOME")+"/images/Green-Growth.jpg";
                TITLE_COLOR = new java.awt.Color(140, 180, 200);
            }
        }

    }
    /**
     *
     * @param  parent               gui container
     * @return                      Jdialog object
     * @throws HeadlessException
     */
    protected JDialog createDialog(Component parent) throws HeadlessException {

        dialog = new JDialog((Frame)parent, GuiTitle, true);
        dialog.setComponentOrientation(this.getComponentOrientation());

        Container contentPane = dialog.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(this, BorderLayout.CENTER);

        if (JDialog.isDefaultLookAndFeelDecorated()) {
            boolean supportsWindowDecorations =
                    UIManager.getLookAndFeel().getSupportsWindowDecorations();
            if (supportsWindowDecorations) {
                dialog.getRootPane().setWindowDecorationStyle(JRootPane.FILE_CHOOSER_DIALOG);
            }
        }

        initComponents();

        dialog.pack();

        dialog.setLocationRelativeTo(parent);

        return dialog;
    }

    /**
     *
     *@param parent                    gui container
     * @param title                    title of the list
     * @param guititle                 title of the dialog gui
     */
    public void showDialog(Component parent, String aiv, String title, String guititle) throws HeadlessException {
        SliderValue = Integer.parseInt(aiv);
        InfoTitle = title;
        GuiTitle = guititle;
        dialog = createDialog(parent);


        dialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                returnStatus = RET_CANCEL;
            }
        });
        dialog.show();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        if(BACKGROUND.equalsIgnoreCase("plain")) jPanel1 = new JPanel();
        else jPanel1 = new ImagePanel(BACKGROUND);

        jLabel1 = new javax.swing.JLabel();

        dialog.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        jLabel1.setFont(new java.awt.Font("Lucida Bright", 1, 14));
        jLabel1.setForeground(TITLE_COLOR);
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText(InfoTitle);

        sliderCurrentValue.setFont(new java.awt.Font("Lucida Bright", 1, 12));
        sliderCurrentValue.setForeground(TITLE_COLOR);
        sliderCurrentValue.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        sliderCurrentValue.setText("Smoothing favtor = "+SliderValue);

// create and add slider ----------------------------
        imSlider = new ScrollSlider(Scrollbar.HORIZONTAL, SliderValue, 1, 0, rcConstants.SMOOTH, rcConstants.SMOOTH);
        imSlider.addAdjustmentListener(new AdjustmentListener() {
            public void adjustmentValueChanged(AdjustmentEvent ae) {
                RepInterval =  imSlider.getValue();
//                if(RepInterval==0)RepInterval=1;
                sliderCurrentValue.setText("Smoothing factor = " + RepInterval);

            }
        });



        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);

        jPanel1Layout.setHorizontalGroup(
                jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(org.jdesktop.layout.GroupLayout.CENTER, jPanel1Layout.createSequentialGroup()
                                .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addContainerGap())
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                                .addContainerGap())
                        .add(jPanel1Layout.createSequentialGroup()
                                .add(imSlider, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 160 , org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel1Layout.createSequentialGroup()
                                .add(sliderCurrentValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cancelButton)
                        .addContainerGap())

                ))
        );

        jPanel1Layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        jPanel1Layout.setVerticalGroup(
                jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(jPanel1Layout.createSequentialGroup()
                        .add(jLabel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(imSlider)
                        .add(sliderCurrentValue, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                .add(cancelButton)
                                .add(okButton))
                        .addContainerGap(23, Short.MAX_VALUE))
        );



        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(dialog.getContentPane());
        dialog.getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                )
        );


        layout.setVerticalGroup(
                layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                        .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                )
        );
    }


    /** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
    public int getReturnStatus() {
        return returnStatus;
    }

    /** @return the float value of the slider. N.B. selected values of the slider are devided by 10 */
    public String getSliderValue() {
        if (RepInterval==0) RepInterval=1;
        return String.valueOf(RepInterval);
    }


    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {
            status = 1;
            doClose(RET_OK);
    }

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {
        status = 0;
        doClose(RET_CANCEL);
    }

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {
        doClose(RET_CANCEL);
    }

    private void doClose(int retStatus) {
        returnStatus = retStatus;
        setVisible(false);
        dialog.dispose();
    }

}
