/*---------------------------------------------------------------------------*
*  Copyright (c) 2005        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.gui;

import jas.hist.*;
import jas.hist.util.ObserverAdapter;

import javax.swing.*;

/**
 * @author Vardan Gyurjyan
 *         Date: Oct 31, 2005
 *         Time: 10:56:24 AM
 */

public class TimeGraphLite
{
    public volatile double value;
    public int number_of_bins;
    public TimeLineDataSource datasource;
    public JASHist	plot;

    TimeGraphLite(final String title,final int nb, double ymax, final JScrollPane pp, final boolean b)
    {
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                number_of_bins = nb;
                plot = new JASHist();

                datasource = new TimeLineDataSource();
                plot.addData(datasource).show(true);
//        plot.setTitle(title);
                plot.setDataAreaBorderType(JASHist.ETCHED);
                plot.getYAxis().setLabel(title);
                plot.getYAxis().setRangeAutomatic(true);
                plot.getYAxis().setAllowSuppressedZero(b);
//        plot.getYAxis().setRange(0, Double.NaN);
                plot.getXAxis().setLabel(" ");
                plot.setAllowUserInteraction(false);
                plot.setDoubleBuffered(true);
//        plot.setBounds(845,693,403,255);
                pp.setDoubleBuffered(true);
//        pp.setFont(new java.awt.Font("Dialog", 0, 10));
                pp.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
                pp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

                pp.setViewportView(plot);

            }
        });
    }

    public void remove(){
        SwingUtilities.invokeLater(new Runnable(){
            public void run(){
                plot.destroy();
            }
        });
    }


// Note	that this class	extends	Observable so that it can notify
    // the plot	(JASHist) when the data	changes.
    public class TimeLineDataSource extends ObserverAdapter implements
            Rebinnable1DHistogramData, HasStyle
    {
        private	double[] data =  new double[number_of_bins];
        private int index=0;


        // Create the HistogramUpdate object that will be sent
        // to all our Observers	(specifically the JASHist).	This
        // HistogramUpdate will	specify	that both the bin contents
        // (data) and the axis min/max (range) have	changed. The
        // boolean argument	to the HistogramUpdate constructor
        // indicates that this is a	"final"	update,	which the
        // JASHist should resond to	immediately.
        int	flags =	HistogramUpdate.DATA_UPDATE	|
                HistogramUpdate.RANGE_UPDATE;
        HistogramUpdate	hu = new HistogramUpdate(flags,true);


        public void update() {
            //filling the data double array
            if(index < number_of_bins) {
                data[index] = value;
                index++;
            }
            else {
                // time to scroll. For that create the new array, ignore the first element
                // of the old data buffer.
                int j=0;
                double[] newData = new double[number_of_bins];
                for(int i=1; i<number_of_bins; i++){
                    newData[i-1] = data[i];
                }
                // add the new data at the end of the new buffer
                newData[number_of_bins-1]=value;
                //export the data
                data = newData;
            }
//if(value>10)System.out.println(" TG: ===> "+index+" "+value);
            setChanged(); // notifyObservers needs this
            notifyObservers(hu);
        }

        public double[][] rebin(int	rBins, double rMin,	double rMax,
                                boolean	wantErrors,	boolean	hurry)
        {
            double[][] result =	{ data };
            return result;
        }
        public String[]	getAxisLabels()	{ return null; }
        public double getMin() { return	0; }
        public double getMax() { return	data.length; }
        public boolean isRebinnable() {	return true; }
        public int getBins() { return data.length; }
        public int getAxisType() { return DOUBLE; }
        public String getTitle() { return ""; }

        public JASHistStyle getStyle()
        {
            JASHist1DHistogramStyle style = new JASHist1DHistogramStyle();
            style.setShowErrorBars(false);
            style.setShowDataPoints(true);
            style.setShowLinesBetweenPoints(true);
            style.setShowHistogramBars(false);
            style.setDataPointSize(1);
            style.setLinesBetweenPointsWidth(2);
            return style;
        }
    }
}
