/*---------------------------------------------------------------------------*
*  Copyright (c) 2008        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.gui.dalogbrowser;

import java.awt.Color;
import java.awt.Component;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jdesktop.application.Action;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.BadLocationException;
import javax.swing.text.Position;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import org.jlab.coda.cMsg.cMsg;
import org.jlab.coda.cMsg.cMsgCallbackAdapter;
import org.jlab.coda.cMsg.cMsgCallbackInterface;
import org.jlab.coda.cMsg.cMsgException;
import org.jlab.coda.cMsg.cMsgMessage;
import org.jlab.coda.runcontrol.gui.*;

/**
 * The application's main frame.
 */
public class DalogBrowserView extends FrameView
        implements TreeSelectionListener {

    /** The name of this browser. 
     * Must be unique for the cMsg server that it will connect
     */
    public String myName;
    /** Hashmap holds component subject of the subscription. The key is the component name*/
    private ConcurrentHashMap<String, String> CompAddress = new ConcurrentHashMap<String, String>();
    /** Hashmap holds expid name and codatypes of the components under expid
     * (those components does not have session or runtype assosiation)*/
    private ConcurrentHashMap<String, ArrayList> Expid_CodaTypes = new ConcurrentHashMap<String, ArrayList>();
    /** Hashmap holds session name and codatypes of the components under sesion
     * (those components does not have runtypes)*/
    private ConcurrentHashMap<String, ArrayList> Session_CodaTypes = new ConcurrentHashMap<String, ArrayList>();
    /** Hashmap holds runtype name and  of the components under runtype */
    private ConcurrentHashMap<String, ArrayList> Config_CodaTypes = new ConcurrentHashMap<String, ArrayList>();
    /** Hashmap holds Expid class objects. The key is the name of the EXPID*/
    private ConcurrentHashMap<String, DLBExpid> EXPIDS = new ConcurrentHashMap<String, DLBExpid>();
    /** Sessions list */
    private List<String> Sessions = new ArrayList<String>();
    /** Runtypes list */
    private List<String> RunTypes = new ArrayList<String>();
    /** ComponentTypes list */
    private List<String> ComponentTypes = new ArrayList<String>();
    /** jTree root objec*/
    private DefaultMutableTreeNode root;
    /** cMsg server */
    private cMsg cMsgSrv;
    /** Dalog cMsg subscription object. This is for the subscription 
     subject = expid/session/runtype/codatype/name
     type = dalog/{i=1|i=2....}*/
    private Object ssObj;
    /** Subject of the ssObj subscription*/
    private String subject;
    /** Type of the ssObj subscription*/
    private String type = "dalog/{i=1|i=2|i=3|i=4|i=5|i=6|i=7|i=8|i=9|i=10|i=11|i=12|i=13|i=14|i=15|i=16}";
    private cMsgCallbackInterface visualizecontrol = new VisualizeCallback();
    /** Selected expid from the tree*/
    private String $expid;
    /** Default name of the root tree*/
    private String rootNodeName = "Afecs Platforms";
    /** Default table model*/
    public javax.swing.table.DefaultTableModel mmm;
    /** Message limit of the table*/
    public int messageLimit = 1000;
    //----------------------------------------------------------------------------    
    /**
     * Constructor
     * @param app
     */
    public DalogBrowserView(SingleFrameApplication app) {
        super(app);
        buildInitialTree();
        initComponents();
        this.getFrame().setTitle("Dalog Browser");
    }

    @Action
    public void myExit() {
        try {
            if(cMsgSrv!=null)cMsgSrv.disconnect();
        } catch (cMsgException e) {
            e.printStackTrace();
        }

        this.getFrame().dispose();
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = DalogBrowserApp.getApplication().getMainFrame();
            aboutBox = new DalogBrowserAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        DalogBrowserApp.getApplication().show(aboutBox);
    }

    @Action
    public void showAddPlatformBox() {
        if (addPlatformBox == null) {
            JFrame mainFrame = DalogBrowserApp.getApplication().getMainFrame();
            addPlatformBox = new DalogBrowserEnterPlatform(mainFrame, this);
            addPlatformBox.setLocationRelativeTo(mainFrame);
        }
        DalogBrowserApp.getApplication().show(addPlatformBox);
    }
    //----------------------------------------------------------------------------
    /**
     * Open connection to the Afecs platform (i.e. expid).
     * @param udl     UDL of the platform of the form
     *                cMsg://host:port/cMsg/expid
     * Todo all the necessary subscriptions
     */
    public void connectToThePlatform(String udl) {
        String tmps = null;
        String expid = null, host = null, port = null;


        // Separate the string after cMsg:// and tokenize it by /
        StringTokenizer st = new StringTokenizer(udl.split("://")[1], "/");
        while (st.hasMoreTokens()) {
            tmps = st.nextToken();
            if (tmps.contains(":")) {
                // This might be a host and port section of the udl
                StringTokenizer st1 = new StringTokenizer(tmps, ":");
                host = st1.nextToken();
                port = st1.nextToken();
            } else {
                //always asign the next token to expid, 
                //since the last one is always expid
                expid = tmps;
            }
        }

        // See if we have already connection with this platform
        if (!EXPIDS.containsKey(expid)) {

//                    System.out.println("debug... " + expid + " " + host + " " + port);

            // Connect to cMsg server of the platform
            try {
                cMsgSrv = new cMsg(udl, myName, " ");
                cMsgSrv.connect();

                // Enable message reception
                cMsgSrv.start();
                // Subscribe to the subject = expid/* and type = dalog/*, 
                // necessary for the tree construction.
                cMsgCallbackInterface dalogMessageCallback = new DalogMessageCallback();
                cMsgSrv.subscribe(expid + "/*", "dalog/*", dalogMessageCallback, null);


            } catch (cMsgException e) {
                e.printStackTrace();
                myExit();
            }

            DLBExpid np = new DLBExpid();
            np.setHost(host);
            np.setPort(port);
            np.setServer(cMsgSrv);

            EXPIDS.put(expid, np);

            addNodeToTheTree(rootNodeName, expid);
            /*
           DefaultMutableTreeNode expidnode = new DefaultMutableTreeNode(expid);
           root.add(expidnode);
           jTree1.validate();
           // expand the tree
           TreeNode base = (TreeNode) jTree1.getModel().getRoot();
           expandAll(jTree1, new TreePath(base), true);
            */

        }
    }
    //----------------------------------------------------------------------------
    /**
     * Method for dalog message tree expansion or collapse.
     * @param tree
     * @param parent
     * @param expand      true: expands, false: collapses
     */
    private void expandAll(JTree tree, TreePath parent, boolean expand) {

        // Traverse children
        TreeNode node = (TreeNode) parent.getLastPathComponent();
        if (node.getChildCount() >= 0) {
            for (Enumeration e = node.children(); e.hasMoreElements();) {
                TreeNode n = (TreeNode) e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                expandAll(tree, path, expand);
            }
        }
        // Expansion or collapse must be done buttom-up
        if (expand) {
            tree.expandPath(parent);
        } else {
            tree.collapsePath(parent);
        }
    }
    //----------------------------------------------------------------------------
    /**
     * Method disconnects from the platform cMsg server.
     * Todo unsubscribe all the subscriptions
     */
    @Action
    public void disconnectFromThePlatform() {
        DLBExpid me = null;
        cMsg connection = null;
        if ($expid != null) {
            if (EXPIDS.containsKey($expid)) {
                try {
                    me = EXPIDS.get($expid);
                    connection = me.getServer();
                    connection.disconnect();

                    // Unsubscribe
                    if (connection != null) {
                        if (ssObj != null) {
                            connection.unsubscribe(ssObj);
                        }
                    }

                    // Remove the node from the jtree
                    removeNodeFromTheTree($expid);

                    // Remove the platform from the hastable
                    EXPIDS.remove($expid);
                    jTextField1.setText("");

                } catch (cMsgException ex) {
                    ex.printStackTrace();
                    Logger.getLogger(DalogBrowserView.class.getName()).
                            log(Level.SEVERE, null, ex);
                }
            }
        }
    }
    //----------------------------------------------------------------------------
    /**
     * Removes the node from the dalog browser tree.
     * If the node has descendants all descendants are removed as well. 
     * @param nodename              The name of the node 
     */
    public void removeNodeFromTheTree(String nodename) {
        int startingRow = 0;

        // Get the default model of the jTree1
        DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();

        // Find node to remove
        TreePath path = jTree1.getNextMatch(nodename, startingRow, Position.Bias.Forward);
        MutableTreeNode node = (MutableTreeNode) path.getLastPathComponent();

        // Remove the node
        model.removeNodeFromParent(node);
        System.out.println(" removed the node - "+nodename);
    }

    /**
     *
     * @param nn node name to be remved
     */
    public void deleteTheNode(String nn){
        removeNode(searchNode(nn));

    }

    /**
     * This method takes the node string and
     * traverses the tree till it finds the node
     * matching the string. If the match is found
     * the node is returned else null is returned
     *
     * @param nodeStr node string to search for
     * @return tree node
     */
    public DefaultMutableTreeNode searchNode(String nodeStr)
    {
        DefaultMutableTreeNode node = null;

        //Get the enumeration
        Enumeration enu = root.breadthFirstEnumeration();

        //iterate through the enumeration
        while(enu.hasMoreElements())
        {
            //get the node
            node = (DefaultMutableTreeNode)enu.nextElement();

            //match the string with the user-object of the node
            if(nodeStr.equals(node.getUserObject().toString()))
            {
                //tree node with string found
                return node;
            }
        }

        //tree node with string node found return null
        return null;
    }
    /**
     * This method removes the passed tree node from the tree
     * and selects appropiate node
     *
     * @param selNode node to be removed
     */
    public void removeNode(DefaultMutableTreeNode selNode)
    {
        if (selNode != null)
        {
            //get the default model of the jTree1
            DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();

            //get the parent of the selected node
            MutableTreeNode parent = (MutableTreeNode)(selNode.getParent());

            // if the parent is not null
            if (parent != null)
            {
                //get the sibling node to be selected after removing the
                //selected node
                MutableTreeNode toBeSelNode = getSibling(selNode);

                //if there are no siblings select the parent node after removing the node
                if(toBeSelNode == null)
                {
                    toBeSelNode = parent;
                }

                //make the node visible by scroll to it
                TreeNode[] nodes = model.getPathToRoot(toBeSelNode);
                TreePath path = new TreePath(nodes);
                jTree1.scrollPathToVisible(path);
                jTree1.setSelectionPath(path);

                //remove the node from the parent
                model.removeNodeFromParent(selNode);
            }
        }
    }

    /**
     * This method returns the previous sibling node
     * if there is no previous sibling it returns the next sibling
     * if there are no siblings it returns null
     *
     * @param selNode selected node
     * @return previous or next sibling, or parent if no sibling
     */
    private MutableTreeNode getSibling(DefaultMutableTreeNode selNode)
    {
        //get previous sibling
        MutableTreeNode sibling = (MutableTreeNode)selNode.getPreviousSibling();

        if(sibling == null)
        {
            //if previous sibling is null, get the next sibling
            sibling    = (MutableTreeNode)selNode.getNextSibling();
        }

        return sibling;
    }
    //----------------------------------------------------------------------------
    /**
     * Adds the node to the root node
     * @param rootnode   the name of the root node
     * @param name       the name of the branch
     */
    public void addNodeToTheTree(String rootnode, String name) {

        int startingRow = 0;

        DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();

        // Find the root node to which the new branch node is to be added
        TreePath path = jTree1.getNextMatch(rootnode, startingRow, Position.Bias.Forward);
//        System.out.println(" Path .... " + path + " rootnode = " + rootnode);
        MutableTreeNode node = (MutableTreeNode) path.getLastPathComponent();

        // Creating branch node of the root node
        DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(name);

        // Insert new session node as last child of the expid node
        model.insertNodeInto(newNode, node, node.getChildCount());
        jTree1.validate();

        // expand the tree
        TreeNode base = (TreeNode) jTree1.getModel().getRoot();
        expandAll(jTree1, new TreePath(base), true);

    }
    /**
     * Adds the component node to the typenode of the specific root (runtype, session, etc)
     * @param grandpa    the name of the parent of the parent node
     * @param pa         the name of the parent node
     * @param name       the name of the component node
     */
    public void addGeneticNode(String grandpa, String pa, String name) {

        DefaultMutableTreeNode tmpnode, typenode = null;

        DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();

        DefaultMutableTreeNode node = searchNode(grandpa);
//        System.out.println(".... "+node.getChildCount());
        node.getChildAt(0);
        for(int i =0; i<node.getChildCount(); i++){
            tmpnode = (DefaultMutableTreeNode)node.getChildAt(i);
            if(tmpnode!=null){
//                System.out.println(".... "+tmpnode.getUserObject().toString()+" :: "+pa);
                if(tmpnode.getUserObject().toString().equals(pa)){
                    typenode = tmpnode;
                }
            }
        }

        if(typenode!=null){
            // Creating branch node of the root node
            DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(name);

            // Insert new session node as last child of the node
            model.insertNodeInto(newNode, typenode, typenode.getChildCount());
            jTree1.validate();

            // expand the tree
            TreeNode base = (TreeNode) jTree1.getModel().getRoot();
            expandAll(jTree1, new TreePath(base), true);
        }
    }
    //----------------------------------------------------------------------------
    /**
     * Create the root of the dalog browser tree.
     */
    public void buildInitialTree() {
        root = new DefaultMutableTreeNode(rootNodeName);
    }
    //----------------------------------------------------------------------------
    /**
     * Based on dalog browser tree selection create a subject and creates a new
     * subscriptions before removing the old subsciptions
     * @param event
     */
    public void valueChanged(TreeSelectionEvent event) {
        StringBuffer sb = new StringBuffer();
        int i = 0;
        Object o = jTree1.getLastSelectedPathComponent();
        if (o != null) {
            for (i = 1; i < jTree1.getSelectionPath().getPathCount(); i++) {
                if (i == 5) {
                    sb.append(jTree1.getSelectionPath().getPathComponent(i));
                } else {
                    sb.append(jTree1.getSelectionPath().getPathComponent(i)).append("/");
                }
            }
            if (i < 6 && i > 1) {
                sb.append("*");
            }
            subject = sb.toString();
            jTextField1.setText(subject);

            // renew the subscription
            renewSubscription();

        }
    }
    //----------------------------------------------------------------------------
    /**
     * Unsubscribes and creates the new subscription to the dalog messages
     * of the AFECS platform.
     */
    private void renewSubscription() {

        // Here we unsubscribe and subscribe to the subject and type
        if (subject != null && !subject.equals("") && type != null) {
            try {
                if (cMsgSrv != null) {
                    if (ssObj != null) {
                        cMsgSrv.unsubscribe(ssObj);
                    }
                    ssObj = cMsgSrv.subscribe(subject, type, visualizecontrol, null);
                }
            } catch (cMsgException e) {
                e.printStackTrace();
            }
//            System.out.println("Subscribing to subject = " + subject + " type = " + type);
        }
    }
    //----------------------------------------------------------------------------
    /**
     * Clears message board
     */
    @Action
    public void clearMessageBoard() {
        jTextPane1.removeAll();
    }

    //----------------------------------------------------------------------------    
    /**
     * Select all dalog message levels
     */
    @Action
    public void selectAll() {
        jRadioButton1.setSelected(true);
        jRadioButton2.setSelected(true);
        jRadioButton3.setSelected(true);
        jRadioButton4.setSelected(true);
        jRadioButton5.setSelected(true);
        jRadioButton6.setSelected(true);
        jRadioButton7.setSelected(true);
        jRadioButton8.setSelected(true);
        jRadioButton9.setSelected(true);
        jRadioButton10.setSelected(true);
        jRadioButton11.setSelected(true);
        jRadioButton12.setSelected(true);
        jRadioButton13.setSelected(true);
        jRadioButton14.setSelected(true);
        jRadioButton15.setSelected(true);
        jRadioButton16.setSelected(true);
        dfs();

    }
    //----------------------------------------------------------------------------    
    /**
     * Deselect all dalog message levels
     */
    @Action
    public void deselectAll() {
        jRadioButton1.setSelected(false);
        jRadioButton2.setSelected(false);
        jRadioButton3.setSelected(false);
        jRadioButton4.setSelected(false);
        jRadioButton5.setSelected(false);
        jRadioButton6.setSelected(false);
        jRadioButton7.setSelected(false);
        jRadioButton8.setSelected(false);
        jRadioButton9.setSelected(false);
        jRadioButton10.setSelected(false);
        jRadioButton11.setSelected(false);
        jRadioButton12.setSelected(false);
        jRadioButton13.setSelected(false);
        jRadioButton14.setSelected(false);
        jRadioButton15.setSelected(false);
        jRadioButton16.setSelected(false);
        dfs();

    }
    //----------------------------------------------------------------------------    
    /**
     * Select all dalog message Info levels
     */
    @Action
    public void selectInfo() {
        jRadioButton1.setSelected(true);
        jRadioButton2.setSelected(true);
        jRadioButton3.setSelected(true);
        jRadioButton4.setSelected(true);
        dfs();

    }
    //----------------------------------------------------------------------------    
    /**
     * Select all dalog message Warning levels
     */
    @Action
    public void selectWarning() {
        jRadioButton5.setSelected(true);
        jRadioButton6.setSelected(true);
        jRadioButton7.setSelected(true);
        jRadioButton8.setSelected(true);
        dfs();

    }
    //----------------------------------------------------------------------------    
    /**
     * Select all dalog message Error levels
     */
    @Action
    public void selectError() {
        jRadioButton9.setSelected(true);
        jRadioButton10.setSelected(true);
        jRadioButton11.setSelected(true);
        jRadioButton12.setSelected(true);
        dfs();

    }
    //----------------------------------------------------------------------------    
    /**
     * Select all dalog message Severe Error levels
     */
    @Action
    public void selectSevereError() {
        jRadioButton13.setSelected(true);
        jRadioButton14.setSelected(true);
        jRadioButton15.setSelected(true);
        jRadioButton16.setSelected(true);
        dfs();

    }

    //----------------------------------------------------------------------------    
    /**
     * Based on the conditions of the radio buttons defines the
     * type (severity level) of the dalog message subscription.
     */
    public void dfs() {

        boolean x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13, x14, x15, x16;
        StringBuffer sb = new StringBuffer();

        x1 = jRadioButton1.isSelected();
        x2 = jRadioButton2.isSelected();
        x3 = jRadioButton3.isSelected();
        x4 = jRadioButton4.isSelected();
        x5 = jRadioButton5.isSelected();
        x6 = jRadioButton6.isSelected();
        x7 = jRadioButton7.isSelected();
        x8 = jRadioButton8.isSelected();
        x9 = jRadioButton9.isSelected();
        x10 = jRadioButton10.isSelected();
        x11 = jRadioButton11.isSelected();
        x12 = jRadioButton12.isSelected();
        x13 = jRadioButton13.isSelected();
        x14 = jRadioButton14.isSelected();
        x15 = jRadioButton15.isSelected();
        x16 = jRadioButton16.isSelected();


        // Define the cMsg subscription type
        sb.append("dalog").append("/{i=0");
        if (x1) {
            sb.append("|i=" + 1);
        }
        if (x2) {
            sb.append("|i=" + 2);
        }
        if (x3) {
            sb.append("|i=" + 3);
        }
        if (x4) {
            sb.append("|i=" + 4);
        }
        if (x5) {
            sb.append("|i=" + 5);
        }
        if (x6) {
            sb.append("|i=" + 6);
        }
        if (x7) {
            sb.append("|i=" + 7);
        }
        if (x8) {
            sb.append("|i=" + 8);
        }
        if (x9) {
            sb.append("|i=" + 9);
        }
        if (x10) {
            sb.append("|i=" + 10);
        }
        if (x11) {
            sb.append("|i=" + 11);
        }
        if (x12) {
            sb.append("|i=" + 12);
        }
        if (x13) {
            sb.append("|i=" + 13);
        }
        if (x14) {
            sb.append("|i=" + 14);
        }
        if (x15) {
            sb.append("|i=" + 15);
        }
        if (x16) {
            sb.append("|i=" + 16);
        }
        if (!x1 && !x2 && !x3 && !x4 && !x5 && !x6 && !x7 && !x8 && !x9 && !x10 && !x11 && !x12 && !x13 && !x14 && !x15 && !x16) {
            sb.append("}");
        } else {
            sb.append("}");
        }
        type = sb.toString();
//        System.out.println(type);

        // renew the subscription
        renewSubscription();

    }
    //----------------------------------------------------------------------------
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTree1 =  new JTree(root);
        jTree1.addTreeSelectionListener(this);
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jPanel1 = new javax.swing.JPanel();
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton2 = new javax.swing.JRadioButton();
        jRadioButton3 = new javax.swing.JRadioButton();
        jRadioButton4 = new javax.swing.JRadioButton();
        jPanel2 = new javax.swing.JPanel();
        jRadioButton5 = new javax.swing.JRadioButton();
        jRadioButton6 = new javax.swing.JRadioButton();
        jRadioButton7 = new javax.swing.JRadioButton();
        jRadioButton8 = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        jRadioButton9 = new javax.swing.JRadioButton();
        jRadioButton10 = new javax.swing.JRadioButton();
        jRadioButton11 = new javax.swing.JRadioButton();
        jRadioButton12 = new javax.swing.JRadioButton();
        jPanel4 = new javax.swing.JPanel();
        jRadioButton13 = new javax.swing.JRadioButton();
        jRadioButton14 = new javax.swing.JRadioButton();
        jRadioButton15 = new javax.swing.JRadioButton();
        jRadioButton16 = new javax.swing.JRadioButton();
        jLabel1 = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jLabel2 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        jSeparator11 = new javax.swing.JSeparator();
        jScrollPane3 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jSeparator12 = new javax.swing.JSeparator();
        jMenuItem9 = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        jSeparator10 = new javax.swing.JSeparator();
        jMenuItem2 = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        jMenuItem3 = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        jMenuItem4 = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JSeparator();
        jMenuItem5 = new javax.swing.JMenuItem();
        jSeparator6 = new javax.swing.JSeparator();
        jMenuItem6 = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JSeparator();
        jMenuItem7 = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JSeparator();
        jSeparator9 = new javax.swing.JSeparator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        jScrollPane4 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();

        mainPanel.setName("mainPanel"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jTree1.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(DalogBrowserApp.class).getContext().getResourceMap(DalogBrowserView.class);
        jTree1.setFont(resourceMap.getFont("PlatformTree.font")); // NOI18N
        jTree1.setName("PlatformTree"); // NOI18N
        jScrollPane1.setViewportView(jTree1);

        jScrollPane2.setBorder(null);
        jScrollPane2.setName("jScrollPane2"); // NOI18N

        jTextPane1.setBackground(resourceMap.getColor("jTextPane1.background")); // NOI18N
        jTextPane1.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
        jTextPane1.setFont(resourceMap.getFont("jTextPane1.font")); // NOI18N
        jTextPane1.setName("jTextPane1"); // NOI18N
        doc = (StyledDocument)jTextPane1.getDocument();
        style = doc.addStyle("StyleName",null);
        jScrollPane2.setViewportView(jTextPane1);

        jPanel1.setBackground(resourceMap.getColor("jPanel1.background")); // NOI18N
        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), resourceMap.getString("jPanel1.border.title"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, resourceMap.getFont("jPanel1.border.titleFont"), resourceMap.getColor("jPanel1.border.titleColor"))); // NOI18N
        jPanel1.setName("jPanel1"); // NOI18N

        jRadioButton1.setText(resourceMap.getString("jRadioButton1.text")); // NOI18N
        jRadioButton1.setIcon(resourceMap.getIcon("jRadioButton1.icon")); // NOI18N
        jRadioButton1.setIconTextGap(1);
        jRadioButton1.setName("jRadioButton1"); // NOI18N
        jRadioButton1.setSelectedIcon(resourceMap.getIcon("info.icon"));
        jRadioButton1.setSelected(true);
        jRadioButton1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton2.setText(resourceMap.getString("jRadioButton2.text")); // NOI18N
        jRadioButton2.setIcon(resourceMap.getIcon("jRadioButton2.icon")); // NOI18N
        jRadioButton2.setIconTextGap(1);
        jRadioButton2.setName("jRadioButton2"); // NOI18N
        jRadioButton2.setSelectedIcon(resourceMap.getIcon("info.icon"));
        jRadioButton2.setSelected(true);
        jRadioButton2.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton3.setText(resourceMap.getString("jRadioButton3.text")); // NOI18N
        jRadioButton3.setIcon(resourceMap.getIcon("jRadioButton3.icon")); // NOI18N
        jRadioButton3.setIconTextGap(1);
        jRadioButton3.setName("jRadioButton3"); // NOI18N
        jRadioButton3.setSelectedIcon(resourceMap.getIcon("info.icon"));
        jRadioButton3.setSelected(true);
        jRadioButton3.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton4.setText(resourceMap.getString("jRadioButton4.text")); // NOI18N
        jRadioButton4.setIcon(resourceMap.getIcon("jRadioButton4.icon")); // NOI18N
        jRadioButton4.setIconTextGap(1);
        jRadioButton4.setName("jRadioButton4"); // NOI18N
        jRadioButton4.setSelectedIcon(resourceMap.getIcon("info.icon"));
        jRadioButton4.setSelected(true);
        jRadioButton4.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
                jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jRadioButton1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton2)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton4))
        );
        jPanel1Layout.setVerticalGroup(
                jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jRadioButton1)
                                .addComponent(jRadioButton2)
                                .addComponent(jRadioButton3)
                                .addComponent(jRadioButton4))
                        .addContainerGap())
        );

        jPanel2.setBackground(resourceMap.getColor("jPanel2.background")); // NOI18N
        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), resourceMap.getString("jPanel2.border.title"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, resourceMap.getFont("jPanel2.border.titleFont"), resourceMap.getColor("jPanel2.border.titleColor"))); // NOI18N
        jPanel2.setName("jPanel2"); // NOI18N

        jRadioButton5.setText(resourceMap.getString("jRadioButton5.text")); // NOI18N
        jRadioButton5.setIcon(resourceMap.getIcon("jRadioButton5.icon")); // NOI18N
        jRadioButton5.setIconTextGap(1);
        jRadioButton5.setName("jRadioButton5"); // NOI18N
        jRadioButton5.setSelectedIcon(resourceMap.getIcon("warning.icon"));
        jRadioButton5.setSelected(true);
        jRadioButton5.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton6.setText(resourceMap.getString("jRadioButton6.text")); // NOI18N
        jRadioButton6.setIcon(resourceMap.getIcon("jRadioButton6.icon")); // NOI18N
        jRadioButton6.setIconTextGap(1);
        jRadioButton6.setName("jRadioButton6"); // NOI18N
        jRadioButton6.setSelectedIcon(resourceMap.getIcon("warning.icon"));
        jRadioButton6.setSelected(true);
        jRadioButton6.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton7.setText(resourceMap.getString("jRadioButton7.text")); // NOI18N
        jRadioButton7.setIcon(resourceMap.getIcon("jRadioButton7.icon")); // NOI18N
        jRadioButton7.setIconTextGap(1);
        jRadioButton7.setName("jRadioButton7"); // NOI18N
        jRadioButton7.setSelectedIcon(resourceMap.getIcon("warning.icon"));
        jRadioButton7.setSelected(true);
        jRadioButton7.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton8.setText(resourceMap.getString("jRadioButton8.text")); // NOI18N
        jRadioButton8.setIcon(resourceMap.getIcon("jRadioButton8.icon")); // NOI18N
        jRadioButton8.setIconTextGap(1);
        jRadioButton8.setName("jRadioButton8"); // NOI18N
        jRadioButton8.setSelectedIcon(resourceMap.getIcon("warning.icon"));
        jRadioButton8.setSelected(true);
        jRadioButton8.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
                jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(jRadioButton5)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton6)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton7)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton8))
        );
        jPanel2Layout.setVerticalGroup(
                jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jRadioButton5)
                        .addComponent(jRadioButton6)
                        .addComponent(jRadioButton7)
                        .addComponent(jRadioButton8))
        );

        jPanel3.setBackground(resourceMap.getColor("jPanel3.background")); // NOI18N
        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), resourceMap.getString("jPanel3.border.title"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, resourceMap.getFont("jPanel3.border.titleFont"), resourceMap.getColor("jPanel3.border.titleColor"))); // NOI18N
        jPanel3.setName("jPanel3"); // NOI18N

        jRadioButton9.setText(resourceMap.getString("jRadioButton9.text")); // NOI18N
        jRadioButton9.setIcon(resourceMap.getIcon("jRadioButton9.icon")); // NOI18N
        jRadioButton9.setIconTextGap(1);
        jRadioButton9.setName("jRadioButton9"); // NOI18N
        jRadioButton9.setSelectedIcon(resourceMap.getIcon("error.icon"));
        jRadioButton9.setSelected(true);
        jRadioButton9.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton10.setText(resourceMap.getString("jRadioButton10.text")); // NOI18N
        jRadioButton10.setIcon(resourceMap.getIcon("jRadioButton10.icon")); // NOI18N
        jRadioButton10.setIconTextGap(1);
        jRadioButton10.setName("jRadioButton10"); // NOI18N
        jRadioButton10.setSelectedIcon(resourceMap.getIcon("error.icon"));
        jRadioButton10.setSelected(true);
        jRadioButton10.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton11.setText(resourceMap.getString("jRadioButton11.text")); // NOI18N
        jRadioButton11.setIcon(resourceMap.getIcon("jRadioButton11.icon")); // NOI18N
        jRadioButton11.setIconTextGap(1);
        jRadioButton11.setName("jRadioButton11"); // NOI18N
        jRadioButton11.setSelectedIcon(resourceMap.getIcon("error.icon"));
        jRadioButton11.setSelected(true);
        jRadioButton11.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton12.setText(resourceMap.getString("jRadioButton12.text")); // NOI18N
        jRadioButton12.setIcon(resourceMap.getIcon("jRadioButton12.icon")); // NOI18N
        jRadioButton12.setIconTextGap(1);
        jRadioButton12.setName("jRadioButton12"); // NOI18N
        jRadioButton12.setSelectedIcon(resourceMap.getIcon("error.icon"));
        jRadioButton12.setSelected(true);
        jRadioButton12.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
                jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel3Layout.createSequentialGroup()
                        .addComponent(jRadioButton9)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton10)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton11)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton12))
        );
        jPanel3Layout.setVerticalGroup(
                jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jRadioButton9)
                        .addComponent(jRadioButton10)
                        .addComponent(jRadioButton11)
                        .addComponent(jRadioButton12))
        );

        jPanel4.setBackground(resourceMap.getColor("jPanel4.background")); // NOI18N
        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), resourceMap.getString("jPanel4.border.title"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, resourceMap.getFont("jPanel4.border.titleFont"), resourceMap.getColor("jPanel4.border.titleColor"))); // NOI18N
        jPanel4.setName("jPanel4"); // NOI18N
        jRadioButton13.setText(resourceMap.getString("jRadioButton13.text")); // NOI18N
        jRadioButton13.setIcon(resourceMap.getIcon("jRadioButton13.icon")); // NOI18N
        jRadioButton13.setIconTextGap(1);
        jRadioButton13.setName("jRadioButton13"); // NOI18N
        jRadioButton13.setSelectedIcon(resourceMap.getIcon("severeerror.icon"));
        jRadioButton13.setSelected(true);
        jRadioButton13.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton14.setText(resourceMap.getString("jRadioButton14.text")); // NOI18N
        jRadioButton14.setIcon(resourceMap.getIcon("jRadioButton14.icon")); // NOI18N
        jRadioButton14.setIconTextGap(1);
        jRadioButton14.setName("jRadioButton14"); // NOI18N
        jRadioButton14.setSelectedIcon(resourceMap.getIcon("severeerror.icon"));
        jRadioButton14.setSelected(true);
        jRadioButton14.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton15.setText(resourceMap.getString("jRadioButton15.text")); // NOI18N
        jRadioButton15.setIcon(resourceMap.getIcon("jRadioButton15.icon")); // NOI18N
        jRadioButton15.setIconTextGap(1);
        jRadioButton15.setName("jRadioButton15"); // NOI18N
        jRadioButton15.setSelectedIcon(resourceMap.getIcon("severeerror.icon"));
        jRadioButton15.setSelected(true);
        jRadioButton15.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        jRadioButton16.setText(resourceMap.getString("jRadioButton16.text")); // NOI18N
        jRadioButton16.setIcon(resourceMap.getIcon("jRadioButton16.icon")); // NOI18N
        jRadioButton16.setIconTextGap(1);
        jRadioButton16.setName("jRadioButton16"); // NOI18N
        jRadioButton16.setSelectedIcon(resourceMap.getIcon("severeerror.icon"));
        jRadioButton16.setSelected(true);
        jRadioButton16.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                defineSeveriy(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
                jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel4Layout.createSequentialGroup()
                        .addComponent(jRadioButton13)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton14)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton15)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jRadioButton16))
        );
        jPanel4Layout.setVerticalGroup(
                jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jRadioButton13)
                        .addComponent(jRadioButton14)
                        .addComponent(jRadioButton15)
                        .addComponent(jRadioButton16))
        );

        jLabel1.setIcon(resourceMap.getIcon("jLabel1.icon")); // NOI18N
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jSeparator1.setName("jSeparator1"); // NOI18N

        jLabel2.setIcon(resourceMap.getIcon("jLabel2.icon")); // NOI18N
        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jTextField1.setFont(resourceMap.getFont("jTextField1.font")); // NOI18N
        jTextField1.setForeground(resourceMap.getColor("jTextField1.foreground")); // NOI18N
        jTextField1.setText(resourceMap.getString("jTextField1.text")); // NOI18N
        jTextField1.setName("jTextField1"); // NOI18N

        jLabel3.setFont(resourceMap.getFont("jLabel3.font")); // NOI18N
        jLabel3.setForeground(resourceMap.getColor("jLabel3.foreground")); // NOI18N
        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jSeparator11.setName("jSeparator11"); // NOI18N

        jScrollPane3.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
        jScrollPane3.setName("jScrollPane3"); // NOI18N

        // Create default table model
        mmm = new javax.swing.table.DefaultTableModel(
                new Object [][] {
                        {null, null, null, null, null, null, null},
                },
                new String [] {
                        "Time", "Expid", "Session", "RunType", "CType"," Name", "Severity","Message"
                }
        )
        {
            Class[] types = new Class [] {
                    java.lang.String.class, java.lang.String.class, java.lang.String.class,
                    java.lang.String.class, java.lang.String.class, java.lang.String.class,
                    java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                    false, false, false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        };
        jTable1.setBorder(javax.swing.BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
        jTable1.setFont(resourceMap.getFont("jTable1.font")); // NOI18N
        jTable1.setModel(mmm);
        TableCellRenderer renderer = new CustomTableCellRenderer();
        try
        {
            jTable1.setDefaultRenderer( Class.forName
                    ( "java.lang.String" ), renderer );
        }
        catch( ClassNotFoundException ex )
        {
            System.exit( 0 );
        }
        jTable1.setName("jTable1"); // NOI18N
        jScrollPane3.setViewportView(jTable1);

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
                mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(mainPanelLayout.createSequentialGroup()
                                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 191, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                        .addGroup(mainPanelLayout.createSequentialGroup()
                                                        .addContainerGap()
                                                        .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                        .addComponent(jLabel1)))
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 810, Short.MAX_VALUE)
                                                .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                                                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                        .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                        .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                        .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 810, Short.MAX_VALUE)))
                                        .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addContainerGap()
                                        .addComponent(jLabel3)
                                        .addGap(38, 38, 38)
                                        .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, 349, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                .addGap(15, 15, 15))
                        .addComponent(jSeparator1, javax.swing.GroupLayout.DEFAULT_SIZE, 1022, Short.MAX_VALUE)
                        .addComponent(jSeparator11, javax.swing.GroupLayout.DEFAULT_SIZE, 1022, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
                mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(jLabel1)
                                .addComponent(jLabel2)
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jPanel1, 0, 61, Short.MAX_VALUE)
                                .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                        .addGap(21, 21, 21)
                        .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(11, 11, 11)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                                        .addComponent(jScrollPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 293, Short.MAX_VALUE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 81, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 380, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jSeparator11, javax.swing.GroupLayout.PREFERRED_SIZE, 4, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(9, 9, 9)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel3))
                        .addContainerGap())
        );

        menuBar.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        menuBar.setFont(resourceMap.getFont("menuBar.font")); // NOI18N
        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setFont(resourceMap.getFont("fileMenu.font")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(DalogBrowserApp.class).getContext().getActionMap(DalogBrowserView.class, this);
        jMenuItem1.setAction(actionMap.get("showAddPlatformBox")); // NOI18N
        jMenuItem1.setFont(resourceMap.getFont("jMenuItem1.font")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        fileMenu.add(jMenuItem1);

        jSeparator12.setName("jSeparator12"); // NOI18N
        fileMenu.add(jSeparator12);

        jMenuItem9.setAction(actionMap.get("disconnectFromThePlatform")); // NOI18N
        jMenuItem9.setFont(resourceMap.getFont("jMenuItem9.font")); // NOI18N
        jMenuItem9.setName("jMenuItem9"); // NOI18N
        fileMenu.add(jMenuItem9);

        jSeparator2.setName("jSeparator2"); // NOI18N
        fileMenu.add(jSeparator2);

        jSeparator10.setName("jSeparator10"); // NOI18N
        fileMenu.add(jSeparator10);

        jMenuItem2.setAction(actionMap.get("selectAll")); // NOI18N
        jMenuItem2.setFont(resourceMap.getFont("jMenuItem2.font")); // NOI18N
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        fileMenu.add(jMenuItem2);

        jSeparator3.setName("jSeparator3"); // NOI18N
        fileMenu.add(jSeparator3);

        jMenuItem3.setAction(actionMap.get("deselectAll")); // NOI18N
        jMenuItem3.setFont(resourceMap.getFont("jMenuItem3.font")); // NOI18N
        jMenuItem3.setName("jMenuItem3"); // NOI18N
        fileMenu.add(jMenuItem3);

        jSeparator4.setName("jSeparator4"); // NOI18N
        fileMenu.add(jSeparator4);

        jMenuItem4.setAction(actionMap.get("selectInfo")); // NOI18N
        jMenuItem4.setFont(resourceMap.getFont("jMenuItem4.font")); // NOI18N
        jMenuItem4.setName("jMenuItem4"); // NOI18N
        fileMenu.add(jMenuItem4);

        jSeparator5.setName("jSeparator5"); // NOI18N
        fileMenu.add(jSeparator5);

        jMenuItem5.setAction(actionMap.get("selectWarning")); // NOI18N
        jMenuItem5.setFont(resourceMap.getFont("jMenuItem5.font")); // NOI18N
        jMenuItem5.setName("jMenuItem5"); // NOI18N
        fileMenu.add(jMenuItem5);

        jSeparator6.setName("jSeparator6"); // NOI18N
        fileMenu.add(jSeparator6);

        jMenuItem6.setAction(actionMap.get("selectError")); // NOI18N
        jMenuItem6.setFont(resourceMap.getFont("jMenuItem6.font")); // NOI18N
        jMenuItem6.setName("jMenuItem6"); // NOI18N
        fileMenu.add(jMenuItem6);

        jSeparator7.setName("jSeparator7"); // NOI18N
        fileMenu.add(jSeparator7);

        jMenuItem7.setAction(actionMap.get("selectSevereError")); // NOI18N
        jMenuItem7.setFont(resourceMap.getFont("jMenuItem7.font")); // NOI18N
        jMenuItem7.setName("jMenuItem7"); // NOI18N
        fileMenu.add(jMenuItem7);

        jSeparator8.setName("jSeparator8"); // NOI18N
        fileMenu.add(jSeparator8);

        jSeparator9.setName("jSeparator9"); // NOI18N
        fileMenu.add(jSeparator9);

        exitMenuItem.setAction(actionMap.get("myExit")); // NOI18N
        exitMenuItem.setFont(resourceMap.getFont("exitMenuItem.font")); // NOI18N
        exitMenuItem.setText(resourceMap.getString("exitMenuItem.text")); // NOI18N
        exitMenuItem.setToolTipText(resourceMap.getString("exitMenuItem.toolTipText")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setFont(resourceMap.getFont("helpMenu.font")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setFont(resourceMap.getFont("aboutMenuItem.font")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        jScrollPane4.setName("jScrollPane4"); // NOI18N

        jList1.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        jList1.setName("jList1"); // NOI18N
        jScrollPane4.setViewportView(jList1);

        setComponent(mainPanel);
        setMenuBar(menuBar);

    }// </editor-fold>//GEN-END:initComponents
    private void defineSeveriy(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_defineSeveriy
        dfs();
    }//GEN-LAST:event_defineSeveriy
//----------------------------------------------------------------------------    

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JList jList1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JMenuItem jMenuItem6;
    private javax.swing.JMenuItem jMenuItem7;
    private javax.swing.JMenuItem jMenuItem9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JRadioButton jRadioButton10;
    private javax.swing.JRadioButton jRadioButton11;
    private javax.swing.JRadioButton jRadioButton12;
    private javax.swing.JRadioButton jRadioButton13;
    private javax.swing.JRadioButton jRadioButton14;
    private javax.swing.JRadioButton jRadioButton15;
    private javax.swing.JRadioButton jRadioButton16;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JRadioButton jRadioButton4;
    private javax.swing.JRadioButton jRadioButton5;
    private javax.swing.JRadioButton jRadioButton6;
    private javax.swing.JRadioButton jRadioButton7;
    private javax.swing.JRadioButton jRadioButton8;
    private javax.swing.JRadioButton jRadioButton9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator10;
    private javax.swing.JSeparator jSeparator11;
    private javax.swing.JSeparator jSeparator12;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JSeparator jSeparator9;
    private javax.swing.JTable jTable1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextPane jTextPane1;
    private javax.swing.JTree jTree1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    // End of variables declaration//GEN-END:variables
    private JDialog aboutBox;
    private JDialog addPlatformBox;
    private StyledDocument doc;
    private Style style;

    /**
     *
     * @author gurjyan
     */
    class VisualizeCallback extends cMsgCallbackAdapter {

        private int messageCount;

        @Override
        public void callback(cMsgMessage msg, Object userObject) {
            messageCount++;
            try {
                String type;
                String sender;
                String txt;
                String week=null;
                String month=null;
                String day=null;
                String hour=null;
                String time=null;
                String sseverity=null;
                String state=null;
                String expid=null;
                String session=null;
                String runtype=null;
                String codaclass=null;
                String name=null;
                String message;
                int severity = 0;

                String t = new Date().toString();
                StringTokenizer tk = new StringTokenizer(t);
                if(msg.getPayloadItem("EXPID")!=null)expid = msg.getPayloadItem("EXPID").getString();
                if(msg.getPayloadItem("session")!=null)session = msg.getPayloadItem("session").getString();
                if(msg.getPayloadItem("config")!=null)runtype = msg.getPayloadItem("config").getString();
                if(msg.getPayloadItem("codaClass")!=null)codaclass = msg.getPayloadItem("codaClass").getString();
                if(msg.getPayloadItem("codaName")!=null)name = msg.getPayloadItem("codaName").getString();
                if(msg.getPayloadItem("severity")!=null)sseverity = msg.getPayloadItem("severity").getString();
                if(msg.getPayloadItem("state")!=null)state = msg.getPayloadItem("state").getString();
                message = msg.getText();
                type = msg.getType();
                sender = msg.getSender();

                // get the time string 
                if (tk.hasMoreTokens()) week = tk.nextToken()+" ";
                if (tk.hasMoreTokens()) month = tk.nextToken()+" ";
                if (tk.hasMoreTokens()) day = tk.nextToken()+" ";
                if (tk.hasMoreTokens()) hour = tk.nextToken()+" ";
                time = hour+" "+month+" "+day;

                txt = time + " : " + sseverity + " : " + codaclass + " : " + expid + " : " + session + " : " + runtype + " : " + name + " :: " + message;

                try {
                    severity = Integer.parseInt(type.substring(type.lastIndexOf("/") + 1));
                } catch (NumberFormatException e) {
                    e.printStackTrace();
                }

// reset color to black
                StyleConstants.setForeground(style, Color.BLACK);
                try {
                    if(doc.getLength()>=1000){
                        doc.remove(0,100);
                    }
                    if (severity <= 4) {
                        StyleConstants.setForeground(style, Color.BLUE);
                        doc.insertString(doc.getLength(), " " + txt + "\n\n", style);
                        mmm.addRow(new Object[]{time, expid, session, runtype, codaclass, name, sseverity, message});
                        jTable1.scrollRectToVisible(jTable1.getCellRect(mmm.getRowCount() - 1, 0, true));
                        jTextPane1.setCaretPosition(jTextPane1.getDocument().getLength());
                    } else if (severity > 4 && severity <= 8) {
                        StyleConstants.setForeground(style, Color.ORANGE);
                        doc.insertString(doc.getLength(), " " + txt + "\n\n", style);
                        mmm.addRow(new Object[]{time, expid, session, runtype, codaclass, name, sseverity, message});
                        jTable1.scrollRectToVisible(jTable1.getCellRect(mmm.getRowCount() - 1, 0, true));
                        jTextPane1.setCaretPosition(jTextPane1.getDocument().getLength());
                    } else if (severity > 8 && severity <= 12) {
                        StyleConstants.setForeground(style, Color.MAGENTA);
                        doc.insertString(doc.getLength(), " " + txt + "\n\n", style);
                        mmm.addRow(new Object[]{time, expid, session, runtype, codaclass, name, sseverity, message});
                        jTable1.scrollRectToVisible(jTable1.getCellRect(mmm.getRowCount() - 1, 0, true));
                        jTextPane1.setCaretPosition(jTextPane1.getDocument().getLength());
                    } else if (severity > 12 && severity <= 16) {
                        StyleConstants.setForeground(style, Color.RED);
                        doc.insertString(doc.getLength(), " " + txt + "\n\n", style);
                        mmm.addRow(new Object[]{time, expid, session, runtype, codaclass, name, sseverity, message});
                        jTable1.scrollRectToVisible(jTable1.getCellRect(mmm.getRowCount() - 1, 0, true));
                    }

                    if (messageCount >= messageLimit) {
                        mmm.removeRow(0);
                    }
                } catch (BadLocationException e) {
                    e.printStackTrace();
                } catch (NullPointerException n) {
                }
                jTextPane1.setCaretPosition(jTextPane1.getDocument().getLength());
            } catch (cMsgException ex) {
                Logger.getLogger(DalogBrowserView.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }


    //----------------------------------------------------------------------------
    /**
     *
     * @author gurjyan
     */
    class DalogMessageCallback extends cMsgCallbackAdapter {


        @Override
        public void callback(cMsgMessage msg, Object userObject) {
            String expid=null,  session=null,  runtype=null,  componentType=null,  componentName=null, subject = null;

            try{
                if(msg.getPayloadItem("EXPID")!=null)expid = msg.getPayloadItem("EXPID").getString();
                if(msg.getPayloadItem("session")!=null)session = msg.getPayloadItem("session").getString();
                if(msg.getPayloadItem("config")!=null)runtype = msg.getPayloadItem("config").getString();
                if(msg.getPayloadItem("codaName")!=null)componentName = msg.getPayloadItem("codaName").getString();
                if(msg.getPayloadItem("codaClass")!=null)componentType = msg.getPayloadItem("codaClass").getString();
                subject = msg.getSubject();
//                System.out.println("cb ...> "+expid+" : "+session+" : "+runtype+" : "+componentType+" : "+componentName);
//                System.out.println("cb <... "+ subject);

            } catch (cMsgException ex) {
                Logger.getLogger(DalogBrowserView.class.getName()).log(Level.SEVERE, null, ex);
            }

            if((!CompAddress.containsKey(componentName))
                    || ( (CompAddress.containsKey(componentName))
                    && (!CompAddress.get(componentName).equals(subject)))) {

                //remove the component from the JTree
//                    removeNodeFromTheTree(componentName);
                deleteTheNode(componentName);

                //update compAddress hashmap
                CompAddress.remove(componentName);
                CompAddress.put(componentName, subject);

                if(expid!=null){
                    if(session!=null){
                        if(!Sessions.contains(session)){
                            addNodeToTheTree(expid, session);
                            Sessions.add(session);
                        }
                        if(runtype!=null){
                            if(!RunTypes.contains(runtype)){
                                addNodeToTheTree(session,runtype);
                                RunTypes.add(runtype);
                            }
                            if(componentType!=null){
                                if(!Config_CodaTypes.containsKey(runtype)){
                                    ArrayList<String> tlist = new ArrayList<String>();
                                    tlist.add(componentType);
                                    Config_CodaTypes.put(runtype,tlist);
                                    addNodeToTheTree(runtype,componentType);
                                    System.out.println("<.h.> "+runtype+" <-> "+ componentType);
                                }
                                else {
                                        ArrayList<String> tl = Config_CodaTypes.get(runtype);
                                   if(!tl.contains(componentType)){
                                            tl.add(componentType);
                                        Config_CodaTypes.put(runtype,tl);
                                        addNodeToTheTree(runtype,componentType);
                                        System.out.println("<.hhh.> "+runtype+" <-> "+ componentType);
                                    }
                                }
                                if(componentName!=null){
                                    addGeneticNode(runtype, componentType, componentName);
//                                    addNodeToTheTree(componentType, componentName);
                                }
                            } else {
                                //codaType is not defined
                                System.out.println("Severe Error: Type is not defined for the component");
                            }
                        } else {
                            //session is defined but runtype is not
                            if(componentType!=null){
                                if(!Session_CodaTypes.containsKey(session)){
                                    ArrayList<String> tlist = new ArrayList<String>();
                                    tlist.add(componentType);
                                    Session_CodaTypes.put(session,tlist);
                                    addNodeToTheTree(session,componentType);
                                }
                                else {
                                    ArrayList<String> tl = Session_CodaTypes.get(runtype);
                               if(!tl.contains(componentType)){
                                        tl.add(componentType);
                                    Session_CodaTypes.put(session,tl);
                                    addNodeToTheTree(session,componentType);
                                    System.out.println("<.hhh. S > "+session+" <-> "+ componentType);
                                }
                                }
                                if(componentName!=null){
                                    addGeneticNode(session, componentType, componentName);
//                                    addNodeToTheTree(componentType, componentName);
                                }
                            } else {
                                //codaType is not defined
                                System.out.println("Severe Error: Type is not defined for the component");
                            }
                        }
                    } else {
                        //session is not defined
                        if(runtype!=null){
                            if(!RunTypes.contains(runtype)){
                                addNodeToTheTree(expid,runtype);
                                RunTypes.add(runtype);
                            }
                            if(componentType!=null){
                                if(!Config_CodaTypes.containsKey(runtype)){
                                    ArrayList<String> tlist = new ArrayList<String>();
                                    tlist.add(componentType);
                                    Config_CodaTypes.put(runtype,tlist);
                                    addNodeToTheTree(runtype,componentType);
                                }
                                else {
                                    ArrayList<String> tl = Config_CodaTypes.get(runtype);
                               if(!tl.contains(componentType)){
                                        tl.add(componentType);
                                    Config_CodaTypes.put(runtype,tl);
                                    addNodeToTheTree(runtype,componentType);
                                    System.out.println("<.hhh.> "+runtype+" <-> "+ componentType);
                                }
                                }
                                if(componentName!=null){
                                    addGeneticNode(runtype, componentType, componentName);
//                                    addNodeToTheTree(componentType, componentName);

                                }
                            } else {
                                //codaType is not defined
                                System.out.println("Severe Error: Type is not defined for the component");
                            }
                        } else {
                            //session and runtype are not defined
                            if(componentType!=null){
                                if(!Expid_CodaTypes.containsKey(expid)){
                                    ArrayList<String> tlist = new ArrayList<String>();
                                    tlist.add(componentType);
                                    Expid_CodaTypes.put(expid,tlist);
                                    addNodeToTheTree(expid,componentType);
                                }
                                else {
                                    ArrayList<String> tl = Expid_CodaTypes.get(expid);
                                     if(!tl.contains(componentType)){
                                        tl.add(componentType);
                                    Expid_CodaTypes.put(expid,tl);
                                    addNodeToTheTree(expid,componentType);
                                    System.out.println("<.hhh. E > "+expid+" <-> "+ componentType);
                                }
                                }
                                if(componentName!=null){
                                    addGeneticNode(expid, componentType, componentName);
                                }
                            } else {
                                //codaType is not defined
                                System.out.println("Severe Error: Type is not defined for the component");
                            }
                        }
                    }
                } else {
                    //expid is not defined
                    System.out.println("Severe Error: Expid is not set in the dalogMsg.");
                }
            }
        }
    }


    /**
     * Inner class for custom cell rendering
     * @author gurjyan
     */
    public class CustomTableCellRenderer extends DefaultTableCellRenderer {

        @Override
        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int column) {
            String tmp = null;

            Component cell = super.getTableCellRendererComponent(table, value,
                    isSelected, hasFocus, row, column);
            if (value instanceof String) {
                String sever = (String) value;
                StringTokenizer st = new StringTokenizer(sever, ".");
                while (st.hasMoreTokens()) {
                    tmp = st.nextToken();
                }
                if (tmp.equals("5")) {
                    cell.setBackground(Color.ORANGE);
                } else if (tmp.equals("6")) {
                    cell.setBackground(Color.ORANGE);
                } else if (tmp.equals("7")) {
                    cell.setBackground(Color.ORANGE);
                } else if (tmp.equals("8")) {
                    cell.setBackground(Color.ORANGE);
                } else if (tmp.equals("9")) {
                    cell.setBackground(Color.MAGENTA);
                } else if (tmp.equals("10")) {
                    cell.setBackground(Color.MAGENTA);
                } else if (tmp.equals("11")) {
                    cell.setBackground(Color.MAGENTA);
                } else if (tmp.equals("12")) {
                    cell.setBackground(Color.MAGENTA);
                } else if (tmp.equals("13")) {
                    cell.setBackground(Color.RED);
                } else if (tmp.equals("14")) {
                    cell.setBackground(Color.RED);
                } else if (tmp.equals("15")) {
                    cell.setBackground(Color.RED);
                } else if (tmp.equals("16")) {
                    cell.setBackground(Color.RED);
                } // cell.setForeground();
                // cell.setFont();
                else {
                    cell.setBackground(new Color(230, 230, 230));
                }
            }
            return cell;
        }
    }
}
