/*
 * WncHmiView.java
 */

package org.jlab.coda.runcontrol.gui.wnchmi;


import org.jdesktop.application.Action;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.SingleFrameApplication;
import org.jlab.coda.cMsg.*;
import org.jlab.coda.runcontrol.gui.DLBExpid;
import org.jlab.coda.runcontrol.gui.DLBSession;
import org.jlab.coda.runcontrol.gui.DLBSupervisor;
import org.jlab.coda.runcontrol.gui.DLBType;
import org.jlab.coda.runcontrol.util.rcConstants;
import org.jlab.coda.runcontrol.util.rcSubscription;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.Position;
import javax.swing.tree.*;
import java.awt.*;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * The application's main frame.
 */
public class WncHmiView extends FrameView implements TreeSelectionListener {

    /** The name of this HMI.
     * Must be unique for the cMsg server that it will connect
     */
    public String myName;

/** WC crate name currently being monitored and controled*/
    public String myWCName;

    /** Hashmap holds Expid class objects. The key is the name of the EXPID*/
    private ConcurrentHashMap<String, DLBExpid> EXPIDS = new ConcurrentHashMap<String, DLBExpid>();
    /** jTree root objec*/
    private DefaultMutableTreeNode root;
    /** cMsg server */
    private cMsg cMsgSrv;
    /** Dalog cMsg subscription object. This is for the subscription 
     subject = expid/session/runtype/codatype/name
     type = dalog/{i=1|i=2....}*/
    private Object ssObj;
    /** Subject of the ssObj subscription*/
    private String subject;
    /** Type of the ssObj subscription*/
    private String type = "dalog/{i=1|i=2|i=3|i=4|i=5|i=6|i=7|i=8|i=9|i=10|i=11|i=12|i=13|i=14|i=15|i=16}";
    private cMsgCallbackInterface visualizecontrol = new VisualizeCallback();
    /** Selected expid from the tree*/
    private String $expid;
    /** Default name of the root tree*/
    private String rootNodeName = "Afecs Platforms";
    /** Default table model*/
    public javax.swing.table.DefaultTableModel myTableModel;
    /** Message limit of the table*/
    public int messageLimit = 1000;


    /**
     * Constructor
     * @param app
     */
    public WncHmiView(SingleFrameApplication app) {
        super(app);
        app.getMainFrame().getContentPane().setPreferredSize(new Dimension(1640, 480));
        buildInitialTree();
        initComponents();
        this.getFrame().setTitle("Wiener Crate Control");
    }


    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = WncHmiApp.getApplication().getMainFrame();
            aboutBox = new WncHmiAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        WncHmiApp.getApplication().show(aboutBox);
    }


    @Action
    public void Exit() {
        try {
            if(cMsgSrv!=null)cMsgSrv.disconnect();
        } catch (cMsgException e) {
            e.printStackTrace();
        }

        this.getFrame().dispose();
    }

    @Action
    public void showAddPlatformBox() {
        if (addPlatformBox == null) {
            JFrame mainFrame = WncHmiApp.getApplication().getMainFrame();
            addPlatformBox = new Connect2Platform(mainFrame, this);
            addPlatformBox.setLocationRelativeTo(mainFrame);
        }
        WncHmiApp.getApplication().show(addPlatformBox);
    }

    /**
     * Based on dalog browser tree selection create a subject and creates a new
     * subscriptions before removing the old subsciptions
     * @param event
     */
    public void valueChanged(TreeSelectionEvent event) {
        StringBuffer sb = new StringBuffer();
        int i = 0;
        Object o = jTree1.getLastSelectedPathComponent();
        if (o != null) {
            for (i = 1; i < jTree1.getSelectionPath().getPathCount(); i++) {
                if (i == 5) {
                    sb.append(jTree1.getSelectionPath().getPathComponent(i));
                    myWCName = jTree1.getSelectionPath().getPathComponent(i).toString();
                    powerOnButton.setEnabled(true);
                    powerOffButton.setEnabled(true);
                } else {
                    sb.append(jTree1.getSelectionPath().getPathComponent(i)).append("/");
                    myWCName = null;
                    powerOnButton.setEnabled(false);
                    powerOffButton.setEnabled(false);
                }
            }
            if (i < 6 && i > 1) {
                sb.append("*");
            } else {
                subject = sb.toString();
                // renew the subscription
                renewSubscription();
            }
        }
    }

    /**
     * Unsubscribes and creates the new subscription to the dalog messages
     * of the AFECS platform.
     */
    private void renewSubscription() {

        // Here we unsubscribe and subscribe to the subject and type 
        if ((subject != null) &&
                (!subject.equals("")) &&
                (type != null) &&
                (subject.contains(rcConstants.wienercrateType))) {
            try {
                if (cMsgSrv != null) {
                    if (ssObj != null) {
                        cMsgSrv.unsubscribe(ssObj);
                    }
                    ssObj = cMsgSrv.subscribe(subject, type, visualizecontrol, null);
                }
            } catch (cMsgException e) {
                e.printStackTrace();
            }
//            System.out.println("DBG .... Subscribing to subject = " + subject + " type = " + type);
        }
    }

    /**
     * Open connection to the Afecs platform (i.e. expid).
     * @param udl     UDL of the platform of the form
     *                cMsg://host:port/cMsg/expid
     * Todo all the necessary subscriptions
     */
    public void connectToThePlatform(String udl) {
        String tmps = null;
        String expid = null, host = null, port = null;


        // Separate the string after cMsg:// and tokenize it by /
        StringTokenizer st = new StringTokenizer(udl.split("://")[1], "/");
        while (st.hasMoreTokens()) {
            tmps = st.nextToken();
            if (tmps.contains(":")) {
                // This might be a host and port section of the udl
                StringTokenizer st1 = new StringTokenizer(tmps, ":");
                host = st1.nextToken();
                port = st1.nextToken();
            } else {
                //always asign the next token to expid,
                //since the last one is always expid
                expid = tmps;
            }
        }

        // See if we have already connection with this platform
        if (!EXPIDS.containsKey(expid)) {

//                    System.out.println("debug... " + expid + " " + host + " " + port);

            // Connect to cMsg server of the platform
            try {
                cMsgSrv = new cMsg(udl, myName, " ");
                cMsgSrv.connect();

                // Enable message reception
                cMsgSrv.start();
                // Subscribe to the subject = expid/* and type = dalog/*,
                // necessary for the tree construction.
                cMsgCallbackInterface dalogMessageCallback = new DalogMessageCallback();
                cMsgSrv.subscribe(expid + "/*", "dalog/*", dalogMessageCallback, null);


            } catch (cMsgException e) {
                e.printStackTrace();
                Exit();
            }

            DLBExpid np = new DLBExpid();
            np.setHost(host);
            np.setPort(port);
            np.setServer(cMsgSrv);

            EXPIDS.put(expid, np);

            addNodeToTheTree(rootNodeName, expid);

        }
    }

    //----------------------------------------------------------------------------
    /**
     * Removes the node from the dalog browser tree.
     * If the node has descendants all descendants are removed as well.
     * @param nodename              The name of the node
     */
    public void removeNodeFromTheTree(String nodename) {
        int startingRow = 0;

        // Get the default model of the jTree1
        DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();

        // Find node to remove
        TreePath path = jTree1.getNextMatch(nodename, startingRow, Position.Bias.Forward);
        MutableTreeNode node = (MutableTreeNode) path.getLastPathComponent();

        // Remove the node
        model.removeNodeFromParent(node);
    }
    //----------------------------------------------------------------------------
    /**
     * Adds the node to the root node
     * @param rootnode   the name of the root node
     * @param name       the name of the branch
     */
    public void addNodeToTheTree(String rootnode, String name) {

        int startingRow = 0;

        DefaultTreeModel model = (DefaultTreeModel) jTree1.getModel();

        try{
            // Find the root node to which the new branch node is to be added
            TreePath path = jTree1.getNextMatch(rootnode, startingRow, Position.Bias.Forward);
//        System.out.println(" Path .... " + path + " rootnode = " + rootnode);
            MutableTreeNode node = (MutableTreeNode) path.getLastPathComponent();

            // Creating branch node of the root node
            DefaultMutableTreeNode newNode = new DefaultMutableTreeNode(name);

            // Insert new session node as last child of the expid node
            model.insertNodeInto(newNode, node, node.getChildCount());
            jTree1.validate();

            // expand the tree
            TreeNode base = (TreeNode) jTree1.getModel().getRoot();
            expandAll(jTree1, new TreePath(base), true);

        } catch(NullPointerException e){e.printStackTrace();}
    }

    /**
     * Method for dalog message tree expansion or collapse.
     * @param tree
     * @param parent
     * @param expand      true: expands, false: collapses
     */
    private void expandAll(JTree tree, TreePath parent, boolean expand) {

        // Traverse children
        TreeNode node = (TreeNode) parent.getLastPathComponent();
        if (node.getChildCount() >= 0) {
            for (Enumeration e = node.children(); e.hasMoreElements();) {
                TreeNode n = (TreeNode) e.nextElement();
                TreePath path = parent.pathByAddingChild(n);
                expandAll(tree, path, expand);
            }
        }
        // Expansion or collapse must be done buttom-up
        if (expand) {
            tree.expandPath(parent);
        } else {
            tree.collapsePath(parent);
        }
    }

    /**
     * Create the root of the dalog browser tree.
     */
    public void buildInitialTree() {
        root = new DefaultMutableTreeNode(rootNodeName);
    }




    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTree1 =  new JTree(root);
        jTree1.addTreeSelectionListener(this);
        jPanel1 = new javax.swing.JPanel();
        powerOnButton = new javax.swing.JButton();
        powerOffButton = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        jTextField2 = new javax.swing.JTextField();
        jTextField3 = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTable1 = new javax.swing.JTable();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        jMenuItem2 = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JSeparator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(WncHmiApp.class).getContext().getResourceMap(WncHmiView.class);
        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setBackground(resourceMap.getColor("mainPanel.background")); // NOI18N

        jTable1.setBackground(resourceMap.getColor("mainPanel.background")); // NOI18N
        jScrollPane1.setBackground(resourceMap.getColor("mainPanel.background")); // NOI18N
        jScrollPane2.setBackground(resourceMap.getColor("mainPanel.background")); // NOI18N

        menuBar.setName("menuBar"); // NOI18N
        menuBar.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        menuBar.setFont(resourceMap.getFont("menuBar.font")); // NOI18N
        menuBar.setBackground(resourceMap.getColor("mainPanel.background")); // NOI18N
        
        jLabel1.setIcon(resourceMap.getIcon("jLabel1.icon")); // NOI18N
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jLabel1.setName("jLabel1"); // NOI18N

        jLabel2.setIcon(resourceMap.getIcon("jLabel2.icon")); // NOI18N
        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        jLabel2.setName("jLabel2"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jTree1.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jTree1.setFont(resourceMap.getFont("jTable1.font")); // NOI18N
        jTable1.setForeground(resourceMap.getColor("jLabel3.foreground")); // NOI18N
        jTree1.setName("jTree1"); // NOI18N
        jScrollPane1.setViewportView(jTree1);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),
                resourceMap.getString("jPanel1.border.title"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION, resourceMap.getFont("jPanel1.border.titleFont"),
                resourceMap.getColor("jPanel1.border.titleColor"))); // NOI18N
        jPanel1.setToolTipText(resourceMap.getString("jPanel1.toolTipText")); // NOI18N
        jPanel1.setAutoscrolls(true);
        jPanel1.setName("jPanel1"); // NOI18N
        jPanel1.setBackground(resourceMap.getColor("jPanel1.background")); // NOI18N

        powerOnButton.setFont(resourceMap.getFont("jButton1.font")); // NOI18N
        powerOnButton.setForeground(resourceMap.getColor("jButton1.foreground")); // NOI18N
        powerOnButton.setIcon(resourceMap.getIcon("jButton1.icon")); // NOI18N
        powerOnButton.setText(resourceMap.getString("jButton1.text")); // NOI18N
//        powerOnButton.setSize(100,4);
        powerOnButton.setToolTipText(resourceMap.getString("jButton1.toolTipText")); // NOI18N
        powerOnButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        powerOnButton.setName("powerOn"); // NOI18N
        powerOnButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                     if(cMsgSrv!=null && cMsgSrv.isConnected() && myWCName !=null){
                    cMsgMessage msg = new cMsgMessage();
                    msg.setSubject(myWCName);
                    msg.setType(rcSubscription.WNCControlType);
                    msg.setText("on");
                         try {
                             cMsgSrv.send(msg);
                             cMsgSrv.flush(1);
                         } catch (cMsgException e) {
                             e.printStackTrace();
                         }
                     }
                }
            });

        powerOffButton.setFont(resourceMap.getFont("jButton1.font")); // NOI18N
        powerOffButton.setForeground(resourceMap.getColor("jButton1.foreground")); // NOI18N
        powerOffButton.setIcon(resourceMap.getIcon("jButton2.icon")); // NOI18N
        powerOffButton.setText(resourceMap.getString("jButton2.text")); // NOI18N
//        powerOffButton.setSize(100,4);
        powerOffButton.setToolTipText(resourceMap.getString("jButton2.toolTipText")); // NOI18N
        powerOffButton.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));
        powerOffButton.setName("powerOff"); // NOI18N
        powerOffButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                     if(cMsgSrv!=null && cMsgSrv.isConnected() && myWCName !=null){
                    cMsgMessage msg = new cMsgMessage();
                    msg.setSubject(myWCName);
                    msg.setType(rcSubscription.WNCControlType);
                    msg.setText("off");
                         try {
                             cMsgSrv.send(msg);
                             cMsgSrv.flush(1);
                         } catch (cMsgException e) {
                             e.printStackTrace();
                         }
                     }
                }
            });

        powerOnButton.setEnabled(false);
        powerOffButton.setEnabled(false);
        
        jLabel3.setFont(resourceMap.getFont("jLabel3.font")); // NOI18N
        jLabel3.setForeground(resourceMap.getColor("jLabel3.foreground")); // NOI18N
        jLabel3.setIcon(resourceMap.getIcon("jLabel3.icon")); // NOI18N
        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jLabel4.setFont(resourceMap.getFont("jLabel4.font")); // NOI18N
        jLabel4.setForeground(resourceMap.getColor("jLabel4.foreground")); // NOI18N
        jLabel4.setIcon(resourceMap.getIcon("jLabel4.icon")); // NOI18N
        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        jLabel5.setFont(resourceMap.getFont("jLabel5.font")); // NOI18N
        jLabel5.setForeground(resourceMap.getColor("jLabel5.foreground")); // NOI18N
        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N

        jTextField1.setFont(resourceMap.getFont("jTextField1.font")); // NOI18N
        jTextField1.setForeground(resourceMap.getColor("jTextField1.foreground")); // NOI18N
        jTextField1.setText(resourceMap.getString("jTextField1.text")); // NOI18N
        jTextField1.setName("jTextField1"); // NOI18N

        jTextField2.setFont(resourceMap.getFont("jTextField2.font")); // NOI18N
        jTextField2.setForeground(resourceMap.getColor("jTextField2.foreground")); // NOI18N
        jTextField2.setText(resourceMap.getString("jTextField2.text")); // NOI18N
        jTextField2.setName("jTextField2"); // NOI18N

        jTextField3.setFont(resourceMap.getFont("jTextField3.font")); // NOI18N
        jTextField3.setForeground(resourceMap.getColor("jTextField3.foreground")); // NOI18N
        jTextField3.setText(resourceMap.getString("jTextField3.text")); // NOI18N
        jTextField3.setName("jTextField3"); // NOI18N

        jLabel6.setFont(resourceMap.getFont("jLabel6.font")); // NOI18N
        jLabel6.setForeground(resourceMap.getColor("jLabel6.foreground")); // NOI18N
        jLabel6.setIcon(resourceMap.getIcon("jLabel4.icon")); // NOI18N
        jLabel6.setText(resourceMap.getString("jLabel6.text")); // NOI18N
        jLabel6.setName("jLabel6"); // NOI18N

        jLabel7.setFont(resourceMap.getFont("jLabel7.font")); // NOI18N
        jLabel7.setForeground(resourceMap.getColor("jLabel7.foreground")); // NOI18N
        jLabel7.setIcon(resourceMap.getIcon("jLabel4.icon")); // NOI18N
        jLabel7.setText(resourceMap.getString("jLabel7.text")); // NOI18N
        jLabel7.setName("jLabel7"); // NOI18N

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
                jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(jLabel3)
                                .addComponent(jLabel4)
                                .addComponent(jLabel5))
                        .addGap(56, 56, 56)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(jPanel1Layout.createSequentialGroup()
                                        .addComponent(jTextField3, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jLabel7))
                                .addGroup(jPanel1Layout.createSequentialGroup()
                                        .addComponent(jTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jLabel6))
                                .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 72, Short.MAX_VALUE)
                                .addComponent(powerOnButton)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(powerOffButton)
                        ))
                        .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
                jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(powerOnButton)
                                .addComponent(powerOffButton)
                                .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGap(21, 21, 21)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel3)
                                .addComponent(jTextField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel4)
                                .addComponent(jTextField2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel6))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel5)
                                .addComponent(jTextField3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel7))
                        .addContainerGap(25, Short.MAX_VALUE))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED),
                resourceMap.getString("jPanel2.border.title"), javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION, resourceMap.getFont("jPanel2.border.titleFont"),
                resourceMap.getColor("jPanel2.border.titleColor"))); // NOI18N
        jPanel2.setName("jPanel2"); // NOI18N
        jPanel2.setBackground(resourceMap.getColor("jPanel1.background")); // NOI18N

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        // Create default table model
        myTableModel = new javax.swing.table.DefaultTableModel(
                new Object [][] {
                        {null, null, null, null, null, null},
                        {null, null, null, null, null, null},
                        {null, null, null, null, null, null},
                        {null, null, null, null, null, null},
                        {null, null, null, null, null, null}
                },
                new String [] {
                        "Module", "Voltage", "Current", "Temperature", "Status", "State"
                }
        ) {
            Class[] types = new Class [] {
                    java.lang.String.class, java.lang.String.class, java.lang.String.class,
                    java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                    false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        };
        jTable1.setName("jTable1"); // NOI18N
        jTable1.setFont(resourceMap.getFont("jTable1.font")); // NOI18N
        jTable1.setForeground(resourceMap.getColor("jTextField2.foreground")); // NOI18N
        jTable1.setModel(myTableModel);
        TableCellRenderer renderer = new CustomTableCellRenderer();
        try
        {
            jTable1.setDefaultRenderer( Class.forName
                    ( "java.lang.String" ), renderer );
        }
        catch( ClassNotFoundException ex )
        {
            System.exit( 0 );
        }
        jScrollPane2.setViewportView(jTable1);

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
                jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel2Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 431, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
                jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(jPanel2Layout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 118, Short.MAX_VALUE)
                        .addContainerGap())
        );

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
                mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(mainPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(jLabel1)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jLabel2))
                                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 163, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addContainerGap())
        );
        mainPanelLayout.setVerticalGroup(
                mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(mainPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addGroup(mainPanelLayout.createSequentialGroup()
                                        .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                        .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGroup(mainPanelLayout.createSequentialGroup()
                                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                        .addComponent(jLabel2)
                                        .addComponent(jLabel1))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 291, Short.MAX_VALUE)))
                        .addContainerGap())
        );



        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setFont(resourceMap.getFont("fileMenu.font")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N
        fileMenu.setBackground(resourceMap.getColor("mainPanel.background")); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(WncHmiApp.class).getContext().getActionMap(WncHmiView.class, this);
        jMenuItem1.setAction(actionMap.get("showAddPlatformBox")); // NOI18N
        jMenuItem1.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.ALT_MASK));
        jMenuItem1.setFont(resourceMap.getFont("jMenuItem1.font")); // NOI18N
        jMenuItem1.setMnemonic('C');
        jMenuItem1.setText(resourceMap.getString("jMenuItem1.text")); // NOI18N
        jMenuItem1.setToolTipText(resourceMap.getString("jMenuItem1.toolTipText")); // NOI18N
        jMenuItem1.setName("jMenuItem1"); // NOI18N
        fileMenu.add(jMenuItem1);

        jSeparator2.setName("jSeparator2"); // NOI18N
        fileMenu.add(jSeparator2);

        jMenuItem2.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.ALT_MASK));
        jMenuItem2.setFont(resourceMap.getFont("jMenuItem2.font")); // NOI18N
        jMenuItem2.setForeground(resourceMap.getColor("jMenuItem2.foreground")); // NOI18N
        jMenuItem2.setMnemonic('D');
        jMenuItem2.setText(resourceMap.getString("jMenuItem2.text")); // NOI18N
        jMenuItem2.setToolTipText(resourceMap.getString("jMenuItem2.toolTipText")); // NOI18N
        jMenuItem2.setName("jMenuItem2"); // NOI18N
        fileMenu.add(jMenuItem2);

        jSeparator3.setName("jSeparator3"); // NOI18N
        fileMenu.add(jSeparator3);

//        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(WncHmiApp.class).getContext().getActionMap(WncHmiView.class, this);
        exitMenuItem.setAction(actionMap.get("Exit")); // NOI18N
        exitMenuItem.setAccelerator(null);
        exitMenuItem.setFont(resourceMap.getFont("exitMenuItem.font")); // NOI18N
        exitMenuItem.setForeground(resourceMap.getColor("exitMenuItem.foreground")); // NOI18N
        exitMenuItem.setIcon(resourceMap.getIcon("exitMenuItem.icon")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setFont(resourceMap.getFont("helpMenu.font")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setFont(resourceMap.getFont("aboutMenuItem.font")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        jSeparator1.setName("jSeparator1"); // NOI18N

        setComponent(mainPanel);
        setMenuBar(menuBar);
        
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton powerOnButton;
    private javax.swing.JButton powerOffButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JTable jTable1;
    private javax.swing.JTextField jTextField1;
    private javax.swing.JTextField jTextField2;
    private javax.swing.JTextField jTextField3;
    private javax.swing.JTree jTree1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    // End of variables declaration//GEN-END:variables

    private JDialog aboutBox;
    private JDialog addPlatformBox;

    /**
     *
     * @author gurjyan
     */
    class VisualizeCallback extends cMsgCallbackAdapter {

        /** on/off status of wiener crate*/
        private int crateIsOn;

        /** channel names*/
        private String[] wn = {"a","a","a","a","a"};

        /** channel set voltage*/
        private float[] wsv = new float[5];

        /** channel output voltage*/
        private float[] wv = new float[5];

        /** channel output current*/
        private float[] wi = new float[5];

        /** power module temperature status*/
        private int[] wt = new int[5] ;

        /** Channel on status*/
        private int[] wcs = new int[5];

        /** time in sec the fan tray was on*/
        private int activeSec;

        /** crate fan speed*/
        private int ftspeed;

        /** formater*/
        private DecimalFormat myFormatter = new DecimalFormat("##.###");


        @Override
        public void callback(cMsgMessage msg, Object userObject) {
            try {
                String type;
                String sender;
                String sseverity=null;
                String state;
                String expid = null;
                String session = null;
                String runtype = null;
                String codatype = null;
                String name = null;
                String message;
                int severity = 0;

                String t = new Date().toString();
                StringTokenizer sk = new StringTokenizer(msg.getSubject(), "/");

                myTableModel.setRowCount(0);

                if (sk.hasMoreTokens()) {
                    expid = sk.nextToken();
                }
                if (sk.hasMoreTokens()) {
                    session = sk.nextToken();
                }
                if (sk.hasMoreTokens()) {
                    runtype = sk.nextToken();
                }
                if (sk.hasMoreTokens()) {
                    codatype = sk.nextToken();
                }
                if (sk.hasMoreTokens()) {
                    name = sk.nextToken();
                }
                message = msg.getText();

                try{
                    if(msg.getPayloadItem("severity")!=null)
                        sseverity = msg.getPayloadItem("severity").getString();
                    if(msg.getPayloadItem("state")!=null)
                        state = msg.getPayloadItem("state").getString();

                    if(msg.getPayloadItem("WienerCrate")!=null)
                        crateIsOn = msg.getPayloadItem("WienerCrate").getInt();
                    if(msg.getPayloadItem("WienerStatus")!=null)
                        wcs = msg.getPayloadItem("WienerStatus").getIntArray();
                    if(msg.getPayloadItem("WienerVoltage")!=null)
                    wv = msg.getPayloadItem("WienerVoltage").getFloatArray();
                    if(msg.getPayloadItem("WienerCurrent")!=null)
                    wi = msg.getPayloadItem("WienerCurrent").getFloatArray();
                    if(msg.getPayloadItem("WienerTemperature")!=null)
                    wt = msg.getPayloadItem("WienerTemperature").getIntArray();
                    if(msg.getPayloadItem("WienerFanSpeed")!=null)
                    ftspeed = msg.getPayloadItem("WienerFanSpeed").getInt();
                    if(msg.getPayloadItem("WienerUpTime")!=null)
                    activeSec = msg.getPayloadItem("WienerUpTime").getInt();
                    if(msg.getPayloadItem("WienerModule")!=null)
                    wn = msg.getPayloadItem("WienerModule").getStringArray();
                    type = msg.getType();
                    sender = msg.getSender();

                    if(crateIsOn==1){
                        jTextField1.setText("On");
                    }
                    else{
                        jTextField1.setText("Off");
                    }
                    jTextField2.setText(Integer.toString(ftspeed));
                    jTextField3.setText(Integer.toString(activeSec));

//                    severity = Integer.parseInt(type.substring(type.lastIndexOf("/") + 1));
                } catch (NumberFormatException e) {
                    e.printStackTrace();
                }
//                System.out.println(" SEVERITY ....... "+sseverity);

                try {
                    for(int i= 0; i<5; i++){
//                        myTableModel.addRow(new Object[]{wn[i], swv[i], swi[i], wt[i],wcs[i]});
                        myTableModel.addRow(new Object[]{wn[i],myFormatter.format(wv[i]) ,myFormatter.format(wi[i]) , wt[i],wcs[i],sseverity});
                    }

                } catch (NullPointerException n) {
                }
            } catch (cMsgException ex) {
                Logger.getLogger(WncHmiView.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }


    //----------------------------------------------------------------------------
    /**
     *
     * @author gurjyan
     */
    class DalogMessageCallback extends cMsgCallbackAdapter {

        private String subject;
        private String expid,  session,  supervisor,  componentType,  componentName;

        @Override
        public void callback(cMsgMessage msg, Object userObject) {
            subject = msg.getSubject();
            StringTokenizer st = new StringTokenizer(subject, "/");
            if (st.hasMoreTokens()) {
                expid = st.nextToken();
                $expid = expid;
            }
            if (st.hasMoreTokens()) {
                session = st.nextToken();
            }
            if (st.hasMoreTokens()) {
                supervisor = st.nextToken();
            }
            if (st.hasMoreTokens()) {
                componentType = st.nextToken();
            }
            if (st.hasMoreTokens()) {
                componentName = st.nextToken();
            }

            if(!componentType.equalsIgnoreCase(rcConstants.wienercrateType)) return;

            if (expid != null) {
                if (!EXPIDS.containsKey(expid)) {
                    System.out.println("Error.... This can not be happening ");
                    return;
                } else {
                    // Get the expid object from the map
                    DLBExpid Expid = EXPIDS.get(expid);
                    // Get the list of all registered sessions for that expid
                    HashMap Sessions = Expid.getSessions();
                    // See if this message session is registered. If it not
                    if (session != null && !Sessions.containsKey(session)) {
                        // Create the DLBSession object
                        DLBSession Session = new DLBSession();
                        Session.setName(session);
                        addNodeToTheTree(expid, session);
                        if (supervisor != null) {
                            // Create DLBSupervisor object
                            DLBSupervisor Supervisor = new DLBSupervisor();
                            Supervisor.setName(supervisor);
//                            System.out.println("Debug ... " + session + " " + supervisor);

                            addNodeToTheTree(session, supervisor);
                            if (componentType != null) {
                                // Create DLBType object
                                DLBType Type = new DLBType();
                                Type.setName(componentType);
                                addNodeToTheTree(supervisor, componentType);
                                if (componentName != null) {
                                    Type.setComponents(componentName);
                                    addNodeToTheTree(componentType, componentName);
                                }
                                Supervisor.setComponentTypes(Type.getName(), Type);
                            }
                            Session.setSupervisors(Supervisor.getName(), Supervisor);
                        }
                        Expid.setSessions(Session.getName(), Session);
                        EXPIDS.remove(expid);
                        EXPIDS.put(expid, Expid);
                    } // Session is registered
                    else {
                        // Session is registered. Get the DLBSession object from the Sessions map.
                        DLBSession oldSession = (DLBSession) Sessions.get(session);
                        // See if supervisor is registered
                        if (supervisor != null) {
                            HashMap Supervisors = oldSession.getSupervisors();
                            // supervisor is not registered
                            if (!Supervisors.containsKey(supervisor)) {
                                // Create DLBSupervisor object
                                DLBSupervisor Supervisor = new DLBSupervisor();
                                Supervisor.setName(supervisor);
                                addNodeToTheTree(session, supervisor);
                                if (componentType != null) {
                                    // Create DLBType object
                                    DLBType Type = new DLBType();
                                    Type.setName(componentType);
                                    addNodeToTheTree(supervisor, componentType);
                                    if (componentName != null) {
                                        Type.setComponents(componentName);
                                        addNodeToTheTree(componentType, componentName);
                                    }
                                    Supervisor.setComponentTypes(Type.getName(), Type);
                                }
                                oldSession.setSupervisors(Supervisor.getName(), Supervisor);
                            } // supervisor is registered
                            else {
                                // Get DLBSupervisor object from the Scs map.
                                DLBSupervisor oldSupervisor = (DLBSupervisor) Supervisors.get(supervisor);
                                // See if componentType is registered
                                if (componentType != null) {
                                    HashMap comTypes = oldSupervisor.getComponentTypes();
                                    // componentType is not registered
                                    if (!comTypes.containsKey(componentType)) {
                                        // Create DLBType object
                                        DLBType Type = new DLBType();
                                        Type.setName(componentType);
                                        addNodeToTheTree(supervisor, componentType);
                                        if (componentName != null) {
                                            Type.setComponents(componentName);
                                            addNodeToTheTree(componentType, componentName);
                                        }
                                        oldSupervisor.setComponentTypes(Type.getName(), Type);
                                    } // componentType is registered
                                    else {
                                        // get the registered DLBType object in the map
                                        DLBType oldType = (DLBType) comTypes.get(componentType);
                                        //see if the componentName is registered with this type
                                        if (componentName != null) {
                                            // get the list of the names with this type
                                            java.util.List comNames = oldType.getComponents();
                                            // see if the name is registered
                                            if (!comNames.contains(componentName)) {
                                                // if it is not
                                                oldType.setComponents(componentName);
                                                addNodeToTheTree(componentType, componentName);
                                                oldSupervisor.setComponentTypes(oldType.getName(), oldType);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Inner class for custom cell rendering
     * @author gurjyan
     */
    public class CustomTableCellRenderer extends DefaultTableCellRenderer {

        @Override
        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int column) {

            Component cell = super.getTableCellRendererComponent(table, value,
                    isSelected, hasFocus, row, column);
            if (value instanceof String) {
                if (value.equals("0")) {
                    cell.setBackground(Color.RED);
                }
                else if(value.equals("noSuchInstance")){
                    cell.setBackground(Color.YELLOW);
                }
                else if(value.toString().contains("WARN")){
                    cell.setBackground(Color.YELLOW);
                }
                else {
                    cell.setBackground(new Color(230, 230, 230));
                }
            }
            return cell;
        }
    }


}
