/*----------------------------------------------------------------------------*
 *  Copyright (c) 2001        Southeastern Universities Research Association, *
 *                            Thomas Jefferson National Accelerator Facility  *
 *                                                                            *
 *    This software was developed under a United States Government license    *
 *    described in the NOTICE file included as part of this distribution.     *
 *                                                                            *
 *    Author:  Vardan Gyurjyan                                                *
 *             gurjyan@jlab.org                  Jefferson Lab, MS-12H        *
 *             Phone: (757) 269-5879             12000 Jefferson Ave.         *
 *             Fax:   (757) 269-5800             Newport News, VA 23606       *
 *                                                                            *
 *----------------------------------------------------------------------------*/
/**
 * CODAPlatformAdmin Ontology class
 *
 * @author Vardan Gyurjyan - JLAB DAQ
 * @version $Date: 2005/09/16 18:30:30 $ $Revision: 1.3 $
 */

package org.jlab.coda.runcontrol.ontologies.CodaPlatform;

import rca.content.onto.BasicOntology;
import rca.content.onto.Ontology;
import rca.content.onto.OntologyException;
import rca.content.schema.AgentActionSchema;
import rca.content.schema.ConceptSchema;
import rca.content.schema.ObjectSchema;
import rca.content.schema.PrimitiveSchema;

public class CodaPlatformAdminOntology extends Ontology {
    /**
     * A symbolic constant, containing the name of this ontology.
     */
    public static final String NAME = "CodaPlatform-Ontology";

    // VOCABULARY
    // Concepts supported by Hepatic agent
    public static final String CODA_AGENT           = "Coda-Agent";
//slots for CODA_AGENT concept
    public static final String CODA_AGENT_NAME      = "Name";
    public static final String CODA_AGENT_NNAME     = "NName";
    public static final String CODA_AGENT_TYPE      = "Type";
    public static final String AGENT_JAVA_CLAS      = "Clas";

    public static final String CODA_CONTAINER       = "Coda-Container";
//slots for CODA_CONTAINER concept
    public static final String CODA_CONTAINER_NAME  = "Name";
    public static final String CODA_CONTAINER_NODE  = "Node";
    public static final String CODA_CONTAINER_TYPE  = "Type";

    public static final String CODA_AP_TYPE         = "Coda-AP-Type";
//slots for CODA_AP_TYPE concept
    public static final String MAIN                 = "Main";
    public static final String DAQ_ROC              = "DaqRoc";
    public static final String SLOW_CONTROL_ROC     = "SlcRoc";
    public static final String EVENT_BUILDER        = "Eb";
    public static final String EVENT_RECORDER       = "Er";
    public static final String EVENT_TRANSFER       = "Et";
    public static final String GENERIC              = "Generic";
    public static final String PLEXUS               = "Plexus";

    // Actions supported by Hepatic agent
    public static final String STARTREMOTECONTAINER = "Start-Remote-Container";
//arguments in the action
    public static final String FILENAME             = "Filename";

    public static final String STARTREMOTECONTAINERAGENTS = "Startremotecontaineragents";
//arguments in the action
    public static final String STRCA_CONTAINERNODE        = "Strca_Containernode";
    public static final String STRCA_AGENTNAMELIST        = "Strca_Agentnamelist";
    public static final String STRCA_AGENTCLASSLIST       = "Strca_Agentclasslist";

    public static final String STOPCONTAINER        = "Stop-Container";
//arguments in the action
    public static final String STP_CONTAINER        = "Container";
    public static final String STP_AGENTS           = "Agents";

    public static final String STARTCODAAGENT       = "Start-Coda-Agent";
//arguments in the action
    public static final String ST_AGENT             = "Agent";
    public static final String ST_CONTAINER         = "Container";

    public static final String STOPCODAAGENT        = "Stop-Coda-Agent";
//arguments in the action
    public static final String SP_AGENT             = "Agent";
    public static final String SP_CONTAINER         = "Container";

    public static final String SUSPENDCODAAGENT     = "Suspend-Coda-Agent";
//arguments in the action
    public static final String SS_AGENT             = "Agent";
    public static final String SS_CONTAINER         = "Container";

    public static final String RESUMECODAAGENT      = "Resume-Coda-Agent";
//arguments in the action
    public static final String RS_AGENT             = "Agent";
    public static final String RS_CONTAINER         = "Container";

    public static final String CLONCODAAGENT        = "Clone-Coda-Agent";
//arguments in the action
    public static final String CL_AGENT             = "Agent";
    public static final String CL_CONTAINER         = "Container";

    public static final String MOVECODAAGENT        = "Move-Coda-Agent";
//arguments in the action
    public static final String MV_AGENT             = "Agent";
    public static final String MV_CONTAINER         = "Container";

    public static final String GETPLATFORMMAP       = "Get-Platform-Map";
//argument in the action
    public static final String PL_MAP               = "PLMap";

//The singleton instance of this ontology
    private static Ontology theInstance = new CodaPlatformAdminOntology();

    /**
     * This method grants access to the singleton instance of the
     * CodaPaltformAdmin ontology.
     *
     * @return An <code>Ontology</code> object, containing the concepts
     * of the CodaPaltformAdmin.
     */
    public static Ontology getInstance() {
        return theInstance;
    }

//    Private constructor
    private CodaPlatformAdminOntology() {
//         CodaPlatformAdminOntology ontology extends basic ontology
        super(NAME, BasicOntology.getInstance());

        try {
            add(new ConceptSchema(CODA_AGENT), CodaAgent.class);
            add(new ConceptSchema(CODA_CONTAINER), CodaContainer.class);
            add(new ConceptSchema(CODA_AP_TYPE), CodaAgentType.class);
            add(new AgentActionSchema(STARTREMOTECONTAINER), StartRemoteContainer.class);
            add(new AgentActionSchema(STARTREMOTECONTAINERAGENTS), StartRemoteContainerAgents.class);
            add(new AgentActionSchema(STOPCONTAINER), StopContainer.class);
            add(new AgentActionSchema(STARTCODAAGENT), StartCodaAgent.class);
            add(new AgentActionSchema(STOPCODAAGENT), StopCodaAgent.class);
            add(new AgentActionSchema(SUSPENDCODAAGENT), SuspendCodaAgent.class);
            add(new AgentActionSchema(RESUMECODAAGENT), ResumeCodaAgent.class);
            add(new AgentActionSchema(CLONCODAAGENT), CloneCodaAgent.class);
            add(new AgentActionSchema(MOVECODAAGENT), MoveCodaAgent.class);
            add(new AgentActionSchema(GETPLATFORMMAP), GetPlatformMap.class);

//             Structure of the schema for the Coda-Agent concept
            ConceptSchema cs = (ConceptSchema) getSchema(CODA_AGENT);
            cs.add(CODA_AGENT_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            cs.add(CODA_AGENT_NNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(AGENT_JAVA_CLAS, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(CODA_AGENT_TYPE, (ConceptSchema) getSchema(CODA_AP_TYPE),ObjectSchema.OPTIONAL);

//             Structure of the schema for the Coda-Container concept
            cs = (ConceptSchema) getSchema(CODA_CONTAINER);
            cs.add(CODA_CONTAINER_NAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(CODA_CONTAINER_NODE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(CODA_CONTAINER_TYPE, (ConceptSchema) getSchema(CODA_AP_TYPE), ObjectSchema.OPTIONAL);

//             Structure of the schema for the Coda-AP_TYPE concept
            cs = (ConceptSchema) getSchema(CODA_AP_TYPE);
            cs.add(MAIN, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(DAQ_ROC, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(SLOW_CONTROL_ROC, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(EVENT_BUILDER, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(EVENT_RECORDER, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(EVENT_TRANSFER, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(GENERIC, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);
            cs.add(PLEXUS, (PrimitiveSchema) getSchema(BasicOntology.BOOLEAN), ObjectSchema.OPTIONAL);

//            Sturcture of the schema for the Start-remote-container action
            AgentActionSchema as = (AgentActionSchema) getSchema(STARTREMOTECONTAINER);
            as.add(FILENAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

            as = (AgentActionSchema) getSchema(STARTREMOTECONTAINERAGENTS);
            as.add(STRCA_CONTAINERNODE,(PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(STRCA_AGENTNAMELIST,(PrimitiveSchema) getSchema(BasicOntology.STRING), 0, ObjectSchema.UNLIMITED);
            as.add(STRCA_AGENTCLASSLIST,(PrimitiveSchema) getSchema(BasicOntology.STRING), 0, ObjectSchema.UNLIMITED); 

//            Sturcture of the schema for the Stop-Remote-Container action
            as = (AgentActionSchema) getSchema(STOPCONTAINER);
            as.add(STP_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER));
            as.add(STP_AGENTS, (ConceptSchema) getSchema(CODA_AGENT), 0, ObjectSchema.UNLIMITED);

//            Sturcture of the schema for the Sart-Coda_Agent action
            as = (AgentActionSchema) getSchema(STARTCODAAGENT);
            as.add(ST_AGENT, (ConceptSchema) getSchema(CODA_AGENT));
            as.add(ST_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER));

//            Sturcture of the schema for the Stop-Coda_Agent action
            as = (AgentActionSchema) getSchema(STOPCODAAGENT);
            as.add(SP_AGENT, (ConceptSchema) getSchema(CODA_AGENT));
            as.add(SP_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER), ObjectSchema.OPTIONAL);

//            Sturcture of the schema for the Suspend-Coda_Agent action
            as = (AgentActionSchema) getSchema(SUSPENDCODAAGENT);
            as.add(SS_AGENT, (ConceptSchema) getSchema(CODA_AGENT));
            as.add(SS_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER), ObjectSchema.OPTIONAL);

//            Sturcture of the schema for the Resume-Coda_Agent action
            as = (AgentActionSchema) getSchema(RESUMECODAAGENT);
            as.add(RS_AGENT, (ConceptSchema) getSchema(CODA_AGENT));
            as.add(RS_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER), ObjectSchema.OPTIONAL);

//            Sturcture of the schema for the Clone_Coda_Agent action
            as = (AgentActionSchema) getSchema(CLONCODAAGENT);
            as.add(CL_AGENT, (ConceptSchema) getSchema(CODA_AGENT));
            as.add(CL_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER), ObjectSchema.OPTIONAL);

//            Sturcture of the schema for the Move_Coda_Agent action
            as = (AgentActionSchema) getSchema(MOVECODAAGENT);
            as.add(MV_AGENT, (ConceptSchema) getSchema(CODA_AGENT));
            as.add(MV_CONTAINER, (ConceptSchema) getSchema(CODA_CONTAINER));

//            Sturcture of the schema for the Get_Platform_Map action
            as = (AgentActionSchema) getSchema(GETPLATFORMMAP);
            as.add(PL_MAP, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
        }
        catch (OntologyException oe) {
            oe.printStackTrace();
        }
    }
}



