/*----------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
/**
 CODACommunications Ontology class
 @author Vardan Gyurjyan - JLAB DAQ
 @version $Date: 2005/09/16 18:35:55 $ $Revision: 1.4 $
 */

package org.jlab.coda.runcontrol.ontologies.CodaRunControl;

import rca.content.onto.BasicOntology;
import rca.content.onto.Ontology;
import rca.content.onto.OntologyException;
import rca.content.schema.AgentActionSchema;
import rca.content.schema.ConceptSchema;
import rca.content.schema.ObjectSchema;
import rca.content.schema.PrimitiveSchema;

public class  CodaRunControlOntology extends Ontology {

    // A symbolic constant, containing the name of this ontology.
    public static final String NAME = "CodaRunControlOntology-Ontology";

// Vocabulay of the cool language

    // ================= Concepts of Control and slots ======================================
    public static final String CONTROL                                = "Control";
    public static final String CONTROL_HASCOMPONENT                   = "Control_Components";
    public static final String CONTROL_HASSYSTEM                      = "Control_Systems";
    public static final String CONTROL_HASSTATE                       = "Control_States";
    public static final String CONTROL_HASRULE                        = "Control_Rules";
    public static final String CONTROL_HASCODAOPTION                  = "Control_Codaoption";
    public static final String CONTROL_HASANAOPTION                   = "Control_Anaoption";
    public static final String CONTROL_HASEROPTION                    = "Control_Eroption";
    public static final String CONTROL_HASEBOPTION                    = "Control_Eboption";
    public static final String CONTROL_HASROCOPTION                   = "Control_Rocoption";
    public static final String CONTROL_HASSLCOPTION                   = "Control_Slcoption";
    public static final String CONTROL_HASTSOPTION                    = "Control_Tsoption";
    public static final String CONTROL_HASETOPTION                    = "Control_Etoption";

    // ================= Concepts of System and slots ======================================
    public static final String SYSTEM                                 = "System";
    public static final String SYSTEM_HASSMSAGENT                     = "System_Supervisor";
    public static final String SYSTEM_ISCONTROLLABLE                  = "System_Iscontrollable";
    public static final String SYSTEM_HASREPORTINGINTERVAL            = "System_Reporinginterval";
    public static final String SYSTEM_HASSTATE                        = "System_State";

    // ================= Concepts of CoolWhile and slots ======================================
    public static final String WHILE                                  = "While";
    public static final String WHILE_STATE                            = "While_State";

    // ================= Concepts of EndWhile and slots ======================================
    public static final String ENDWHILE                               = "Endwhile";
    public static final String ENDWHILE_VALUE                         = "Endwhile_Value";

    // ================= Concepts of CoolIf and slots ======================================
    public static final String IF                                     = "If";
    public static final String IF_STATE                               = "If_State";
    public static final String IF_CODASTATE                           = "If_Codastate";

    // ================= Concepts of CoolAndIf and slots ======================================
    public static final String ANDIF                                  = "Andif";
    public static final String ANDIF_STATE                            = "Andif_State";

    // ================= Concepts of CoolOrIf and slots ======================================
    public static final String ORIF                                   = "Orif";
    public static final String ORIF_STATE                             = "Orif_State";

    // ================= Concepts of CoolMove and slots ======================================
//    public static final String MOVETO                                 = "Moveto";
//    public static final String MOVETO_STATE                           = "Moveto_State";

    // ================= Concepts of CoolExecute and slots ======================================
    public static final String EXECUTE                                = "Execute";
    public static final String EXECUTE_PROCESS                        = "Execute_Process";

    // ================= Concepts of CoolWait and slots ======================================
    public static final String WAIT                                   = "Wait";
    public static final String WAIT_SEC                               = "Wait_Sec";

    // ================= Concepts of CoolLoop and slots ======================================
    public static final String LOOP                                   = "Loop";
    public static final String LOOP_PARAM                             = "Loop_Param";

    // ================= Concepts of EndLoop and slots ======================================
    public static final String ENDLOOP                                = "Endloop";
    public static final String ENDLOOP_VALUE                          = "Endloop_Value";


    // ================= Concepts of ElseIf and slots ======================================
    public static final String ELSEIF                                 = "Elseif";
    public static final String ELSEIF_STATE                           = "Elseif_State";

    // ================= Concepts of Else and slots ======================================
    public static final String ELSE                                   = "Else";
    public static final String ELSE_VALUE                             = "Else_Value";

    // ================= Concepts of EndIf and slots ======================================
    public static final String ENDIF                                  = "Endif";
    public static final String ENDIF_VALUE                            = "Endif_Value";

    // ================= Concepts of Component and slots ====================================
    public static final String COMPONENT                              = "Component";
    public static final String COMPONENT_HASNAME                      = "Component_Name";
    public static final String COMPONENT_HASPRIORITY                  = "Component_Priority";
    public static final String COMPONENT_HASTYPE                      = "Component_Type";
    public static final String COMPONENT_HASIPC                       = "Component_Ipc";
    public static final String COMPONENT_HASCODE                      = "Component_Code";
    public static final String COMPONENT_HASSESSION                   = "Component_Session";
    public static final String COMPONENT_HASRUNTYPE                   = "Component_Runtype";
    public static final String COMPONENT_HASNODE                      = "Component_Node";
    public static final String COMPONENT_HASEXTERNALPROCESS           = "Component_Processes";
    public static final String COMPONENT_REPRESENTSCODA2CLIENT        = "Component_Representscoda2client";
    public static final String COMPONENT_HASLOGINGLEVEL               = "Component_Loginglevel";
    public static final String COMPONENT_REPRESENTSIPADDRESS          = "Component_Representsipaddress";
    public static final String COMPONENT_HASOPTION                    = "Component_Option";
    public static final String COMPONENT_CONTROLSCOMPONENT            = "Component_Controls";
    public static final String COMPONENT_REPRESENTSOBJECTTYPE         = "Component_Objecttype";
    public static final String COMPONENT_FILTERSDALOG                 = "Component_Filtersdalog";
    public static final String COMPONENT_DALOGTHROTTLELEVEL           = "Component_DalogThrottlelevel";
    public static final String COMPONENT_REPORTINGINTERVAL            = "Component_Reportinginterval";
    public static final String COMPONENT_REPORTSTO                    = "Component_Reportsto";
    public static final String COMPONENT_CLIENTHOST                   = "Component_ClientHost";
    public static final String COMPONENT_CLIENTPORT                   = "Component_ClientPort";


    // ================= Concepts of Rule and slots ====================================
    public static final String RULE                                 = "Rule";
    public static final String RULE_ITERATION                       = "Rule_Iteration";
    public static final String RULE_IF                              = "Rule_If";
    public static final String RULE_WHILE                           = "Rule_While";
    public static final String RULE_ENDWHILE                        = "Rule_Endwhile";
    public static final String RULE_ANDIF                           = "Rule_Andif";
    public static final String RULE_ORIF                            = "Rule_Orif";
    public static final String RULE_ENDIF                           = "Rule_Endif";
    public static final String RULE_ELSE                            = "Rule_Else";
    public static final String RULE_ELSEIF                          = "Rule_Elseif";
    public static final String RULE_THENMOVETO                      = "Rule_Thenmoveto";
    public static final String RULE_ELSEMOVETO                      = "Rule_Elsemoveto";
    public static final String RULE_ELSEIFMOVETO                    = "Rule_Elseifmoveto";
    public static final String RULE_WHILEMOVETO                     = "Rule_Whilemoveto";
    public static final String RULE_THENEXECUTEPROCESS              = "Rule_Thenexecute";
    public static final String RULE_ELSEEXECUTEPROCESS              = "Rule_Elseexecute";
    public static final String RULE_ELSEIFEXECUTEPROCESS            = "Rule_Elseifexecute";
    public static final String RULE_WHILEEXECUTEPROCESS             = "Rule_Whileexecute";
    public static final String RULE_LOOP                            = "Rule_Loop";
    public static final String RULE_ENDLOOP                         = "Rule_Endloop";
    public static final String RULE_WAIT                            = "Rule_Wait";


    // ================= Concepts of State and slots ====================================
    public static final String COOLSTATE                              = "CoolState";
    public static final String STATE_ACHIEVEDTHROUGH                  = "State_Achievedthrough";
    public static final String STATE_ISCRITICAL                       = "State_Iscritical";
    public static final String STATE_HASSTATENAME                     = "State_Name";
    public static final String STATE_HASSEVERITY                      = "State_Severity";
    public static final String STATE_ISFOR                            = "State_Isfor";
    public static final String STATE_DEPENDSON                        = "State_Dependson";

    // ================= Concepts of Process and slots ======================================
    public static final String PROCESS                                = "Process";
    public static final String PROCESS_ISPARTOFSTATE                  = "Process_Ispartofstate";
    public static final String PROCESS_HASCOMMANDNAME                 = "Process_Commandname";
    public static final String PROCESS_HASPROCESSIPC                  = "Process_Ipc";
    public static final String PROCESS_HASTIMEOUT                     = "Process_Timeout";
    public static final String PROCESS_HASPERIODICITY                 = "Process_Periodicity";
    public static final String PROCESS_HASDELAY                       = "Process_Delay";
    public static final String PROCESS_HASINPUTDATA                   = "Process_Inputdata";
    public static final String PROCESS_HASOUTPUTDATA                  = "Process_Outputdata";
    public static final String PROCESS_SUBSCRIBESTO                   = "Process_Subscribesto";
    public static final String PROCESS_SENDSTO                        = "Process_Sendsto";
    public static final String PROCESS_ISSTATETRANSITION              = "Process_Isstatetransition";
    public static final String PROCESS_SETS                           = "Process_Sets";
    public static final String PROCESS_GETS                           = "Process_Gets";
    public static final String PROCESS_TRAPS                          = "Process_Traps";
    public static final String PROCESS_STOP                           = "Process_Stop";
    public static final String PROCESS_HASERRORSTATUS                 = "Process_Errorstatus";
    public static final String PROCESS_HASERRORINDEX                  = "Process_Errorindex";
    public static final String PROCESS_HASEREQUESTID                  = "Process_Requestid";

    // ================= Concepts of Publisher and slots ====================================
    public static final String PUBLISHER                              = "Publisher";
    public static final String PUBLISHER_HASUDL                       = "Publisher_Udl";
    public static final String PUBLISHER_HASSENDSUBJECT               = "Publisher_Sendsubject";
    public static final String PUBLISHER_HASSENDTYPE                  = "Publisher_Sendtype";
    public static final String PUBLISHER_HASSUBSCRIBESUBJECT          = "Publisher_Subscribesubject";
    public static final String PUBLISHER_HASSUBSCRIBETYPE             = "Publisher_Subscribetype";

    // ================= Concepts of Option and slots ====================================
    public static final String OPTION                                 = "Option";
    public static final String OPTION_HASSCHEDULER                    = "Option_Scheduler";
    public static final String OPTION_HASDATAFILE                     = "Option_Datafile";
    public static final String OPTION_HASCODA2COMPONENT               = "Option_Hascoda2component";
    public static final String OPTION_HASAUTOSTART                    = "Option_Hasautostart";
    public static final String OPTION_HASSTARTRUN                     = "Option_Hasstartrun";
    public static final String OPTION_HASDATALIMIT                    = "Option_Datalimit";
    public static final String OPTION_HASEVENTLIMIT                   = "Option_Eventlimit";
    public static final String OPTION_HASCONFIGFILE                   = "Option_Configfile";
    public static final String OPTION_HASDATAFILEPREFIX               = "Option_Datafileprefix";
    public static final String OPTION_HASCONFIGSTRING                 = "Option_Configstring";
    public static final String OPTION_CONFIGISXML                     = "Option_Configisxml";
    public static final String OPTION_CONFIGISMIB                     = "Option_Configismib";

    // ================= Concepts of Option and slots ====================================
    public static final String COOLSCHEDULER                              = "Scheduler";
    public static final String COOLSCHEDULER_HASEXTERNALRUNCONFIGPROCESS  = "Scheduler_ProcessList";
    public static final String COOLSCHEDULER_HASSTOPAFTEREVTCOUNT         = "Scheduler_StopafterList";
    public static final String COOLSCHEDULER_HASPAUSEAFTEREVTCOUNT        = "Scheduler_PauseafterList";

    // ================= Concepts of Node and slots ====================================
    public static final String NODE                                   = "Node";
    public static final String NODE_HASNODENAME                       = "Node_Name";
    public static final String NODE_HASIP                             = "Node_Ip";
    public static final String NODE_ISLOCAL                           = "Node_Local";
    public static final String NODE_HASUSERNAME                       = "Node_User";
    public static final String NODE_HASPASSWORD                       = "Node_Passwd";

    // ================= Concepts of Data and slots ====================================
    public static final String DATA                                   = "Data";
    public static final String DATA_HASTIMESTAMP                      = "Data_Timestamp";
    public static final String DATA_HASDATATYPE                       = "Data_Type";
    public static final String DATA_HASCARDINALITY                    = "Data_Cardinality";
    public static final String DATA_HASSEMANTICS                      = "Data_Semantics";
    public static final String DATA_HASCONTENT                        = "Data_Content";

    // ==================== Agent platform run control specific internal concepts ==============
    //         Concept STATUSQUANT and slots
    public static final String STATUSQUANT                  = "Statusquant";
    public static final String STATUSQUANT_STATUS           = "Statusquant_Status";
    public static final String STATUSQUANT_SEVERITY         = "Statusquant_Severity";
    public static final String STATUSQUANT_PRODUCER         = "Statusquant_Producer";
    public static final String STATUSQUANT_TIMESTAMP        = "Statusquant_Timestamp";
    public static final String STATUSQUANT_TRANSITION       = "Statusquant_Transition";
    //         Concept DATAQUANT and slots
    public static final String DATAQUANT                    = "Dataquant";
    public static final String DATAQUANT_DATA               = "Dataquant_Data";
    public static final String DATAQUANT_PRODUCER           = "Dataquant_Producer";
    public static final String DATAQUANT_PROCESS            = "Dataquant_Process";
    public static final String DATAQUANT_PROCESSSTATE       = "Dataquant_Processstate";
    public static final String DATAQUANT_TIMESTAMP          = "Dataquant_Timestamp";
    //         Concept Receiver and slot
    public static final String RECEIVER                     = "Receiver";
    public static final String RECEIVER_COMPONENT           = "Receiver_Component";
    //         Concept Producer and slot
    public static final String PRODUCER                     = "Producer";
    public static final String PRODUCER_COMPONENT           = "Producer_Component";
    //         Concept TimeStamp and slot
    public static final String TIMESTAMP                    = "Timestamp";
    public static final String TIMESTAMP_VALUE              = "Timestamp_Value";
    //         Concept SetPreConfigure and slots
    public static final String SETPRECONFIGURE              = "Setpreconfigure";
    public static final String SETPRECONFIGURE_PROCESS      = "Processes";
    //         Concept SetConfigure and slots
    public static final String SETCONFIGURE                 = "Setconfigure";
    public static final String SETCONFIGURE_PROCESS         = "Processes";
    //         Concept SetPostConfigure and slots
    public static final String SETPOSTCONFIGURE             = "Setpostconfigure";
    public static final String SETPOSTCONFIGURE_PROCESS     = "Processes";
    //         Concept SetPreDownload and slots
    public static final String SETPREDOWNLOAD               = "Setpredownload";
    public static final String SETPREDOWNLOAD_PROCESS       = "Processes";
    //         Concept SetDownload and slots
    public static final String SETDOWNLOAD                  = "Setdownload";
    public static final String SETDOWNLOAD_PROCESS          = "Processes";
    //         Concept SetPostDownload and slots
    public static final String SETPOSTDOWNLOAD              = "Setpostdownload";
    public static final String SETPOSTDOWNLOAD_PROCESS      = "Processes";
    //         Concept SetPrePrestart and slots
    public static final String SETPREPRESTART               = "Setpreprestart";
    public static final String SETPREPRESTART_PROCESS       = "Processes";
    //         Concept SetPrestart and slots
    public static final String SETPRESTART                  = "Setprestart";
    public static final String SETPRESTART_PROCESS          = "Processes";
    //         Concept SetPostPrestart and slots
    public static final String SETPOSTPRESTART              = "Setpostprestart";
    public static final String SETPOSTPRESTART_PROCESS      = "Processes";
    //         Concept SetPreGo and slots
    public static final String SETPREGO                     = "Setprego";
    public static final String SETPREGO_PROCESS             = "Processes";
    //         Concept SetGo and slots
    public static final String SETGO                        = "Setgo";
    public static final String SETGO_PROCESS                = "Processes";
    //         Concept SetReset and slots
    public static final String SETRESET                     = "Setreset";
    public static final String SETRESET_PROCESS             = "Processes";
    //         Concept SetPostGo and slots
    public static final String SETPOSTGO                    = "Setpostgo";
    public static final String SETPOSTGO_PROCESS            = "Processes";
    //         Concept SetPreGo and slots
    public static final String SETPREEND                    = "Setpreend";
    public static final String SETPREEND_PROCESS            = "Processes";
    //         Concept SetGo and slots
    public static final String SETEND                       = "Setend";
    public static final String SETEND_PROCESS               = "Processes";
    //         Concept SetPostGo and slots
    public static final String SETPOSTEND                   = "Setpostend";
    public static final String SETPOSTEND_PROCESS           = "Processes";
    //         Concept SetInPause and slots
    public static final String SETINPAUSE                    = "Setinpause";
    public static final String SETINPAUSE_PROCESS            = "Processes";
    //         Concept SetInResume and slots
    public static final String SETINRESUME                   = "Setinresume";
    public static final String SETINRESUME_PROCESS           = "Processes";

    //         Vocabulay of the actions
    //         Action Codainit and slots
    public static final String CODAINIT                       = "Codainit";
    public static final String CODAINIT_CLIENTHOST            = "Codainit_Clienthost";
    public static final String CODAINIT_CLIENTPORT            = "Codainit_Clientport";
    //         Action Differentiate and slots
    public static final String DIFFERENTIATE                  = "Differentiate";
    public static final String DIFFERENTIATE_COMPONENT        = "Differentiate_Component";
    public static final String DIFFERENTIATE_OPTION           = "Differentiate_Option";
    //         Action CheckDifferentiate and slots
    public static final String CHECKDIFFERENTIATE             = "Checkdifferentiate";
    //         Action AcceptEnding and slots
    public static final String ACCEPTENDING                   = "Acceptending";
    //         Action SOPREPORTING and slots
    public static final String STOPREPORTING                  = "Stopreporting";
    //         Action ExecuteCommand and slots
    public static final String EXECUTECOMMAND                 = "Executecommand";
    public static final String EXECUTECOMMAND_PROCESS         = "Processes";
    //         Action StopMonitoring and slots
    public static final String STOPMONITORING                 = "Stopmonitoring";
    public static final String STOPMONITORING_PROCESSNAME     = "Stopmonitoring_Processname";
    public static final String STOPMONITORING_AGENTNAME       = "Stopmonitoring_Agentname";
    //         Action ResumeMonitoring and slots
    public static final String RESUMEMONITORING               = "Resumemonitoring";
    public static final String RESUMEMONITORING_PROCESSNAME   = "Resumemonitoring_Processname";
    public static final String RESUMEMONITORING_AGENTNAME     = "Resumemonitoring_Agentname";
    //         Action KillMonitoring and slots
    public static final String KILLMONITORING                 = "Killmonitoring";
    public static final String KILLMONITORING_PROCESSNAME     = "Killmonitoring_Processname";
    public static final String KILLMONITORING_AGENTNAME       = "Killmonitoring_Agentname";
    //         Action AcceptData and slots
    public static final String ACCEPTDATA                     = "Acceptdata";
    public static final String ACCEPTDATA_DATA                = "Acceptdata_Data";
    public static final String ACCEPTDATA_PRODUCER            = "Acceptdata_Producer";
    public static final String ACCEPTDATA_PROCESS             = "Acceptdata_Process";
    public static final String ACCEPTDATA_PROCESSSTATE        = "Acceptdata_Processstate";
    public static final String ACCEPTDATA_TIMESTAMP           = "Acceptdata_Timestamp";
    //         Action AcceptGroupData and slots
    public static final String ACCEPTGROUPDATA                = "Acceptgroupdata";
    public static final String ACCEPTGROUPDATA_DATA           = "Acceptgroupdata_Data";
    //         Action AcceptTsData and slots
    public static final String ACCEPTTSDATA                   = "Accepttsdata";
    public static final String ACCEPTTSDATA_DATA              = "Accepttsdata_Data";
    public static final String ACCEPTTSDATA_PRODUCER          = "Accepttsdata_Producer";
    public static final String ACCEPTTSDATA_TIMESTAMP         = "Accepttsdata_Timestamp";
    //         Action AcceptStatus and slots
    public static final String ACCEPTSTATUS                   = "Acceptstatus";
    public static final String ACCEPTSTATUS_STATUS            = "Acceptstatus_Status";
    public static final String ACCEPTSTATUS_SEVERITY          = "Acceptstatus_Severity";
    public static final String ACCEPTSTATUS_PRODUCER          = "Acceptstatus_Producer";
    public static final String ACCEPTSTATUS_TIMESTAMP         = "Acceptstatus_Timestamp";
    public static final String ACCEPTSTATUS_TRANSITION        = "Acceptstatus_Transition";
    //         Action AcceptGroupStatus and slots
    public static final String ACCEPTGROUPSTATUS              = "Acceptgroupstatus";
    public static final String ACCEPTGROUPSTATUS_STATUS       = "Acceptgroupstatus_Status";
    //         Action AcceptSTATE and slots
    public static final String ACCEPTSTATE                    = "Acceptstate";
    public static final String ACCEPTSTATE_STATE              = "Acceptstate_State";
    public static final String ACCEPTSTATE_SEVERITY           = "Acceptstate_Severity";
    public static final String ACCEPTSTATE_PRODUCER           = "Acceptstate_Producer";
    public static final String ACCEPTSTATE_TIMESTAMP          = "Acceptstate_Timestamp";
    //         Action AcceptComponent and slots
    public static final String ACCEPTCOMPONENT                = "Acceptcomponent";
    public static final String ACCEPTCOMPONENT_COMPONENT      = "Acceptcomponent_Component";

    //         Action AcceptSystem and slots
    public static final String ACCEPTSYSTEM                   = "Acceptsystem";
    public static final String ACCEPTSYSTEM_SYSTEM            = "Acceptsystem_System";

    //         Action AcceptStates and slots
    public static final String ACCEPTSTATES                   = "Acceptstates";
    public static final String ACCEPTSTATES_LIST              = "Acceptstates_List";

    //         Action AcceptRules and slots
    public static final String ACCEPTRULES                    = "Acceptrules";
    public static final String ACCEPTRULES_LIST               = "Acceptrules_List";

    //         Action PreConfigure and slots
    public static final String PRECONFIGURE                   = "Preconfigure";
    public static final String PRECONFIGURE_TIMEOUT           = "Timeout";
    public static final String PRECONFIGURE_SESSION           = "Preconfigure_Session";
    public static final String PRECONFIGURE_RUNTYPE           = "Preconfigure_Runtype";
    //         Action Configure and slots
    public static final String CONFIGURE                      = "Configure";
    public static final String CONFIGURE_TIMEOUT              = "Timeout";
    public static final String CONFIGURE_SESSION              = "Configure_Session";
    public static final String CONFIGURE_RUNTYPE              = "Configure_Runtype";
    //         Action PostConfigure and slots
    public static final String POSTCONFIGURE                  = "Postconfigure";
    public static final String POSTCONFIGURE_TIMEOUT          = "Timeout";
    public static final String POSTCONFIGURE_SESSION          = "Postconfigure_Session";
    public static final String POSTCONFIGURE_RUNTYPE          = "Postconfigure_Runtype";
    //         Action PreDownload and slots
    public static final String PREDOWNLOAD                    = "Predownload";
    public static final String PREDOWNLOAD_TIMEOUT            = "Timeout";
    public static final String PREDOWNLOAD_SESSION            = "Predownload_Session";
    public static final String PREDOWNLOAD_RUNTYPE            = "Predownload_Runtype";
    //         Action Download and slots
    public static final String DOWNLOAD                       = "Download";
    public static final String DOWNLOAD_TIMEOUT               = "Timeout";
    public static final String DOWNLOAD_SESSION               = "Download_Session";
    public static final String DOWNLOAD_RUNTYPE               = "Download_Runtype";
    //         Action PostDownload and slots
    public static final String POSTDOWNLOAD                   = "Posdownload";
    public static final String POSTDOWNLOAD_TIMEOUT           = "Timeout";
    public static final String POSTDOWNLOAD_SESSION           = "Postdownload_Session";
    public static final String POSTDOWNLOAD_RUNTYPE           = "Postdownload_Runtype";
    //         Action PrePrestart and slots
    public static final String PREPRESTART                    = "Preprestart";
    public static final String PREPRESTART_TIMEOUT            = "Timeout";
    public static final String PREPRESTART_ENDAFTER           = "Endafter";
    public static final String PREPRESTART_SESSION            = "Preprestart_Session";
    public static final String PREPRESTART_RUNTYPE            = "Preprestart_Runtype";
    //         Action Prestart and slots
    public static final String PRESTART                       = "Prestart";
    public static final String PRESTART_TIMEOUT               = "Timeout";
    public static final String PRESTART_SESSION               = "Prestart_Session";
    public static final String PRESTART_RUNTYPE               = "Prestart_Runtype";
    //         Action PostPrestart and slots
    public static final String POSTPRESTART                   = "Postprestart";
    public static final String POSTPRESTART_TIMEOUT           = "Timeout";
    public static final String POSTPRESTART_SESSION           = "Postprestart_Session";
    public static final String POSTPRESTART_RUNTYPE           = "Postprestart_Runtype";
    //         Action PreGo and slots
    public static final String PREGO                   = "Prego";
    public static final String PREGO_TIMEOUT           = "Timeout";
    public static final String PREGO_SESSION           = "Prego_Session";
    public static final String PREGO_RUNTYPE           = "Prego_Runtype";
    public static final String PREGO_RUNNUMBER         = "Prego_Runnumber";
    //         Action Go and slots
    public static final String GO                      =  "Go";
    public static final String GO_TIMEOUT              = "Timeout";
    public static final String GO_SESSION              = "Go_Session";
    public static final String GO_RUNTYPE              = "Go_Runtype";
    //         Action Reset and slots
    public static final String RESET                   = "Reset";
    public static final String RESET_TIMEOUT           = "Timeout";
    public static final String RESET_SESSION           = "Reset_Session";
    public static final String RESET_RUNTYPE           = "reset_Runtype";
    //         Action Exit and slots
    public static final String EXIT                    = "Exit";
    public static final String EXIT_TIMEOUT            = "Timeout";
    public static final String EXIT_SESSION            = "Exit_Session";
    public static final String EXIT_RUNTYPE            = "Exit_Runtype";
    //         Action PostGo and slots
    public static final String POSTGO                  = "Postgo";
    public static final String POSTGO_TIMEOUT          = "Timeout";
    public static final String POSTGO_SESSION          = "Postgo_Session";
    public static final String POSTGO_RUNTYPE          = "Postgo_Runtype";
    //         Action PreEnd and slots
    public static final String PREEND                  = "Preend";
    public static final String PREEND_TIMEOUT          = "Timeout";
    public static final String PREEND_SESSION          = "Preend_Session";
    public static final String PREEND_RUNTYPE          = "Preend_Runtype";
    //         Action End and slots
    public static final String END                     = "End";
    public static final String END_TIMEOUT             = "Timeout";
    public static final String END_SESSION             = "End_Session";
    public static final String END_RUNTYPE             = "End_Runtype";
    //         Action PostEnd and slots
    public static final String POSTEND                 = "Postend";
    public static final String POSTEND_TIMEOUT         = "Timeout";
    public static final String POSTEND_SESSION         = "Postend_Session";
    public static final String POSTEND_RUNTYPE         = "Postend_Runtype";
    //         Action Pause and slots
    public static final String PAUSE                   = "Pause";
    public static final String PAUSE_TIMEOUT           = "Timeout";
    public static final String PAUSE_SESSION           = "Pause_Session";
    public static final String PAUSE_RUNTYPE           = "Pause_Runtype";
    //         Action Resume and slots
    public static final String RESUME                  = "Resume";
    public static final String RESUME_TIMEOUT          = "Timeout";
    public static final String RESUME_SESSION          = "Resume_Session";
    public static final String RESUME_RUNTYPE          = "Resume_Runtype";
    //         Action ReportRunStatus and slots
    public static final String REPORTRUNSTATUS         = "Reportrunstatus";
    public static final String REPORTRUNSTATUS_TIMEOUT = "Reportrunstatus_Timeout";
    //         Action ResultNotify and slots
    public static final String RESULTNOTIFY                 = "Resultnotify";
    public static final String RESULTNOTIFY_COMPONENT       = "Resultnotify_Component";

    //         Action ReportState and slots
    public static final String REPORTSTATE             = "Reportstate";
    public static final String REPORTSTATE_TO          = "Reportstate_To";
    public static final String REPORTSTATE_THROUGHACL  = "Reportstate_Acl";
    public static final String REPORTSTATE_INTERVAL    = "Reportstate_Interval";
    public static final String REPORTSTATE_TIMEOUT     = "Reportstate_Timeout";
    //         Action ReportRunNumber and slots
    public static final String REPORTRUNNUMBER         = "Reportrunnumber";
    public static final String REPORTRUNNUMBER_TIMEOUT = "Reportrunnumber_Timeout";
    //         Action ReportSession and slots
    public static final String REPORTSESSION           = "Reportsession";
    public static final String REPORTSESSION_TIMEOUT   = "Reportsession_Timeout";
    //         Action ReportRunType and slots
    public static final String REPORTRUNTYPE           = "Reportruntype";
    public static final String REPORTRUNTYPE_TIMEOUT   = "Reportruntype_Timeout";
    //         Action ReportRunStatistics and slots
    public static final String REPORTRUNSTATISTICS           = "Reportrunstatistics";
    public static final String REPORTRUNSTATISTICS_TIMEOUT   = "Reportrunstatistics_Timeout";
    //         Action StartConfig and slots
    public static final String STARTCONFIG                   = "Startconfig";
    public static final String STARTCONFIG_RUNTYPE           = "Startconfig_Runtype";
    public static final String STARTCONFIG_SESSION           = "Startconfig_Session";
    public static final String STARTCONFIG_SOURCE            = "Startconfig_Source";
    public static final String STARTCONFIG_FILENAME          = "Startconfig_Filename";
    public static final String STARTCONFIG_RCNODE            = "Startconfig_Rcnode";
    //    Action recordNewRun and slots
    public static final String RECORDNEWRUN                  = "Recordnewrun";
    public static final String RECORDNEWRUN_SESSION          = "Recordnewrun_Session";
    public static final String RECORDNEWRUN_RUNTYPE          = "Recordnewrun_Runtype";
    //    Action acceptRunNumber and slots
    public static final String ACCEPTRUNNUMBER               = "Acceptrunnumber";
    public static final String ACCEPTRUNNUMBER_VALUE         = "Acceptrunnumber_Value";
    public static final String ACCEPTRUNNUMBER_SESSION       = "Acceptrunnumber_Session";
    //    Action disconnect and slots
    public static final String DISCONNECT                    = "Disconnect";
    public static final String DISCONNECT_SESSION            = "Disconnect_Session";

    //    Action moveto and slots
    public static final String ACTIONMOVETO                  = "ActionMoveto";
    public static final String ACTIONMOVETO_STATE            = "ActionMoveto_State";

    //    Action execute and slots
    public static final String ACTIONEXECUTE                 = "ActionExecute";
    public static final String ACTIONEXECUTE_PROCESS         = "ActionExecute_Process";


    //The singleton instance of this ontology
    private static Ontology theInstance = new CodaRunControlOntology();

    /**
     This method grants access to the singleton instance of the
     CodaRunControlOntology ontology.
     @return An <code>Ontology</code> object, containing the concepts
     of the CodaRunControlOntology.
     */
    public static Ontology getInstance() {
        return theInstance;
    }

    //    Private constructor
    private CodaRunControlOntology() {
//         CodaRunControlOntology ontology extends basic ontology
        super(NAME, BasicOntology.getInstance());

        try {
            add(new ConceptSchema(CONTROL), CoolControl.class);
            add(new ConceptSchema(COOLSTATE), CoolState.class);
            add(new ConceptSchema(RULE), CoolRule.class);
            add(new ConceptSchema(IF), CoolIf.class);
            add(new ConceptSchema(WHILE), CoolWhile.class);
            add(new ConceptSchema(ENDWHILE), CoolEndWhile.class);
            add(new ConceptSchema(ANDIF), CoolAndIf.class);
            add(new ConceptSchema(ORIF), CoolOrIf.class);
            add(new ConceptSchema(ELSEIF), CoolElseIf.class);
//            add(new ConceptSchema(MOVETO), CoolMoveTo.class);
            add(new ConceptSchema(EXECUTE), CoolExecute.class);
            add(new ConceptSchema(LOOP), CoolLoop.class);
            add(new ConceptSchema(WAIT), CoolWait.class);
            add(new ConceptSchema(ENDLOOP), CoolEndLoop.class);
            add(new ConceptSchema(ELSE), CoolElse.class);
            add(new ConceptSchema(ENDIF), CoolEndIf.class);
            add(new ConceptSchema(COMPONENT), CoolComponent.class);
            add(new ConceptSchema(SYSTEM), CoolSystem.class);
            add(new ConceptSchema(PROCESS), CoolProcess.class);
            add(new ConceptSchema(PUBLISHER), CoolPublisher.class);
            add(new ConceptSchema(OPTION), CoolOption.class);
            add(new ConceptSchema(COOLSCHEDULER), CoolScheduler.class);
            add(new ConceptSchema(DATA), CoolData.class);
            add(new ConceptSchema(NODE), CoolNode.class);
            add(new ConceptSchema(RECEIVER), Receiver.class);
            add(new ConceptSchema(PRODUCER), Producer.class);
            add(new ConceptSchema(TIMESTAMP),CodaTimeStamp.class);
            add(new ConceptSchema(STATUSQUANT),StatusQuant.class);
            add(new ConceptSchema(DATAQUANT),DataQuant.class);

            add(new AgentActionSchema(RESULTNOTIFY), ResultNotify.class);
            add(new AgentActionSchema(DIFFERENTIATE), Differentiate.class);
            add(new AgentActionSchema(CODAINIT), CodaInit.class);
            add(new AgentActionSchema(CHECKDIFFERENTIATE), CheckDifferentiate.class);
            add(new AgentActionSchema(ACCEPTENDING), AcceptEnding.class);
            add(new AgentActionSchema(STOPREPORTING), StopReporting.class);
            add(new AgentActionSchema(SETPRECONFIGURE), SetPreConfigure.class);
            add(new AgentActionSchema(SETCONFIGURE), SetConfigure.class);
            add(new AgentActionSchema(SETPOSTCONFIGURE), SetPostConfigure.class);
            add(new AgentActionSchema(SETPREDOWNLOAD), SetPreDownload.class);
            add(new AgentActionSchema(SETDOWNLOAD), SetDownload.class);
            add(new AgentActionSchema(SETPOSTDOWNLOAD), SetPostDownload.class);
            add(new AgentActionSchema(SETPREPRESTART), SetPrePrestart.class);
            add(new AgentActionSchema(SETPRESTART), SetPrestart.class);
            add(new AgentActionSchema(SETPOSTPRESTART), SetPostPrestart.class);
            add(new AgentActionSchema(SETPREGO), SetPreGo.class);
            add(new AgentActionSchema(SETGO), SetGo.class);
            add(new AgentActionSchema(SETRESET), SetReset.class);
            add(new AgentActionSchema(SETPOSTGO), SetPostGo.class);
            add(new AgentActionSchema(SETPREEND), SetPreEnd.class);
            add(new AgentActionSchema(SETEND), SetEnd.class);
            add(new AgentActionSchema(SETPOSTEND), SetPostEnd.class);
            add(new AgentActionSchema(PRECONFIGURE), PreConfigure.class);
            add(new AgentActionSchema(CONFIGURE), Configure.class);
            add(new AgentActionSchema(POSTCONFIGURE), PostConfigure.class);
            add(new AgentActionSchema(PREDOWNLOAD), PreDownload.class);
            add(new AgentActionSchema(DOWNLOAD), Download.class);
            add(new AgentActionSchema(POSTDOWNLOAD), PostDownload.class);
            add(new AgentActionSchema(PREPRESTART), PrePrestart.class);
            add(new AgentActionSchema(PRESTART), Prestart.class);
            add(new AgentActionSchema(POSTPRESTART), PostPrestart.class);
            add(new AgentActionSchema(PREGO), PreGo.class);
            add(new AgentActionSchema(GO), Go.class);
            add(new AgentActionSchema(RESET), Reset.class);
            add(new AgentActionSchema(EXIT), Exit.class);
            add(new AgentActionSchema(POSTGO), PostGo.class);
            add(new AgentActionSchema(PREEND), PreEnd.class);
            add(new AgentActionSchema(END), End.class);
            add(new AgentActionSchema(POSTEND), PostEnd.class);
            add(new AgentActionSchema(PAUSE), Pause.class);
            add(new AgentActionSchema(RESUME), Resume.class);
            add(new AgentActionSchema(EXECUTECOMMAND), ExecuteCommand.class);
            add(new AgentActionSchema(SETINPAUSE), SetInPause.class);
            add(new AgentActionSchema(SETINRESUME), SetInResume.class);
            add(new AgentActionSchema(ACCEPTDATA), AcceptData.class);
            add(new AgentActionSchema(ACCEPTGROUPDATA), AcceptGroupData.class);
            add(new AgentActionSchema(ACCEPTTSDATA), AcceptTsData.class);
            add(new AgentActionSchema(ACCEPTSTATUS), AcceptStatus.class);
            add(new AgentActionSchema(ACCEPTSTATE), AcceptState.class);
            add(new AgentActionSchema(ACCEPTSTATUS), AcceptStatus.class);
            add(new AgentActionSchema(ACCEPTGROUPSTATUS), AcceptGroupStatus.class);
            add(new AgentActionSchema(ACCEPTCOMPONENT), AcceptComponent.class);
            add(new AgentActionSchema(ACCEPTSYSTEM), AcceptSystem.class);
            add(new AgentActionSchema(ACCEPTSTATES), AcceptStates.class);
            add(new AgentActionSchema(ACCEPTRULES), AcceptRules.class);
            add(new AgentActionSchema(STOPMONITORING), StopMonitoring.class);
            add(new AgentActionSchema(RESUMEMONITORING), ResumeMonitoring.class);
            add(new AgentActionSchema(KILLMONITORING), KillMonitoring.class);
            add(new AgentActionSchema(REPORTRUNSTATUS), ReportRunStatus.class);
            add(new AgentActionSchema(REPORTSTATE), ReportState.class);
            add(new AgentActionSchema(REPORTRUNNUMBER), ReportRunNumber.class);
            add(new AgentActionSchema(REPORTRUNTYPE), ReportRunType.class);
            add(new AgentActionSchema(REPORTRUNSTATISTICS), ReportRunStatistics.class);
            add(new AgentActionSchema(REPORTSESSION), ReportSession.class);
            add(new AgentActionSchema(STARTCONFIG), StartConfig.class);
            add(new AgentActionSchema(RECORDNEWRUN), RecordNewRun.class);
            add(new AgentActionSchema(ACCEPTRUNNUMBER), AcceptRunNumber.class);
            add(new AgentActionSchema(DISCONNECT), Disconnect.class);
            add(new AgentActionSchema(ACTIONMOVETO), CoolActionMoveTo.class);
            add(new AgentActionSchema(ACTIONEXECUTE), CoolActionExecute.class);

//    Structure of the schema for the CONTROL concept
            ConceptSchema cs = (ConceptSchema) getSchema(CONTROL);
            cs.add(CONTROL_HASCOMPONENT, (ConceptSchema) getSchema(COMPONENT), 0, ObjectSchema.UNLIMITED);
            cs.add(CONTROL_HASSYSTEM, (ConceptSchema) getSchema(SYSTEM), 0, ObjectSchema.UNLIMITED);
            cs.add(CONTROL_HASSTATE, (ConceptSchema) getSchema(COOLSTATE), 0, ObjectSchema.UNLIMITED);
            cs.add(CONTROL_HASRULE, (ConceptSchema) getSchema(RULE),0,ObjectSchema.UNLIMITED);
            cs.add(CONTROL_HASCODAOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASANAOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASEROPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASEBOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASROCOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASSLCOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASTSOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(CONTROL_HASETOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);

//    Structure of the schema for the IF concept
            cs = (ConceptSchema) getSchema(IF);
            cs.add(IF_STATE, (ConceptSchema) getSchema(COOLSTATE), ObjectSchema.MANDATORY);
            cs.add(IF_CODASTATE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

//    Structure of the schema for the WHILE concept
            cs = (ConceptSchema) getSchema(WHILE);
            cs.add(WHILE_STATE, (ConceptSchema) getSchema(COOLSTATE), ObjectSchema.MANDATORY);

//    Structure of the schema for the ENDWHILE concept
            cs = (ConceptSchema) getSchema(ENDWHILE);
            cs.add(ENDWHILE_VALUE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

//    Structure of the schema for the ANDIF concept
            cs = (ConceptSchema) getSchema(ANDIF);
            cs.add(ANDIF_STATE, (ConceptSchema) getSchema(COOLSTATE), ObjectSchema.MANDATORY);

//    Structure of the schema for the ORIF concept
            cs = (ConceptSchema) getSchema(ORIF);
            cs.add(ORIF_STATE, (ConceptSchema) getSchema(COOLSTATE), ObjectSchema.MANDATORY);

//    Structure of the schema for the ELSEIF concept
            cs = (ConceptSchema) getSchema(ELSEIF);
            cs.add(ELSEIF_STATE, (ConceptSchema) getSchema(COOLSTATE), ObjectSchema.MANDATORY);

//    Structure of the schema for the MOVETO concept
//            cs = (ConceptSchema) getSchema(MOVETO);
//            cs.add(MOVETO_STATE, (ConceptSchema) getSchema(STATE), ObjectSchema.MANDATORY);

//    Structure of the schema for the EXECUTE concept
            cs = (ConceptSchema) getSchema(EXECUTE);
            cs.add(EXECUTE_PROCESS, (ConceptSchema) getSchema(PROCESS), ObjectSchema.MANDATORY);

//    Structure of the schema for the LOOP concept
            cs = (ConceptSchema) getSchema(LOOP);
            cs.add(LOOP_PARAM, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);

//    Structure of the schema for the WAIT concept
            cs = (ConceptSchema) getSchema(WAIT);
            cs.add(WAIT_SEC, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);

//    Structure of the schema for the ENDLOOP concept
            cs = (ConceptSchema) getSchema(ENDLOOP);
            cs.add(ENDLOOP_VALUE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

//    Structure of the schema for the ELSE concept
            cs = (ConceptSchema) getSchema(ELSE);
            cs.add(ELSE_VALUE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

//    Structure of the schema for the ENDIF concept
            cs = (ConceptSchema) getSchema(ENDIF);
            cs.add(ENDIF_VALUE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);


//    Structure of the schema for the SYSTEM concept
            cs = (ConceptSchema) getSchema(SYSTEM);
            cs.add(SYSTEM_HASSMSAGENT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(SYSTEM_ISCONTROLLABLE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(SYSTEM_HASREPORTINGINTERVAL, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(SYSTEM_HASSTATE, (ConceptSchema) getSchema(COOLSTATE), 0, ObjectSchema.UNLIMITED);

//    Structure of the schema for the COMPONENT concept
            cs = (ConceptSchema) getSchema(COMPONENT);
            cs.add(COMPONENT_HASNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASPRIORITY, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASIPC, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASCODE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASSESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASRUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASNODE, (ConceptSchema) getSchema(NODE), ObjectSchema.OPTIONAL);


            cs.add(COMPONENT_HASEXTERNALPROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
            cs.add(COMPONENT_REPRESENTSCODA2CLIENT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASLOGINGLEVEL, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_REPRESENTSIPADDRESS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_HASOPTION, (ConceptSchema) getSchema(OPTION), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_CONTROLSCOMPONENT, (PrimitiveSchema) getSchema(BasicOntology.STRING),0,ObjectSchema.UNLIMITED);
            cs.add(COMPONENT_REPRESENTSOBJECTTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_REPORTINGINTERVAL, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_FILTERSDALOG, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_REPORTSTO, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_DALOGTHROTTLELEVEL, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_CLIENTPORT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_CLIENTHOST, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_CLIENTHOST, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(COMPONENT_CLIENTPORT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);


//    Structure of the schema for the RULE concept
            cs = (ConceptSchema) getSchema(RULE);
            cs.add(RULE_IF, (ConceptSchema) getSchema(COOLSTATE),ObjectSchema.OPTIONAL);
            cs.add(RULE_WHILE, (ConceptSchema) getSchema(COOLSTATE),ObjectSchema.OPTIONAL);
            cs.add(RULE_ENDIF, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(RULE_ENDWHILE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(RULE_ELSE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(RULE_ELSEIF, (ConceptSchema) getSchema(COOLSTATE),ObjectSchema.OPTIONAL);
            cs.add(RULE_LOOP, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            cs.add(RULE_ENDLOOP, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(RULE_ITERATION, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(RULE_WAIT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);

            cs.add(RULE_ORIF, (ConceptSchema) getSchema(COOLSTATE), 0, ObjectSchema.UNLIMITED);
            cs.add(RULE_ANDIF, (ConceptSchema) getSchema(COOLSTATE),0,ObjectSchema.UNLIMITED);
            cs.add(RULE_THENMOVETO, (ConceptSchema) getSchema(COOLSTATE),0,ObjectSchema.UNLIMITED);
            cs.add(RULE_ELSEMOVETO, (ConceptSchema) getSchema(COOLSTATE),0,ObjectSchema.UNLIMITED);
            cs.add(RULE_ELSEIFMOVETO, (ConceptSchema) getSchema(COOLSTATE),0,ObjectSchema.UNLIMITED);
            cs.add(RULE_WHILEMOVETO, (ConceptSchema) getSchema(COOLSTATE),0,ObjectSchema.UNLIMITED);
            cs.add(RULE_THENEXECUTEPROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
            cs.add(RULE_ELSEEXECUTEPROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
            cs.add(RULE_ELSEIFEXECUTEPROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
            cs.add(RULE_WHILEEXECUTEPROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);

//    Structure of the schema for the STATE concept
            cs = (ConceptSchema) getSchema(COOLSTATE);
            cs.add(STATE_HASSTATENAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(STATE_HASSEVERITY, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(STATE_ISCRITICAL, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(STATE_ACHIEVEDTHROUGH, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
            cs.add(STATE_DEPENDSON, (ConceptSchema) getSchema(COOLSTATE), 0, ObjectSchema.UNLIMITED);
            cs.add(STATE_ISFOR, (ConceptSchema) getSchema(COMPONENT), ObjectSchema.OPTIONAL);

//    Structure of the schema for the PROCESS concept
            cs = (ConceptSchema) getSchema(PROCESS);
            cs.add(PROCESS_ISPARTOFSTATE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASCOMMANDNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASPROCESSIPC, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASTIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASPERIODICITY, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASDELAY, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASINPUTDATA, (ConceptSchema) getSchema(DATA), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASOUTPUTDATA, (ConceptSchema) getSchema(DATA), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_SUBSCRIBESTO, (ConceptSchema) getSchema(PUBLISHER), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_SENDSTO, (ConceptSchema) getSchema(PUBLISHER), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_ISSTATETRANSITION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_SETS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_GETS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_TRAPS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_STOP, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASERRORSTATUS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASERRORINDEX, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(PROCESS_HASEREQUESTID, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);

//    Structure of the schema for the Publisher concept
            cs = (ConceptSchema) getSchema(PUBLISHER);
            cs.add(PUBLISHER_HASUDL, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(PUBLISHER_HASSENDSUBJECT, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(PUBLISHER_HASSENDTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PUBLISHER_HASSUBSCRIBESUBJECT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(PUBLISHER_HASSUBSCRIBETYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);

//    Structure of the schema for the OPTION concept
            cs = (ConceptSchema) getSchema(OPTION);
            cs.add(OPTION_HASSCHEDULER, (ConceptSchema) getSchema(COOLSCHEDULER), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASDATAFILE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASCODA2COMPONENT, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASAUTOSTART, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASSTARTRUN, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASDATALIMIT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASEVENTLIMIT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASCONFIGFILE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASDATAFILEPREFIX, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_HASCONFIGSTRING, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_CONFIGISXML, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(OPTION_CONFIGISMIB, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

//    Structure of the schema for the OPTION concept
            cs = (ConceptSchema) getSchema(COOLSCHEDULER);
            cs.add(COOLSCHEDULER_HASEXTERNALRUNCONFIGPROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
            cs.add(COOLSCHEDULER_HASSTOPAFTEREVTCOUNT, (PrimitiveSchema) getSchema(BasicOntology.STRING), 0, ObjectSchema.UNLIMITED);
            cs.add(COOLSCHEDULER_HASPAUSEAFTEREVTCOUNT, (PrimitiveSchema) getSchema(BasicOntology.STRING), 0, ObjectSchema.UNLIMITED);

//    Structure of the schema for the NODE concept
            cs = (ConceptSchema) getSchema(NODE);
            cs.add(NODE_HASNODENAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            cs.add(NODE_HASIP, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(NODE_ISLOCAL, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(NODE_HASUSERNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(NODE_HASPASSWORD, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);

//    Structure of the schema for the DATA concept
            cs = (ConceptSchema) getSchema(DATA);
            cs.add(DATA_HASTIMESTAMP, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(DATA_HASDATATYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(DATA_HASCARDINALITY, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(DATA_HASSEMANTICS, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(DATA_HASCONTENT, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);

//    Structure of the schema for the CHECKWITH concept
//            cs = (ConceptSchema) getSchema(CHECKWITH);
//            cs.add(CHECKWITH_COMMAND, (ConceptSchema) getSchema(COMMAND));
//    Structure of the schema for the RECEIVER concept
            cs = (ConceptSchema) getSchema(RECEIVER);
            cs.add(RECEIVER_COMPONENT, (ConceptSchema) getSchema(COMPONENT));
//    Structure of the schema for the PRODUCER concept
            cs = (ConceptSchema) getSchema(PRODUCER);
            cs.add(PRODUCER_COMPONENT, (ConceptSchema) getSchema(COMPONENT));
//    Structure of the schema for the TIMESTAMP concept
            cs = (ConceptSchema) getSchema(TIMESTAMP);
            cs.add(TIMESTAMP_VALUE, (PrimitiveSchema) getSchema(BasicOntology.DATE));

//    Structure of the schema for the STATUSQUANT concept
            cs = (ConceptSchema) getSchema(STATUSQUANT);
            cs.add(STATUSQUANT_STATUS, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            cs.add(STATUSQUANT_SEVERITY, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            cs.add(STATUSQUANT_PRODUCER, (ConceptSchema) getSchema(PRODUCER));
            cs.add(STATUSQUANT_TIMESTAMP, (ConceptSchema) getSchema(TIMESTAMP), ObjectSchema.OPTIONAL);
            cs.add(STATUSQUANT_TRANSITION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

//    Structure of the schema for the DATAQUANT concept
            cs = (ConceptSchema) getSchema(DATAQUANT);
            cs.add(DATAQUANT_DATA, (ConceptSchema) getSchema(DATA));
            cs.add(DATAQUANT_PRODUCER, (ConceptSchema) getSchema(PRODUCER), ObjectSchema.OPTIONAL);
            cs.add(DATAQUANT_PROCESS, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            cs.add(DATAQUANT_PROCESSSTATE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            cs.add(DATAQUANT_TIMESTAMP, (ConceptSchema) getSchema(TIMESTAMP), ObjectSchema.OPTIONAL);

//    Structure of the schema for the DIFFERENTIATE action
            AgentActionSchema as = (AgentActionSchema) getSchema(DIFFERENTIATE);
            as.add(DIFFERENTIATE_COMPONENT, (ConceptSchema) getSchema(COMPONENT));
            as.add(DIFFERENTIATE_OPTION, (ConceptSchema) getSchema(OPTION));
//    Structure of the schema for the CODAINIT action
            as = (AgentActionSchema) getSchema(CODAINIT);
            as.add(CODAINIT_CLIENTHOST, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(CODAINIT_CLIENTPORT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
//    Structure of the schema for the CHECKDIFFERENTIATE action
            as = (AgentActionSchema) getSchema(CHECKDIFFERENTIATE);
//    Structure of the schema for the ACCEPTENDING action
            as = (AgentActionSchema) getSchema(ACCEPTENDING);
//    Structure of the schema for the STOPREPORTING action
            as = (AgentActionSchema) getSchema(STOPREPORTING);
//    Structure of the schema for the SETPRECONFIGURE AgentAction
            as = (AgentActionSchema) getSchema(SETPRECONFIGURE);
            as.add(SETPRECONFIGURE_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETCONFIGURE AgentAction
            as = (AgentActionSchema) getSchema(SETCONFIGURE);
            as.add(SETCONFIGURE_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPOSTCONFIGURE AgentAction
            as = (AgentActionSchema) getSchema(SETPOSTCONFIGURE);
            as.add(SETPOSTCONFIGURE_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPREDOWNLOAD AgentAction
            as = (AgentActionSchema) getSchema(SETPREDOWNLOAD);
            as.add(SETPREDOWNLOAD_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETDOWNLOAD AgentAction
            as = (AgentActionSchema) getSchema(SETDOWNLOAD);
            as.add(SETDOWNLOAD_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPOSTDOWNLOAD AgentAction
            as = (AgentActionSchema) getSchema(SETPOSTDOWNLOAD);
            as.add(SETPOSTDOWNLOAD_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPREPRESTART AgentAction
            as = (AgentActionSchema) getSchema(SETPREPRESTART);
            as.add(SETPREPRESTART_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPRESTART AgentAction
            as = (AgentActionSchema) getSchema(SETPRESTART);
            as.add(SETPRESTART_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPOSTPRESTART AgentAction
            as = (AgentActionSchema) getSchema(SETPOSTPRESTART);
            as.add(SETPOSTPRESTART_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPREGO AgentAction
            as = (AgentActionSchema) getSchema(SETPREGO);
            as.add(SETPREGO_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETGO AgentAction
            as = (AgentActionSchema) getSchema(SETGO);
            as.add(SETGO_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPOSTGO AgentAction
            as = (AgentActionSchema) getSchema(SETPOSTGO);
            as.add(SETPOSTGO_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPREEND AgentAction
            as = (AgentActionSchema) getSchema(SETPREEND);
            as.add(SETPREEND_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETEND AgentAction
            as = (AgentActionSchema) getSchema(SETEND);
            as.add(SETEND_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETRESET AgentAction
            as = (AgentActionSchema) getSchema(SETRESET);
            as.add(SETRESET_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETPOSTEND AgentAction
            as = (AgentActionSchema) getSchema(SETPOSTEND);
            as.add(SETPOSTEND_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETINPAUSE AgentAction
            as = (AgentActionSchema) getSchema(SETINPAUSE);
            as.add(SETINPAUSE_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the SETINRESUME AgentAction
            as = (AgentActionSchema) getSchema(SETINRESUME);
            as.add(SETINRESUME_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the EXECUTECOMMAND action
            as = (AgentActionSchema) getSchema(EXECUTECOMMAND);
            as.add(EXECUTECOMMAND_PROCESS, (ConceptSchema) getSchema(PROCESS), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the STOPMONITORING action
            as = (AgentActionSchema) getSchema(STOPMONITORING);
            as.add(STOPMONITORING_PROCESSNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(STOPMONITORING_AGENTNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
//    Structure of the schema for the RESUMEMONITORING action
            as = (AgentActionSchema) getSchema(RESUMEMONITORING);
            as.add(RESUMEMONITORING_PROCESSNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(RESUMEMONITORING_AGENTNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
//    Structure of the schema for the KILLMONITORING action
            as = (AgentActionSchema) getSchema(KILLMONITORING);
            as.add(KILLMONITORING_PROCESSNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(KILLMONITORING_AGENTNAME, (PrimitiveSchema) getSchema(BasicOntology.STRING));
//    Structure of the schema for the ACCEPTDATA action
            as = (AgentActionSchema) getSchema(ACCEPTDATA);
            as.add(ACCEPTDATA_DATA, (ConceptSchema) getSchema(DATA));
            as.add(ACCEPTDATA_PRODUCER, (ConceptSchema) getSchema(PRODUCER), ObjectSchema.OPTIONAL);
            as.add(ACCEPTDATA_PROCESS, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(ACCEPTDATA_PROCESSSTATE, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            as.add(ACCEPTDATA_TIMESTAMP, (ConceptSchema) getSchema(TIMESTAMP), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACCEPTGROUPDATA action
            as = (AgentActionSchema) getSchema(ACCEPTGROUPDATA);
            as.add(ACCEPTGROUPDATA_DATA, (ConceptSchema) getSchema(DATAQUANT), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the ACCEPTTSDATA action
            as = (AgentActionSchema) getSchema(ACCEPTTSDATA);
            as.add(ACCEPTTSDATA_DATA, (ConceptSchema) getSchema(DATA));
            as.add(ACCEPTTSDATA_PRODUCER, (ConceptSchema) getSchema(PRODUCER), ObjectSchema.OPTIONAL);
            as.add(ACCEPTTSDATA_TIMESTAMP, (ConceptSchema) getSchema(TIMESTAMP), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACCEPTSTATUS action
            as = (AgentActionSchema) getSchema(ACCEPTSTATUS);
            as.add(ACCEPTSTATUS_STATUS, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(ACCEPTSTATUS_SEVERITY, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(ACCEPTSTATUS_PRODUCER, (ConceptSchema) getSchema(PRODUCER));
            as.add(ACCEPTSTATUS_TIMESTAMP, (ConceptSchema) getSchema(TIMESTAMP), ObjectSchema.OPTIONAL);
            as.add(ACCEPTSTATUS_TRANSITION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACCEPTSTATE action
            as = (AgentActionSchema) getSchema(ACCEPTSTATE);
            as.add(ACCEPTSTATE_STATE, (ConceptSchema) getSchema(COOLSTATE));
            as.add(ACCEPTSTATE_SEVERITY, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(ACCEPTSTATE_PRODUCER, (ConceptSchema) getSchema(PRODUCER), ObjectSchema.OPTIONAL);
            as.add(ACCEPTSTATE_TIMESTAMP, (ConceptSchema) getSchema(TIMESTAMP), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACCEPTGROUPSTATUS action
            as = (AgentActionSchema) getSchema(ACCEPTGROUPSTATUS);
            as.add(ACCEPTGROUPSTATUS_STATUS, (ConceptSchema) getSchema(STATUSQUANT), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the ACCEPTCOMPONENT action
            as = (AgentActionSchema) getSchema(ACCEPTCOMPONENT);
            as.add(ACCEPTCOMPONENT_COMPONENT, (ConceptSchema) getSchema(COMPONENT));
//    Structure of the schema for the ACCEPTSYSTEM action
            as = (AgentActionSchema) getSchema(ACCEPTSYSTEM);
            as.add(ACCEPTSYSTEM_SYSTEM, (ConceptSchema) getSchema(SYSTEM));
//    Structure of the schema for the ACCEPTSTATES action
            as = (AgentActionSchema) getSchema(ACCEPTSTATES);
            as.add(ACCEPTSTATES_LIST, (ConceptSchema) getSchema(COOLSTATE), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the ACCEPTRULES action
            as = (AgentActionSchema) getSchema(ACCEPTRULES);
            as.add(ACCEPTRULES_LIST, (ConceptSchema) getSchema(RULE), 0, ObjectSchema.UNLIMITED);
//    Structure of the schema for the PRECONFIGURE action
            as = (AgentActionSchema) getSchema(PRECONFIGURE);
            as.add(PRECONFIGURE_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PRECONFIGURE_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PRECONFIGURE_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the CONFIGURE action
            as = (AgentActionSchema) getSchema(CONFIGURE);
            as.add(CONFIGURE_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(CONFIGURE_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(CONFIGURE_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the POSTCONFIGURE action
            as = (AgentActionSchema) getSchema(POSTCONFIGURE);
            as.add(POSTCONFIGURE_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(POSTCONFIGURE_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(POSTCONFIGURE_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the PREDOWNLOAD action
            as = (AgentActionSchema) getSchema(PREDOWNLOAD);
            as.add(PREDOWNLOAD_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PREDOWNLOAD_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PREDOWNLOAD_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the DOWNLOAD action
            as = (AgentActionSchema) getSchema(DOWNLOAD);
            as.add(DOWNLOAD_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(DOWNLOAD_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(DOWNLOAD_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the POSTDOWNLOAD action
            as = (AgentActionSchema) getSchema(POSTDOWNLOAD);
            as.add(POSTDOWNLOAD_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(POSTDOWNLOAD_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(POSTDOWNLOAD_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the PREPRESTART action
            as = (AgentActionSchema) getSchema(PREPRESTART);
            as.add(PREPRESTART_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PREPRESTART_ENDAFTER, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PREPRESTART_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PREPRESTART_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the PRESTART action
            as = (AgentActionSchema) getSchema(PRESTART);
            as.add(PRESTART_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PRESTART_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PRESTART_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the POSTPRESTART action
            as = (AgentActionSchema) getSchema(POSTPRESTART);
            as.add(POSTPRESTART_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(POSTPRESTART_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(POSTPRESTART_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the PREGO action
            as = (AgentActionSchema) getSchema(PREGO);
            as.add(PREGO_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PREGO_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PREGO_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(PREGO_RUNNUMBER, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
//    Structure of the schema for the GO action
            as = (AgentActionSchema) getSchema(GO);
            as.add(GO_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(GO_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(GO_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the RESET action
            as = (AgentActionSchema) getSchema(RESET);
            as.add(RESET_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(RESET_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(RESET_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the EXIT action
            as = (AgentActionSchema) getSchema(EXIT);
            as.add(EXIT_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(EXIT_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(EXIT_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the POSTGO action
            as = (AgentActionSchema) getSchema(POSTGO);
            as.add(POSTGO_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(POSTGO_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(POSTGO_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the PREEND action
            as = (AgentActionSchema) getSchema(PREEND);
            as.add(PREEND_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PREEND_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PREEND_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the END action
            as = (AgentActionSchema) getSchema(END);
            as.add(END_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(END_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(END_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the POSTEND action
            as = (AgentActionSchema) getSchema(POSTEND);
            as.add(POSTEND_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(POSTEND_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(POSTEND_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the PAUSE action
            as = (AgentActionSchema) getSchema(PAUSE);
            as.add(PAUSE_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(PAUSE_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(PAUSE_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the RESUME action
            as = (AgentActionSchema) getSchema(RESUME);
            as.add(RESUME_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(RESUME_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(RESUME_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the REPORTRUNSTATUS action
            as = (AgentActionSchema) getSchema(REPORTRUNSTATUS);
            as.add(REPORTRUNSTATUS_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
//    Structure of the schema for the RESULTNOTIFY action
            as = (AgentActionSchema) getSchema(RESULTNOTIFY);
            as.add(RESULTNOTIFY_COMPONENT, (ConceptSchema) getSchema(COMPONENT), ObjectSchema.OPTIONAL);
//    Structure of the schema for the REPORTSTATE action
            as = (AgentActionSchema) getSchema(REPORTSTATE);
            as.add(REPORTSTATE_TO, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(REPORTSTATE_THROUGHACL, (PrimitiveSchema) getSchema(BasicOntology.STRING),ObjectSchema.OPTIONAL);
            as.add(REPORTSTATE_INTERVAL, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
            as.add(REPORTSTATE_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER),ObjectSchema.OPTIONAL);
//    Structure of the schema for the REPORTRUNNUMBER action
            as = (AgentActionSchema) getSchema(REPORTRUNNUMBER);
            as.add(REPORTRUNNUMBER_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
//    Structure of the schema for the REPORTRUNTYPE action
            as = (AgentActionSchema) getSchema(REPORTRUNTYPE);
            as.add(REPORTRUNTYPE_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
//    Structure of the schema for the REPORTRUNSTATISTICS action
            as = (AgentActionSchema) getSchema(REPORTRUNSTATISTICS);
            as.add(REPORTRUNSTATISTICS_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
//    Structure of the schema for the REPORTSESSION action
            as = (AgentActionSchema) getSchema(REPORTSESSION);
            as.add(REPORTSESSION_TIMEOUT, (PrimitiveSchema) getSchema(BasicOntology.INTEGER));
//    Structure of the schema for the STARTCONFIG action
            as = (AgentActionSchema) getSchema(STARTCONFIG);
            as.add(STARTCONFIG_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(STARTCONFIG_SOURCE, (PrimitiveSchema) getSchema(BasicOntology.STRING));
            as.add(STARTCONFIG_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.MANDATORY);
            as.add(STARTCONFIG_FILENAME, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(STARTCONFIG_RCNODE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the RECORDNERUN action
            as = (AgentActionSchema) getSchema(RECORDNEWRUN);
            as.add(RECORDNEWRUN_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
            as.add(RECORDNEWRUN_RUNTYPE, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACCEPTRUNNUMBER action
            as = (AgentActionSchema) getSchema(ACCEPTRUNNUMBER);
            as.add(ACCEPTRUNNUMBER_VALUE, (PrimitiveSchema) getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
            as.add(ACCEPTRUNNUMBER_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the DISCONNECT action
            as = (AgentActionSchema) getSchema(DISCONNECT);
            as.add(DISCONNECT_SESSION, (PrimitiveSchema) getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACTIONMOVETO action
            as = (AgentActionSchema) getSchema(ACTIONMOVETO);
            as.add(ACTIONMOVETO_STATE, (ConceptSchema) getSchema(COOLSTATE), ObjectSchema.OPTIONAL);
//    Structure of the schema for the ACTIONEXECUTE action
            as = (AgentActionSchema) getSchema(ACTIONEXECUTE);
            as.add(ACTIONEXECUTE_PROCESS, (ConceptSchema) getSchema(PROCESS), ObjectSchema.OPTIONAL);
        }
        catch (OntologyException oe) {
            oe.printStackTrace();
        }
    }
}
