package org.jlab.coda.runcontrol.util.coolparser;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.graph.query.Predicate;

/**
 * @author: gurjyan
 * @date: Mar 18, 2008
 * @time: 11:19:04 AM
 */
public class CoolParserVer2 {

    Model m, m1, m2;
    String roc, eb, er, option, fileOutput, supervisor;
    String cool;

    // cool language specific resources
    Resource control;
    Resource component;
    Resource roccomponent1;
    Resource ebcomponent1;
    Resource ercomponent1;
    Resource option1;
    Resource fileOutput1;
    Resource supervisor1;

    // cool language specific properties
    Property hasComponent;
    Property hasIpc;
    Property hasType;
    Property hasName;
    Property hasPriority;

    public CoolParserVer2(){
        m1 = ModelFactory.createDefaultModel();
        m2 = ModelFactory.createDefaultModel();
        //N.B. in the URI "_" is not allowed
        cool = "http://COOLHOME/schema/cool#";
        roc = "http://COOLHOME/rocs#";
        eb = "http://COOLHOME/ebs#";
        er = "http://COOLHOME/ers#";
        option = "http://COOLHOME/options#";
        supervisor = "http://COOLHOME/rcs#";
        fileOutput = "http://COOLHOME/fo#";

        // cool language specific resources
        control = m1.createResource(cool+"Control");
        component = m1.createResource(cool+"Component");
        roccomponent1 = m2.createResource(roc+"ROC2");
        ebcomponent1 = m2.createResource(eb+"EB1");
        ercomponent1 = m2.createResource(er+"ER1");
        option1 = m2.createResource(option+"ER1");
        supervisor1 = m2.createResource(supervisor+"RC");

        // cool language specific properties
        hasComponent = m1.createProperty(cool+"hasComponent");
        hasIpc = m2.createProperty(cool+"hasIpc");
        hasType = m2.createProperty(cool+"hasType");
        hasName = m2.createProperty(cool+"hasName");
        hasPriority = m2.createProperty(cool+"hasPriority");

    }
    public static void main(String[] args){

        CoolParserVer2 myParser = new CoolParserVer2();
        myParser.createModel();
        myParser.parseModel();

    }

    public void createModel(){
        m1.add(control,hasComponent,roccomponent1).
                add(control,hasComponent,ebcomponent1);


        m2.add(roccomponent1,hasIpc,"dpsh").
                add(roccomponent1, hasType,"ROC").
                add(roccomponent1, hasName, "ROC2").
                add(roccomponent1, hasPriority,44).
                add(ebcomponent1,hasIpc,"dpsh").
                add(ebcomponent1, hasType,"EB").
                add(ebcomponent1, hasName, "EB1").
                add(ebcomponent1, hasPriority,33);


        m1.setNsPrefix("cool", cool);
        m1.setNsPrefix("roc", roc);
        m2.setNsPrefix("cool", cool);
        m2.setNsPrefix("roc", roc);

//        m1.write(System.out);
//        m2.write(System.out);

        m = m1.union(m2);
        m.setNsPrefix("cool", cool);
        m.setNsPrefix("roc", roc);

        m.write(System.out);

    }

    public void parseModel() {
        Resource subject=null;
        Property predicate=null;
        RDFNode object=null;

        StmtIterator iter = m.listStatements(subject, predicate, object);
         while (iter.hasNext()){
             Statement stmt = iter.nextStatement();
        System.out.println(stmt.getSubject() +"    "+stmt.getPredicate()+"    "+stmt.getObject());
    }
    }
}
