/*---------------------------------------------------------------------------*
*  Copyright (c) 2005        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.util.datatypes;

import org.jlab.coda.runcontrol.util.rcConstants;

/**
 * @author Vardan Gyurjyan
 *         Date: Jul 28, 2007
 *         Time: 8:15:02 AM
 */
public class CompMonitor {

    /** Instant event rate*/
    float evtrate_inst=0;
    /** Average event rate*/
    float evtrate_aver=0;
    /** Partial average event rate*/
    float evtrate_pa=0;
// array of evt rates
    public float[] evtR = new float[rcConstants.SMOOTH];


    /** Instant data rate*/
    double datarate_inst=0;
    /** Average data rate*/
    double datarate_aver=0;
    /** Partial average data rate*/
    double datarate_pa=0;
// array of data rates
public double[] dataR = new double[rcConstants.SMOOTH];

    /** Current event number*/
    int evtnum=0;
    /** Calculate average values after this many events*/
    int statistics;
    /** stores the sum of the evt rates*/
    float evtrate_sum;
    /** stores the sum of the data rates*/
    double datarate_sum;
    /** counts number of setEvtRate calls*/
    int evtraterecords = 1;
    /** counts number of setDataRate calls*/
    int dataraterecords = 1;



    /**
     * sets the value which will be used to calculate the average evt and data rates.
     * @param st
     */
    public void setAverageOver(int st){
        statistics = st;
    }

/** resets all the variables. This is used at go*/
    public void reset(){
    evtrate_inst=0;
    datarate_inst=0;
    evtrate_aver=0;
    datarate_aver=0;
    evtnum=0;
    evtrate_sum = 0;
    datarate_sum = 0;
    evtraterecords = 1;
    dataraterecords = 1;
}

    /**
     * Record the event rate and count the number of calls of this method.
     * If the number of calls is equal to the variable statistics then calculate
     * average value of the event rate.
     * @param evtrate
     */
    public void setEvtRate(float evtrate){
        evtrate_inst = evtrate;
    }

    public void setAverEvtRate(float evtrate){
        evtrate_aver = evtrate;
    }

    public void setPAEvtRate(float evtrate){
        evtrate_pa = evtrate;
    }


    /**
     * Record the data rate and count the number of calls of this method.
     * If the number of calls is equal to the variable statistics then calculate
     * average value of the data rate.
     * @param datarate
     */
    public void setDataRate(double datarate){
        datarate_inst = datarate;
    }

    public void setAverDataRate(double datarate){
        datarate_aver = datarate;
    }

    public void setPADataRate(double datarate){
        datarate_pa = datarate;
    }

    /**
     * Stores the event number
     * @param evtn
     */
    public void setEvtNumber(int evtn){
        evtnum = evtn;
    }

    /**
     * @return average data rate
     */
    public double getDataRate(){
//        return datarate_aver;
        return datarate_inst;
    }
    /**
     * @return Average data rate
     */
    public double getAverDataRate(){
        return datarate_aver;
    }

    /**
     * @return Average data rate
     */
    public double getPADataRate(){
        return datarate_pa;
    }


    /**
     * @return  average event rate
     */
    public float getEvtRate(){
//        return evtrate_aver;
        return evtrate_inst;
    }

    /**
     * @return  average event rate
     */
    public float getAverEvtRate(){
        return evtrate_aver;
    }

    /**
     * @return  average event rate
     */
    public float getPAEvtRate(){
        return evtrate_pa;
    }


    /**
     * @return recorded event number
     */
    public int getEvtNumber(){
        return evtnum;
    }

    public void arrayShift(){
        float tmpe[] = new float[rcConstants.SMOOTH];
        double tmpd[] = new double[rcConstants.SMOOTH];
        for (int i=0;i<rcConstants.SMOOTH-1;i++){
            tmpe[i+1] = evtR[i];
            tmpd[i+1] = dataR[i];
        }
        evtR = tmpe;
        dataR = tmpd;
    }

}
