/*---------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Carl Timmer                                                    *
*             timmer@jlab.org                   Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5130             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

// add methods updateSessionTable, updateRuntypeTable, recreateConfigTable,
// recreateSessionTable. etc. vhg 12-Mar-2004

package org.jlab.coda.runcontrol.util.db;

import java.sql.*;
import java.util.List;
import java.util.ArrayList;

/**
 * Communicate various SQL commands to a single instance (singleton)
 * via a JDBC database connection object.
 *
 * @author  Carl Timmer
 */
public class DatabaseCommunication {

    /** JDBC connection to the database. */
    private static Connection con;

    /** Runcontrol session. */
    private static String session;

    /** Set the runcontrol session relevant to these SQL queries. */
    public static void setSession(String s) {
        session = s;
    }

    /** Get the runcontrol session used for SQL queries. */
    public static String getSession() {
        return session;
    }

    /** A private constructor insures no instantiation of this class. */
    private DatabaseCommunication() {}

    /** Make sure a connection exists to the database. */
    private static void establishConnection() throws RCException {
        // Already got a connection
        if (con != null) return;

        // Get a db connection
        try {
            con = DatabaseConnection.getConnection();
        }
        catch (DBException ex) {
            throw new RCException("First establish a connection to the database with \"DatabaseConnection.createConnection(...)\" \n" +
                    ex.getMessage());
        }
    }

    /** Check to see if the session is known. */
    private static boolean sessionKnown() {
        if (session == null) {
            session = System.getenv("SESSION");
            if (session == null) {
                return false;
            }

        }
        return true;
    }


    /** Check if session is registered in the sessions table , and if not insert a record. */
    public static void updateSessionsTable(String session, String node) throws RCException {

        establishConnection();
        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT name FROM sessions WHERE name = '" + session + "'");
            if(!rs.next()) {

                stmt.executeUpdate("INSERT INTO sessions VALUES ( '"+session+
                        "',1,' "+node+" gurjyan 1925 102','yes','vg.log','runControl',1, ' ')");
            }
            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    /** Check if session is registered in the sessions table. */
    public static boolean checkSessionsTable(String session) throws RCException {
        boolean flag;
        establishConnection();
        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT name FROM sessions WHERE name = '" + session + "'");
            flag = rs.next();
            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
        return flag;
    }

    /** Check if runtype is registered in the runTypes table , and if not insert a record. */
    public static void updaterunTypesTable(String runtype) throws RCException {

        establishConnection();
        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT name FROM runTypes WHERE name = '" + runtype + "'");
            if(!rs.next()) {
                stmt.executeUpdate("INSERT INTO runTypes VALUES ( '"+runtype+"',0,'yes', ' ')");
                stmt.executeUpdate("UPDATE sessions SET config = '"+runtype+"' WHERE name = '"+session+"'");
            }
            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    /** Check if runtype is registered in the runTypes table */
    public static boolean checkRunTypesTable(String runtype) throws RCException {
        boolean flag;

        establishConnection();
        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT name FROM runTypes WHERE name = '" + runtype + "'");
            flag = rs.next();
            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
        return flag;
    }


    public static List getRuntypes(){
// Reads the runtypes table and returns all the runtypes from the db.
        List<String> l = new ArrayList<String>();
        try {
            establishConnection();
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT * FROM runTypes");
            while(rs.next()) {
                String tmps =rs.getString("name").trim();
//                System.out.println(tmps);
                l.add(tmps);
            }
            rs.close();
            stmt.close();
        } catch (RCException e) {
            e.printStackTrace();
        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return l;
    }

    /** Recreate config table with the name runType */
    public static void recreateConfigTable(String runtype) throws RCException {

        Statement stmt = null;
        boolean tableexist = false;
        establishConnection();
        try {
            DatabaseMetaData dbmd = con.getMetaData();
            String[]types = {"TABLE"};
            ResultSet rst = dbmd.getTables(null,null,"%",types);
            while(rst.next()){
                String tableName = rst.getString(3).trim();
                if(tableName.equals(runtype)) tableexist = true;
            }
            rst.close();
            if(!tableexist){

                stmt = con.createStatement();
                stmt.executeUpdate("CREATE TABLE "+runtype+" (name CHAR(32) NOT NULL PRIMARY KEY," +
                        "code CHAR(512) NOT NULL, inputs CHAR(400) NOT NULL,outputs CHAR(400) NOT NULL," +
                        "first CHAR(32) NOT NULL,next CHAR(32) NOT NULL,inuse CHAR(32) NOT NULL)");
                stmt.close();
            }
            else {
                stmt = con.createStatement();
                stmt.executeUpdate("DROP TABLE "+runtype);
                stmt.executeUpdate("CREATE TABLE "+runtype+" (name CHAR(32) NOT NULL PRIMARY KEY," +
                        "code CHAR(512) NOT NULL, inputs CHAR(400) NOT NULL,outputs CHAR(400) NOT NULL," +
                        "first CHAR(32) NOT NULL,next CHAR(32) NOT NULL,inuse CHAR(32) NOT NULL)");
                stmt.close();
            }
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    /** Recreate options table for required runtype */
    public static void recreateOptionsTable(String runtype) throws RCException {

        Statement stmt = null;
        boolean tableexist = false;
        establishConnection();
        try {
            DatabaseMetaData dbmd = con.getMetaData();
            String[]types = {"TABLE"};
            ResultSet rst = dbmd.getTables(null,null,"%",types);
            while(rst.next()){
                String tableName = rst.getString(3).trim();
                if(tableName.equals(runtype+"_option")) tableexist = true;
            }
            rst.close();
            if(!tableexist){
                stmt = con.createStatement();
                stmt.executeUpdate("CREATE TABLE "+runtype+"_option (name CHAR(32) NOT NULL," +
                        "value CHAR(80) NOT NULL) ");
                stmt.close();
            }
            else {
//                stmt = con.createStatement();
//                stmt.executeUpdate("DROP TABLE "+runtype+"_option");
//                stmt.executeUpdate("CREATE TABLE "+runtype+"_option (name CHAR(32) NOT NULL," +
//                        "value CHAR(80) NOT NULL) ");
//                stmt.close();
            }
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    public static boolean checkComponent(String component) throws RCException {
        boolean found = false;
        establishConnection();
        try {
            Statement stm = con.createStatement();
            ResultSet rs = stm.executeQuery("SELECT * FROM process");
            while(rs.next()) {
                String s = rs.getString("name");
                if(s.trim().equals(component)) found = true;
            }
            stm.close();
/*
            if(!found){
                stm = con.createStatement();
                String sql = "INSERT INTO process VALUES( ' "+component+
                        "', 0, 'none', '"+type+"', 'none', 0, 'none', 0, 'none', 'none ' )";
                stm.executeUpdate(sql);
            }
*/
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return found;
    }

    /** Execute query */
    public static void execDBQuery(String query) throws RCException {

        establishConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.executeQuery(query);
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    /** Execute update */
    public static void execDBUpdate(String query) throws RCException {

        establishConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.executeUpdate(query);
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    /** This methods gets the runNumber from the database "sessions" table for this session. */
    public static int getRunNumber() throws RCException {
        int runNumber = 0;

        if (!sessionKnown()) {
            throw new RCException("Need to define SESSION environmental variable " +
                    "or call the \"DatabaseCommunication.setSession\" method");
        }

        establishConnection();

        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT runNumber FROM sessions WHERE name = '" + session + "'");
            while (rs.next()) {
                runNumber = rs.getInt("runNumber");
                break;
            }
            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }

        return runNumber;
    }

    /** This methods gets the runNumber from the database "sessions" table for the given session. */
    public static int getRunNumber(String sess) throws RCException {
        int runNumber = 0;
        establishConnection();
        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT runNumber FROM sessions WHERE name = '" + sess + "'");
            while (rs.next()) {
                runNumber = rs.getInt("runNumber");
                break;
            }
            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }

        return runNumber;
    }

    /** This method sets the runNumber in the database "sessions" table for this session. */
    public static void setRunNumber(int runNumber) throws RCException {
        if (!sessionKnown()) {
            throw new RCException("Need to define SESSION environmental variable " +
                    "or call the \"DatabaseCommunication.setSession\" method");
        }
        establishConnection();

        try {
            Statement stmt = con.createStatement();
            stmt.executeUpdate("UPDATE sessions SET runNumber = " + runNumber +
                    "WHERE name = '" + session + "'");
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }

        return;
    }

    /** This method sets the runNumber in the database "sessions" table for given session. */
    public static void setRunNumber(int runNumber, String sess) throws RCException {
        establishConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.executeUpdate("UPDATE sessions SET runNumber = " + runNumber +
                    " WHERE name = '" + sess + "'");
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }

        return;
    }
    /** This method sets the config in the database "sessions" table for given session. */
    public static void setSessionConfig(String runtype, String sess) throws RCException {
        establishConnection();
        try {
            Statement stmt = con.createStatement();
            stmt.executeUpdate("UPDATE sessions SET config = '" + runtype +
                    "' WHERE name = '"+sess+"'");
            stmt.close();
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }

        return;
    }


}
