/*---------------------------------------------------------------------------*
*  Copyright (c) 2004        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.util.db;

import org.jlab.coda.runcontrol.util.rcConstants;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.sql.*;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class Db2Cool {
    /** JDBC connection to the database. */
    private static Connection con;
    private String session;
    private static String coolhome;
    private InetAddress addr;
    private String ipaddr;
    private DbComponent myRCComponent;
    // is not used for now. the host name of the main platform will be taken as a
    // parameter to this program
    private static String ROCCONTAINERHOST;
    private static String SLCONTAINERHOST;

    public Db2Cool(){
        String l = null;
        session = System.getenv("SESSION");
        if (session==null)System.err.println("SESSION environmental variable is not defined");
        coolhome = System.getenv("COOL_HOME");
        if (coolhome==null){
            System.err.println("COOL_HOME environmental variable is not defined");
            coolhome = "./";
        }
        try {
            addr = InetAddress.getLocalHost();
            ipaddr = addr.getHostAddress();
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }
        l = addr.getHostName();
//        MAINCONTAINERHOST = l.substring(0,l.lastIndexOf(".jlab.org"));
    }

    public static void main(String[] args){
        String tmps, tmpruntype = "all";
        List l;

        Db2Cool mydbcool = new Db2Cool();

        if(args.length==1 && ((args[0].equalsIgnoreCase("-h"))||(args[0].equalsIgnoreCase("-help")))){
            System.out.println("\nusage:\n\ndb2cool [-runtype coda_runtype] " +
                    "[-roccontainer roc_agents_host_name] [-slcontainer slcontrol_agents_host_name]" +
                    " [-o cool_output_dir]\n" );
            System.exit(0);
        } else if(args.length==2){
            if(args[0].equalsIgnoreCase("-runtype")) tmpruntype = args[1];
            else {
                System.out.println("\nusage:\n\ndb2cool [-runtype coda_runtype] " +
                        "[-roccontainer roc_agents_host_name] [-slcontainer slcontrol_agents_host_name] [-o cool_output_dir]\n" );
                System.exit(0);
            }
            mydbcool.getDbConnection();
            l = mydbcool.getRuntypes();
            Iterator it = l.iterator();
            if(tmpruntype.equalsIgnoreCase("all")){
                while (it.hasNext()){
                    tmps = (String)it.next();
                    System.out.println("cool: "+tmps+" "+coolhome);
                    mydbcool.generateCool(tmps);
                }
            } else {
                System.out.println("cool: "+tmpruntype);
                mydbcool.generateCool(tmpruntype);
            }
        } else if(args.length==4){
            if(args[0].equalsIgnoreCase("-runtype")) tmpruntype = args[1];
            else {
                System.out.println("\nusage:\n\ndb2cool [-runtype coda_runtype] " +
                        "[-roccontainer roc_agents_host_name] [-slcontainer slcontrol_agents_host_name] [-o cool_output_dir]\n" );
                System.exit(0);
            }
            if(args[2].equalsIgnoreCase("-roccontainer")) ROCCONTAINERHOST = args[3];
            else if(args[2].equalsIgnoreCase("-slcontainer")) SLCONTAINERHOST = args[3];
            else if(args[2].equalsIgnoreCase("-o")) coolhome = args[3];
            System.out.println("DDD... "+coolhome);

            mydbcool.getDbConnection();
            l = mydbcool.getRuntypes();
            Iterator it = l.iterator();
            if(tmpruntype.equalsIgnoreCase("all")){
                while (it.hasNext()){
                    tmps = (String)it.next();
                    System.out.println("cool: "+tmps+" "+coolhome);
                    mydbcool.generateCool(tmps);
                }
            } else {
                System.out.println("cool: "+tmpruntype);
                mydbcool.generateCool(tmpruntype);
            }
        } else if(args.length==6){
            if(args[0].equalsIgnoreCase("-runtype")) tmpruntype = args[1];
            else {
                System.out.println("\nusage:\n\ndb2cool [-runtype coda_runtype] " +
                        "[-roccontainer roc_agents_host_name] [-slcontainer slcontrol_agents_host_name] [-o cool_output_dir]\n" );
                System.exit(0);
            }
            if(args[2].equalsIgnoreCase("-roccontainer")) ROCCONTAINERHOST = args[3];
            else if(args[2].equalsIgnoreCase("-slcontainer")) SLCONTAINERHOST = args[3];
            else if(args[2].equalsIgnoreCase("-o")) coolhome = args[3];
            if(args[4].equalsIgnoreCase("-roccontainer")) ROCCONTAINERHOST = args[5];
            else if(args[4].equalsIgnoreCase("-slcontainer")) SLCONTAINERHOST = args[5];
            else if(args[4].equalsIgnoreCase("-o")) coolhome = args[5];
            mydbcool.getDbConnection();
            l = mydbcool.getRuntypes();
            Iterator it = l.iterator();
            if(tmpruntype.equalsIgnoreCase("all")){
                while (it.hasNext()){
                    tmps = (String)it.next();
                    System.out.println("cool: "+tmps);
                    mydbcool.generateCool(tmps);
                }
            } else {
                System.out.println("cool: "+tmpruntype);
                mydbcool.generateCool(tmpruntype);
            }
        } else if(args.length==8){
            if(args[0].equalsIgnoreCase("-runtype")) tmpruntype = args[1];
            else {
                System.out.println("\nusage:\n\ndb2cool [-runtype coda_runtype] " +
                        "[-roccontainer roc_agents_host_name] [-slcontainer slcontrol_agents_host_name] [-o cool_output_dir]\n" );
                System.exit(0);
            }
            if(args[2].equalsIgnoreCase("-roccontainer")) ROCCONTAINERHOST = args[3];
            else if(args[2].equalsIgnoreCase("-slcontainer")) SLCONTAINERHOST = args[3];
            else if(args[2].equalsIgnoreCase("-o")) coolhome = args[3];
            if(args[4].equalsIgnoreCase("-roccontainer")) ROCCONTAINERHOST = args[5];
            else if(args[4].equalsIgnoreCase("-slcontainer")) SLCONTAINERHOST = args[5];
            else if(args[4].equalsIgnoreCase("-o")) coolhome = args[5];
            if(args[6].equalsIgnoreCase("-roccontainer")) ROCCONTAINERHOST = args[7];
            else if(args[6].equalsIgnoreCase("-slcontainer")) SLCONTAINERHOST = args[7];
            else if(args[6].equalsIgnoreCase("-o")) coolhome = args[7];
            mydbcool.getDbConnection();
            l = mydbcool.getRuntypes();
            Iterator it = l.iterator();
            if(tmpruntype.equalsIgnoreCase("all")){
                while (it.hasNext()){
                    tmps = (String)it.next();
                    System.out.println("cool: "+tmps);
                    mydbcool.generateCool(tmps);
                }
            } else {
                System.out.println("cool: "+tmpruntype);
                mydbcool.generateCool(tmpruntype);
            }
        } else {
            System.out.println("\nusage:\n\ndb2cool [-runtype coda_runtype] " +
                    "[-roccontainer roc_agents_host_name] [-slcontainer slcontrol_agents_host_name] [-o cool_output_dir]\n" );
            System.exit(0);
        }
    }


    public void getDbConnection(){
// Make connection to the database.
        String url = System.getenv("CODA_DB_URL");
        String driver = System.getenv("CODA_DB_DRIVER");
        String usr = System.getenv("CODA_DB_USER");
        String passwd = System.getenv("CODA_DB_PASSWORD");

        if(usr==null)usr=" ";
        if(passwd==null)passwd=" ";
        try {
            DatabaseConnection.createConnection(url,driver,usr,passwd);
        } catch (DBException e) {
            e.printStackTrace();
        }
    }

    /** Make sure a connection exists to the database. */
    private static void connect() throws RCException {
        // Already got a connection
        if (con != null) return;

        // Get a db connection
        try {
            con = DatabaseConnection.getConnection();
        }
        catch (DBException ex) {
            throw new RCException("First establish a connection to the database with \"DatabaseConnection.createConnection(...)\" \n" +
                    ex.getMessage());
        }
    }

    public List getRuntypes(){
// Reads the runtypes table and returns all the runtypes from the db.
        List<String> l = new ArrayList<String>();
        try {
            connect();
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT * FROM runTypes");
            while(rs.next()) {
                String tmps =rs.getString("name").trim();
//                System.out.println(tmps);
                l.add(tmps);
            }
            rs.close();
            stmt.close();
        } catch (RCException e) {
            e.printStackTrace();
        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return l;
    }

    public List getDbComponents(String cnf){
        List<DbComponent> l = new ArrayList<DbComponent>();
        DbComponent mycnf;
        String tmpname, tmpprocname, tmptype, tmpstate, tmpscript;
        Statement stmt;
        Statement stmt2;
        ResultSet rs;
        ResultSet rs2;
        boolean tableexist = false;
        myRCComponent = new DbComponent();
        myRCComponent.setName(cnf);
        try {
            connect();
// see if the cnf configuration table exist
            DatabaseMetaData dbmd = con.getMetaData();
            String[]types = {"TABLE"};
            ResultSet rst = dbmd.getTables(null,null,"%",types);
            while(rst.next()){
                String tableName = rst.getString(3).trim();
//                System.out.println("*"+tableName+"*"+cnf+"*");
                if(tableName.equals(cnf)) tableexist = true;
            }
            rst.close();
            if(tableexist){
                stmt = con.createStatement();
//            Statement stmt2 = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                rs = stmt.executeQuery("SELECT * FROM "+cnf);
//            rs2 = stmt2.executeQuery("SELECT * FROM process");
                while(rs.next()){
                    mycnf = new DbComponent();
                    tmpname = rs.getString("name").trim();
                    mycnf.setName(tmpname);
                    mycnf.setCode(rs.getString("code"));
                    mycnf.setInputs(rs.getString("inputs"));
                    mycnf.setOutputs(rs.getString("outputs"));
                    mycnf.setFirst(rs.getString("first"));
                    mycnf.setNext(rs.getString("next"));
                    stmt2 = con.createStatement();
                    rs2 = stmt2.executeQuery("SELECT * FROM process");
                    while(rs2.next()){
                        tmpprocname = rs2.getString("name").trim();
                        if(tmpprocname.equals(tmpname)){
                            tmptype=rs2.getString("type").trim();
                            if(tmptype != null) {
                                mycnf.setType(tmptype);
                                if(tmptype.equalsIgnoreCase(rcConstants.analysisType))mycnf.setPriority(11);
                                else if(tmptype.equalsIgnoreCase(rcConstants.erType))mycnf.setPriority(22);
                                else if(tmptype.equalsIgnoreCase(rcConstants.ebType))mycnf.setPriority(33);
                                else if(tmptype.equalsIgnoreCase(rcConstants.rocType))mycnf.setPriority(44);
                                else if(tmptype.equalsIgnoreCase(rcConstants.slowcontrolType))mycnf.setPriority(55);
                                else if(tmptype.equalsIgnoreCase(rcConstants.tsType))mycnf.setPriority(66);
                                else if(tmptype.equalsIgnoreCase(rcConstants.rcSupervisorType))mycnf.setPriority(77);
                            } else System.err.println( "type is not defined for the component "+" "+tmpprocname);
                        }
                    }
                    rs2.close();
                    stmt2.close();


// see if the script table for particular configuration exist
                    tableexist = false;
                    dbmd = con.getMetaData();
                    rst = dbmd.getTables(null,null,"%",types);
                    while(rst.next()){
                        String tableName = rst.getString(3).trim();
                        if(tableName.equals(cnf+"_script")) tableexist = true;
                    }
                    rst.close();
                    if(tableexist){
                        stmt2 = con.createStatement();
                        rs2 = stmt2.executeQuery("SELECT * FROM "+cnf+"_script");
                        while(rs2.next()){
                            tmpprocname = rs2.getString("name").trim();
                            if(tmpprocname.equals(tmpname)){
                                tmpstate=rs2.getString("state").trim();
                                tmpscript=rs2.getString("script").trim();
// System.out.println(tmpname+" "+tmpstate+" "+tmpscript);
                                if(tmpstate.equalsIgnoreCase("download"))mycnf.setSrcPredownload(tmpscript);
                                if(tmpstate.equalsIgnoreCase("prestart"))mycnf.setSrcPreprestart(tmpscript);
                                if(tmpstate.equalsIgnoreCase("go"))mycnf.setSrcPrego(tmpscript);
                                if(tmpstate.equalsIgnoreCase("end"))mycnf.setSrcPreend(tmpscript);
                            }
                        }
                        rs2.close();
                        stmt2.close();
//                rs2.first();
                    }
//                    System.out.println(mycnf.getName());
                    l.add(mycnf);
                }
                rs.close();
                stmt.close();
            }
        } catch (RCException e) {
            e.printStackTrace();
        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return l;
    }

    public DbOptions getDbOptions(String cnf){
        String tmpname;
        boolean tableexist = false;

        DbOptions myopt = new DbOptions();
        try {
            connect();
// see if the cnf configuration table exist
            DatabaseMetaData dbmd = con.getMetaData();
            String[]types = {"TABLE"};
            ResultSet rst = dbmd.getTables(null,null,"%",types);
            while(rst.next()){
                String tableName = rst.getString(3).trim();
                if(tableName.equals(cnf)) tableexist = true;
            }
            rst.close();
            if(tableexist){
                Statement stmt = con.createStatement();
                ResultSet rs;
                rs = stmt.executeQuery("SELECT * FROM "+cnf+"_option");
                while(rs.next()){
                    tmpname = rs.getString("name").trim();
                    if(tmpname.equalsIgnoreCase("dataLimit")) myopt.setOptDatalimit(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("eventLimit")) myopt.setOptEventlimit(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("tokenInterval")) myopt.setOptTokeninterval(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("SPLITMB")) myopt.setOptSplitmb(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("dataFile")) myopt.setOptDatafile(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("Boot")) myRCComponent.setSrcPostconfigure(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("Download")) myRCComponent.setSrcPostdownload(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("Prestart")) myRCComponent.setSrcPostprestart(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("Go")) myRCComponent.setSrcPostgo(rs.getString("value").trim());
                    else if(tmpname.equalsIgnoreCase("End")) myRCComponent.setSrcPostend(rs.getString("value").trim());
                }
                stmt.close();
                rs.close();
            }
        } catch (RCException e) {
            e.printStackTrace();
        }
        catch (SQLException ex) {
            ex.printStackTrace();
        }
        return myopt;
    }

    public void generateCool(String cnf){
        List components;
        DbOptions opt;
        DbComponent tmpcomp;
        try{
            components = getDbComponents(cnf);
            if(components == null) {
                System.err.println("Error: DB contains null components for runtype = "+cnf);
                return;
            }
            Iterator it = components.iterator();
            opt = getDbOptions(cnf);
            if(opt == null) System.err.println("Warning: DB contains null options table for runtype = "+cnf);

            System.out.println("DDD... "+coolhome);

            BufferedWriter out = new BufferedWriter(new FileWriter(coolhome+"/"+cnf+".rdf"));
            out.write("<rdf:RDF\n");
            out.write("xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"\n");
            out.write("xmlns:cool=\"http://COOLHOME/schema/cool#\"\n");
            out.write(">\n");
            out.write("\n");
            out.write("    <!-- Convention\n" +
                    "            The cool configuration name, the control name and\n" +
                    "            the name of the RC component should be the same.\n" +
                    "            This name defines the runtype of the experiment.\n" +
                    "    -->");
            out.write("\n");
        out.write("<rdf:Description rdf:about=\"http://COOLHOME/schema/cool#Control\">\n");

//            out.write("<cool:Control rdf:ID=\""+cnf+"\">\n");

            while(it.hasNext()){
                tmpcomp = (DbComponent)it.next();
                out.write("    <cool:hasComponent rdf:resource=\"#" +tmpcomp.getName().toUpperCase()+ "\"/>\n");
            }
            out.write("    <cool:hasComponent rdf:resource=\"#RC\"/>\n");
            out.write("    <cool:hasCodaOption rdf:resource=\"#OPTION\"/>\n");

            out.write("</rdf:Description>\n");
            
//            out.write("</cool:Control>\n");

            out.write("\n");
            if(opt!=null ){
                out.write("<cool:Option rdf:ID=\"OPTION\">\n");
                out.write("    <cool:hasCoda2Component>true</cool:hasCoda2Component>\n");
                if(opt.getOptDatalimit()!=null)
                    out.write("    <cool:hasDataLimit>"+opt.getOptDatalimit().trim()+"</cool:hasDataLimit>\n");
                if(opt.getOptEventlimit()!=null)
                    out.write("    <cool:hasEventLimit>"+opt.getOptEventlimit().trim()+"</cool:hasEventLimit>\n");
                if(opt.getOptTokeninterval()!=null)
                    out.write("    <cool:hasTokenInterval>"+opt.getOptTokeninterval().trim()+"</cool:hasTokenInterval>\n");
                if(opt.getOptDatafile()!=null)
                    out.write("    <cool:hasDataFile>"+opt.getOptDatafile().trim()+"</cool:hasDataFile>\n");
                if(opt.getOptSplitmb()!=null)
                    out.write("    <cool:Splitmb>"+opt.getOptSplitmb().trim()+"</cool:Splitmb>\n");
                out.write("</cool:Option>\n");
            }
            out.write("\n");
            it = components.iterator();
            while(it.hasNext()){
                tmpcomp = (DbComponent)it.next();
// file component
                if((tmpcomp.getName().startsWith("file"))||(tmpcomp.getName().startsWith("coda"))){
                    out.write("<cool:Component rdf:ID=\""+tmpcomp.getName().toUpperCase()+"\">\n");
                    out.write("    <cool:hasIpc>cmsg</cool:hasIpc>\n");
                    out.write("    <cool:hasType>FILE</cool:hasType>\n");
                    out.write("    <cool:hasName>"+tmpcomp.getName()+"</cool:hasName>\n");
                    out.write("    <cool:hasPriority>888</cool:hasPriority>\n");
                    if(tmpcomp.getCode()!=null)
                        out.write("    <cool:hasCode>"+tmpcomp.getCode().trim()+"</cool:hasCode>\n");
                    out.write("</cool:Component>\n");
                    out.write("\n");
                }
                else {
// coda components
                    if(tmpcomp.getName()!=null)
                        out.write("<cool:Component rdf:ID=\""+tmpcomp.getName().toUpperCase()+"\">\n");
                    // components in the database does not have IPC definition
                    out.write("    <cool:hasIpc>dpsh</cool:hasIpc>\n");
                    out.write("    <cool:representsCoda2Client>true</cool:representsCoda2Client>\n");
                    if(tmpcomp.getType()!=null)
                        out.write("    <cool:hasType>"+tmpcomp.getType().trim()+"</cool:hasType>\n");
                    if(tmpcomp.getName()!=null)
                        out.write("    <cool:hasName>"+tmpcomp.getName().trim()+"</cool:hasName>\n");
                    if(tmpcomp.getCode()!=null)
                        out.write("    <cool:hasCode>"+tmpcomp.getCode().trim()+"</cool:hasCode>\n");
                    if(tmpcomp.getPriority()!=0)
                        out.write("    <cool:hasPriority>"+tmpcomp.getPriority()+"</cool:hasPriority>\n");
                    if((tmpcomp.getSrcPreconfigure()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Preconfigure_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcconfigure()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Configure_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcPostconfigure()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Postconfigure_"+tmpcomp.getName()+"\"/>\n");
                    }

                    if((tmpcomp.getSrcPredownload()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Predownload_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcdownload()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Download_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcPostdownload()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Postdownload_"+tmpcomp.getName()+"\"/>\n");
                    }

                    if((tmpcomp.getSrcPreprestart()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Preprestart_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcprestart()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Prestart_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcPostprestart()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Postprestart_"+tmpcomp.getName()+"\"/>\n");
                    }

                    if((tmpcomp.getSrcPrego()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Prego_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcgo()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Go_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcPostgo()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Postgo_"+tmpcomp.getName()+"\"/>\n");
                    }

                    if((tmpcomp.getSrcPreend()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Preend_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcend()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#End_"+tmpcomp.getName()+"\"/>\n");
                    }
                    if((tmpcomp.getSrcPostend()!=null)) {
                        out.write("    <cool:hasExternalProcess rdf:resource=\"#Postend_"+tmpcomp.getName()+"\"/>\n");
                    }
                    // Node is where the agent will physically run, i.e. container node
                    // node name and IP address of the node where from this program is called.
//                    System.out.println(tmpcomp+" "+tmpcomp.getName()+" "+tmpcomp.getType());
                    if((tmpcomp.getType().equalsIgnoreCase(rcConstants.rocType)) && (ROCCONTAINERHOST!=null)){
                        out.write("    <cool:hasNode rdf:resource=\"#ND_ROC\"/>\n");
                        out.write("</cool:Component>\n");
                        out.write("\n");
                    }  else  if((tmpcomp.getType().equalsIgnoreCase(rcConstants.slowcontrolType)) && (SLCONTAINERHOST!=null)){
                        out.write("    <cool:hasNode rdf:resource=\"#ND_SLC\"/>\n");
                        out.write("</cool:Component>\n");
                        out.write("\n");
                    }
                    else {
                        out.write("</cool:Component>\n");
                        out.write("\n");
                    }
                }
                if((tmpcomp.getSrcPreconfigure()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Preconfigure_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>preconfigure</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPreconfigure()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcconfigure()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Configure_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>configure</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcconfigure()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcPostconfigure()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Postconfigure_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>postconfigure</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPostconfigure()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }

                if((tmpcomp.getSrcPredownload()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Predownload_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>predownload</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPredownload()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcdownload()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Download_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>download</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcdownload()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcPostdownload()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Postdownload_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>postdownload</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPostdownload()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }

                if((tmpcomp.getSrcPreprestart()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Preprestart_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>preprestart</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPreprestart()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcprestart()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Prestart_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>prestart</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcprestart()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcPostprestart()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Postprestart_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>postprestart</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPostprestart()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }

                if((tmpcomp.getSrcPrego()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Prego_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>prego</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPrego()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcgo()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Go_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>go</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcgo()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcPostgo()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Postgo_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>postgo</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPostgo()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }

                if((tmpcomp.getSrcPreend()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Preend_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>preend</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPreend()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcend()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"End_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>end</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcend()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                if((tmpcomp.getSrcPostend()!=null)) {
// Process description
                    out.write("   <cool:Process rdf:ID=\"Postend_"+tmpcomp.getName()+"\">\n");
                    out.write("       <cool:isPartOfState>postend</cool:isPartOfState>\n");
                    out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                    out.write("       <cool:hasCommandName>"+tmpcomp.getSrcPostend()+"</cool:hasCommandName>\n");
                    out.write("   </cool:Process>\n");
                    out.write("\n");
                }
                out.write("\n");
            }

// runcontrol component
            out.write("<cool:Component rdf:ID=\"RC\">\n");
            out.write("    <cool:hasIpc>cmsg</cool:hasIpc>\n");
            out.write("    <cool:hasType>RCS</cool:hasType>\n");
            out.write("    <cool:hasName>"+cnf+"</cool:hasName>\n");
            out.write("    <cool:hasPriority>77</cool:hasPriority>\n");
            if((myRCComponent.getSrcPreconfigure()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Preconfigure_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcconfigure()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Configure_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcPostconfigure()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Postconfigure_"+myRCComponent.getName()+"\"/>\n");
            }

            if((myRCComponent.getSrcPredownload()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Predownload_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcdownload()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Download_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcPostdownload()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Postdownload_"+myRCComponent.getName()+"\"/>\n");
            }

            if((myRCComponent.getSrcPreprestart()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Preprestart_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcprestart()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Prestart_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcPostprestart()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Postprestart_"+myRCComponent.getName()+"\"/>\n");
            }

            if((myRCComponent.getSrcPrego()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Prego_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcgo()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Go_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcPostgo()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Postgo_"+myRCComponent.getName()+"\"/>\n");
            }

            if((myRCComponent.getSrcPreend()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Preend_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcend()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#End_"+myRCComponent.getName()+"\"/>\n");
            }
            if((myRCComponent.getSrcPostend()!=null)) {
                out.write("    <cool:hasExternalProcess rdf:resource=\"#Postend_"+myRCComponent.getName()+"\"/>\n");
            }
            out.write("</cool:Component>\n");
            out.write("\n");
// Runcontrol (thalamus) related processes
            if((myRCComponent.getSrcPreconfigure()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Preconfigure_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>preconfigure</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPreconfigure()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcconfigure()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Configure_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>configure</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcconfigure()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcPostconfigure()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Postconfigure_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>postconfigure</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPostconfigure()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }

            if((myRCComponent.getSrcPredownload()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Predownload_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>predownload</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPredownload()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcdownload()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Download_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>download</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcdownload()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcPostdownload()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Postdownload_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>postdownload</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPostdownload()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }

            if((myRCComponent.getSrcPreprestart()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Preprestart_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>preprestart</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPreprestart()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcprestart()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Prestart_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>prestart</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcprestart()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcPostprestart()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Postprestart_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>postprestart</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPostprestart()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }

            if((myRCComponent.getSrcPrego()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Prego_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>prego</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPrego()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcgo()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Go_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>go</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcgo()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcPostgo()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Postgo_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>postgo</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPostgo()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }

            if((myRCComponent.getSrcPreend()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Preend_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>preend</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPreend()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcend()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"End_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>end</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcend()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }
            if((myRCComponent.getSrcPostend()!=null)) {
// Process description
                out.write("   <cool:Process rdf:ID=\"Postend_"+myRCComponent.getName()+"\">\n");
                out.write("       <cool:isPartOfState>postend</cool:isPartOfState>\n");
                out.write("       <cool:hasProcessIpc>shell</cool:hasProcessIpc>\n");
                out.write("       <cool:hasCommandName>"+myRCComponent.getSrcPostend()+"</cool:hasCommandName>\n");
                out.write("   </cool:Process>\n");
                out.write("\n");
            }

            if(ROCCONTAINERHOST!=null){
// Node on which ROC component is running.
                out.write("<cool:Node rdf:ID=\"ND_ROC\">\n");
                out.write("    <cool:hasNodeName>"+ROCCONTAINERHOST +"</cool:hasNodeName>\n");
//                out.write("    <cool:hasIP>"+ipaddr+"</cool:hasIP>\n");
                out.write("    <cool:isLocal>yes</cool:isLocal>\n");
                out.write("</cool:Node>\n");
                out.write("\n");
            }
            if(SLCONTAINERHOST!=null){
// Node on which ROC component is running.
                out.write("<cool:Node rdf:ID=\"ND_ROC\">\n");
                out.write("    <cool:hasNodeName>"+SLCONTAINERHOST +"</cool:hasNodeName>\n");
//                out.write("    <cool:hasIP>"+ipaddr+"</cool:hasIP>\n");
                out.write("    <cool:isLocal>yes</cool:isLocal>\n");
                out.write("</cool:Node>\n");
                out.write("\n");
            }
//            out.write("</cool:Component>\n");
            out.write("\n");
            out.write("</rdf:RDF>\n");

            out.close();
        } catch(IOException e){
            e.printStackTrace();
        }
    }
}
