/*---------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Carl Timmer                                                    *
*             timmer@jlab.org                   Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5130             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package org.jlab.coda.runcontrol.util.db;

import java.sql.*;

/**
 * This class corresponds to an entry in the "process" table in the
 * database containing runcontrol information. Each table named after
 * the runType has a list of the components from the "process" table
 * that it uses. This class describes the characteristics of a
 * runcontrol computer process.
 *
 * @author Carl Timmer
 */
public class DbRunComponent extends RunComponent {
    // JDBC connection to the database
    private Connection con;

    public DbRunComponent(String componentName) throws RCException {
        setName(componentName);
        try {
            con = DatabaseConnection.getConnection();
        }
        catch (DBException ex) {
            throw new RCException(ex.getMessage());
        }
        getData();
    }

    public DbRunComponent(String componentName, Connection connection) throws RCException {
        setName(componentName);
        con = connection;
        getData();
    }

    // get data from the database "process" table
    public void getData() throws RCException {
        try {
            Statement stmt = con.createStatement();
            ResultSet rs;
            rs = stmt.executeQuery("SELECT * FROM process WHERE name = '" + getName() + "'");
            while (rs.next()) {
                setCommand(rs.getString("cmd"));
                setType(rs.getString("type"));
                setState(rs.getString("state"));
                setHost(rs.getString("host"));
                setPort(rs.getInt("port"));
                setId(rs.getInt("id"));
                setPid(rs.getInt("pid"));
                setIsInUse(rs.getString("inuse"));
            }
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }
    }

}
