/*---------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Carl Timmer                                                    *
*             timmer@jlab.org                   Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5130             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package org.jlab.coda.runcontrol.util.db;

import org.jlab.coda.runcontrol.util.rcConstants;

import java.util.*;
import java.sql.*;

/**
 * This class stores data obtained from the runcontrol database.
 *
 * @author Carl Timmer
 */
public class DbRunTypeData implements DataRetriever {
    private Connection con;

    public DbRunTypeData() throws RCException {
        try {
            con = DatabaseConnection.getConnection();
        }
        catch (DBException ex) {
            throw new RCException(ex.getMessage());
        }
    }

    public RunTypeData getData(String runType) throws RCException {

        RunTypeData rtd = new RunTypeData();

        try {
            // Obtain data from "runType" db table
            String s;
            Statement stmt = con.createStatement();
            ResultSet rs = stmt.executeQuery("SELECT inuse FROM runTypes WHERE name = '"
                    + runType + "'");
            while (rs.next()) {
                s = rs.getString("inuse");
                if (s.equalsIgnoreCase("no")) {
                    rtd.setIsInUse(false);
                }
                else {
                    rtd.setIsInUse(true);
                }
            }

            // Obtain data from the $(RUNTYPE) table and the "process" table
            // -- a list of run components -- and  store it in a hash table.
            String type;
            DbRunComponent rcomp;
            stmt = con.createStatement();
            rs = stmt.executeQuery("SELECT name FROM " + runType);
            while (rs.next()) {
                s  = rs.getString("name");
                // Constructing this object fills it with
                // data from the "process" table
                rcomp = new DbRunComponent(s, con);
                rtd.getAllComponents().put(s, rcomp);
                type = rcomp.getType();
                if(type!=null) {
                    if (type.equals(rcConstants.rocType)) {
                        rtd.getRocComponents().put(s, rcomp);
                    }
                    else if (type.equals(rcConstants.ebType)) {
                        rtd.getEbComponents().put(s, rcomp);
                    }
                    else if (type.equals(rcConstants.erType)) {
                        rtd.getErComponents().put(s, rcomp);
                    }
                    else if (type.equals(rcConstants.tsType)) {
                        rtd.getTsComponents().put(s, rcomp);
                    }
                }
            }

            // Obtain data from $(RUNTYPE)_option db table
            int val;
            stmt = con.createStatement();
            rs = stmt.executeQuery("SELECT * FROM " + runType +
                    "_option WHERE name = 'dataLimit' OR name = 'eventLimit'");
            while (rs.next()) {
                s = rs.getString("name");
                val = rs.getInt("value");
                if (s.equalsIgnoreCase("dataLimit")) {
                    rtd.setDataLimit(val);
                }
                else if (s.equalsIgnoreCase("eventLimit")) {
                    rtd.setEventLimit(val);
                }
            }

            // Obtain data from $(RUNTYPE)_script db table which is a
            // script to be run before a particular transition is started.
            // Although the database associates each script with a particular
            // component, this is only a kludge and should be ignored.
            String state, script;
            stmt = con.createStatement();
            rs = stmt.executeQuery("SELECT state,script FROM " + runType + "_script");
            while (rs.next()) {
                // "state" is the key. "script" is the value.
                state  = rs.getString("state");
                script = rs.getString("script");
                if (script == null || script.equals("")) {
                    continue;
                }
                rtd.getScripts().put(state, script);
            }
        }
        catch (SQLException ex) {
            throw new RCException(ex.getMessage());
        }

        return rtd;
    }

}
