/*---------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Carl Timmer                                                    *
*             timmer@jlab.org                   Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5130             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package org.jlab.coda.runcontrol.util.db;


/**
 * This abstract class represents a generic component
 * managed by run control.
 *
 * @author Carl Timmer  
 */
public abstract class RunComponent {
    
    String name;
    String priority;
    String command;
    String type;
    String state;
    String host;
    int id;
    int port;
    int pid;
    boolean isInUse;
    
        
    public String toString() {
        String s = "name = " + name + 
	           "\npriority = " + priority +
	           "\ncmd = " + command +
	           "\ntype = " + type +
	           "\nstate = " + state +
	           "\nhost = " + host +
	           "\nid = " + id +
	           "\nport = " + port +
	           "\npid = " + pid +
	           "\nisInUse = " + isInUse + "\n";
	return s;
    }
    
    
    // get data from the database "process" table or some other source
    public abstract void getData() throws RCException ;
    
    // getters and setters
    
    public void   setName(String name) { this.name = name; }
    public String getName() { return name; }

    public void   setPriority(String priority) { this.priority = priority; }
    public String getPriority() { return priority; }

    public void   setCommand(String command) { this.command = command; }
    public String getCommand() { return command; }

    public void   setType(String type) { this.type = type; }
    public String getType() { return type; }

    public void   setState(String state) { this.state = state; }
    public String getState() { return state; }

    public void   setHost(String host) { this.host = host; }
    public String getHost() { return host; }

    public int  getId() { return id; }
    public void setId(int id) {
	if (id < 0) {return;}
	this.id = id;
    }

    public int  getPort() { return port; }
    public void setPort(int port) {
	if ((id < 0) || (id > 65535)) {return;}
	this.port = port;
    }

    public int  getPid() { return pid; }
    public void setPid(int pid) {
	if (pid < 0) {return;}
	this.pid = pid;
    }

    public boolean getIsInUse() { return isInUse; }
    public void setIsInUse(boolean isInUse) { this.isInUse = isInUse; }
    public void setIsInUse(String inUse) {
	if (inUse.equalsIgnoreCase("no")) {
	    this.isInUse = false;
	    return;
	}
	this.isInUse = true;
    }

}
