/*---------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Carl Timmer                                                    *
*             timmer@jlab.org                   Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5130             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package org.jlab.coda.runcontrol.util.db;

import java.util.HashMap;
import java.util.LinkedHashMap;

/**
 * Contains all the data associated with an experiment configuration (or runType).
 * This class is used to store data after reading it from a database. It is also
 * used to store data after reading it from a COOL file with the purpose of then
 * writing it out to a database.
 *
 * @author  Carl Timmer
 */
public class RunTypeData {    
    /** Name of the runType whose data we're storing. */
    String runType;
    
    // This runType/configuration is being used
    boolean isInUse;
    
    // Collections of runComponents. These are objects of class RunComponent which is
    // extended by the DbRunComponent class or possibly a COOLRunComponent in the future.
    // Key is the name of the component (String), and value is the RunComponent object.
    HashMap allComponents = new HashMap(50);
    HashMap rocComponents = new HashMap(30);
    HashMap ebComponents  = new HashMap(10);
    HashMap erComponents  = new HashMap(5);
    HashMap tsComponents  = new HashMap(5);
    
    // Contains scripts to be run in the proper order.
    // Key is the transition. Value is the script.
    LinkedHashMap scripts = new LinkedHashMap(10);

    int dataLimit;
    int eventLimit;
    
    // getters & setters
    public HashMap getAllComponents() { return allComponents; }
    public HashMap getRocComponents() { return rocComponents; }
    public HashMap getEbComponents()  { return ebComponents;  }
    public HashMap getErComponents()  { return erComponents;  }
    public HashMap getTsComponents()  { return tsComponents;  }
    
    public HashMap getScripts() { return scripts;  }
    
    public void   setRunType(String runType) { this.runType = runType; }
    public String getRunType() { return runType; }
    
    public void    setIsInUse(boolean isInUse) { this.isInUse = isInUse; }
    public boolean getIsInUse() { return isInUse; }

    public void setDataLimit(int dataLimit) { this.dataLimit = dataLimit; }
    public int  getDataLimit() { return dataLimit; }

    public void setEventLimit(int eventLimit) { this.eventLimit = eventLimit; }
    public int  getEventLimit() { return eventLimit; }

}
