/*---------------------------------------------------------------------------*
*  Copyright (c) 2005        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.util;

/**
 *  This class defines some useful constants. These constants correspond
 *  to identical constants defined in the C implementation of cMsgOntology.
 *
 */
public final class rcConstants {

    private rcConstants(){}

    /** sliding average max value (smooting factor)*/
    public static final int SMOOTH = 20;

    /** Number of threads in the thread pool, used in StemSell agent*/
    public static final int NTHP = 7;

    public static final String failed = "FAILED";
    public static final String undefined = "UNDEFINED";
    public static final String justborn = "JUSTBORN";

    // Agent to Medulla reporting periond
    public static int AgentReportingPeriod = 1000; // 1 sec.

    // keep updating the gui after the end sec.
    public static final int update_after_end = 5000;

    /* when in the go state if physical component is not responding for
this amount of sec. agent will request ending the run*/
    public static  int END_REQUEST_TIME = 60;

    /** Maximum allowed synchronous process execution time (e.g. asking/poling periodically the status
     * information from the physical component.). In another words this is the time agent will wait
     * for physical component to respond to the request.*/
    public static int LONG_EXEC_TIME = 30000;

    /** A maximum timeout in seconds for all agents to report the coda
     * transition execution status to Medulla. */
    public static int TIMEOUT = 1200;

    /** sleep after sending reset command to the representative agent*/
    public static int SLEEP_AFTER_EVERY_RESET = 10;

    /** the number of the fails after which Medula will not wait the component to end.  */
    public static int FAILS_NUMBER = 2;

    /** message priorities, used in the agent client communication protocole*/
    public static int priority_low        = 1;
    public static int priority_medium     = 2;
    public static int priority_high       = 3;

    /** requests to the physical client*/
    public static int init_request        = 789;
    public static int exit_request        = 788;

    /** requests from the physical client*/
    public static int new_reg_request        = 689;
    public static int update_reg_request     = 688;


    /** Coda state transition enumaration*/
    public static final int preconfigure  = 0;
    public static final int configure     = 1;
    public static final int postconfigure = 2;

    public static final int predownload   = 3;
    public static final int download      = 4;
    public static final int postdownload  = 5;

    public static final int preprestart   = 6;
    public static final int prestart      = 7;
    public static final int postprestart  = 8;

    public static final int prego         = 9;
    public static final int go            = 10;
    public static final int postgo        = 11;

    public static final int preend        = 12;
    public static final int end           = 13;
    public static final int postend       = 14;

    public static final int reset         = 15;
    public static final int pause         = 16;
    public static final int resume        = 17;

    /** Medulla will broadcast the monitored statistics information to Plexuses every mbroadcast_period/1000 sec.*/
    public static final int mbroadcast_period = 1000;

/** Default reporting interval for the real world physical clients*/
      public static final String DefaultReportingInterval = "1.0";

    /** Coda state transition commands. Those are the transition commands ( or part of the command string)
     * sent by Thalamus which are not described in the cool file and are executed independnet of the user configuration.
     * This way user will not warry of describing cdoa basic state transtions, necessary to run the DAQ.*/
    public static final String CMD_preconfigure  = "preconfigure";
    public static final String CMD_configure     = "configure";
    public static final String CMD_postconfigure = "postconfigure";

    public static final String CMD_predownload   = "predownload";
    public static final String CMD_download      = "download";
    public static final String CMD_postdownload  = "postdownload";

    public static final String CMD_preprestart   = "preprestart";
    public static final String CMD_prestart      = "prestart";
    public static final String CMD_postprestart  = "postprestart";

    public static final String CMD_prego         = "prego";
    public static final String CMD_go            = "go";
    public static final String CMD_postgo        = "postgo";

    public static final String CMD_preend        = "preend";
    public static final String CMD_end           = "end";
    public static final String CMD_postend       = "postend";

    public static final String CMD_exit          = "exit";
    public static final String CMD_pause         = "pause";
    public static final String CMD_resume        = "resume";


    /** expected component statuses after execution of certain coda transitions*/
    public static final String unknown               = "unknown";
    public static final String booted                = "booted";

    public static final String CMD_preconfigure_ok   = "preconfigured";
    public static final String CMD_configure_ok      = "configured";
    public static final String CMD_postconfigure_ok  = "postconfigured";

    public static final String CMD_predownload_ok    = "predownloaded";
    public static final String CMD_download_ok       = "downloaded";
    public static final String CMD_postdownload_ok   = "postdownloaded";

    public static final String CMD_preprestart_ok    = "preprestarted";
    public static final String CMD_prestart_ok       = "paused";
    public static final String CMD_postprestart_ok   = "postprestarted";

    public static final String CMD_prego_ok          = "pregoed";
    public static final String CMD_go_ok             = "active";
    public static final String CMD_postgo_ok         = "postgoed";

    public static final String CMD_preend_ok         = "preended";
    public static final String CMD_end_ok            = "downloaded";
    public static final String CMD_postend_ok        = "postended";

    public static final String CMD_reset_ok          = "resetted";
    public static final String CMD_pause_ok          = "paused";
    public static final String CMD_resume_ok         = "active";


    // type=run/control/* commands
    public static final String CMD_SetRunNumber         = "setRunNumber";
    public static final String CMD_GetRunNumber         = "getRunNumber";
    public static final String CMD_EnableDLL            = "enableDLL";
    public static final String CMD_DisableDLL           = "disableDLL";
    public static final String CMD_GetRunType           = "getRunType";
    public static final String CMD_SetRunType           = "setRunType";
//    public static final String CMD_SetRSInterval        = "setRSInterval";

    // type=supervisor/control/*
    public static final String CMD_ReportRunStatus      = "reportRunStatus";
    public static final String CMD_ReportRunNumber      = "reportRunNumber";
    public static final String CMD_ReportRunType        = "reportRunType";
    public static final String CMD_ReportSession        = "reportSession";
    public static final String CMD_ReporRunStatistics   = "reportRunstatistics";
    public static final String CMD_AutoStartTrue        = "autoStartTrue";
    public static final String CMD_AutoStartFalse       = "autoStartFalse";
    public static final String CMD_ReportComponents     = "reportComponents";
    public static final String CMD_autoreset            = "autoReset";
    public static final String CMD_componentKill        = "componentKill";
    public static final String CMD_reportInterval       = "reportInterval";

    // gui specific
    public static final String CMD_ReportActiveRunType  = "reportActiveRuntype";
    public static final String CMD_DisconnectRunType    = "disconnectRuntype";

    // run/monitor/* commands

    // sessin/control/* commands
    public static final String CMD_SetState                 = "setState";
    public static final String CMD_GetState                 = "getState";
    public static final String CMD_SetSession               = "setSession";
    public static final String CMD_GetSession               = "getSession";
    public static final String CMD_ReliaseSession           = "reliaseSession";
    public static final String CMD_Start                    = "start";
    public static final String CMD_Stop                     = "stop";
    public static final String CMD_Exit                     = "exit";
    public static final String CMD_Exec                     = "exec";
    public static final String CMD_StartReporting           = "startReporting";
    public static final String CMD_StopReporting            = "stopReporting";
    public static final String CMD_SetInterval              = "setInterval"; //used also for supervisor/control/*
    public static final String CMD_reset                    = "reset";


    // coda/info/* commands
    public static final String CMD_GetObjectType            = "getObjecttype";
    public static final String CMD_StartReportState         = "startReportstate";
    public static final String CMD_StopReportState          = "stopReportstate";

/** After component starts monitoring and broadcasting its statistics info it also sends id */
//    public static final String CMP_monitor_id          = "active";

    /** Major cMsgOntology version number. */
    public static final int    domain_version      = 1;
    /** Minor cMsgOntology version number. */
    public static final int    minorVersion = 9;

    public static final String info     = "info";
    public static final String warning  = "warning";
    public static final String error    = "error";

    // sleep before teaching the agents and after they were created on the platform in milisec.
    public static final int sleepAfterCreate_Cortex = 100;
    public static final int sleepAfterKill_Cortex = 100;
    public static final int sleepBeforeInit_Cortex = 200;
    public static final int sleepBeforeTeaching_Cortex = 100;
    public static final int sleepForMedullaDiff = 1000;

    // making sure Hepatic already created agents
    public static final int sleepBeforeDifferentiation_Cortex = 100;
    public static final int sleepAfterDifferentiation_Cortex = 500;

    //@todo think on it
    public static int noInfo       = -1;
    public static int debugInfo    = 0;
    public static int debugWarning = 1;
    public static int debugError   = 2;

    public static int maxcomponents = 1000;

    // Stemcell will report to medula if component will not report within this mant seconds.
    public static final int ACTIVE_BUT_NOTREPORTING = 15;

    // Medulla will report to gui in case pre or post transition is not completed after this amount/10 sec.
    public static final int PREPROST_NOTREPORTING = 150;

    // Cortex agent creation and differentiation progress report to gui after this many components
    public static final int CORTEX_DIF_GUI_REPORT = 50;
    public static final int CORTEX_CREATE_GUI_REPORT = 50;

    // If bargraph has more then this number of bins plot as points
    public static int BAR_OR_POINT = 100;

    // component types. N.B. listed according to their priorities
    public static final String  userType                   = "USR";
    public static final String  analysisType               = "ANA";
    public static final String  statemachinesupervisorType = "SMS";
    public static final String  slowcontrolType            = "SLC";
    public static final String  erType                     = "ER";
    public static final String  ebType                     = "EB";
    public static final String  emuType                    = "EMU";
    public static final String  rocType                    = "ROC";
    public static final String  tsType                     = "TS";
    public static final String rcSupervisorType            = "RCS";
    public static final String  wienercrateType            = "WNC";

/** Wiener crate states */
    public static final String WienerUnknown       = "Wunknown";
    public static final String WienerOn            = "Won";
    public static final String WienerOff           = "Woff";
    public static final String WienerError         = "Werror";
    public static final String WienerWarning       = "Wwarning";
    public static final String AgentStateActive    = "Wactive";
    public static final String AgentStatePasive    = "Wpasive";
    public static final float wienerDeltaV         = 0.8f;

}
