package org.jlab.coda.runcontrol.util;

import org.jlab.coda.cMsg.cMsgNetworkConstants;

/**
 * Copyright (c)            Southeastern Universities Research Association,
 * Thomas Jefferson National Accelerator Facility
 * <p/>
 * This software was developed under a United States Government license
 * described in the NOTICE file included as part of this distribution.
 * <p/>
 * </p>
 * Header file for cMsg subscription subjects and types
 * RC Broadcast domain UDL is of the form:
 *        cMsg:rcb://<udpPort>?expid=<expid>
 *
 *    The intial cMsg:rcb:// is stripped off by the top layer API
 *
 *    Remember that for this domain:
 *    1) udp listening port is optional and defaults to 6543 (cMsgNetworkConstants.rcBroadcastPort)
 *    2) the experiment id is given by the optional parameter expid. If none is
 *       given, the environmental variable EXPID is used. if that is not defined,
 *       an exception is thrown
 *
 * </p>
 * Author: gurjyan
 * Date: Jul 13, 2006
 * Time: 10:28:31 AM
 */

public final class rcSubscription {

    public static String expid;
    public static String cMsgTcpPort;
    public static String cMsgHost;
    public static String cMsgUdpPort;
    public static String rcDomainUdpPort;
    public static String PlatformcMsgServerUDL4Gui;

    // do not let any one to instantiate this class
    private rcSubscription(){
    }

    static{
        expid = System.getenv("EXPID");
            if(expid==null) {
                System.out.println("Fatal-error: EXPID is not defined.");
                System.exit(-1);
            }
            cMsgHost = System.getenv("PCMSG_HOST");
            cMsgUdpPort = System.getenv("CMSG_BROADCAST_PORT");
             if(cMsgUdpPort==null)cMsgUdpPort = Integer.toString(cMsgNetworkConstants.nameServerBroadcastPort);

            cMsgTcpPort = System.getenv("CMSG_PORT");
             if(cMsgTcpPort==null)cMsgTcpPort = Integer.toString(cMsgNetworkConstants.tcpServerPort);

            rcDomainUdpPort = System.getenv("RC_BROADCAST_PORT");
             if(rcDomainUdpPort==null)rcDomainUdpPort = Integer.toString(cMsgNetworkConstants.rcBroadcastPort);

        System.out.println("Configuring AFECS platform..."+
                "\nExperiment ID   = "+expid+
                "\ncMsgTcpPort     = "+cMsgTcpPort+
                "\ncMsgUdpPort     = "+cMsgUdpPort+
                "\nRcDomainUdpPort = "+rcDomainUdpPort);
//        if(cMsgHost==null)
//            PlatformcMsgServerUDL4Gui = "cMsg:cMsg://broadcast:"+cMsgTcpPort+"/cMsg/"+expid+"/?broadcastTO=5";
//        else
//            PlatformcMsgServerUDL4Gui = "cMsg:cMsg://"+cMsgHost+":"+cMsgTcpPort+"/cMsg/"+expid;
    }

    /** Medulla connection information connecting cMsg domain name server: UDL*/
//    public static final String PlatformcMsgServerUDL = "cMsg:cMsg://localhost/cMsg/"+System.getenv("EXPID");
//    public static final String PlatformcMsgServerUDL = "cMsg:cMsg://broadcast:"+cMsgUdpPort+"/cMsg/"+expid+"/?broadcastTO=5";
    public static final String PlatformcMsgServerUDL = "cMsg:cMsg://localhost:"+cMsgTcpPort+"/cMsg/"+expid;
    /** name*/
    public static final String PlatformcMsgServerName = "cMsgPlatform";
    /** description*/
    public static final String PlatformcMsgServerDescription = "RC platform cMsg server";

    /** RC broadcast domain server UDL*/
    public static final String RcBServerUDL            = "cMsg:rcb://"+rcDomainUdpPort;
    /** RC broadcast domain server name*/
    public static final String RcBServerName           = "RcBroadcastDomain";
    /** RC broadcastc domain server description*/
    public static final String RcBServerDescription    = "UDP Broadcast Listener in the Cortex";

    /** RC domain server UDL*/
    public static final String RcServerUDL             = "cMsg:rcs://";
    /** RC  domain server name*/
    public static final String RcServerName            = "RcDomain";
    /** RC  domain server description*/
    public static final String RcServerDescription     = "cMsg RC domain server in the StemCell";


    /** RcGui cMsg (cMsg domain) subscription subjects*/
    public static final String RcGuiSubjInfo           = "INFO";
    public static final String RcGuiSubjError          = "ERROR";
    public static final String RcGuiSubjWarning        = "WARNING";
    public static final String RcGuiSubjTsData         = "TsData";
    
    public static final String Hepatic                 = "Hepatic";
    public static final String Stemcell                = "StemCell";

    /** RcGui cMsg (cMsg domain) subscription types*/
    public static final String RcGuiTypeTransition         = "transition";
    public static final String RcGuiTypeRunStatus          = "runStatus";
    public static final String RcGuiTypeRunType            = "runType";
    public static final String RcGuiTypeSession            = "session";
    public static final String RcGuiTypeRunNumber          = "runNumber";
    public static final String RcGuiTypeRunStatistics      = "runStatistics";
    public static final String RcGuiTypeComponentCount     = "componentCount";
    public static final String RcGuiTypeComponentNames     = "componentNames";
    public static final String RcGuiTypeStemCell           = "stemcell";
    public static final String RcGuiTypeRegisteredSessions = "reportRegisteredSessions";
    public static final String RcGuiTypeConfigurations     = "reportConfigurations";
    public static final String RcGuiTypeDataFile           = "datafile";
    public static final String RcGuiTypeContainer          = "container";
    public static final String RcGuiTypeCompRepInterval    = "acceptcompints";
    public static final String RcGuiTypeAutoStart          = "autostart";
    public static final String RcGuiTypeStartRun           = "startrun";
    public static final String RcGuiTypeEventLimit         = "eventlimit";
    public static final String RcGuiTypeFileName           = "filename";


    /** Cortex cMsg (cMsg domain) subscription subject used for UI communication (configure message)*/
    public static final String CrtSubject              = "Cortex";
    /** Cortex cMsg (cMsg domain) subscription type used for UI communication*/
    public static final String CrtType                 = "Control";

    /** Cortex cMsg (RcB domain) subscription subjects*/
    public static final String CrtRcBSubject           = "Registration";
    /** Cortex cMsg (RcB domain) subscription types*/
    public static final String CrtRcBType              = "Component registration request";

    public static final String RCReport = "rcreport/status";
    public static final String Dalog    = "rcreport/dalog";
    public static final String Response = "response";
    public static final String Retire   = "retire";  // type indicating that control system disconnects
    public static final String Employ = "employ";  // type indicating that control system has been created
    public static final String DalogSupervisor = "dalogsupervisor"; // subject for the supervisor that reports the set of the agents under its control

    public static final String CodaInfoType                   = "coda/info";
    public static final String SessionControlType             = "session/control";
    public static final String RunTransitionType              = "run/transition";
//    public static final String RunMonitorType                 = "run/monitor";
    public static final String RunControlType                 = "run/control";
    public static final String SupervisorControlType          = "supervisor/control";

// This subscription is unusual. Agent will send a control message to this callback which will start
// continuous monitoring of the coda component data. N.B. client will not lanch a new threa, but rather will
// use the callback thread. Now, when the new mesage will arrive to this callback client will check its
// callback cueue and will reconfigure monitoring process without exiting the callback.

    // test subject and type necessary to test if the servers are active
    public static final String SubjectTest             = "test";
    public static final String TypeTest                = "test";

// subject and type of the logging service
    public static final String logSubject              = "logsubject";
    public static final String logType                 = "logtype";

// dalog message specific
    public static final String dalogMessageSeverity          = "severity";
    public static final String dalogMessageComponentClass    = "codaClass";

    // wiener crate specific
    public static final String WNCControlType                = "wnc/control";

}
