/*----------------------------------------------------------------------------*
*  Copyright (c) 2003        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package org.jlab.coda.runcontrol.util.sh;


import java.lang.*;
import java.io.*;
import java.util.ArrayList;

/**
 *  Class SysExe designed to run UNIX and VxWorks executables.
 *  For VxWorks connection it is using java DPLight class by G. Heyes.
 *
 * @author Vardan H Gyurjyan. 11.19.99 CLAS Online Group.
 */
public class SysExe {

    /** private variable defines the state of the connection*/
    private static  boolean connected;

    private InputStream iostream;

    private String[] envs;

    private ArrayList envVarsList = new ArrayList();

//    private final String prefix = "/bin/csh -c ";

    /*
    public SysExe() {
    String result = "";

    Process p = null;
    Runtime r = Runtime.getRuntime();
    try {
    p = r.exec("printenv");
    } catch (IOException e) {
    }

    iostream = p.getInputStream();

    try {
    BufferedReader myReader = new BufferedReader(new InputStreamReader(iostream));
    while (result!=null){
    result=myReader.readLine();
    if(result !=null)envVarsList.add(result);
    }
    myReader.close();

    } catch (IOException e) {
    e.printStackTrace();
    }
    envs = (String[]) envVarsList.toArray(new String[envVarsList.size()]);
    for(int i = 0;i<envs.length;i++)
    System.out.println(envs[i]);
    }
    */
    /**
     * Method is used to execute the UNIX void commands. Returned int value will indicate the status of the execution.
     *
     * @param command UNIX executable name.
     * @return status  0: Ok, -1: failed
     */
    public  int run_o_unix (String command) {
        int status = 0;
        Runtime r = Runtime.getRuntime();
//        System.out.println("=> "+command);
        try {
            Process p = r.exec (command);

            try {
                p.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }


        } catch (IOException e) {
            System.out.println("run_unix exception: " +e);
            status = -1;
        }
//        System.out.println(status);
        return status;
    }

    /**
     * Method is used to execute the UNIX commands. Returned InputStream connected
     * to the standard output of the child process.
     *
     * @param command UNIX executable name.
     * @return stream result
     */
    public  InputStream run_o_unix_stream (String command) {
        Process p = null;
        Runtime r = Runtime.getRuntime();
        try {
            p = r.exec (command);

            try {
                p.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (IOException e) {
            System.out.println("run_unix exception: " +e);
        }
        assert p != null;
        return p.getInputStream();
    }



    /**
     * Method is used to execute the UNIX void commands. Returned int value will indicate the status of the execution.
     *
     * @param command UNIX executable name.
     * @return status  0: Ok, -1: failed
     */
    public int run_unix (String[] command){
        int status = 0;

        for (int i=0;i<command.length; i++){
            if(command[i].startsWith("$")) {
                if(command[i].contains("/")){
                    int pos = command[i].indexOf("/");
                    String nst1 = command[i].substring(1,pos);
                    String nst2 = command[i].substring(pos);
                    command[i] = System.getenv(nst1)+nst2;
//                System.out.println("=> "+command[i]);
                } else {
                    String nst = System.getenv(command[i].substring(1));
                    command[i] = nst;
//                    System.out.println("=> "+command[i]);
                }
            }
        }

        ProcessBuilder pb = new ProcessBuilder(command);

        try {
            Process p = pb.start();
            try {
                p.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (IOException e) {
            status = -1;
            e.printStackTrace();
        }
        return status;
    }



    /**
     * Method is used to execute the UNIX commands. Returned InputStream connected
     * to the standard output of the child process.
     *
     * @param command UNIX executable name.
     * @return stream result
     */
    public  InputStream run_unix_stream (String[] command) {

        Process p = null;
        for (int i=0;i<command.length; i++){
            if(command[i].startsWith("$")) {
                if(command[i].contains("/")){
                    int pos = command[i].indexOf("/");
                    String nst1 = command[i].substring(1,pos);
                    String nst2 = command[i].substring(pos);
                    command[i] = System.getenv(nst1)+nst2;
                } else {
                    String nst = System.getenv(command[i].substring(1));
                    command[i] = nst;
                }
            }
        }

//        for(int i=0;i<command.length;i++)System.out.println("=> "+command[i]);

        ProcessBuilder pb = new ProcessBuilder(command);

        try {
            p = pb.start();
//            System.out.println(command.length+" command[1]"+" statrted");
            try {
                p.waitFor();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
        if(p!=null)return p.getInputStream();
        else return null;
    }

}
