package org.jlab.coda.runcontrol.util.snmp;


import net.percederberg.mibble.*;
import net.percederberg.mibble.snmp.SnmpObjectType;
import net.percederberg.mibble.value.ObjectIdentifierValue;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

/**
 * @author: gurjyan
 * @date: Jan 10, 2008
 * @time: 12:15:10 PM
 */
public class MibReader {


    public Mib loadMib(File file)
            throws FileNotFoundException, MibLoaderException {

        MibLoader  loader = new MibLoader();

        loader.addDir(file.getParentFile());
        try {
            return loader.load(file);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    public HashMap<String, AfecsMibObject> extractData(Mib mib) {
        HashMap<String, AfecsMibObject>    map = new HashMap<String, AfecsMibObject>();
        Iterator iter = mib.getAllSymbols().iterator();
        MibSymbol  symbol;
        ObjectIdentifierValue   value;
        SnmpObjectType type;
        AfecsMibObject ob;

        while (iter.hasNext()) {
            symbol = (MibSymbol) iter.next();

            value = extractOid(symbol);
            type = extractObjectType(symbol);

            if (value != null) {
                ob = new AfecsMibObject();
                ob.setOid(value);

                if(type!=null){
                    ob.setAccess(type.getAccess());
                    ob.setDefaultValue(type.getDefaultValue());
                    ob.setDescription(type.getDescription());
                    ob.setName(type.getName());
                    ob.setReference(type.getReference());
                    ob.setStatus(type.getStatus());
                    ob.setSyntax(type.getSyntax());
                    ob.setUnit(type.getUnits());
                }
                map.put(symbol.getName(), ob);
            }
        }
        return map;
    }

    public ObjectIdentifierValue extractOid(MibSymbol symbol) {
        MibValue value;

        if (symbol instanceof MibValueSymbol) {
            value = ((MibValueSymbol) symbol).getValue();
            if (value instanceof ObjectIdentifierValue) {
                return (ObjectIdentifierValue) value;
            }
        }
        return null;
    }


    public SnmpObjectType extractObjectType(MibSymbol symbol) {
        MibType type;

        if (symbol instanceof MibValueSymbol) {
            type = ((MibValueSymbol) symbol).getType();
            if (type instanceof SnmpObjectType) {
                return (SnmpObjectType) type;
            }
        }
        return null;
    }

    public static void main(String[] args){

        File myMibFile = new File(System.getenv("COOL_HOME")+"/mibs/"+args[0]);
        Mib myMib = null;
        HashMap mibVarMap = null;
        Iterator it;
        String s;
        AfecsMibObject o;
        MibReader myMibReader = new MibReader();

        //  Read the Mib file
        try {
            myMib = myMibReader.loadMib(myMibFile);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (MibLoaderException e) {
            e.printStackTrace();
        }

        // Extract all the OID's
        if(myMib!=null)  mibVarMap = myMibReader.extractData(myMib);

        // Print the vales
        if(mibVarMap!=null && !mibVarMap.isEmpty()) {
            it = mibVarMap.keySet().iterator();
            while(it.hasNext()){
                s = (String)it.next();
                o = (AfecsMibObject)mibVarMap.get(s);
                System.out.println(s+" "+o.getOid());
                System.out.println("access = "+o.getAccess());
                System.out.println("default value = "+o.getDefaultValue());
                System.out.println("description = "+o.getDescription());
                System.out.println("name = "+o.getName());
                System.out.println("reference = "+o.getReference());
                System.out.println("status = "+o.getStatus());
                System.out.println("unit = "+o.getUnit());
                System.out.println();
            }
        }
    }

}
