package org.jlab.coda.runcontrol.util.snmp;

import org.snmp4j.*;
import org.snmp4j.transport.*;
import org.snmp4j.event.*;
import org.snmp4j.mp.*;
import org.snmp4j.security.*;
import org.snmp4j.smi.*;

import java.io.IOException;

/**
 * SNMP test class.
 * User: gurjyan
 * Date: May 21, 2007
 * Time: 11:50:54 AM
 */
@SuppressWarnings({"deprecation"})
public class SnmpModule {
    public Snmp snmp;
    public Address targetAddress;
    public volatile static String mibend = "empty";
    public static int knel = 100;
    public static String myIPAddress = "129.57.29.202";

    public SnmpModule(){
        targetAddress = GenericAddress.parse("udp:129.57.29.202/161");
        TransportMapping transport;
        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            USM usm = new USM(SecurityProtocols.getInstance(),
                    new OctetString(MPv3.createLocalEngineID()), 0);
            SecurityModels.getInstance().addSecurityModel(usm);
            transport.listen();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    public static void main(String[] args){

        SNMPSetInt(myIPAddress,"1.3.6.1.4.1.19947.1.1.1.0", Integer.parseInt(args[0]));

        if(args.length==1 && args[0].equalsIgnoreCase("m")){
            SnmpModule myTest = new SnmpModule();
            myTest.scansnmp("1.3.6");
            while(!mibend.equals("endOfMibView")){
                try {
                    Thread.sleep(knel);
                } catch (InterruptedException e) {
                    e.printStackTrace();  }
            }
        }
    }



    public void afadd(){
        // add user to the USM
        snmp.getUSM().addUser(new OctetString("MD5DES"),
                new UsmUser(new OctetString("MD5DES"),
                        AuthMD5.ID,
                        new OctetString("MD5DESUserAuthPassword"),
                        PrivDES.ID,
                        new OctetString("MD5DESUserPrivPassword")));
        // create the target
        UserTarget target = new UserTarget();
        target.setAddress(targetAddress);
        target.setRetries(1);
        target.setTimeout(5000);
        target.setVersion(SnmpConstants.version3);
        target.setSecurityLevel(SecurityLevel.AUTH_PRIV);
        target.setSecurityName(new OctetString("MD5DES"));

        // create the PDU
        PDU pdu = new ScopedPDU();
        pdu.add(new VariableBinding(new OID("1.3.6")));
        pdu.setType(PDU.GETNEXT);

        // send the PDU
        ResponseEvent response = null;
        try {
            response = snmp.send(pdu, target);
        } catch (IOException e) {
            e.printStackTrace();
        }
        // extract the response PDU (could be null if timed out)
        PDU responsePDU = response.getResponse();
        // extract the address used by the agent to send the response:
        Address peerAddress = response.getPeerAddress();

    }


    public void afsync() {
//        An asynchronous SNMPv1 request is sent by the following code:

        // setting up target
        CommunityTarget target = new CommunityTarget();
        target.setCommunity(new OctetString("public"));
        target.setAddress(targetAddress);
        target.setRetries(2);
        target.setTimeout(1500);
        target.setVersion(SnmpConstants.version1);
        // creating PDU
        PDU pdu = new PDU();
        pdu.add(new VariableBinding(new OID(new int[] {1,3,6,1,2,1,1,1})));
        pdu.add(new VariableBinding(new OID(new int[] {1,3,6,1,2,1,1,2})));
        pdu.setType(PDU.GETNEXT);

        // sending request
        ResponseListener listener = new ResponseListener() {
            public void onResponse(ResponseEvent event) {
                // Always cancel async request when response has been received
                // otherwise a memory leak is created! Not canceling a request
                // immediately can be useful when sending a request to a broadcast
                // address.
                ((Snmp)event.getSource()).cancel(event.getRequest(), this);
                System.out.println("Received response PDU is: "+event.getResponse());
            }
        };
        try {
            snmp.sendPDU(pdu, target, null, listener);
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    public void scansnmp(String oid) {
//        An asynchronous SNMPv1 request is sent by the following code:

        // setting up target
        CommunityTarget target = new CommunityTarget();
        target.setCommunity(new OctetString("public"));
        target.setAddress(targetAddress);
        target.setRetries(2);
        target.setTimeout(1500);
        target.setVersion(SnmpConstants.version2c);
        // creating PDU
        PDU pdu = new PDU();
        pdu.add(new VariableBinding(new OID(oid)));
        pdu.setType(PDU.GETNEXT);



        // sending request
        ResponseListener listener = new ResponseListener() {
            PDU respdu;
            public void onResponse(ResponseEvent event) {
                // Always cancel async request when response has been received
                // otherwise a memory leak is created! Not canceling a request
                // immediately can be useful when sending a request to a broadcast
                // address.
                ((Snmp)event.getSource()).cancel(event.getRequest(), this);
                respdu = event.getResponse();

                if(respdu!=null){
                    try {
                        Thread.sleep(knel);
                    } catch (InterruptedException e) { e.printStackTrace();  }

                    scansnmp(respdu.get(0).getOid().toString());
                    mibend = respdu.get(0).getVariable().toString();
                    System.out.println(respdu.get(0).getOid()+
                            "           "+
                            respdu.get(0).getVariable().getSyntaxString()+
                            "           "+
                            respdu.get(0).getVariable());

                }
            }
        };
        try {
            snmp.sendPDU(pdu, target, null, listener);
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    public void aftrap(){
// Traps (notifications) and other SNMP PDUs can be received by adding the following code to the first code snippet above:

        CommandResponder trapPrinter = new CommandResponder() {
            public synchronized void processPdu(CommandResponderEvent e) {
                PDU command = e.getPDU();
                if (command != null) {
                    System.out.println(command.toString());
                }
            }
        };
        snmp.addCommandResponder(trapPrinter);
    }


    /**
     * @param oid  of the SNMP channel
     */
    public static void SNMPSetInt(String ip, String oid, int val) {
        /** SNMP communication object*/
        Snmp snmp = null;

        /** SNMP address of the wienere crate*/
        Address targetAddress;

        ResponseEvent response = null;

        targetAddress = GenericAddress.parse("udp:"+ ip +"/161");

        TransportMapping transport;
        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            USM usm = new USM(SecurityProtocols.getInstance(),
                    new OctetString(MPv3.createLocalEngineID()), 0);
            SecurityModels.getInstance().addSecurityModel(usm);
            transport.listen();
        } catch (IOException e) {
            e.printStackTrace();
        }

        // Setting up target
        CommunityTarget target = new CommunityTarget();
        target.setCommunity(new OctetString("guru"));
        target.setAddress(targetAddress);
        target.setRetries(3);
        target.setTimeout(1500);
        target.setVersion(SnmpConstants.version2c);

        // Creating PDU
        PDU pdu = new PDU();

        // Adding variable
        VariableBinding vb = new VariableBinding(new OID(oid));
//        Variable variable = new Integer32(Integer.parseInt(val));
        Variable variable = new Integer32(val);
        vb.setVariable(variable);

        pdu.add(vb);
        pdu.setType(PDU.SET);
        try {
            if (snmp!=null) response = snmp.send(pdu, target);
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            System.out.println(response.getResponse().getErrorStatusText());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }




}

