/*----------------------------------------------------------------------------*
*  Copyright (c) 2004        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-12H        *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5800             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package org.jlab.coda.runcontrol.util.tcp;

import org.jlab.coda.runcontrol.util.db.DatabaseConnection;
import org.jlab.coda.runcontrol.util.db.DBException;
import org.jlab.coda.runcontrol.util.rcConstants;
import org.jlab.coda.runcontrol.util.rcException;

import java.io.*;
import java.net.Socket;
import java.net.UnknownHostException;
import java.net.InetAddress;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * Simplest java client for the TcpServer. It has only one synchronized static method.
 *
 * @author Vardan Gyurjyan
 * @since
 */
public final class JtcpClient {

    /** Verbosity level defined in the rcConstants class. */
    private static int verbose = rcConstants.debugError;
    private static Connection con;

    private DataOutputStream request;
    private BufferedReader response;

    Socket vxservsock;

    int port;
    String host;

    public JtcpClient (String componentName) throws rcException {

        /* Open connection to database*/
        String url = System.getenv("CODA_DB_URL");
        String driver = System.getenv("CODA_DB_DRIVER");
        String usr = System.getenv("CODA_DB_USER");
        String passwd = System.getenv("CODA_DB_PASSWORD");

        if(usr==null)usr=" ";
        if(passwd==null)passwd=" ";
        try {
            DatabaseConnection.createConnection(url,driver,usr,passwd);
        } catch (DBException e) {
            e.printStackTrace();
            throw new rcException("Jtcp_Error: Can not create DB connection. Check DB related env. variable settings.");
        }

        if(con == null){
            // Get a db connection
            try {
                con = DatabaseConnection.getConnection();
            }
            catch (DBException ex) {
                System.out.println(ex.getMessage());
                throw new rcException("Jtcp_Error: Can not get DB connection.");
            }
        }
        /* get hostname and portnumber */
        try{
            Statement stmt = con.createStatement();
            ResultSet rs;

            rs = stmt.executeQuery("SELECT * FROM process WHERE name = '"+componentName+"'");
            while(rs.next()) {
//                port =Integer.parseInt(rs.getString("inuse").trim());
                port =rs.getInt("inuse");
//                if(verbose>=rcConstants.debugInfo)System.out.println(port);
            }

            rs = stmt.executeQuery("SELECT * FROM process WHERE name = '"+componentName+"'");
            while(rs.next()) {
                host =rs.getString("host").trim();
//                if(verbose>=rcConstants.debugInfo)System.out.println(host);
            }

            rs.close();
            stmt.close();
        }
        catch (SQLException ex) {
            ex.printStackTrace();
            throw new rcException("Jtcp_Error: Getting host and port from DB.");
        }

    }


    public String vxcmd(int flag, String command)
    throws rcException {

        StringBuffer strbuf = new StringBuffer();

        /* Connect to the tcpServer*/
        try{
            vxservsock = new Socket(host,port);
            vxservsock.setTcpNoDelay(true);
//            vxservsock.setKeepAlive(true);
//            System.err.println(command);
            if((command.equalsIgnoreCase("heartbeat")) || (command.equalsIgnoreCase("status")) ||(command.equalsIgnoreCase("statistics")) )
           vxservsock.setSoTimeout(3000);
            if(verbose>=rcConstants.debugInfo) {

                InetAddress remhost = vxservsock.getInetAddress();
//                System.out.println("Connected to remote host " +remhost);

                int servport = vxservsock.getPort();
//                System.out.println("Server listens on the port "+servport);

                int localport = vxservsock.getLocalPort();
//                System.out.println("my port is "+localport);

                InetAddress localaddress = vxservsock.getLocalAddress();
//                System.out.println("my local address is "+localaddress);

            }
            /*open the output stream on the socket*/
            request = new DataOutputStream(new BufferedOutputStream(vxservsock.getOutputStream()));

            /* open the input stream on the socket*/
            response = new BufferedReader(new InputStreamReader(vxservsock.getInputStream()));

        } catch (UnknownHostException e) {
            e.printStackTrace();
            throw new rcException();
        }
        catch (IOException e){
            e.printStackTrace();
            throw new rcException();
        }

        /* send the request to the server. reply flag */
        try {
            request.writeInt(flag);

            /* write the length of the message*/
            request.writeInt(command.length());

            /* write the actual message */
            request.write(command.getBytes("ASCII"));
            request.writeByte('\0');

            request.flush();

            if(verbose>=rcConstants.debugInfo) System.out.println(flag+" "+command.length()+" "+command);

            /*read the response  back*/
            String str;
            while(((str = response.readLine()) != null) && (str.indexOf("value =") <0)) {
                strbuf.append(str+"\n");
            }
        } catch (IOException e) {
            e.printStackTrace();
            try {
                request.close();
                response.close();
                vxservsock.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
        finally{
            try {
                /* close everything */
                request.close();
                response.close();
                vxservsock.close();
            }
            catch(IOException e){
                e.printStackTrace();
            }
        }

        /* return the result string*/
        return strbuf.toString().trim();
    }
}
