/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.abs;

import rca.content.AgentAction;

/**
 * An abstract descriptor that can hold an agent action
 * expression.
 * Original author Federico Bergenti - Universita` di Parma
 */
public class AbsAgentAction extends AbsConcept implements AbsContentElement, AgentAction {
	
	private boolean isAContentExpression = false;
	
    /**
     * Construct an Abstract descriptor to hold an agent action of
     * the proper type, e.g. SELL, BUY....
     * @param typeName The name of the type of the agent action held by 
     * this abstract descriptor.
     */
    public AbsAgentAction(String typeName) {
        super(typeName);
    }

    /**
     * Set an attribute of the agent action held by this
     * abstract descriptor.
     * @param name The name of the attribute to be set.
     * @param value The new value of the attribute.
     */
    public void set(String name, AbsPredicate value) {
        super.set(name, value);
 		} 
 		
    // Easy way to access the Java class representing AbsAgentAction.
 		// Useful in MIDP where XXX.class is not available
		private static Class absAgentActionClass = null;
 		public static Class getJavaClass() {
 			if (absAgentActionClass == null) {
 				try {
 					absAgentActionClass = Class.forName("rca.content.abs.AbsAgentAction");
 				}
 				catch (Exception e) {
 					// Should never happen
 					e.printStackTrace();
 				}
 			}
 			return absAgentActionClass;
 		}

 		/**
  	 * @see AbsContent.isAContentExpression
  	 */
  	public boolean isAContentExpression() {
  		return isAContentExpression;
  	}
  	
  	/**
  	 * @see AbsContent.setIsAContentExpression(boolean flag)
  	 */
  	public void setIsAContentExpression(boolean flag) {
  		isAContentExpression = flag;
  	}
}

