/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.abs;

import rca.content.Concept;

/**
 * An abstract descriptor that can hold a concept
 * expression.
 * Original author Federico Bergenti - Universita` di Parma
 */
public class AbsConcept extends AbsPrimitiveSlotsHolder implements AbsTerm, Concept {

    /**
     * Construct an Abstract descriptor to hold a concept of
     * the proper type (e.g. PERSON, ADDRESS...).
     * @param typeName The name of the type of the concept held by 
     * this abstract descriptor.
     */
    public AbsConcept(String typeName) {
        super(typeName);
    }

    /**
     * Sets an attribute of the concept held by this
     * abstract descriptor.
     * @param name The name of the attribute to be set.
     * @param value The new value of the attribute.
     */
    public void set(String name, AbsTerm value) {
        super.set(name, value);
    } 

    /**
     * Gets the value of an attribute of the concept 
     * held by this abstract descriptor.
     * @param name The name of the attribute.
     * @return value The value of the attribute.
     */
    public AbsTerm getAbsTerm(String name) {
        return (AbsTerm)getAbsObject(name);
    }

    // Easy way to access the Java class representing AbsConcept.
 		// Useful in MIDP where XXX.class is not available
		private static Class absConceptClass = null;
 		public static Class getJavaClass() {
 			if (absConceptClass == null) {
 				try {
 					absConceptClass = Class.forName("rca.content.abs.AbsConcept");
 				}
 				catch (Exception e) {
 					// Should never happen
 					e.printStackTrace();
 				}
 			}
 			return absConceptClass;
 		}

}

