/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.abs;

import rca.util.leap.Serializable;

/**
 * The common ancestor of all abstract descriptors
 * Original author Federico Bergenti - Universita` di Parma
 * Original author Giovanni Caire - TILAB
 */
public interface AbsObject extends Serializable {
    /**
     * @return The name of the type of the object held by this
     * abstract descriptor.
     */
    public String getTypeName();
    
    /**
     * Gets the value of an attribute of the object held by this
     * abstract descriptor.
     * @param name The name of the attribute.
     * @return value The value of the attribute.
     */
    public AbsObject getAbsObject(String name);

    /**
     * @return the name of all attributes.
     */
    public String[] getNames();

    /**
     * Tests if the object is grounded, i.e., if no one of its attributes 
     * is associated with a variable
     * @return <code>true</code> if the object is grounded.
     */
    public boolean isGrounded();

    /**
     * Gets the number of attributes.
     * @return the number of attributes.
     */
    public int getCount();

    /**
     * @deprecated Use <code>toString()</code> instead
     */
    /*public void dump();*/
}

