/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.abs;

import rca.content.Predicate;

/**
 * An abstract descriptor that can hold a predicate expression.
 * Original author Paola Turci, Federico Bergenti - Universita` di Parma
 */
public class AbsPredicate extends AbsPrimitiveSlotsHolder implements AbsContentElement, Predicate {

	private boolean isAContentExpression = false;
		private boolean isMeta = false;

    /**
     * Construct an Abstract descriptor to hold a predicate of
     * the proper type (e.g. FATHER_OF, WORKS_FOR...).
     * @param typeName The name of the type of the predicate held by 
     * this abstract descriptor.
     */
    public AbsPredicate(String typeName) {
        super(typeName);
    }

    /**
     * Sets an attribute of the predicate held by this
     * abstract descriptor.
     * @param name The name of the attribute to be set.
     * @param value The new value of the attribute.
     */
    public void set(String name, AbsObject value) {
        super.set(name, value);
    } 

    /**
     * Gets the value (casted as an AbsTerm) of an attribute of 
     * the predicate held by this abstract descriptor.
     * This method is here mainly for backward compatibility.
     * @param name The name of the attribute.
     * @return value The value of the attribute.
     */
    public AbsTerm getAbsTerm(String name) {
        return (AbsTerm) getAbsObject(name);
    } 
    
    // Easy way to access the Java class representing AbsPredicate.
 		// Useful in MIDP where XXX.class is not available
		private static Class absPredicateClass = null;
 		public static Class getJavaClass() {
 			if (absPredicateClass == null) {
 				try {
 					absPredicateClass = Class.forName("rca.content.abs.AbsPredicate");
 				}
 				catch (Exception e) {
 					// Should never happen
 					e.printStackTrace();
 				}
 			}
 			return absPredicateClass;
 		}
 		/**
  	 * @see AbsContentElement.isAContentExpression
  	 */
  	public boolean isAContentExpression() {
  		return isAContentExpression;
  	}
  	
  	/**
  	 * @see AbsContentElement.setIsAContentExpression(boolean flag)
  	 */
  	public void setIsAContentExpression(boolean flag) {
  		isAContentExpression = flag;
  	}


		/** Return true if this object represents a meta formula (symbol ??x) rather than a concrete formula.
		 * This method is currently used only by the semantics framework.
		 * @return true if this object represents a meta formula
		 * @since RCA3.4
		 **/
		public final boolean isMetaFormula() {return isMeta;}

		/** Sets the value of isMetaFormula, where the default is false.
		 * This method is currently used only by the semantics framework.
		 * @since RCA3.4
		 **/
		public final void setIsMetaFormula(boolean isMeta) { this.isMeta = isMeta;}
}

