/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.lang;

import rca.content.onto.*;
import rca.content.abs.*;

/**
 * Base class for content language codecs that transform 
 * AbsContentElements to/from sequences of bytes
 * Original author Giovanni Caire - TILAB
 */
public abstract class ByteArrayCodec extends Codec{

    /**
     * Construct a ByteArrayCodec object with the given name
     */
    public ByteArrayCodec(String name) {
    	super(name);
    }

    /**
     * Encodes a content into a byte array.
     * @param content the content as an abstract descriptor.
     * @return the content as a byte array.
     * @throws CodecException
     */
    public abstract byte[] encode(AbsContentElement content) 
            throws CodecException;

    /**
     * Encodes a content into a byte array.
     * @param ontology the ontology 
     * @param content the content as an abstract descriptor.
     * @return the content as a byte array.
     * @throws CodecException
     */
    public abstract byte[] encode(Ontology ontology, AbsContentElement content) 
            throws CodecException;

    /**
     * Decodes the content to an abstract description.
     * @param content the content as a byte array.
     * @return the content as an abstract description.
     * @throws CodecException
     */
    public abstract AbsContentElement decode(byte[] content) 
            throws CodecException;

    /**
     * Decodes the content to an abstract description.
     * @param ontology the ontology.
     * @param content the content as a byte array.
     * @return the content as an abstract description.
     * @throws CodecException
     */
    public abstract AbsContentElement decode(Ontology ontology, byte[] content) 
            throws CodecException;

}

