/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.lang;

import rca.content.onto.*;
import rca.util.WrapperException;
import rca.util.leap.Serializable;

/**
 * Generic base class for al content language codecs
 * Original author Federico Bergenti - Universita` di Parma
 */
public abstract class Codec implements Serializable {

    /**
     * Class CodecException.
     *
     * Original author Federico Bergenti
     */
    public static class CodecException extends WrapperException {

        /**
         * Constructor
         *
         * @param message the message.
         *
         */
        public CodecException(String message) {
            super(message);
        }

	 			/**
	   			 Construct a new <code>CodecException</code>
	         @param msg The message for this exception.
	         @param t The exception wrapped by this object.
				 */
				public CodecException(String msg, Throwable t) {
	    		super(msg, t);
				}		

    }

    /** This string is the prefix of all the unnamed slots of a Frame **/
    public static String UNNAMEDPREFIX = "_JADE.UNNAMED"; 

    private String name = null;

    /**
     * Construct a Codec object with the given name
     */
    protected Codec(String name) {
        this.name = name;
    }

    /**
     * Gets the name of this codec.
     * @return the name of this codec.
     */
    public String getName() {
        return name;
    } 

    /**
     * @return the ontology containing the schemas of the operator
     * defined in this language
     */
    public Ontology getInnerOntology() {
    	return null;
    }
}

