/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.lang;

import rca.content.onto.*;
import rca.content.abs.*;

/**
 * Base class for content language codecs that transform 
 * AbsContentElements to/from sequences of strings
 * Original author Giovanni Caire - TILAB
 */
public abstract class StringCodec extends Codec{

    /**
     * Construct a StringCodec object with the given name
     */
    public StringCodec(String name) {
    	super(name);
    }

    /**
     * Encodes a content into a string.
     * @param content the content as an abstract descriptor.
     * @return the content as a string.
     * @throws CodecException
     */
    public abstract String encode(AbsContentElement content) 
            throws CodecException;

    /**
     * Encodes a content into a string using a given ontology.
     * @param ontology the ontology 
     * @param content the content as an abstract descriptor.
     * @return the content as a string.
     * @throws CodecException
     */
    public abstract String encode(Ontology ontology, AbsContentElement content) 
            throws CodecException;

    /**
     * Decodes the content to an abstract description.
     * @param content the content as a string.
     * @return the content as an abstract description.
     * @throws CodecException
     */
    public abstract AbsContentElement decode(String content) 
            throws CodecException;

    /**
     * Decodes the content to an abstract description using a 
     * given ontology.
     * @param ontology the ontology.
     * @param content the content as a string.
     * @return the content as an abstract description.
     * @throws CodecException
     */
    public abstract AbsContentElement decode(Ontology ontology, String content) 
            throws CodecException;

}

