/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.lang.sl;

import rca.content.onto.*;
import rca.core.CaseInsensitiveString;

/**
 * Ontology containing schemas for the SL0 language operators.
 * see rca.content.Ontology
 * Original author Giovanni Caire - TILAB
 */
class SL0Ontology extends Ontology implements SL0Vocabulary {
	// NAME
  public static final String ONTOLOGY_NAME = rca.domain.FIPANames.Ontology.SL0_ONTOLOGY;
	
  // The singleton instance of this ontology
	private static Ontology theInstance = new SL0Ontology(ONTOLOGY_NAME, BasicOntology.getInstance(), null);
	
	public static Ontology getInstance() {
		return theInstance;
	}
	
  /**
   * Constructor
   */
  protected SL0Ontology(String name, Ontology base, Introspector intro) {
  	super(name, base, intro);
	}

	boolean isUnaryLogicalOp(String symbol) {
		// There are no logical operators in SL0
		return false;
	}
	
	boolean isBinaryLogicalOp(String symbol) {
		// There are no logical operators in SL0
		return false;
	}
	
	boolean isQuantifier(String symbol) {
		// There are no quantifiers in SL0
		return false;
	}
	
	boolean isModalOp(String symbol) {
		// There are no modal operators in SL0
		return false;
	}
	
	boolean isActionOp(String symbol) {
		return CaseInsensitiveString.equalsIgnoreCase(DONE, symbol); 
	}
	
	boolean isBinaryTermOp(String symbol) {
		return (CaseInsensitiveString.equalsIgnoreCase(EQUALS, symbol) || 
			CaseInsensitiveString.equalsIgnoreCase(RESULT, symbol));
	}
	
	boolean isSLFunctionWithoutSlotNames(String symbol) {
		return CaseInsensitiveString.equalsIgnoreCase(ACTION, symbol); 
	}
}
