/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.lang.sl;

import rca.content.onto.*;
import rca.content.schema.*;
import rca.content.abs.AbsPredicate;
import rca.core.CaseInsensitiveString;

/**
 * Ontology containing schemas for the SL1 language operators.
 * see rca.content.Ontology
 * Original author Giovanni Caire - TILAB
 */
class SL1Ontology extends SL0Ontology implements SL1Vocabulary {
	// NAME
  public static final String ONTOLOGY_NAME = rca.domain.FIPANames.Ontology.SL1_ONTOLOGY;;
	
  // The singleton instance of this ontology
  // Note that the SL0Ontology does not add any schema to the BasicOntology
	private static Ontology theInstance = new SL1Ontology(ONTOLOGY_NAME, BasicOntology.getInstance(), null);
	
	public static Ontology getInstance() {
		return theInstance;
	}
	
  /**
   * Constructor
   */
  protected SL1Ontology(String name, Ontology base, Introspector intro) {
  	super(name, base, intro);
  	
  	try {
  		add(new PredicateSchema(AND), AbsPredicate.getJavaClass());
  		add(new PredicateSchema(OR), AbsPredicate.getJavaClass());
	  	add(new PredicateSchema(NOT), AbsPredicate.getJavaClass());
  	
  		PredicateSchema ps = (PredicateSchema) getSchema(AND);
  		ps.add(AND_LEFT, (PredicateSchema) PredicateSchema.getBaseSchema());
	  	ps.add(AND_RIGHT, (PredicateSchema) PredicateSchema.getBaseSchema());
  	
  		ps = (PredicateSchema) getSchema(OR);
  		ps.add(OR_LEFT, (PredicateSchema) PredicateSchema.getBaseSchema());
	  	ps.add(OR_RIGHT, (PredicateSchema) PredicateSchema.getBaseSchema());
  		
  		ps = (PredicateSchema) getSchema(NOT);
	  	ps.add(NOT_WHAT, (PredicateSchema) PredicateSchema.getBaseSchema());
    } 
    catch (OntologyException oe) {
      oe.printStackTrace();
    } 
  	
	}

	boolean isUnaryLogicalOp(String symbol) {
		return (CaseInsensitiveString.equalsIgnoreCase(NOT, symbol));
	}
	
	boolean isBinaryLogicalOp(String symbol) {
		return (CaseInsensitiveString.equalsIgnoreCase(AND, symbol) || 
			CaseInsensitiveString.equalsIgnoreCase(OR, symbol));
	}
	
}
