/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.onto;

import rca.content.abs.AbsObject;

/** 
   This interface must be implemented by ontological classes that
   belong to an ontology using the <code>MicroIntrospector</code>.
   It includes methods by means of which an object can be converted
   into/from an abstract descriptor.
   @see rca.content.onto.MicroIntrospector
   Original author Giovanni Caire - TILAB
 */
public interface Introspectable {

    /**
     * Externalise this object into the given abstract descriptor
     * @param abs The abstract descriptor this object must externalise 
     * itself into.
     * @param onto The reference ontology 
     * @throws OntologyException If some error occurs during the externalisation
     */
    public void externalise(AbsObject abs, Ontology onto) throws OntologyException; 

    /**
     * Internalise this object from a given abstract descriptor 
     * @param abs The abstract descriptor this object must internalise 
     * itself from
     * @param onto The reference ontology 
     * @throws UngroundedException If the abstract descriptor  
     * contains a variable
     * @throws OntologyException If some error occurs during the internalisation
     */
    public void internalise(AbsObject abs, Ontology onto) throws UngroundedException, OntologyException; 
}
