/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.onto.basic;

import rca.content.*;
import rca.content.abs.*;
import rca.content.onto.*;
import rca.core.AID;

/**
   This class implements the <code>action</code> operator of the
   FIPA SL0 action.
   Original author Giovanni Caire - TILAB
 */
public class Action implements AgentAction, Introspectable {
	private AID actor;
	private Concept action;
	
	public Action() {
		actor = null;
		action = null;
	}
	
	public Action(AID id, Concept a) {
		setActor(id);
		setAction(a);
	}
	
	public AID getActor() {
		return actor;
	}
	
	public void setActor(AID id) {
		actor = id;
	}	
	
	public Concept getAction() {
		return action;
	}
	
	public void setAction(Concept a) {
		action = a;
	}
	
  public void externalise(AbsObject abs, Ontology onto) throws OntologyException {
  	try {
  		AbsAgentAction absAction = (AbsAgentAction) abs;
  		absAction.set(BasicOntology.ACTION_ACTOR, (AbsTerm) onto.fromObject(getActor()));
  		absAction.set(BasicOntology.ACTION_ACTION, (AbsTerm) onto.fromObject(getAction()));
  	}
  	catch (ClassCastException cce) {
  		throw new OntologyException("Error externalising Action");
  	}
  }

  public void internalise(AbsObject abs, Ontology onto) throws UngroundedException, OntologyException {
    try {
  		AbsAgentAction absAction = (AbsAgentAction) abs;
  		setActor((AID) onto.toObject(absAction.getAbsObject(BasicOntology.ACTION_ACTOR))); 
  		setAction((Concept) onto.toObject(absAction.getAbsObject(BasicOntology.ACTION_ACTION))); 
  	}
  	catch (ClassCastException cce) {
  		throw new OntologyException("Error internalising Action");
  	}
  }
}
