/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.schema;

import rca.content.abs.*;
import rca.content.onto.OntologyException;

/**
 * This class represents the schema of a generic content element in
 * an ontology.
 * Original author Federico Bergenti - Universita` di Parma
 */
public class ContentElementSchema extends ObjectSchemaImpl {
    public static final String         BASE_NAME = "ContentElement";
    private static ContentElementSchema baseSchema = new ContentElementSchema();

    /**
     * Construct a schema that vinculates an entity to be a generic
     * content element
     */
    private ContentElementSchema() {
        super(BASE_NAME);
    }

    /**
     * Creates a <code>ContentElementSchema</code> with a given type-name.
     * @param typeName The name of this <code>ContentElementSchema</code>.
     */
    protected ContentElementSchema(String typeName) {
        super(typeName);
    }

    /**
     * Retrieve the generic base schema for all content elements.
     * @return the generic base schema for all content elements.
     */
    public static ObjectSchema getBaseSchema() {
        return baseSchema;
    } 

    /**
     * Creates an Abstract descriptor to hold a content element of
     * the proper type.
     */
    public AbsObject newInstance() throws OntologyException {
			throw new OntologyException("AbsContentElement cannot be instantieted"); 
    } 
    
  	/**
  	   Return true if 
  	   - s is the base schema for the XXXSchema class this schema is
  	     an instance of (e.g. s is ConceptSchema.getBaseSchema() and this 
  	     schema is an instance of ConceptSchema)
  	   - s is the base schema for a super-class of the XXXSchema class
  	     this schema is an instance of (e.g. s is TermSchema.getBaseSchema()
  	     and this schema is an instance of ConceptSchema)
  	 */
  	protected boolean descendsFrom(ObjectSchema s) {
  		if (s != null) {
  			if (s.equals(getBaseSchema())) {
	  			return true;
  			}
  			return super.descendsFrom(s);
  		}
  		else {
  			return false;
  		}
  	}
}
