/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.schema.facets;

import rca.content.onto.*;
import rca.content.schema.*;
import rca.content.abs.*;

/**
 * This facet forces an AbsAggregate to contain a number of 
 * elements that is comprised between a given min and a given max.
 * Original author Giovanni Caire - TILAB
 */
public class CardinalityFacet implements Facet {
    private int cardMin;
    private int cardMax;

    /**
       Construct a <code>CardinalityFacet</code> that forces
       the number of elements in an AbsAggregate to be within
       a given range
       @param cardMin The range lower bound
       @param cardMax The range upper bound
     */
    public CardinalityFacet(int cardMin, int cardMax) {
        this.cardMin = cardMin;
        this.cardMax = cardMax;
    }

    /**
       Check that
       the number of elements in an AbsAggregate is within
       a given range
       @param value The value to be checked
       @throws OntologyException If the value is not valid
     */
    public void validate(AbsObject value, Ontology onto) throws OntologyException {
        if (value instanceof AbsVariable) {
            // A variable can always be used where whatever term is required
            return;
        }

      if (!(value instanceof AbsAggregate)) {
          throw new OntologyException(value+" is not an AbsAggregate");
      }

      int size = ((AbsAggregate) value).size();
      if (size < cardMin) {
          throw new OntologyException(value+" includes less elements than required ("+cardMin+")");
      }
      if (cardMax != ObjectSchema.UNLIMITED && size > cardMax) {
          throw new OntologyException(value+" includes more elements than allowed ("+cardMax+")");
      }
    }
}