/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.content.schema.facets;

import rca.content.onto.*;
import rca.content.schema.*;
import rca.content.abs.*;
import rca.util.leap.Iterator;

/**
 * This facet forces the elements in an AbsAggregate
 * to be compliant to a given schema.
 * Original author Giovanni Caire - TILAB
 */
public class TypedAggregateFacet implements Facet {
	private ObjectSchema type;
	
	/**
	   Construct a <code>TypedAggregateFacet</code> that forces
	   the elements in an AbsAggregate to be instances of a given 
	   schema
	 */
	public TypedAggregateFacet(ObjectSchema s) {
		type = s;
	}
	
	/**
	  Get the schema associated to this facet
	*/
	public ObjectSchema getType() {
		return type;
	}
	/**
	   Check whether a given value for the slot this Facet applies
	   to is valid.
	   @param value The value to be checked
	   @throws OntologyException If the value is not valid
	 */
	public void validate(AbsObject value, Ontology onto) throws OntologyException {
		if (value instanceof AbsVariable) {
			// A variable can always be used where whatever term is required
			return;
		}
		
  	if (!(value instanceof AbsAggregate)) {
  		throw new OntologyException(value+" is not an AbsAggregate");
  	}
  	
  	AbsAggregate agg = (AbsAggregate) value;
		Iterator it = agg.iterator();
		while (it.hasNext()) {
			AbsTerm el = (AbsTerm) it.next();
			ObjectSchema s = onto.getSchema(el.getTypeName());
			if (!s.isCompatibleWith(type)) {
				throw new OntologyException("Schema "+s+" for element "+el+" is not compatible with type "+type);
			}
		}
	}
}