/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE

import rca.lang.acl.ACLMessage;

import rca.util.leap.List;

import rca.security.JADESecurityException;
import rca.security.JADEPrincipal;
import rca.security.Credentials;


/**
  Original author Giovanni Rimassa - Universita' di Parma
  @version $Date: 2005-06-09 14:20:07 +0200 (Thu, 09 Jun 2005) $ $Revision: 5712 $
*/

public interface AgentContainer {
  static final String MAIN_CONTAINER_NAME = "Main-Container";
  static final String AUX_CONTAINER_NAME = "Container";



    ContainerID getID();
    String getPlatformID();
    NodeDescriptor getNodeDescriptor();
    MainContainer getMain();
    ServiceManager getServiceManager();
    ServiceFinder getServiceFinder();

    AID getAMS();
    AID getDefaultDF();

    void initAgent(AID agentID, Agent instance,  
                   JADEPrincipal ownerPrincipal, Credentials initialCredentials                  
                   ) throws NameClashException, IMTPException, NotFoundException, JADESecurityException;
    void powerUpLocalAgent(AID agentID) throws NotFoundException;
    Agent addLocalAgent(AID id, Agent a);
    void removeLocalAgent(AID id);
    Agent acquireLocalAgent(AID id);
    void releaseLocalAgent(AID id);
    AID[] agentNames();

    //#MIDP_EXCLUDE_BEGIN
    void fillListFromMessageQueue(List messages, Agent a);
    void fillListFromReadyBehaviours(List behaviours, Agent a);
    void fillListFromBlockedBehaviours(List behaviours, Agent a);

    //void commitMigration(Agent instance);
    //void abortMigration(Agent instance);
    //#MIDP_EXCLUDE_END

    void addAddressToLocalAgents(String address);
    void removeAddressFromLocalAgents(String address);
    boolean postMessageToLocalAgent(ACLMessage msg, AID receiverID);
    boolean livesHere(AID id);
    Location here();

    void shutDown();

    void becomeLeader();
}
