/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#MIDP_EXCLUDE_FILE
//#APIDOC_EXCLUDE_FILE

import rca.domain.FIPAAgentManagement.AMSAgentDescription;
import rca.security.Credentials;
import rca.security.JADEPrincipal;



/**
   Hold all information about an agent
   Original author Giovanni Rimassa - Universita' di Parma
   Original author Giovanni Caire - TILAB
   @version $Date: 2005-10-07 10:36:28 +0200 (Fri, 07 Oct 2005) $ $Revision: 5795 $
*/

public class AgentDescriptor {

    public static final boolean NATIVE_AGENT = false;
    public static final boolean FOREIGN_AGENT = true;



    private AMSAgentDescription description;
    //  private AgentProxy proxy;
    private boolean foreign;
    private ContainerID containerID;
    private JADEPrincipal principal;
    private Credentials amsDelegation;


    public AgentDescriptor() {
	this(NATIVE_AGENT);
    }

    public AgentDescriptor(boolean isForeign) {
	foreign = isForeign;
    }

    // AMS description
    public void setDescription(AMSAgentDescription dsc) {
	description = dsc;
    }

    public AMSAgentDescription getDescription() {
	return description;
    }

    // Is this agent a foreign agent?
    public boolean isForeign() {
	return foreign;
    }


    // Is this agent a native agent?
    public boolean isNative() {
	return !foreign;
    }
    
   // Container ID
    public void setContainerID(ContainerID cid) {
	containerID = cid;
    }

    public ContainerID getContainerID() {
	return containerID;
    }

    // Agent principal
    public void setPrincipal(JADEPrincipal p) {
	principal = p;
    }

    public JADEPrincipal getPrincipal() {
	return principal;
    }

    // AMS delegation
    public void setAMSDelegation(Credentials cf) {
	amsDelegation = cf;
    }

    public Credentials getAMSDelegation() {
	return amsDelegation;
    }

}
