/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#MIDP_EXCLUDE_FILE
//#APIDOC_EXCLUDE_FILE

import java.lang.reflect.*;

/**
   This class is used internally by the framework and is not accessible
   to users.
   Original author Giovanni Caire - TILAB
 */
public class CallbackInvokator implements rca.util.leap.Serializable {
	// Package-scoped constructor to avoid creation outside the 
	// container
	CallbackInvokator() {
	}
	
	public void invokeCallbackMethod(Agent a, String name) {
		recursiveInvoke(a, a.getClass(), name);
	}
	
	private void recursiveInvoke(Agent a, Class agentClass, String name) {
		Method callbackMethod = null;
		try {
			callbackMethod = agentClass.getDeclaredMethod(name, (Class[]) null);
			//#J2ME_EXCLUDE_BEGIN
			//#DOTNET_EXCLUDE_BEGIN
			boolean accessibilityChanged = false;
			if (!callbackMethod.isAccessible()) {
				try {
					callbackMethod.setAccessible(true);
					accessibilityChanged = true;
				}
				catch (SecurityException se) {
					System.out.println("Callback method "+name+"() of agent "+a.getName()+" not accessible.");
				}
			}					
			//#DOTNET_EXCLUDE_END
			//#J2ME_EXCLUDE_END
			try { 			
				callbackMethod.invoke(a, (Object[]) null);
				//#J2ME_EXCLUDE_BEGIN
				//#DOTNET_EXCLUDE_BEGIN
				// Restore accessibility if changed
				if (accessibilityChanged) {
					callbackMethod.setAccessible(false);
				}
				//#DOTNET_EXCLUDE_END
				//#J2ME_EXCLUDE_END
			}
			catch (Exception e) {
				System.out.println("Error executing callback method "+name+"() of agent "+a.getName()+". "+e);
			}
		}
		catch (NoSuchMethodException e) {
			// Callback method not defined. Try in the superclass if any.	
			// Otherwise just ignore it
			Class superClass = agentClass.getSuperclass();
			if (superClass != null) {
				recursiveInvoke(a, superClass, name);
			}
		}
		catch (Exception e1) {
			e1.printStackTrace();
		}
	}
}
