/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;


//#APIDOC_EXCLUDE_FILE


/**
   The <code>Command</code> interface has to be implemented by all
   rca kernel-level commands, used by the various platform services.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.

*/
public interface Command {

    /**
       Query the name of this command object.

       @return A string containing the name of the given command.
    */
    String getName();

    /**
       Query the service this command object belongs to.

       @return The name of the service this command object belongs to.
    */
    String getService();

    /**
       Obtain the parameter list, as an array of Java objects.
       @return The parameters associated with this command object.
    */
    Object[] getParams();

    /**
       Assign a return value to this command, so that the original
       command issuer can retrieve it.
       @param rv The desired return value for this command object.
    */
    void setReturnValue(Object rv);

    /**
       Obtain the return value for this command.
       @return The value that is to be returned back to the issuer of
       this <code>Command</code> object.
    */
    Object getReturnValue();

    /**
        Get the JADEPrincipal of the actor, respnsible for this Command object.
        @return
      */
     rca.security.JADEPrincipal getPrincipal();
     void setPrincipal( rca.security.JADEPrincipal p );
 
     /**
        Get the credentials that the actor of this command
        wants to use while processing this command.
        @return
      */
     rca.security.Credentials getCredentials();
     void setCredentials( rca.security.Credentials creds );

}
