/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#J2ME_EXCLUDE_FILE

class FullResourceManager implements ResourceManager {
	private static final String USER_AGENTS_GROUP_NAME = "JADE User Agents";
	private static final String SYSTEM_AGENTS_GROUP_NAME = "JADE System Agents";
	private static final String CRITICAL_THREADS_GROUP_NAME = "JADE Time-critical Threads";
  
	private ThreadGroup parent;
	private ThreadGroup agentThreads;
  private ThreadGroup systemAgentThreads;
  private ThreadGroup criticalThreads;
      
    
  public FullResourceManager() {
  	parent = new ThreadGroup("FIXME: Dummy name");
  	agentThreads = new ThreadGroup(parent, USER_AGENTS_GROUP_NAME);
    agentThreads.setMaxPriority(Thread.NORM_PRIORITY);
  	
  	systemAgentThreads = new ThreadGroup(parent, SYSTEM_AGENTS_GROUP_NAME);
    systemAgentThreads.setMaxPriority(Thread.NORM_PRIORITY);
    
  	criticalThreads = new ThreadGroup(parent, CRITICAL_THREADS_GROUP_NAME);
    criticalThreads.setMaxPriority(Thread.MAX_PRIORITY);
  }
  
  public Thread getThread(int type, String name, Runnable r) {
  	Thread t = null;
  	switch (type) {
  	case USER_AGENTS:
  		t = new Thread(agentThreads, r);
      t.setPriority(agentThreads.getMaxPriority());
  		break;
  	case SYSTEM_AGENTS:
  		t = new Thread(systemAgentThreads, r);
      t.setPriority(systemAgentThreads.getMaxPriority());
  		break;
  	case TIME_CRITICAL:
  		t = new Thread(criticalThreads, r);
      t.setPriority(criticalThreads.getMaxPriority());
  		break;
  	}
  	if (t != null) {
  		t.setName(name);
  	}
  	
  	return t;
  }
  
  public void releaseResources() {
  	if (parent != null) {
	    parent.interrupt();
  	}
    
    agentThreads = null;
    systemAgentThreads = null;
		criticalThreads = null;
		parent = null;
  }
  	
}
  	
  		
  		
  
  
      