/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#MIDP_EXCLUDE_FILE

import rca.util.leap.Map;
import rca.util.leap.HashMap;

import java.util.concurrent.ConcurrentHashMap;
import java.util.Iterator;


/**
   The table holding information about all agents known to the
   platform (both agents living in the platform and remote agents
   registered with the platform AMS).
   Original author Giovanni Caire - TILAB
   Original author Giovanni Rimassa - Universita` di Parma
 */
class GADT {
//  private Map agents = new HashMap();
  private ConcurrentHashMap agents = new ConcurrentHashMap();


  public AgentDescriptor put(AID aid, AgentDescriptor a) {
      Row r;
//      AID t;
//      System.out.println("DEBUG=-> RequestedNameInAID  = "+aid.getName());

    synchronized(agents) {

/////////////////////////////////////////////
//        for (Object o : agents.keySet()) {
//            t = (AID) o;
//            System.out.println("DEBUG=-> Registered agent  " + t.getName());
//            if(t.getName().equals(aid.getName()))r = (Row)agents.get(aid);
//
//        }
///////////////////////////////////////////
          r = (Row)agents.get(aid);
    }

    if(r == null) {
          agents.put(aid, new Row(a));
          return null;
    }
    else {
          r.lock();

          agents.put(aid, new Row(a));
          AgentDescriptor old = r.get();

          r.unlock();
          return old;
    }
  }

  public AgentDescriptor remove(AID key) {
    Row r;
    synchronized(agents) {
            r = (Row)agents.get(key);
    }
    if (r == null) {
            return null;
    }
    else {
            r.lock();

            agents.remove(key);
            AgentDescriptor a = r.get();
            // Clear the row value, to avoid pending acquire() using the
            // removed agent descriptor...
            r.clear();

            r.unlock();
            return a;
    }
  }

  // The caller must call release() after it has finished with the row
  public AgentDescriptor acquire(AID key) {
    Row r;
    synchronized(agents) {
            r = (Row)agents.get(key);
    }
    if(r == null) {
            return null;
    }
    else {
            r.lock();
            return r.get();
    }
  }

  public void release(AID key) {
    Row r;
    synchronized(agents) {
            r = (Row)agents.get(key);
    }
    if(r != null) {
      r.unlock();
    }
  }

  public AID[] keys() {
    synchronized(agents) {
      Object[] objs = agents.keySet().toArray();
      AID[] result = new AID[objs.length];
      System.arraycopy(objs, 0, result, 0, result.length);
      return result;
    }
  }

  public AgentDescriptor[] values() {
    synchronized(agents) {
            Object[] objs = agents.values().toArray();
            AgentDescriptor[] result = new AgentDescriptor[objs.length];
            for(int i = 0; i < objs.length; i++) {
            Row r = (Row)objs[i];
            result[i] = r.get();
            }
            return result;
    }
  }

  /**
     Inner class Row.
     Rows of the GADT are protected by a recursive mutex lock
   */
  private static class Row {
    private AgentDescriptor value;
    private Thread owner;
    private long depth;

    public Row(AgentDescriptor a) {
      value = a;
      depth = 0;
    }

    public synchronized AgentDescriptor get() {
      return value;
    }

    public synchronized void clear() {
      value = null;
    }

    public synchronized void lock() {
      try {
        Thread me = Thread.currentThread();
                while((owner != null) && (owner != me)) {
                  wait();
                }

                owner = me;
        ++depth;
      }
      catch(InterruptedException ie) {
                return;
      }

    }

    public synchronized void unlock() {
      // Must be owner to unlock
      if(owner != Thread.currentThread()) {
          return;
      }
      --depth;
      if(depth == 0 || value == null) {
          // Note that if the row has just been cleared we must wake up
          // hanging threads even if depth is > 0, otherwise they will
          // hang forever
        owner = null;
        notifyAll();
      }
    }

  } // End of Row inner class
}
