/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


import rca.util.leap.Serializable;


/**
   The <code>HorizontalCommand</code> interface represents those
   kernel-level commands that are exchanged among the slices of a
   given rca service, and they are sent across the network by the
   platform IMTP.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.

*/
public interface HorizontalCommand extends Command, Serializable {


    /**
       Access the service object this command belongs to. A command
       object belongs exactly to one single service.

       @return The name of a <code>Service</code> object, such that
       this command is one of the allowed commands for that service.
    */
    String getService();

    /**
       Query the interaction this command object is a part of. A
       command object is part of exactly one single interaction. Some
       services can generate interactions made by many commands, which
       can be grouped together because calling
       <code>getInteraction()</code> on them yields the same result.

       @return A <code>String</code> serving as a service-unique
       identifier for the interaction of this command object.
    */
    String getInteraction();

}
