/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

import rca.util.leap.Serializable;
import rca.content.Concept;

/**
  
  Abstract interface to represent rca network locations. This interface can
  be used to access information about the various places where a rca mobile
  agent can migrate.

  Original author Giovanni Rimassa - Universita` di Parma
  @version $Date: 2002-02-04 11:31:40 +0100 (Mon, 04 Feb 2002) $ $Revision: 3043 $
*/
public interface Location extends Serializable, Concept {

  /**
    Read a unique ID for the location.
    @return A <code>String</code> representing the location.
  */
  String getID();
  
  /**
    Read the name of a location.
    @return A name for this location. The name has only a local meaning.
  */
  String getName();

  /**
    Read the protocol for a location.
    @return The name of the protocol used to reach this location.
  */
  String getProtocol();

  /**
    Read the address for a location.
    @return The transport address of this location (in the specified protocol).
  */
  String getAddress();

}
