/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


import rca.util.leap.Serializable;


/**
   This interface represents a node of a rca platform (i.e. a
   component that can host a slice of a platform-level service).
   Concrete nodes for a platform are created by the concrete IMTP
   manager.

   @see rca.core.IMTPManager

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public interface Node extends Serializable {

    void setName(String name);
    String getName();
    
    boolean hasPlatformManager();

    void exportSlice(String serviceName, Service.Slice localSlice);
    void unexportSlice(String serviceName);

    /**
       Accepts a command. If this node is a proxy, the
       <code>accept()</code> method is a remote method, forwarding the
       command to the remote location it represents.
       @param cmd The horizontal command to process.
       @return The object that is the result of processing the command.
       @throws IMTPException If a communication error occurs while
       contacting the remote node.
    */
    Object accept(HorizontalCommand cmd) throws IMTPException;

    /**
       Performs a ping operation on this node, to check whether it is
       still alive.

       @param hang If <code>true</code>, the call hangs until the node
       exits or is interrupted.
       @return If the node is currently terminating, <code>true</code>
       is returned, else <code>false</code>
    */
    boolean ping(boolean hang) throws IMTPException;

    void interrupt() throws IMTPException;
    void exit() throws IMTPException;
    void platformManagerDead(String deadPmAddress, String notifyingPmAddr) throws IMTPException;
}
