/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


/**
   This interface is used to notify the rca kernel of important
   events related to the platform nodes.
   <ul>
   <li>Node addition.</li>
   <li>Node removal.</li>
   <li>An existing node has become suddenly unreachable.</li>
   <li>An existing node has become reachable again.</li>
   </ul>

   Original author Giovanni Rimassa - FRAMeTech s.r.l.

   @see rca.core.ServiceManagerImpl
*/
public interface NodeEventListener {

    /**
       This method is invoked when a new node has joined the platform.

       @param n The newly added platform node.
    */
    void nodeAdded(Node n);

    /**
       This method is invoked when a node has left the platform.

       @param n The node that has just left.
    */
    void nodeRemoved(Node n);

    /**
       This method is invoked when the local node discovers that a
       platform node is not reachable anymore. The discovery is due to
       some IMTP-specific network mechanism, and <em>does not</em>
       mean that the remote node has failed. A subsequent
       <code>nodeReachable()</code> call might be issued when the IMTP
       connection becomes working again.

       @param n The node that has become unreachable.
    */
    void nodeUnreachable(Node n);

    /**
       This method is invoked when the local node discovers that a
       platform node, that was previously unreachable, has become
       reachable again.

       @param n The node that has become reachable again.
    */
    void nodeReachable(Node n);

}
