/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

import rca.domain.FIPANames;

/**
 * Description here
 * 
 * Original author <a href="mailto:Joan.Ametller@uab.es">Joan Ametller Esquerra</a>
 * Original author Carles Garrigues
 * Original author <a href="mailto:Jordi.Cucurull@uab.es">Jordi Cucurull Juan</a>
 * 
 */
public class PlatformID implements Location {
	private static final String NO_NAME = "__NO_NAME__";
	
	public PlatformID() {	
		_amsAID = new AID(FIPANames.AMS+'@'+NO_NAME, AID.ISGUID);
	}
	
	public PlatformID(AID amsAID){
		_amsAID = amsAID;
	}
	
	/* (non-Javadoc)
	 * @see rca.core.Location#getID()
	 */
	public String getID() {
		return _amsAID.getName();
	}
	
	/* (non-Javadoc)
	 * @see rca.core.Location#getName()
	 */
	public String getName() {
		String name = _amsAID.getName();
		return name.substring(name.lastIndexOf('@') + 1);
	}
	
	/* (non-Javadoc)
	 * @see rca.core.Location#getProtocol()
	 */
	public String getProtocol() {
		String protocol = null;
		String address = getAddress();
		if (address != null) {
			int index = address.indexOf(':');
			if (index > 0) {
				protocol = address.substring(0, index);
			}
		}
		return protocol;
	}
	
	/* (non-Javadoc)
	 * @see rca.core.Location#getAddress()
	 */
	public String getAddress() {
		return (String)_amsAID.getAllAddresses().next();
	}
	
	public AID getAmsAID(){
		return _amsAID;
	}
	
	public void setID(String id) {
		_amsAID.setName(id);
	}
	
	public void setName(String name) {
		_amsAID.setName(FIPANames.AMS+'@'+name);
	}
	
	public void setProtocol(String protocol) {
		// Do nothing: this is just for ontology support
	}
	
	public void setAddress(String address) {
		_amsAID.clearAllAddresses();
		_amsAID.addAddresses(address);
	}
	
	public void setAmsAID(AID amsAID) {
		_amsAID = amsAID;
	}
	
	private AID _amsAID;
}
