/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE

import rca.security.JADESecurityException;

import java.util.Vector;

/**
   Original author Giovanni Caire - TILAB
 */
public interface PlatformManager {
	static final String NO_NAME = "No-Name";
	static final String AUX_NODE_NAME = "Aux-Node";
	
    String getPlatformName() throws IMTPException;
    
	  String getLocalAddress();	
	  void setLocalAddress(String addr);
	  
    /**
       @param dsc The Descriptor of the new Node
       @param services The services currently installed on the new Node
       @param propagated Flag indicating whether the new-node event 
       was a propagated event within the replication mechanism
     */
    String addNode(NodeDescriptor dsc, Vector nodeServices, boolean propagated) throws IMTPException, ServiceException, JADESecurityException;
    void removeNode(NodeDescriptor dsc, boolean propagated) throws IMTPException, ServiceException;
    void addSlice(ServiceDescriptor service, NodeDescriptor dsc, boolean propagated)  throws IMTPException, ServiceException;
    void removeSlice(String serviceKey, String sliceKey, boolean propagated)  throws IMTPException, ServiceException;
    void addReplica(String newAddr, boolean propagated)  throws IMTPException, ServiceException;
    void removeReplica(String address, boolean propagated)  throws IMTPException, ServiceException;
    
    Service.Slice findSlice(String serviceKey, String sliceKey) throws IMTPException, ServiceException;
    Vector findAllSlices(String serviceKey) throws IMTPException, ServiceException;
    
    void adopt(Node n, Node[] children) throws IMTPException;
    void ping() throws IMTPException;
}
