/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


/**
   This is the interface that must be implemented by a class 
   managing Thread resources on a <code>Container</code>
   @see FullResourceManager
   Original author Giovanni Caire - TILAB
 */
public interface ResourceManager {
	// Constants identifying the types of Thread that can be 
	// requested to the ResourceManager 
	public static final int USER_AGENTS = 0;
	public static final int SYSTEM_AGENTS = 1;
	public static final int TIME_CRITICAL = 2;

	/** 
	   Return a Thread without starting it.
	   @param type The type of the Thread that will be returned: valid 
	   types are <code>USER_AGENTS</code>, <code>SYSTEM_AGENTS</code>,
	   <code>TIME_CRITICAL</code>.
	   @param r The <code>Runnable</code> object that will executed by the 
	   returned <code>Thread</code>.
	 */
  public Thread getThread(int type, String name, Runnable r);
  public void releaseResources();
}
