/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


/**

   The <code>ServiceFinder</code> interface serves as an access point
   for kernel-level service discovery. From a service finder object
   one can look up a given service by name, or can directly ask for a
   slice of the given service.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.

   @see rca.core.Service

*/
public interface ServiceFinder {

    static final String MAIN_SLICE = "$$$Main-Slice$$$";
    static final String THIS_SLICE = "$$$This-Slice$$$";

    /**
       Looks up a platform service by name.

       @param key The name of the service. Concrete syntax for service
       names is left up to concrete services.
    */
    Service findService(String key) throws IMTPException, ServiceException;

    /**
       Looks up a specific service slice by name.

       @param key A structured name identifying both a service and a
       slice within it. Concrete syntax for service names is left up
       to concrete services.
    */
    Service.Slice findSlice(String serviceKey, String sliceKey) throws IMTPException, ServiceException;

    /**
       Retrieves all the slices of a service currently active on this platform.
       @param serviceKey The name of the service. Concrete syntax for
       service names is left up to concrete services.
       @return An array of <code>Service.Slice</code> objects,
       containing all the slices of the requested service.
    */
    Service.Slice[] findAllSlices(String serviceKey) throws IMTPException, ServiceException;

}
