/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


/**
   The <code>Sink</code> interface has to be implemented by all the
   components that process rca kernel-level commands in an exclusive
   and terminal way.

   For each kind of vertical command, there must be at most one sink
   that is registered for that command.  Instead, many command filters
   can be applied to a single command.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.

   @see rca.core.Filter
*/
public interface Sink {

    static final boolean COMMAND_SOURCE = false;
    static final boolean COMMAND_TARGET = true;

    /**
       Definitely consume a command object. This method is invoked by
       the kernel when all incoming filters have been applied to the
       incoming vertical command.

       @param cmd A <code>VerticalCommand</code> describing what
       operation has been requested from previous layers (that can be
       the actual prime source of the command or members of the filter
       chain).
    */
    void consume(VerticalCommand cmd);

}
