/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE

/**
 An implementation of the <code>Slice</code> interface,
 supporting routed dispatching of horizontal commands.
*/
public class SliceProxy implements Service.Slice 
{

	public SliceProxy() 
	{
		this(null, null);
	}

	public SliceProxy(Service svc, Node n) 
	{
		myService = svc;
		myNode = n;
	}

	public Service getService() 
	{
		return myService;
	}

	public Node getNode() throws ServiceException 
	{
		return myNode;
	}

	public void setNode(Node n) 
	{
		myNode = n;
	}

	/**
	 Try to serve an incoming horizontal command, routing it to
	 a remote slice implementation.

	 @param cmd The command to serve, possibly through the network.
	 */
	public VerticalCommand serve(HorizontalCommand cmd) 
	{
		try 
		{
			cmd.setReturnValue(myNode.accept(cmd));
		}
		catch(IMTPException imtpe) 
		{
			cmd.setReturnValue(new ServiceException("An error occurred while routing the command to the remote implementation", imtpe));
		}
		// No local processing of this command is required
		return null;
	}

	private Node myNode;
	private transient Service myService;

}
