/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core;

//#APIDOC_EXCLUDE_FILE


/**
This class is used by the rca internal timing system to schedule
actions after a given amount of time. It should not
be used by application developers. 
Original author Giovanni Rimassa - Universita' di Parma
@version $Date: 2003-12-23 12:46:50 +0100 (Tue, 23 Dec 2003) $ $Revision: 4712 $
*/
public class Timer /*implements Comparable*/ {

  private long expireTimeMillis;
  private boolean fired;
  private TimerListener owner;

  public Timer(long when, TimerListener tl) {
    expireTimeMillis = when;
    owner = tl;
    fired = false;
  }

  /*public int compareTo(Object o) {
    if(equals(o))
      return 0;
    else {
      Timer t = (Timer)o;
      return (expireTimeMillis <= t.expireTimeMillis) ? -1 : 1;
    }
  }*/

  public boolean equals(Object o) {
    Timer t = (Timer)o;
    return (expireTimeMillis == t.expireTimeMillis);
  }


  // Called by the TimerDispatcher

  boolean isExpired() {
    return expireTimeMillis < System.currentTimeMillis();
  }
  
  void fire() {
  	if (!fired) {
  		fired = true;
	  	owner.doTimeOut(this);
  	}
  }

  long expirationTime() {
    return expireTimeMillis;
  }

  void setExpirationTime(long t) {
    expireTimeMillis = t;
  }

}
