/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.behaviours;

import rca.core.Agent;


/**
   Atomic behaviour that must be executed forever. This abstract class
   can be extended by application programmers to create behaviours
   that keep executing continuously (e.g. simple reactive behaviours).


   Original author Giovanni Rimassa - Universita' di Parma
   @version $Date: 2003-11-18 14:24:25 +0100 (Tue, 18 Nov 2003) $ $Revision: 4550 $

 */
public abstract class CyclicBehaviour extends SimpleBehaviour {

  /**
     Default constructor. It does not set the owner agent.
  */
  public CyclicBehaviour() {
    super();
  }

  /**
     This constructor sets the owner agent for this
     <code>CyclicBehaviour</code>.
     @param a The agent this behaviour must belong to.
  */
  public CyclicBehaviour(Agent a) {
    super(a);
  }

  /**
     This is the method that makes <code>CyclicBehaviour</code>
     cyclic, because it always returns <code>false</code>.
     @return Always <code>false</code>.
  */
  public final boolean done() {
    return false;
  }

}

