/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.behaviours;

import rca.core.Agent;

/**
   Atomic behaviour that executes just once. This abstract class can
   be extended by application programmers to create behaviours for
   operations that need to be done just one time.
   
   
   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2000-09-12 15:24:08 +0200 (Tue, 12 Sep 2000) $ $Revision: 1857 $

*/
public abstract class OneShotBehaviour extends SimpleBehaviour {

  /**
     Default constructor. It does not set the owner agent.
  */
  public OneShotBehaviour() {
    super();
  }

  /**
     This constructor sets the owner agent for this
     <code>OneShotBehaviour</code>.
     @param a The agent this behaviour belongs to.
  */
  public OneShotBehaviour(Agent a) {
    super(a);
  }

  /**
     This is the method that makes <code>OneShotBehaviour</code>
     one-shot, because it always returns <code>true</code>.
     @return Always <code>true</code>.
  */
  public final boolean done() {
    return true;
  }

}
