/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.behaviours;

//#APIDOC_EXCLUDE_FILE

import rca.core.Agent;

import rca.lang.acl.ACLMessage;

/**
   Behaviour for sending an ACL message. This class encapsulates a
   <code>send()</code> as an atomic operation. This behaviour sends a
   given ACL message and terminates.
   @see rca.core.behaviours.ReceiverBehaviour
   @see rca.core.Agent#send(ACLMessage msg)
   @see rca.lang.acl.ACLMessage
   
   
   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2003-11-25 09:24:45 +0100 (Tue, 25 Nov 2003) $ $Revision: 4601 $

 */
public final class SenderBehaviour extends OneShotBehaviour {


  // The ACL message to send
	/**
	@serial
	*/
  private ACLMessage message;

  /**
     Send a given ACL message. This constructor creates a
     <code>SenderBehaviour</code> which sends an ACL message.
     @param a The agent this behaviour belongs to, and that will
     <code>send()</code> the message.
     @param msg An ACL message to send.
  */
  public SenderBehaviour(Agent a, ACLMessage msg) {
    super(a);
    message = msg;
    if (msg != null)
	message.setSender(myAgent.getAID());
  }

  /**
     Actual behaviour implementation. This method sends an ACL
     message, using either the given <code>AgentGroup</code> or the
     <code>:receiver</code> message slot to get the message recipient
     names.
  */
  public void action() {
    myAgent.send(message);
  }

}
