/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.behaviours;

import rca.core.Agent;


/**
   An atomic behaviour. This abstract class models behaviours that are
   made by a single, monolithic task and cannot be interrupted.
   
   
   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2000-10-09 09:03:44 +0200 (Mon, 09 Oct 2000) $ $Revision: 1919 $

*/
public abstract class SimpleBehaviour extends Behaviour {

  /**
     Default constructor. It does not set the owner agent for this
     behaviour.
  */
  public SimpleBehaviour() {
    super();
  }

  /**
     This constructor sets the owner agent for this behaviour.
     @param a The agent this behaviour belongs to.
  */
  public SimpleBehaviour(Agent a) {
    super(a);
  }    

  /**
     Resets a <code>SimpleBehaviour</code>. 
  */
  public void reset() {
    super.reset();
  }

}
