/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.event;

import rca.core.AID;
import rca.core.BehaviourID;
import rca.core.ContainerID;
import rca.core.AgentState;

//__SECURITY__BEGIN
import rca.security.JADEPrincipal;
//__SECURITY__END

/**
   This class represents an event related to the agent life-cycle and
   configuration.

   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2004-04-23 11:08:16 +0200 (Fri, 23 Apr 2004) $ $Revision: 4988 $
 */
public class AgentEvent extends JADEEvent {

  public static final int CHANGED_AGENT_STATE = 1;
  public static final int ADDED_BEHAVIOUR = 2;
  public static final int REMOVED_BEHAVIOUR = 3;
  public static final int CHANGED_BEHAVIOUR_STATE = 4;
  public static final int CHANGED_AGENT_PRINCIPAL = 5;

  //private int myID;

  private AID agent;
  private BehaviourID behaviour = null;
  private AgentState from = null;
  private AgentState to = null;
  private String behaviourFrom = null;
  private String behaviourTo = null;
//__SECURITY__BEGIN
  private JADEPrincipal oldPrincipal = null;
  private JADEPrincipal newPrincipal = null;
//__SECURITY__END

  public AgentEvent(int id, AID aid, AgentState f, AgentState t, ContainerID cid) {
    super(id, cid);
    //myID = id;
    if(!isChangedAgentState()) {
      throw new InternalError("Bad Event kind: it must be a 'changed-agent-state' event.");
    }
    agent = aid;
    behaviour = null;
    from = f;
    to = t;
  }

//__SECURITY__BEGIN
  public AgentEvent(int id, AID aid, JADEPrincipal f, JADEPrincipal t, ContainerID cid) {
    super(id, cid);
    //myID = id;
    if(!isChangedAgentPrincipal()) {
      throw new InternalError("Bad Event kind: it must be a 'changed-agent-principal' event.");
    }
    agent = aid;
    oldPrincipal = f;
    newPrincipal = t;
  }
//__SECURITY__END

  public AgentEvent(int id, AID aid, BehaviourID bid, ContainerID cid) {
    super(id, cid);
    //myID = id;
    if(isChangedAgentState() || isChangedBehaviourState()) {
      throw new InternalError("Bad Event kind: it must be an 'added/removed-behaviour' event.");
    }
    agent = aid;
    behaviour = bid;
    from = null;
    to = null;
  }

  public AgentEvent(int id, AID aid, BehaviourID bid, String strFrom, String strTo, ContainerID cid) {
    super(id, cid);
    //myID = id;
    if(!isChangedBehaviourState()) {
      throw new InternalError("Bad Event kind: it must be a 'changed-behaviour-state' event.");
    }

    agent = aid;
    behaviour = bid;
    
    from = null;
    to = null;
    
    behaviourFrom = strFrom;
    behaviourTo = strTo;

  }

  public AID getAgent() {
    return agent;
  }

  public BehaviourID getBehaviour() {
    return behaviour;
  }

  public AgentState getFrom() {
    return from;
  }

  public AgentState getTo() {
    return to;
  }

  public String getBehaviourFrom() {
      return behaviourFrom;
  }
  
  public String getBehaviourTo() {
      return behaviourTo;
  }

//__SECURITY__BEGIN
  public JADEPrincipal getOldPrincipal() {
    return oldPrincipal;
  }

  public JADEPrincipal getNewPrincipal() {
    return newPrincipal;
  }
//__SECURITY__END

  private boolean isChangedAgentPrincipal() {
    return type == CHANGED_AGENT_PRINCIPAL; 
  }

  private boolean isChangedAgentState() {
    return type == CHANGED_AGENT_STATE; 
  }

  private boolean isChangedBehaviourState() {
    return type == CHANGED_BEHAVIOUR_STATE;
  }

}
