/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.event;

import rca.core.ContainerID;
import rca.core.Channel;

import rca.domain.FIPAAgentManagement.Envelope;

/**
   This class represents an event related to the MTP configuration.

   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2002-08-28 17:14:13 +0200 (Wed, 28 Aug 2002) $ $Revision: 3354 $
 */
public class MTPEvent extends JADEEvent {

  public static final int ADDED_MTP = 1;
  public static final int REMOVED_MTP = 2;
  public static final int MESSAGE_IN = 3;
  public static final int MESSAGE_OUT = 4;

  //private int myID; // The actual type of the event
  private Channel chan;
  private Envelope env;
  private byte[] payload;

  public MTPEvent(int id, ContainerID cid, Channel ch) {
    super(id, cid);
    //myID = id;
    if(!isInstall()) {
      throw new InternalError("Bad event kind: it must be an MTP installation related kind.");
    }
    chan = ch;
    env = null;
    payload = null;
  }

  public MTPEvent(int id, ContainerID cid, Envelope e, byte[] pl) {
    super(id, cid);
    //myID = id;
    if(!isCommunication()) {
      throw new InternalError("Bad event kind: it must be a communication related kind.");
    }
    chan = null;
    env = e;
    payload = pl;
  }

  public Channel getChannel() {
    return chan;
  }

  public Envelope getEnvelope() {
    return env;
  }

  public byte[] getPayload() {
    return payload;
  }

  public boolean isInstall() {
    return (type == ADDED_MTP) || (type == REMOVED_MTP);
  }

  public boolean isCommunication() {
    return (type == MESSAGE_IN) || (type == MESSAGE_OUT);
  }

  // Nothing to add to superclass

}
