/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.event;

import rca.core.AID;
import rca.core.ContainerID;
import rca.core.Channel;
import rca.lang.acl.ACLMessage;

/**
   This class represents the events related to ACL message passing.

   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2004-10-05 10:35:16 +0200 (Tue, 05 Oct 2004) $ $Revision: 5412 $

 */
public class MessageEvent extends JADEEvent {

  public static final int SENT_MESSAGE = 1;
  public static final int POSTED_MESSAGE = 2;
  public static final int RECEIVED_MESSAGE = 3;
  public static final int ROUTED_MESSAGE = 4;

  private ACLMessage message = null;
  private AID sender = null;
  private AID receiver = null;
  private Channel from = null;
  private Channel to = null;

  public MessageEvent(int id, ACLMessage msg, AID s, AID r, ContainerID cid) {
    super(id, cid);
    if(isRouting()) {
      throw new InternalError("Bad event kind: it must not be a 'message-routed' event.");
    }
    message = msg;
    sender = s;
    receiver = r;
  }

  public MessageEvent(int id, ACLMessage msg, Channel f, Channel t, ContainerID cid) {
    super(id, cid);
    if(!isRouting()) {
      throw new InternalError("Bad event kind: it must be a 'message-routed' event.");
    }
    message = msg;
    from = f;
    to = t;
  }

  public ACLMessage getMessage() {
    return message;
  }

  public AID getSender() {
  	if (sender != null) {
	    return sender;
  	}
  	else {
  		return message.getSender();
  	}
  }

  public AID getReceiver() {
    return receiver;
  }
  
  public AID getAgent() {
  	if (type == SENT_MESSAGE) {
  		return getSender();
  	}
  	else {
  		return getReceiver();
  	}
  }

  public Channel getFrom() {
    return from;
  }

  public Channel getTo() {
    return to;
  }

  public boolean isRouting() {
    return type == ROUTED_MESSAGE;
  }

}
