/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.event;

//#J2ME_EXCLUDE_FILE


import rca.core.Node;
import rca.core.GenericCommand;
import rca.core.SliceProxy;
import rca.core.ServiceException;
import rca.core.AID;
import rca.core.IMTPException;


/**
   The remote proxy for the rca kernel-level service managing
   the event notification subsystem installed in the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public class NotificationProxy extends SliceProxy implements NotificationSlice {


    public void sniffOn(AID snifferName, AID targetName) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_SNIFFON, NotificationSlice.NAME, null);
	    cmd.addParam(snifferName);
	    cmd.addParam(targetName);


	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void sniffOff(AID snifferName, AID targetName) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_SNIFFOFF, NotificationSlice.NAME, null);
	    cmd.addParam(snifferName);
	    cmd.addParam(targetName);


	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void debugOn(AID introspectorName, AID targetName) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEBUGON, NotificationSlice.NAME, null);
	    cmd.addParam(introspectorName);
	    cmd.addParam(targetName);


	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void debugOff(AID introspectorName, AID targetName) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEBUGOFF, NotificationSlice.NAME, null);
	    cmd.addParam(introspectorName);
	    cmd.addParam(targetName);


	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

}
