/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.event;


//#MIDP_EXCLUDE_FILE


import rca.core.Service;
import rca.core.AID;
import rca.core.IMTPException;


/**
   The horizontal interface for the rca kernel-level service managing
   the event notification subsystem installed in the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public interface NotificationSlice extends Service.Slice {


    /**
       The name of this service.
    */
    static final String NAME = "rca.core.event.Notification";

    /**
       This command name represents the action of activating an
       instance of the Sniffer agent.
    */
    static final String SNIFF_ON = "Sniff-On";

    /**
       This command name represents the action of deactivating a
       previously started instance of the Sniffer agent.
    */
    static final String SNIFF_OFF = "Sniff-Off";

    /**
       This command name represents the action of activating an
       instance of the Introspector agent.
    */
    static final String DEBUG_ON = "Debug-On";

    /**
       This command name represents the action of deactivating a
       previously started instance of the Introspector agent.
    */
    static final String DEBUG_OFF = "Debug-Off";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>MessagePosted</code> event.
    */
    static final String NOTIFY_POSTED = "Notify-Posted";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>MessageReceived</code> event.
    */
    static final String NOTIFY_RECEIVED = "Notify-Received";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>ChangedAgentState</code> event.
    */
    static final String NOTIFY_CHANGED_AGENT_STATE = "Notify-Changed-Agent-State";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>ChangedAgentPrincipal</code> event.
    */
    static final String NOTIFY_CHANGED_AGENT_PRINCIPAL = "Notify-Changed-Agent-Principal";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>BehaviourAdded</code> event.
    */
    static final String NOTIFY_BEHAVIOUR_ADDED = "Notify-BehaviourAdded";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>BehaviourRemoved</code> event.
    */
    static final String NOTIFY_BEHAVIOUR_REMOVED = "Notify-Behaviour-Removed";

    /**
       This command name represents the action of submitting to the
       Notification Service a <code>ChangedBehaviourState</code> event.
    */
    static final String NOTIFY_CHANGED_BEHAVIOUR_STATE = "Notify-Changed-Behaviour-State";



    // Constants for the names of horizontal commands associated to methods
    static final String H_SNIFFON = "1";
    static final String H_SNIFFOFF = "2";
    static final String H_DEBUGON = "3";
    static final String H_DEBUGOFF = "4";

    void sniffOn(AID snifferName, AID targetName) throws IMTPException;
    void sniffOff(AID snifferName, AID targetName) throws IMTPException;
    void debugOn(AID introspectorName, AID targetName) throws IMTPException;
    void debugOff(AID introspectorName, AID targetName) throws IMTPException;

}
