/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
 
package rca.core.management;

//#J2ME_EXCLUDE_FILE

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;
/**
 * This ClassLoader is intended to be used to load agent classes
 * packed within a jar file. If the specified jar does not exist this ClassLoader
 * will attempt to load classes from the system ClassLoader.
 * Original author <a href="mailto:Joan.Ametller@uab.es">Joan Ametller</a>
 * 
 */
public class JarClassLoader extends ClassLoader {
  
	public static final int BUFFER_SIZE = 1024;
	
  /**
   * @param Path and name of the JAR file
   * @throws IOException If there are problems opening the file
   */
  public JarClassLoader(File f, ClassLoader parent) throws IOException{
    super(parent);
    if (f != null) {
    	_file = new JarFile(f);
    }
  	/*File file = new File(filename);
    if(file.exists()){
      _file = new JarFile(new File(filename));
    }*/
  }
  
  /**
   * Method to close file descriptor for the JAR used by this ClassLoader.
   * If this method is invoked, no more classes from the JAR file will be loaded
   * using this ClassLoader.
   * @throws IOException File cannot be closed
   */
  public void close() throws IOException{
    if(_file != null) _file.close();
  }
  
  protected Class findClass(String className) throws ClassNotFoundException{
    if (_file != null) {
	  	ZipEntry zEntry = _file.getEntry(className.replace('.', '/') + ".class");
	    try{
	      InputStream is = _file.getInputStream(zEntry);
	      byte[] rawClass = readFully(is);
	      is.close();
	      return defineClass(className, rawClass, 0, rawClass.length);
	    }
	    catch(IOException ioe){
	      throw new ClassNotFoundException("IOError while reading jar file for class "+className+". "+ioe);
	    }
    }
    else {
    	throw new ClassNotFoundException(className);
    }
  }
  
  private byte[] readFully(InputStream is) throws IOException{
  	byte[] buffer = new byte[BUFFER_SIZE];
  	ByteArrayOutputStream baos = new ByteArrayOutputStream();
  	int read = 0;
  	
  	while((read=is.read(buffer))>= 0) baos.write(buffer,0,read);
  	
  	return baos.toByteArray();
  }
  
  private JarFile _file = null;

}
