/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.core.messaging;

import rca.domain.FIPAAgentManagement.Envelope;
import rca.core.AID;
import rca.lang.acl.ACLMessage;
import rca.util.leap.Serializable;

import rca.security.JADEPrincipal;
import rca.security.Credentials;

//#MIDP_EXCLUDE_BEGIN
import rca.lang.acl.LEAPACLCodec;
import java.io.ObjectOutputStream;
import java.io.IOException;
//#MIDP_EXCLUDE_END

/**
 * Generic class to manage a unified representation of messages
 * (ACLMessage or Payload+Envelope)
 *
 * Original author Jerome Picault - Motorola Labs
 * @version $Date: 2005-09-21 16:28:47 +0200 (Wed, 21 Sep 2005) $ $Revision: 5787 $
 * 
 */
public class GenericMessage implements Serializable {

  private transient ACLMessage msg;
  private Envelope env;
  private byte[] payload;
  private transient JADEPrincipal senderPrincipal;
  private transient Credentials senderCredentials;
  private boolean isAMSFailure = false;
  private transient boolean foreignReceiver = false;
  private String traceID = null;

  public GenericMessage(){
  }
  
  public GenericMessage(ACLMessage msg){
    this.msg = msg;
  }

  public GenericMessage(Envelope env, byte[] payload){
    this.env = env;
    this.payload = payload;
  }

  public final byte[] getPayload(){
    return payload;
  }

  public final Envelope getEnvelope(){
    return env;
  }

  public final ACLMessage getACLMessage(){
    return msg;
  }

  public final void setACLMessage(ACLMessage msg){
    this.msg = msg;
  }

  public final void update(ACLMessage msg, Envelope env, byte[]payload){
    this.msg = msg;
    this.env = env;
    this.payload = payload;
  }

  final void setSenderPrincipal(JADEPrincipal senderPrincipal) {
  	this.senderPrincipal = senderPrincipal;
  }
  
  final JADEPrincipal getSenderPrincipal() {
  	return senderPrincipal;
  }
  
  final void setSenderCredentials(Credentials senderCredentials) {
  	this.senderCredentials = senderCredentials;
  }
  
  final Credentials getSenderCredentials() {
  	return senderCredentials;
  }

  public final boolean isAMSFailure() {
    return isAMSFailure;
  }

  public final void setAMSFailure(boolean b) {
    isAMSFailure=b;
  }
  
  final boolean hasForeignReceiver() {
  	return foreignReceiver;
  }
  
  final void setForeignReceiver(boolean b) {
  	foreignReceiver = b;
  }
  
  public final String getTraceID() {
  	return traceID;
  }
  
  public final void setTraceID(String id) {
  	traceID = id;
  }
  
	//#MIDP_EXCLUDE_BEGIN
  private void writeObject(ObjectOutputStream out) throws IOException {
  	// Updates the payload if not present, before serialising
    if (payload==null){
      payload = (new LEAPACLCodec()).encode(msg, null);
    }
    out.defaultWriteObject();
  }
	//#MIDP_EXCLUDE_END

  public final AID getSender(){
    if (msg!=null) return msg.getSender();
    else if (env!=null) return env.getFrom();
    else return null;
  }

  // DEBUG
  public String toString(){
    return "GenericMessage\n\t"+msg+"\n\t"+env+"\n\t"+((payload==null)?"null payload":payload.toString())+"\n";
  }

  public final int length() {
  	int length = 0;
		if (payload != null) {
			length = payload.length;
		}
		else {
			if (msg != null) {
				byte[] content = msg.getByteSequenceContent();
				if (content != null) {
					length = content.length;
				}
			}
		}
		return length;
  }
}
