/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.messaging;

import rca.core.Service;
import rca.core.AID;
import rca.core.ContainerID;
import rca.core.IMTPException;
import rca.core.ServiceException;
import rca.core.NotFoundException;
import rca.security.JADESecurityException;

import rca.mtp.MTPDescriptor;
import rca.mtp.MTPException;

import rca.domain.FIPAAgentManagement.Envelope;

/**
   The horizontal interface for the rca kernel-level service managing
   the message passing subsystem installed in the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public interface MessagingSlice extends Service.Slice {

  // Constants for the names of the service vertical commands

  /**
     The name of this service.
  */
  static final String NAME = "rca.core.messaging.Messaging";

  /**
     This command name represents the action of sending an ACL
     message from an agent to another.
  */
  static final String SEND_MESSAGE = "Send-Message";

  /**
     This command name represents the action of sending back a
     FAILURE ACL message to notify the message originator of a
     failed delivery.
  */
  static final String NOTIFY_FAILURE = "Notify-Failure";

  /**
     This command name represents the <code>install-mtp</code>
     action.
  */
  static final String INSTALL_MTP = "Install-MTP";

  /**
     This command name represents the <code>uninstall-mtp</code>
     action.
  */
  static final String UNINSTALL_MTP = "Uninstall-MTP";

  /**
     This command name represents the <code>new-mtp</code>
     event.
  */
  static final String NEW_MTP = "New-MTP";

  /**
     This command name represents the <code>dead-mtp</code>
     action.
  */
  static final String DEAD_MTP = "Dead-MTP";

  /**
     This command name represents the <code>set-platform-addresses</code>
     action.
  */
  static final String SET_PLATFORM_ADDRESSES = "Set-Platform-Addresses";



  // Constants for the names of horizontal commands associated to methods
  static final String H_DISPATCHLOCALLY = "1";
  static final String H_ROUTEOUT = "2";
  static final String H_GETAGENTLOCATION = "3";
  static final String H_INSTALLMTP = "4";
  static final String H_UNINSTALLMTP ="5";
  static final String H_NEWMTP = "6";
  static final String H_DEADMTP = "7";
  static final String H_ADDROUTE = "8";
  static final String H_REMOVEROUTE = "9";

  void dispatchLocally(AID senderAID, GenericMessage msg, AID receiverID) throws IMTPException, NotFoundException, JADESecurityException;
  void routeOut(Envelope env, byte[] payload, AID receiverID, String address) throws IMTPException, MTPException;
  ContainerID getAgentLocation(AID agentID) throws IMTPException, NotFoundException;

  MTPDescriptor installMTP(String address, String className) throws IMTPException, ServiceException, MTPException;
  void uninstallMTP(String address) throws IMTPException, ServiceException, NotFoundException, MTPException;

  void newMTP(MTPDescriptor mtp, ContainerID cid) throws IMTPException, ServiceException;
  void deadMTP(MTPDescriptor mtp, ContainerID cid) throws IMTPException, ServiceException;

  void addRoute(MTPDescriptor mtp, String sliceName) throws IMTPException, ServiceException;
  void removeRoute(MTPDescriptor mtp, String sliceName) throws IMTPException, ServiceException;

}
