/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.messaging;

//#J2ME_EXCLUDE_FILE


import rca.lang.acl.ACLMessage;


/**
   This interface can be used 

*/
public interface PersistentDeliveryFilter {

    /**
       The constant to specifiy an immediate failure notification
       (i.e. no message buffering).
    */
    static final long NOW = 0;

    /**
       The constant to specify no failure notification (i.e. message
       buffering for infinitely long).
    */
    static final long NEVER = -1;

    /**
       The application-specific method to control which messages are
       to be buffered and for how long.

       @param msg The undelivered ACL message, that is to be tested
       against this filter and possibly buffered.
       @return The delay, in milliseconds, within which the message is
       to be delivered. If the message is still undelivered after that
       delay, the delivery process aborts and a <code>failure</code>
       message is sent back to the message originator. The specially
       defined <code>NOW</code> and <code>NEVER</code> constants can
       be used to request immediate abort or message storage for
       unlimited time.
    */
    long delayBeforeExpiration(ACLMessage msg);

}
