/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.messaging;

//#J2ME_EXCLUDE_FILE

import rca.core.Node;
import rca.core.SliceProxy;
import rca.core.GenericCommand;
import rca.core.AID;
import rca.core.IMTPException;
import rca.core.ServiceException;
import rca.core.NotFoundException;


/**
   The remote proxy for the rca kernel-level service managing a
   persistent message delivery subsystem installed in the platform.
   A persistent message delivery is able to deliver an ACL message
   across an agent lifecycle, as long as the agent ID stays the same.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public class PersistentDeliveryProxy extends SliceProxy implements PersistentDeliverySlice {


    /*public void activateMsgStore(String name) throws IMTPException, NameClashException {
	try {
	    GenericCommand cmd = new GenericCommand(H_ACTIVATEMSGSTORE, NAME, null);
	    cmd.addParam(name);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NameClashException) {
		    throw (NameClashException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void deactivateMsgStore(String name) throws IMTPException, NotFoundException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEACTIVATEMSGSTORE, NAME, null);
	    cmd.addParam(name);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NotFoundException) {
		    throw (NotFoundException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void registerTemplate(String storeName, MessageTemplate mt) throws IMTPException, NotFoundException {
	try {
	    GenericCommand cmd = new GenericCommand(H_REGISTERTEMPLATE, NAME, null);
	    cmd.addParam(storeName);
	    cmd.addParam(mt);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NotFoundException) {
		    throw (NotFoundException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void deregisterTemplate(String storeName, MessageTemplate mt) throws IMTPException, NotFoundException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEREGISTERTEMPLATE, NAME, null);
	    cmd.addParam(storeName);
	    cmd.addParam(mt);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NotFoundException) {
		    throw (NotFoundException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }*/

    public boolean storeMessage(String storeName, GenericMessage msg, AID receiver) throws IMTPException, NotFoundException {
    try {
        GenericCommand cmd = new GenericCommand(H_STOREMESSAGE, NAME, null);
        cmd.addParam(storeName);
        // NOTE that we can't send the GenericMessage directly as a parameter
        // since we would loose the embedded ACLMessage
        cmd.addParam(msg.getACLMessage());
        cmd.addParam(msg.getEnvelope());
        cmd.addParam(msg.getPayload());
        cmd.addParam(receiver);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else if(result instanceof NotFoundException) {
            throw (NotFoundException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }
        return ((Boolean)result).booleanValue();
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    public void flushMessages(AID receiver) throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_FLUSHMESSAGES, NAME, null);
        cmd.addParam(receiver);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }

    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

}
