/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.messaging;

//#J2ME_EXCLUDE_FILE

import rca.core.Service;
import rca.core.AID;
import rca.core.IMTPException;
import rca.core.NotFoundException;



/**
   The horizontal interface for the rca kernel-level service managing
   the message passing subsystem installed in the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public interface PersistentDeliverySlice extends Service.Slice {

    // Constants for the names of the service vertical commands

    /**
       The name of this service.
    */
    static final String NAME = "rca.core.messaging.PersistentDelivery";

    /**
       This command name represents the action of activating a message
       store for undelivered ACL messages, on the specified node.
    */
    //static final String ACTIVATE_MESSAGE_STORE = "Activate-Message-Store";

    /**
       This command name represents the action of deactivating a message
       store for undelivered ACL messages, on the specified node.
    */
    //static final String DEACTIVATE_MESSAGE_STORE = "Deactivate-Message-Store";

    /**
       This command name represents the action of adding a given message
       template to a message store, so that undelivered ACL messages
       addressed to that ID will be retained.
    */
    //static final String REGISTER_MESSAGE_TEMPLATE = "Register-Message-Template";

    /**
       This command name represents the action of removing a given
       message template from a message store. This stops the retention
       of undelivered ACL messages for that agent ID.
    */
    //static final String DEREGISTER_MESSAGE_TEMPLATE = "Deregister-Message-Template";


    // Constants for the names of horizontal commands associated to methods
    //static final String H_ACTIVATEMSGSTORE = "1";
    //static final String H_DEACTIVATEMSGSTORE = "2";
    //static final String H_REGISTERTEMPLATE = "3";
    //static final String H_DEREGISTERTEMPLATE = "4";
    static final String H_STOREMESSAGE = "5";
    static final String H_FLUSHMESSAGES = "6";


    /*void activateMsgStore(String name) throws IMTPException, NameClashException;
    void deactivateMsgStore(String name) throws IMTPException, NotFoundException;

    void registerTemplate(String storeName, MessageTemplate mt) throws IMTPException, NotFoundException, NameClashException;
    void deregisterTemplate(String storeName, MessageTemplate mt) throws IMTPException, NotFoundException;
    */

    boolean storeMessage(String storeName, GenericMessage msg, AID receiver) throws IMTPException, NotFoundException;
    void flushMessages(AID receiver) throws IMTPException;

}
