/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.replication;

import rca.core.Node;
import rca.core.SliceProxy;
import rca.core.GenericCommand;
import rca.core.IMTPException;
import rca.core.ServiceException;


/**
   The remote proxy for the rca kernel-level service distributing the
   <i>Service Manager</i> address list throughout the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public class AddressNotificationProxy extends SliceProxy implements AddressNotificationSlice {

    public void addServiceManagerAddress(String addr) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_ADDSERVICEMANAGERADDRESS, NAME, null);
	    cmd.addParam(addr);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }

	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    /*public void removeServiceManagerAddress(String addr) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_REMOVESERVICEMANAGERADDRESS, NAME, null);
	    cmd.addParam(addr);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }

	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }*/

    public String getServiceManagerAddress() throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_GETSERVICEMANAGERADDRESS, NAME, null);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }

	    return (String)result;

	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

}
