/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.replication;

import rca.core.Service;
import rca.core.IMTPException;


/**
   The horizontal interface for the rca kernel-level service
   distributing the <i>Service Manager</i> address list throughout the
   platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public interface AddressNotificationSlice extends Service.Slice {

    // Constants for the names of the service vertical commands

    /**
       The name of this service.
    */
    static final String NAME = "rca.core.replication.AddressNotification";

    /**
       This command name represents the notification of the addition
       of a new address to the <i>Service Manager</i> of the platform.
    */
    //static final String SM_ADDRESS_ADDED = "SM-Address-Added";

    /**
       This command name represents the notification of the removal of
       an address from the <i>Service Manager</i> of the platform.
    */
    //static final String SM_ADDRESS_REMOVED = "SM-Address-Removed";



    // Constants for the names of horizontal commands associated to methods
    static final String H_ADDSERVICEMANAGERADDRESS = "1";
    //static final String H_REMOVESERVICEMANAGERADDRESS = "2";
    static final String H_GETSERVICEMANAGERADDRESS = "3";

    void addServiceManagerAddress(String addr) throws IMTPException;
    //void removeServiceManagerAddress(String addr) throws IMTPException;
    String getServiceManagerAddress() throws IMTPException;


}
