/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.replication;

//#MIDP_EXCLUDE_FILE

import rca.core.Node;
import rca.core.SliceProxy;
import rca.core.GenericCommand;
import rca.core.AID;
import rca.core.ContainerID;
import rca.core.IMTPException;
import rca.core.ServiceException;
import rca.core.NotFoundException;


/**
   The remote proxy for the rca kernel-level service managing
   the main-container replication subsystem installed in the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public class MainReplicationProxy extends SliceProxy implements MainReplicationSlice {


    public int getLabel() throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_GETLABEL, NAME, null);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }

        return ((Integer)result).intValue();
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    public String getPlatformManagerAddress() throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_GETPLATFORMMANAGERADDRESS, NAME, null);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }

        return (String)result;
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    public void addReplica(String sliceName, String smAddr, int sliceIndex) throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_ADDREPLICA, NAME, null);
        cmd.addParam(sliceName);
        cmd.addParam(smAddr);
        cmd.addParam(new Integer(sliceIndex));

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    public void removeReplica(String smAddr, int sliceIndex) throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_REMOVEREPLICA, NAME, null);
        cmd.addParam(smAddr);
        cmd.addParam(new Integer(sliceIndex));

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    public void fillGADT(AID[] agents, ContainerID[] containers) throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_FILLGADT, NAME, null);
        cmd.addParam(agents);
        cmd.addParam(containers);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    /*public void bornAgent(AID name, ContainerID cid, JADEPrincipal principal String ownership) throws IMTPException, NameClashException, NotFoundException, JADESecurityException {
	try {
	    GenericCommand cmd = new GenericCommand(H_BORNAGENT, NAME, null);
	    cmd.addParam(name);
	    cmd.addParam(cid);
	    cmd.addParam(ownership);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NameClashException) {
		    throw (NameClashException)result;
		}
		else if(result instanceof NotFoundException) {
		    throw (NotFoundException)result;
		}
		else if(result instanceof JADESecurityException) {
		    throw (JADESecurityException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void deadAgent(AID name) throws IMTPException, NotFoundException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEADAGENT, NAME, null);
	    cmd.addParam(name);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NotFoundException) {
		    throw (NotFoundException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }*/

    public void suspendedAgent(AID name) throws IMTPException, NotFoundException {
    try {
        GenericCommand cmd = new GenericCommand(H_SUSPENDEDAGENT, NAME, null);
        cmd.addParam(name);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else if(result instanceof NotFoundException) {
            throw (NotFoundException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    /*
    public void resumedAgent(AID name) throws IMTPException, NotFoundException {
	try {
	    GenericCommand cmd = new GenericCommand(H_RESUMEDAGENT, NAME, null);
	    cmd.addParam(name);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else if(result instanceof NotFoundException) {
		    throw (NotFoundException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void newMTP(MTPDescriptor mtp, ContainerID cid) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_NEWMTP, NAME, null);
	    cmd.addParam(mtp);
	    cmd.addParam(cid);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }

    public void deadMTP(MTPDescriptor mtp, ContainerID cid) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEADMTP, NAME, null);
	    cmd.addParam(mtp);
	    cmd.addParam(cid);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }*/

    public void newTool(AID tool) throws IMTPException {
    try {
        GenericCommand cmd = new GenericCommand(H_NEWTOOL, NAME, null);
        cmd.addParam(tool);

        Node n = getNode();
        Object result = n.accept(cmd);
        if((result != null) && (result instanceof Throwable)) {
        if(result instanceof IMTPException) {
            throw (IMTPException)result;
        }
        else {
            throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
        }
        }
    }
    catch(ServiceException se) {
        throw new IMTPException("Unable to access remote node", se);
    }
    }

    /*
    public void deadTool(AID tool) throws IMTPException {
	try {
	    GenericCommand cmd = new GenericCommand(H_DEADTOOL, NAME, null);
	    cmd.addParam(tool);

	    Node n = getNode();
	    Object result = n.accept(cmd);
	    if((result != null) && (result instanceof Throwable)) {
		if(result instanceof IMTPException) {
		    throw (IMTPException)result;
		}
		else {
		    throw new IMTPException("An undeclared exception was thrown", (Throwable)result);
		}
	    }
	}
	catch(ServiceException se) {
	    throw new IMTPException("Unable to access remote node", se);
	}
    }
    */

}
