/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.core.replication;

//#MIDP_EXCLUDE_FILE

import rca.core.Service;
import rca.core.AID;
import rca.core.ContainerID;
import rca.core.IMTPException;
import rca.core.NotFoundException;


/**
   The horizontal interface for the rca kernel-level service managing
   the main-container replication subsystem installed in the platform.

   Original author Giovanni Rimassa - FRAMeTech s.r.l.
*/
public interface MainReplicationSlice extends Service.Slice {

    // Constants for the names of the service vertical commands

    /**
       The name of this service.
    */
    static final String NAME = "rca.core.replication.MainReplication";


    // Constants for the names of horizontal commands associated to methods
    static final String H_GETLABEL = "1";
    static final String H_GETPLATFORMMANAGERADDRESS = "2";
    static final String H_ADDREPLICA = "3";
    static final String H_REMOVEREPLICA = "4";
    static final String H_FILLGADT = "5";
    static final String H_BORNAGENT = "6";
    static final String H_DEADAGENT = "7";
    static final String H_SUSPENDEDAGENT = "8";
    static final String H_RESUMEDAGENT = "9";
    static final String H_NEWMTP = "10";
    static final String H_DEADMTP = "11";
    static final String H_NEWTOOL = "12";
    static final String H_DEADTOOL = "13";

    // NOTE that some horizontal command do not have a corresponding
    // method in the SliceProxy since they are always sent in 
    // broadcast and therefore the serve() method of the SliceProxy
    // is called directly.
    int getLabel() throws IMTPException;
    String getPlatformManagerAddress() throws IMTPException;
    void addReplica(String sliceName, String smAddr, int sliceIndex) throws IMTPException;
    void removeReplica(String smAddr, int sliceIndex) throws IMTPException;

    void fillGADT(AID[] agents, ContainerID[] containers) throws IMTPException;

    //void bornAgent(AID name, ContainerID cid, Credentials creds) throws IMTPException, NameClashException, NotFoundException, JADESecurityException;
    //void deadAgent(AID name) throws IMTPException, NotFoundException;
    void suspendedAgent(AID name) throws IMTPException, NotFoundException;
    //void resumedAgent(AID name) throws IMTPException, NotFoundException;

    //void newMTP(MTPDescriptor mtp, ContainerID cid) throws IMTPException;
    //void deadMTP(MTPDescriptor mtp, ContainerID cid) throws IMTPException;

    void newTool(AID tool) throws IMTPException;
    //void deadTool(AID tool) throws IMTPException;

}
