/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain;

//#APIDOC_EXCLUDE_FILE
//#MIDP_EXCLUDE_FILE

import rca.domain.FIPAAgentManagement.*;
import rca.domain.FIPAException;
import rca.core.AID;
import rca.gui.GuiEvent;

/**
 * This interface must be implemented by a GUI that wants to interact
 * with the DF agent. Two implementations of this interface have been
 * realized: the class rca.domain.df (used by the DF agent itself) and
 * the class rca.applet.DFAppletCommunicator (used by the DFApplet).
 * Original author Fabio Bellifemine - CSELT - 25/8/1999
 * @version $Date: 2003-11-18 17:26:01 +0100 (Tue, 18 Nov 2003) $ $Revision: 4564 $
 */

public interface DFGUIAdapter {

    // GUI event types
  public static final int EXIT = 0;
  public static final int CLOSEGUI = 1;

  //DFGUI event Type
  public static final int REGISTER = 1001;
  public static final int DEREGISTER = 1002;
  public static final int MODIFY = 1003;
  public static final int SEARCH = 1004;
  public static final int FEDERATE = 1005;

  //only used by applet
  public static final int REFRESHAPPLET = 1006;


  /**
  * This method notifies an event to the df.
  */
  void postGuiEvent(GuiEvent ev);

  /**
   * This method returns the agent description of an agent registered with the DF given the agent name
   */
  DFAgentDescription getDFAgentDsc(AID name) throws FIPAException;


  /**
  * This method returns the description of this df.
  */
  DFAgentDescription getDescriptionOfThisDF();

  /**
  * This method returns the description used by the df to federate with 
  * a given parent DF.
  */
  DFAgentDescription getDescriptionOfThisDF(AID parent);
}
