/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.DFGUIManagement;

import rca.domain.FIPAAgentManagement.*;

import rca.content.schema.*;
import rca.content.onto.*;

/**
   
   Original author Elisabetta Cortese - TiLab S.p.A.
   @version $Date: 2003-08-26 11:15:34 +0200 (Tue, 26 Aug 2003) $ $Revision: 4243 $
*/

/**
   This class represents the ontology
   <code>DFApplet-management</code>, containing all rca extensions
   related to applet management. There is only a single instance of
   this class.
   <p>
   The package contains one class for each Frame in the ontology.
   <p>
 
*/


public class DFAppletOntology extends Ontology implements DFAppletVocabulary {

  private static Ontology theInstance = new DFAppletOntology();
  /**
     This method grants access to the unique instance of the
     ontology.
     @return An <code>Ontology</code> object, containing the concepts
     of the ontology.
  */
  public static Ontology getInstance() {
    return theInstance;
  }

  private DFAppletOntology() {
	  super(NAME, FIPAManagementOntology.getInstance(), new ReflectiveIntrospector());

    try {
	  	add(new AgentActionSchema(GETDESCRIPTION), GetDescription.class);
	  	add(new AgentActionSchema(FEDERATE), Federate.class);
	  	add(new AgentActionSchema(REGISTERWITH), RegisterWith.class);
	  	add(new AgentActionSchema(DEREGISTERFROM), DeregisterFrom.class);
	  	add(new AgentActionSchema(MODIFYON), ModifyOn.class);
	  	add(new AgentActionSchema(SEARCHON), SearchOn.class);
	  	add(new AgentActionSchema(GETPARENTS), GetParents.class);
	  	add(new AgentActionSchema(GETDESCRIPTIONUSED), GetDescriptionUsed.class);

	  	AgentActionSchema as = (AgentActionSchema)getSchema(FEDERATE);
	  	as.add(FEDERATE_DF, (TermSchema) getSchema(BasicOntology.AID));
	  	as.add(FEDERATE_DESCRIPTION, (TermSchema)getSchema(DFAGENTDESCRIPTION), ObjectSchema.OPTIONAL);
	  	
	  	as = (AgentActionSchema)getSchema(REGISTERWITH);
	  	as.add(REGISTERWITH_DF, (TermSchema) getSchema(BasicOntology.AID));
	  	as.add(REGISTERWITH_DESCRIPTION, (TermSchema)getSchema(DFAGENTDESCRIPTION));
	  
	  	as = (AgentActionSchema)getSchema(DEREGISTERFROM);
	  	as.add(DEREGISTERFROM_DF, (TermSchema) getSchema(BasicOntology.AID));
	  	as.add(DEREGISTERFROM_DESCRIPTION, (TermSchema) getSchema(DFAGENTDESCRIPTION));
	  
	  	as = (AgentActionSchema)getSchema(MODIFYON);
	  	as.add(MODIFYON_DF, (TermSchema) getSchema(BasicOntology.AID));
	  	as.add(MODIFYON_DESCRIPTION, (TermSchema) getSchema(DFAGENTDESCRIPTION));
	  
	  	as = (AgentActionSchema)getSchema(SEARCHON);
	  	as.add(SEARCHON_DF, (TermSchema) getSchema(BasicOntology.AID));
	  	as.add(SEARCHON_DESCRIPTION, (TermSchema)getSchema(DFAGENTDESCRIPTION));
	  	as.add(SEARCHON_CONSTRAINTS, (TermSchema)getSchema(SEARCHCONSTRAINTS));

	  	as = (AgentActionSchema)getSchema(GETDESCRIPTIONUSED);
	  	as.add(GETDESCRIPTIONUSED_PARENTDF, (TermSchema) getSchema(BasicOntology.AID));
		}
    catch(OntologyException oe) {
      	oe.printStackTrace();
    }
  
  }

}
