/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.domain;

//#J2ME_EXCLUDE_FILE
//#APIDOC_EXCLUDE_FILE

import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

/**
 * <code>DFHSQLKB</code> realizes a knowledge base used by the DF which stores its content
 * in an HSQL database, running in the same VM. The SQL commands are optimized for the HSQL database.
 * 
 * Original author Roland Mungenast - Profactor
 */
public class DFHSQLKB extends DFDBKB {

  private static String db_driver = "org.hsqldb.jdbcDriver";
  private static String db_url    = "jdbc:hsqldb:file:dfdb";
  private static String db_user   = "sa";
  private static String db_passwd = "";
  
  protected final static String CACHE_SCALE      = "8"; 
  protected final static String CACHE_SIZE_SCALE = "8";
  protected final static String GC_INTERVAL      = "10000";
  
  
  /**
   * Constructor
   * @param maxResultLimit rca internal limit for the number of maximum search results.
   * @param cleanTables specifies whether the KB should delete all existing tables for the DF at startup
   * @throws SQLException if a database access error occurs
   */
  public DFHSQLKB(int maxResultLimit, boolean cleanTables) throws SQLException {
    super(maxResultLimit, db_driver, db_url, db_user, db_passwd, cleanTables);
  }
  
 
  protected void setDBConnection(String drv, String url, String user, String passwd) throws SQLException {
    Properties props = new Properties();
    props.put("user", user);
    props.put("passwd", passwd);
    props.put("hsqldb.cache_scale", CACHE_SCALE);
    props.put("hsqldb.cache_size_scale", CACHE_SIZE_SCALE);
    props.put("hsqldb.gc_interval", GC_INTERVAL);
    
    conn = DriverManager.getConnection(url, props);
  }
 
  protected String getLongVarCharType() {
    return "LONGVARCHAR";
  }
  
  protected void createTable(Statement stmt, String name, String[] entries) throws SQLException {
    String sql = "CREATE CACHED TABLE " + name + " (";
    for (int i = 0; i < entries.length; i++) {
      sql += entries[i];
      if (i < entries.length - 1)
        sql += ", ";
      else
        sql += ")";
    }
    stmt.executeUpdate(sql);
    conn.commit();
  }
}