/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain;

//#MIDP_EXCLUDE_FILE

import rca.content.Concept;
import rca.content.onto.basic.Action;
import rca.content.onto.basic.Done;
import rca.lang.acl.ACLMessage;
import rca.lang.acl.MessageTemplate;
import rca.domain.JADEAgentManagement.*;
import rca.domain.FIPAAgentManagement.UnsupportedFunction;
import rca.security.JADESecurityException;

/**
   This behaviour serves the actions of the rca management ontology
   supported by the DF.
   It extends RequestManagementBehaviour and implements performAction() to 
   i) call the method of the DF corresponding to the requested 
   action and ii) prepare the result notification.
   Original author Tiziana Trucco - TILAB
   Original author Fabio Bellifemine - TILAB
   Original author Giovanni Caire - TILAB
 */
class DFJadeAgentManagementBehaviour extends RequestManagementBehaviour {

	private df theDF;
	
  protected DFJadeAgentManagementBehaviour(df a, MessageTemplate mt){
		super(a,mt);
		theDF = a;
  }

  /**
     Call the proper method of the DF and prepare the notification 
     message
   */
  protected ACLMessage performAction(Action slAction, ACLMessage request) throws JADESecurityException, FIPAException {
  	Concept action = slAction.getAction();
  	
  	// SHOW_GUI
  	if (action instanceof ShowGui) {
  		theDF.showGuiAction((ShowGui) action, request.getSender());
  	}
  	else {
  		throw new UnsupportedFunction();
  	}
  	
  	// Prepare the notification
  	ACLMessage notification = request.createReply();
  	notification.setPerformative(ACLMessage.INFORM);
  	Done d = new Done(slAction);
  	try {
	  	theDF.getContentManager().fillContent(notification, d);
  	}
  	catch (Exception e) {
  		// Should never happen
  		e.printStackTrace();
  	} 
  	return notification;
  }
}
