/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.domain;

//#J2ME_EXCLUDE_FILE
//#APIDOC_EXCLUDE_FILE

import java.sql.SQLException;

import rca.domain.KBManagement.DBKB;
import rca.domain.KBManagement.MemKB;

/**
 * The <code>DFKBFactory</code> class creates 
 * knowledge base objects used by the DF. 
 * 
 * <p>
 * To run rca with your own knowledge base implementation a new
 * sub class of <code>DFKBFactory</code>, overriding the appropriate method(s), 
 * has to be implemented and specified by the command line parameter 
 * <code>-jade_domain_df_kb-factory</code>.
 * </p>
 * Original author Roland Mungenast - Profactor
 * @since rca 3.3
 */
public class DFKBFactory {
  
  /**
   * Returns the memory based knowledge base which will be used by the DF
   * @param maxResultLimit internal limit for the maximum number of search results
   */
  protected MemKB getDFMemKB(int maxResultLimit) {
    return new DFMemKB(maxResultLimit);
  }
  
  /**
   * Returns the database based knowledge base which will be used by the DF
   * @param maxResultLimit rca internal limit for the maximum number of search results
   * @param driver database driver
   * @param url database url
   * @param user user for the database access
   * @param passwd password for the database access
   * @param cleanTables specifies whether the KB should delete all existing tables for the DF at startup
   * @throws SQLException if the database cannot be initialized
   */
  protected DBKB getDFDBKB(int maxResultLimit, String driver, String url, String user, String passwd, boolean cleanTables) throws SQLException {
    if (url == null)
      return new DFHSQLKB(maxResultLimit, cleanTables);
    else
      return new DFDBKB(maxResultLimit, driver, url, user, passwd, cleanTables);
  }
}


