/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.FIPAAgentManagement;

import rca.core.AID;
import rca.content.Concept;

/**
* This class implements the concept of the fipa-agent-management ontology
* representing the description of an Agent in the AMS catalogue.
* @see rca.domain.FIPAAgentManagement.FIPAManagementOntology
* @see rca.domain.AMSService
* Original author Fabio Bellifemine - CSELT S.p.A.
* @version $Date: 2005-09-27 17:55:12 +0200 (Tue, 27 Sep 2005) $ $Revision: 5790 $
*/
public class AMSAgentDescription implements Concept {

    /**
       String constant for the <code>initiated</code> agent life-cycle
       state.
    */
    public static final String INITIATED = "initiated";

    /**
       String constant for the <code>active</code> agent life-cycle
       state.
    */
    public static final String ACTIVE = "active";

    /**
       String constant for the <code>suspended</code> agent life-cycle
       state.
    */
    public static final String SUSPENDED = "suspended";

    /**
       String constant for the <code>waiting</code> agent life-cycle
       state.
    */
    public static final String WAITING = "waiting";

    /**
       String constant for the <code>transit</code> agent life-cycle
       state.
    */
    public static final String TRANSIT = "transit";
    
    /**
    String constant for the <code>latent</code> agent life-cycle
    state. rca specific state indicating an agent waiting to be restored after a
    crash of the main container
    */
    public static final String LATENT = "latent";

    private AID name;
    private String ownership;
    private String state;

    /**
       Default constructor.
    */
    public AMSAgentDescription() {
    }

    /**
       Set the <code>name</code> slot of this object.
       @param n The agent identifier for the name.
    */
    public void setName(AID n){
	name = n;
    }

    /**
       Set the <code>ownership</code> slot of this object.
       @param n The string for the ownership.
    */
    public void setOwnership(String n) {
	ownership = n;
    }

    /**
       Set the <code>state</code> slot of this object.
       @param n The string for the state.
    */
    public void setState(String n) {
	state = n;
    }

    /**
       Retrieve the <code>name</code> slot of this object.
       @return The value of the <code>name</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getName(){
	return name;
    }

   /**
       Retrieve the <code>ownership</code> slot of this object.
       @return The value of the <code>ownership</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getOwnership(){
	return ownership;
    }

   /**
       Retrieve the <code>state</code> slot of this object.
       @return The value of the <code>state</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getState(){
	return state;
    }

}
