/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.FIPAAgentManagement;

import rca.content.Concept;
import rca.util.leap.List;
import rca.util.leap.Iterator;
import rca.util.leap.ArrayList;

/**
   This class implements the concept of the fipa-agent-management ontology
   representing the description of an agent platform as it can be retrieved
   from the AMS.
   @see rca.domain.FIPAAgentManagement.FIPAManagementOntology
   Original author Giovanni Rimassa - Universita' di Parma
   Original author Alessandro Chiarotto - TILAB
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class APDescription implements Concept {

    private String name;
    private List services = new ArrayList(1); 


    /**
       Default constructor.
    */
    public APDescription() {
    }

    /**
       Set the <code>name</code> slot of this object.
       @param n The string for the platform name.
    */
    public void setName(String n) {
	name = n;
    }

    /**
       Retrieve the <code>name</code> slot of this object.
       @return The value of the <code>name</code> slot of this
       platform description, or <code>null</code> if no value was set.
    */
    public String getName() {
	return name;
    }

    /**
       Add a service to the <code>ap-services</code> slot collection
       of this object.
       @param a The platform service to add to the collection.
    */  
    public void addAPServices(APService a) {
	services.add(a);
    }

    /**
       Remove a service from the <code>ap-services</code> slot
       collection of this object.
       @param a The platform service to remove from the collection.
       @return A boolean, telling whether the element was present in
       the collection or not.
    */
    public boolean removeAPServices(APService a) {
      return services.remove(a);
    }

    /**
       Remove all services from the <code>ap-services</code> slot
       collection of this object.
    */
    public void clearAllAPServices(){
	services.clear();
    }

    /**
       Access all services from the <code>ap-services</code> slot
       collection of this object.
       @return An iterator over the services collection.
    */
    public Iterator getAllAPServices(){
	return services.iterator();
    }

    /**
     * Retrieve a string representation for this platform description.
     * @return an SL0-like String representation of this object 
     **/
    public String toString() {
	StringBuffer str = new StringBuffer("( ap-description ");
	if ((name!=null)&&(name.length()>0))
	    str.append(" :name " + name);
        APService s=null;
        str.append(" :ap-services (set");
        for (Iterator i=services.iterator(); i.hasNext(); ) {
            s=(APService)(i.next());
            str.append(" "+s.toString());
        }
	str.append("))");
    	return str.toString();
    }

    
}
