/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.FIPAAgentManagement;

import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;

import rca.content.Concept;

/**
   This class implements the concept of the fipa-agent-management ontology
   representing the description of a platform service. 
   Original author Fabio Bellifemine - CSELT
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $

 */
public class APService implements Concept {

  private List addresses = new ArrayList(); 
  private String name;
  private String type;

    /**
       Default constructor. Necessary for ontological classes.
    */
    public APService() {
    }
  
    /**
     * Constructor. Create a new APService where name and type get the same value (i.e.
     * the passed type parameter).
     **/
    public APService(String type, String[] addresses) {
	name=type;
	this.type=type;
	for (int i=0; i<addresses.length; i++)
	    this.addresses.add(addresses[i]);
    }

    /**
       Set the <code>name</code> slot of this object.
       @param n The string for the platform service name.
    */
    public void setName(String n) {
	name = n;
    }

    /**
       Retrieve the <code>name</code> slot of this object.
       @return The value of the <code>name</code> slot of this
       platform service description, or <code>null</code> if no value
       was set.
    */
    public String getName() {
	return name;
    }

    /**
       Set the <code>type</code> slot of this object.
       @param t The string for the platform service type.
    */
    public void setType(String t) {
	type = t;
    }

    /**
       Retrieve the <code>type</code> slot of this object.
       @return The value of the <code>type</code> slot of this
       platform service description, or <code>null</code> if no value
       was set.
    */
    public String getType() {
	return type;
    }

    /**
       Add a service to the <code>addresses</code> slot collection
       of this object.
       @param a The address to add to the collection.
    */
    public void addAddresses(String address) {
	addresses.add(address);
    }

    /**
       Remove a service from the <code>addresses</code> slot
       collection of this object.
       @param a The address to remove from the collection.
       @return A boolean, telling whether the element was present in
       the collection or not.
    */
    public boolean removeAddresses(String address) {
	return addresses.remove(address);
    }

    /**
       Remove all addresses from the <code>addresses</code> slot
       collection of this object.
    */
    public void clearAllAddresses() {
	addresses.clear();
    }

    /**
       Access all addresses from the <code>addresses</code> slot
       collection of this object.
       @return An iterator over the addresses collection.
    */
    public Iterator getAllAddresses() {
	return addresses.iterator();
    }

    /**
     * Retrieve a string representation for this platform service
     * description.
     * @return an SL0-like String representation of this object 
     **/
    public String toString() {
	StringBuffer str = new StringBuffer("( ap-service ");
        if ((name!=null)&&(name.length()>0))
	    str.append(" :name " + name);
        if ((type!=null)&&(type.length()>0))
	    str.append(" :type " + type);
        String s;
        str.append(" :addresses (sequence");
        for (Iterator i=addresses.iterator(); i.hasNext(); ) {
            s=(String)(i.next());
            str.append(" "+s.toString());
        }
	str.append("))");
	return str.toString();
    }


}
