/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/



package rca.domain.FIPAAgentManagement;

import rca.content.Concept;


/** 
 This class implements the <code>property</code> type, a pair of a
 name and value.
 @see rca.domain.FIPAAgentManagement.FIPAManagementOntology
 Original author Fabio Bellifemine - CSELT S.p.A.
 @version $Date: 2004-07-09 13:04:29 +0200 (Fri, 09 Jul 2004) $ $Revision: 5200 $
*/
public class Property implements Concept {

    private String name;
    private Object value;

    /**
       Default constructor. A default constructor is needed for rca
       ontological classes.
    */
    public Property() {
    }

    /**
       Create a property object, with the given name and value pair.
       @param name The name of the property.
       @param value The Java object associated with the given name.
    */
    public Property(String name, Object value) {
	this.name = name;
	this.value = value;
    }

    /**
       Set the name of the property object.
       @param n The new name for this property.
    */
    public void setName(String n) {
	name = n;
    }

    /**
       Retrieve the name of this property object.
       @return The string that is the name of this property, or
       <code>null</code> if no name was set.
    */
    public String getName() {
	return name;
    }

    /**
       Set the value for this property object, attached to the
       property name.
       @param o The new Java object to attach to the property name.
    */
    public void setValue(Object o) {
	value = o;
    }

    /**
       Retrieve the value of this property object, associated with the
       property name.
       @return The value of this property, or <code>null</code> if no
       value was set.
    */
    public Object getValue() {
	return value;
    }

    //#J2ME_EXCLUDE_BEGIN
    // For persistence only
    private void setsvalue(java.io.Serializable s) {
    	value = s;
    }
    
    private java.io.Serializable getsvalue() {
    	return (java.io.Serializable) value;
    }
    //#J2ME_EXCLUDE_END
}
