/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.domain.FIPAAgentManagement;

import rca.util.leap.Serializable;
import java.util.Date;

import rca.content.*;

/** 
* This class implements the <code>received-object</code> object from
* the FIPA Agent Management ontology.
* @see rca.domain.FIPAAgentManagement.FIPAManagementOntology
* Original author Fabio Bellifemine - CSELT S.p.A.
* @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class ReceivedObject implements Concept, Serializable {

    private String by;
    private String from;
    private Date date;
    private String id;
    private String via;


    /**
     * The constructor initializes the date to current time and all
     * the Strings to empty strings.
     **/
    public ReceivedObject() {
	date = new Date();
	by = new String();
	from = new String();
	id = new String();
	via = new String();
    }


    /**
       Set the <code>by</code> slot of this object.
       @param b The identifier for the ACC that received the envelope
       containing this object.
    */
    public void setBy(String b) {
    	by = b;
    }

    /**
       Retrieve the <code>by</code> slot of this object. This slot
       identifies the ACC that received the envelope containing this
       object.
       @return The value of the <code>by</code> slot of this object,
       or <code>null</code> if no value was set.
    */
    public String getBy() {
	return by;
    }

    /**
       Set the <code>from</code> slot of this object.
       @param f The identifier for the ACC that sent the envelope
       containing this object.
    */
    public void setFrom(String f) {
	from = f;
    }

    /**
       Retrieve the <code>from</code> slot of this object. This slot
       identifies the ACC that sent the envelope containing this
       object.
       @return The value of the <code>from</code> slot of this object,
       or <code>null</code> if no value was set.
    */
    public String getFrom() {
	return from;
    }

    /**
       Set the <code>date</code> slot of this object.
       @param d The date when the envelope containing this object was
       sent.
    */
    public void setDate(Date d) {
	date = d;
    }

    /**
       Retrieve the <code>date</code> slot of this object. This slot
       identifies the date when the envelope containing this object
       was sent.
       @return The value of the <code>date</code> slot of this object,
       or <code>null</code> if no value was set.
    */
    public Date getDate() {
	return date;
    }

    /**
       Set the <code>id</code> slot of this object.
       @param i A unique id for the envelope containing this object.
    */
    public void setId(String i) {
	id = i;
    }

    /**
       Retrieve the <code>id</code> slot of this object. This slot
       uniquely identifies the envelope containing this object.
       @return The value of the <code>id</code> slot of this object,
       or <code>null</code> if no value was set.
    */
    public String getId() {
	return id;
    }

    /**
       Set the <code>via</code> slot of this object.
       @param v The name of the MTP over which the envelope containing
       this object was sent.
    */
    public void setVia(String v) {
	via = v;
    }

    /**
       Retrieve the <code>via</code> slot of this object. This slot
       describes the MTP over which the envelope containing this
       object was sent.
       @return The value of the <code>via</code> slot of this
       envelope, or <code>null</code> if no value was set.
    */
    public String getVia() {
	return via;
    }

    /**
     * Retrieve a string representation for this received object.
     * @return an SL0-like String representation of this object 
     **/
    public String toString() {
	String s = new String("(ReceivedObject ");
	if (date != null)
	    s = s + " :date "+date.toString();
	if ((by != null) && (by.trim().length()>0))
	    s = s + " :by "+by;
	if ((from != null) && (from.trim().length()>0))
	    s = s + " :from "+from;
	if ((id != null) && (id.trim().length()>0))
	    s = s + " :id "+id;
	if ((via != null) && (via.trim().length()>0))
	    s = s + " :via "+via;
	return s;
    }


}
