/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.JADEAgentManagement;

import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;

import rca.core.ContainerID;
import rca.content.AgentAction;

import rca.security.JADEPrincipal;
import rca.security.Credentials;


/**
  This class represents the <code>create-agent</code> action of 
  the <code>JADE-agent-management ontology</code>.
  This action can be requested to the rca AMS to create a new agent
  on a given container.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2004-04-28 11:04:38 +0200 (Wed, 28 Apr 2004) $ $Revision: 4999 $
*/
public class CreateAgent implements AgentAction {

    private String agentName;
    private String className;
    private List arguments = new ArrayList();
    private ContainerID container;
    //#MIDP_EXCLUDE_BEGIN
    private JADEPrincipal owner;
    private Credentials initialCredentials;
    //#MIDP_EXCLUDE_END

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public CreateAgent() {
    }

    /**
       Set the <code>agent-name</code> slot of this action.
       @param an The local name (i.e. without the platform ID) of the
       agent to create.
    */
    public void setAgentName(String an) {
	agentName = an;
    }

    /**
       Retrieve the value of the <code>agent-name</code> slot of this
       event, containing the local name (i.e. without the platform ID)
       of the agent to create.
       @return The value of the <code>agent-name</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getAgentName() {
	return agentName;
    }

    /**
       Set the <code>class-name</code> slot of this action.
       @param cn The name of the Java class implementing the agent to
       create.
    */
    public void setClassName(String cn) {
	className = cn;
    }

    /**
       Retrieve the value of the <code>class-name</code> slot of this
       event, containing the name of the Java class implementing the
       agentto create.
       @return The value of the <code>class-name</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getClassName() {
	return className;
    }

    /**
       Set the <code>container</code> slot of this action.
       @param cid The container identifier of the container where the
       agent is to be created.
    */
    public void setContainer(ContainerID cid) {
	container = cid;
    }

    /**
       Retrieve the value of the <code>container</code> slot of this
       event, containing the container identifier of the container
       where the agent is to be created.
       @return The value of the <code>container</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getContainer() {
	return container;
    }

    //#MIDP_EXCLUDE_BEGIN
    /**
       Set the principal of the owner of the agent to be created.
       @param p The principal of the owner of the agent to be created.
    */
    public void setOwner(JADEPrincipal p) {
	owner = p;
    }

    /**
       @return The principal of the owner of the agent to be created.
    */
    public JADEPrincipal getOwner() {
	return owner;
    }

    /**
       Set the initial credentials to be granted to the agent to be created.
       @param c The initial credentials to be granted to the agent to be created.
    */
    public void setInitialCredentials(Credentials c) {
	initialCredentials = c;
    }

    /**
       @return The initial credentials to be granted to the agent to be created.
    */
    public Credentials getInitialCredentials() {
	return initialCredentials;
    }
    //#MIDP_EXCLUDE_END

    /**
       Add an object to the <code>arguments</code> slot collection of
       this object.
       @param a The object to add to the collection.
    */
    public void addArguments(Object a) {
	arguments.add(a);
    }
  
    /**
       Remove all objects from the <code>arguments</code> slot
       collection of this object.
    */
    public Iterator getAllArguments() {
  	return arguments.iterator();
    }

}
