/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.JADEAgentManagement;

//#APIDOC_EXCLUDE_FILE

import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;

import rca.core.AID;
import rca.content.AgentAction;

/**

  This class represents the <code>debug-on</code> action, requesting
  a debugger to start observing a set of agents in the platform.

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class DebugOn implements AgentAction {

    private AID debugger;
    private List debuggedAgents = new ArrayList();
    private String password;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public DebugOn() {
    }

    /**
       Set the <code>debugger</code> slot of this action.
       @param id The agent identifier of the debugger agent.
    */
    public void setDebugger(AID id) {
	debugger = id;
    }

    /**
       Retrieve the value of the <code>debugger</code> slot of this
       action, containing the agent identifier of the debugger agent.
       @return The value of the <code>debugger</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getDebugger() {
	return debugger;
    }

    /**
       Remove all agent identifiers from the
       <code>debugged-agents</code> slot collection of this object.
    */
    public void clearAllDebuggedAgents() {
	debuggedAgents.clear();
    }

    /**
       Add an agent identifier to the <code>debugged-agents</code>
       slot collection of this object.
       @param id The agent identifier to add to the collection.
    */
    public void addDebuggedAgents(AID id) {
	debuggedAgents.add(id);
    }

    /**
       Remove an agent identifier from the
       <code>debugged-agents</code> slot collection of this object.
       @param id The agent identifier to remove from the collection.
       @return A boolean, telling whether the element was present in
       the collection or not.
    */
    public boolean removeDebuggedAgents(AID id) {
	return debuggedAgents.remove(id);
    }

    /**
       Access all agent identifiers from the
       <code>debugged-agents</code> slot collection of this object.
       @return An iterator over the properties collection.
    */
    public Iterator getAllDebuggedAgents() {
	return debuggedAgents.iterator();
    }

    //#APIDOC_EXCLUDE_BEGIN

    /**
       This method is called by the AMS in order to prepare an RMI call.
       The <code>getAllDebuggedAgents()</code> cannot be used as it returns
       an <code>Iterator</code> that is not serializable.
    */
    public ArrayList getCloneOfDebuggedAgents() {
	return (ArrayList) ((ArrayList) debuggedAgents).clone();
    }

    //#APIDOC_EXCLUDE_END

    /**
       Set the <code>password</code> slot of this action.
       @param p The password used to authenticate the principal
       requesting this action.
    */
    public void setPassword(String p) {
	password = p;
    }

    /**
       Retrieve the value of the <code>password</code> slot of this
       action, containing the password used to authenticate the
       principal requesting this action.
       @return The value of the <code>password</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getPassword() {
	return password;
    }

}
