/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.JADEAgentManagement;

import rca.core.ContainerID;
import rca.content.AgentAction;

/**
  This class represents the <code>install-MTP</code> action of 
  the <code>JADE-agent-management ontology</code>.
  This action can be requested to the rca AMS to install a new MTP for
  inter-platform communication on a given container.


   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class InstallMTP implements AgentAction {

    private String address;
    private ContainerID container;
    private String className;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public InstallMTP() {
    }

    /**
       Set the <code>address</code> slot of this action.
       @param a The address URL of the MTP endpoint to install.
    */
    public void setAddress(String a) {
	address = a;
    }

    /**
       Retrieve the value of the <code>address</code> slot of this
       action, containing the address URL of the MTP to install.
       @return The value of the <code>address</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getAddress() {
	return address;
    }

    /**
       Set the <code>container</code> slot of this action.
       @param cid The container identifier of the container where the
       new MTP is to be deployed.
    */
    public void setContainer(ContainerID cid) {
	container = cid;
    }

    /**
       Retrieve the value of the <code>container</code> slot of this
       action, containing the container identifier of the container
       where the new MTP is to be deployed.
       @return The value of the <code>container</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getContainer() {
	return container;
    }

    /**
       Set the <code>class-name</code> slot of this action.
       @param a The name of the Java class implementing the MTP
       endpoint to install.
    */
    public void setClassName(String a) {
	className = a;
    }

    /**
       Retrieve the value of the <code>class-name</code> slot of this
       action, containing the name of the Java class implementing the
       MTP endpoint to install.
       @return The value of the <code>class-name</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getClassName() {
	return className;
    }

}
