/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.JADEAgentManagement;

import rca.core.AID;
import rca.content.AgentAction;

/**
  This class represents the <code>kill-agent</code> action of 
  the <code>JADE-agent-management ontology</code>.
  This action can be requested to the rca AMS to kill an active agent.

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/

public class KillAgent implements AgentAction {

    private AID agent;
    private String password;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public KillAgent() {
    }

    /**
       Set the <code>agent</code> slot of this action.
       @param id The agent identifier of the agent to terminate.
    */
    public void setAgent(AID id) {
	agent = id;
    }

    /**
       Retrieve the value of the <code>agent</code> slot of this
       event, containing the agent identifier of the agent to
       terminate.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgent() {
	return agent;
    }

    /**
       Set the <code>password</code> slot of this action.
       @param p The password to authenticate the principal requesting
       the agent termination.
    */
    public void setPassword(String p) {
	password = p;
    }

    /**
       Retrieve the value of the <code>password</code> slot of this
       event, containing the password to authenticate the principal
       requesting the agent termination.
       @return The value of the <code>password</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getPassword() {
	return password;
    }

}
