/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.JADEAgentManagement;

import rca.core.ContainerID;
import rca.content.AgentAction;

/**
  This class represents the <code>kill-container</code> action of 
  the <code>JADE-agent-management ontology</code>.
  This action can be requested to the rca AMS to kill an active container.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class KillContainer implements AgentAction {


    private ContainerID container;
    private String password;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public KillContainer() {
    }

    /**
       Set the <code>container</code> slot of this action.
       @param id The container identifier of the container to
       terminate.
    */
    public void setContainer(ContainerID cid) {
	container = cid;
    }

    /**
       Retrieve the value of the <code>container</code> slot of this
       event, containing the container identifier of the container to
       terminate.
       @return The value of the <code>container</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getContainer() {
	return container;
    }

    /**
       Set the <code>password</code> slot of this action.
       @param p The password to authenticate the principal requesting
       the container termination.
    */
    public void setPassword(String p) {
	password = p;
    }

    /**
       Retrieve the value of the <code>password</code> slot of this
       event, containing the password to authenticate the principal
       requesting the container termination.
       @return The value of the <code>password</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getPassword() {
	return password;
    }

}
