/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.JADEAgentManagement;

import rca.core.*;
import rca.content.*;

/**
  This class represents the <code>query-agents-on-location</code> action of 
  the <code>JADE-agent-management ontology</code>.
  This action can be requested to the rca AMS to retrieve the list
  of agents running on a given container. 

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class QueryAgentsOnLocation implements AgentAction {

    private Location location;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public QueryAgentsOnLocation() {
    }

    /**
       Set the <code>location</code> slot of this action.
       @param loc The location for which the agent list is requested.
    */  
    public void setLocation(Location loc) {
	location = loc;
    }

    /**
       Retrieve the value of the <code>location</code> slot of this
       event, containing the location for which the agent list is
       requested.
       @return The value of the <code>location</code> slot, or
       <code>null</code> if no value was set.
    */
    public Location getLocation() {
	return location;
    }

}
