/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.JADEAgentManagement;

//#APIDOC_EXCLUDE_FILE

import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;

import rca.core.AID;
import rca.content.AgentAction;

/**

   This class represents the <code>sniff-off</code> action, requesting
   a sniffer to stop observing a set of agents in the platform.

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class SniffOff implements AgentAction {

    private AID sniffer;
    private List sniffedAgents = new ArrayList();
    private String password;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public SniffOff() {
    }

    /**
       Set the <code>sniffer</code> slot of this action.
       @param id The agent identifier of the sniffer agent.
    */
    public void setSniffer(AID id) {
	sniffer = id;
    }

    /**
       Retrieve the value of the <code>sniffer</code> slot of this
       action, containing the agent identifier of the sniffer agent.
       @return The value of the <code>sniffer</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getSniffer() {
	return sniffer;
    }

    /**
       Remove all agent identifiers from the
       <code>sniffed-agents</code> slot collection of this object.
    */
    public void clearAllSniffedAgents() {
	sniffedAgents.clear();
    }

    /**
       Add an agent identifier to the <code>sniffed-agents</code> slot
       collection of this object.
       @param id The agent identifier to add to the collection.
    */
    public void addSniffedAgents(AID id) {
	sniffedAgents.add(id);
    }

    /**
       Remove an agent identifier from the <code>sniffed-agents</code>
       slot collection of this object.
       @param id The agent identifier to remove from the collection.
       @return A boolean, telling whether the element was present in
       the collection or not.
    */
    public boolean removeSniffedAgents(AID id) {
	return sniffedAgents.remove(id);
    }

    /**
       Access all agent identifiers from the
       <code>sniffed-agents</code> slot collection of this object.
       @return An iterator over the properties collection.
    */
    public Iterator getAllSniffedAgents() {
	return sniffedAgents.iterator();
    }

    //#APIDOC_EXCLUDE_BEGIN
    /**
       This method is called by the AMS in order to prepare an RMI call.
       The <code>getAllSniffedAgents()</code> cannot be used as it returns
       an <code>Iterator</code> that is not serializable.
    */
    public ArrayList getCloneOfSniffedAgents(){
  	return (ArrayList) ((ArrayList) sniffedAgents).clone();
    }
    //#APIDOC_EXCLUDE_END

    /**
       Set the <code>password</code> slot of this action.
       @param p The password used to authenticate the principal
       requesting this action.
    */
    public void setPassword(String p) {
	password = p;
    }

    /**
       Retrieve the value of the <code>password</code> slot of this
       action, containing the password used to authenticate the
       principal requesting this action.
       @return The value of the <code>password</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getPassword() {
	return password;
    }

}
