/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.JADEAgentManagement;

import rca.core.*;
import rca.content.*;


/**
  This class represents the <code>where-is-agent</code> action of 
  the <code>JADE-agent-management ontology</code>.
  This action can be requested to the rca AMS to know the container
  a given agent is currently running on. 

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-24 14:47:00 +0100 (Mon, 24 Nov 2003) $ $Revision: 4597 $
*/
public class WhereIsAgentAction implements AgentAction {


    private AID agentName;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public WhereIsAgentAction() {
    }

    /**
       Set the <code>agent-identifier</code> slot of this action.
       @param id The agent identifier for the agent whose location is
       requested.
    */
    public void setAgentIdentifier(AID id) {
	agentName = id;
    }

    /**
       Retrieve the value of the <code>agent-identifier</code> slot of
       this action, containing the agent identifier for the agent
       whose location is requested.
       @return The value of the <code>agent-identifier</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgentIdentifier() {
	return agentName;
    }

}
