/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.KBManagement;

//#MIDP_EXCLUDE_FILE

import rca.domain.FIPAAgentManagement.NotUnderstoodException;
import rca.proto.SubscriptionResponder;
import rca.util.leap.List;
import rca.util.Logger;

import java.util.Enumeration;


/**
 * Original author Elisabetta Cortese - TILab
 *
 */


/** Base class for AMS and DF Knowledge Base*/
public abstract class KB {
	protected LeaseManager lm;
	protected SubscriptionResponder sr;
	private Logger logger = Logger.getMyLogger(this.getClass().getName());
	private int maxResultLimit = -1;
	
	public KB(int maxResultLimit) {
		this.maxResultLimit = maxResultLimit;
	}
	
	public void setSubscriptionResponder(SubscriptionResponder sResp){
		sr = sResp;
	}
	
	public void setLeaseManager(LeaseManager leaseMng){
		lm = leaseMng;
	}
	
	public Object register(Object name, Object fact) {
		// We don't want to register a fact whose lease time has 
		// already expired
		if (lm.isExpired(lm.getLeaseTime(fact))) {
			if(logger.isLoggable(Logger.WARNING))
				logger.log(Logger.WARNING,"Fact with lease time already expired");
			return null;
		}
		
		// Now apply lease manager policy on requested lease time.
		lm.grantLeaseTime(fact);
		
		Object previous = insert(name, fact);
		if (previous != null && lm.isExpired(lm.getLeaseTime(previous))) {
			previous = null;
		}
		return previous;
	}
	
	public Object deregister(Object name) {
		Object obj = remove(name);
		if (obj != null && lm.isExpired(lm.getLeaseTime(obj))) {
			obj = null;
		}
		return obj;
	}
	
	protected abstract Object insert(Object name, Object fact);
	protected abstract Object remove(Object name);
	
	public List search(Object template) {
		return search(template, maxResultLimit);
	}
	
	public abstract List search(Object template, int maxResult);
	public abstract KBIterator iterator(Object template);
	
	public abstract void subscribe(Object template, SubscriptionResponder.Subscription s) throws NotUnderstoodException;
	public abstract Enumeration getSubscriptions();
	public abstract void unsubscribe(SubscriptionResponder.Subscription sub);
}