/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.KBManagement;

//#MIDP_EXCLUDE_FILE

import java.util.Enumeration;
import java.util.Hashtable;

import rca.util.leap.Map;
import rca.util.leap.HashMap;
import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;
import rca.util.Logger;

import rca.domain.FIPAAgentManagement.DFAgentDescription;
import rca.domain.FIPAAgentManagement.NotUnderstoodException;

import rca.lang.acl.ACLMessage;
import rca.proto.SubscriptionResponder;
import rca.core.AID;

/**
 * Common base class for AMS and DF Knowledge Base
 * 
 * Original author Elisabetta Cortese - TILab
 */
public abstract class MemKB extends KB {

	protected Map facts = new HashMap();
	protected Hashtable subscriptions = new Hashtable();
	protected LeaseManager lm;
	protected int currentReg = 0;
	protected SubscriptionResponder sr;

	protected final static int MAX_REGISTER_WITHOUT_CLEAN = 100;

	private Logger logger = Logger.getMyLogger(this.getClass().getName());
	
	
	public MemKB(int maxResultLimit) {
	    super(maxResultLimit);
	}

	protected Object insert(Object name, Object fact) {
		currentReg ++;
		if(currentReg > MAX_REGISTER_WITHOUT_CLEAN){
			clean();
			currentReg = 0;
		}
     
	  return facts.put(name, fact);
	}

	protected Object remove(Object name) {
	    return facts.remove(name);
	}

	  // This abstract method has to perform pattern matching
	protected abstract boolean match(Object template, Object fact);


	protected abstract void clean();

	public List search(Object template, int maxResults) {
		List result = new ArrayList();
	    Iterator it = facts.values().iterator();
		int found = 0;
	    while(it.hasNext() && ((maxResults < 0) || (found < maxResults)) ) {
	        Object fact = it.next();
		    if(match(template, fact)){
				result.add(fact);
				found ++;
		    }
      	}
        return result;
	}
	
	/**
	   Iterated search is only supported in a DB-based KB.
	   Throw a RuntimeException. The requester will get back a FAILURE
	 */
	public KBIterator iterator(Object template) {
		throw new RuntimeException("Iterated search non supported");
	}

	//
	public void subscribe(Object dfd, SubscriptionResponder.Subscription s) throws NotUnderstoodException{
		try{
			DFAgentDescription dfdTemplate = (DFAgentDescription) dfd;
			ACLMessage aclSub = (ACLMessage) s.getMessage();
			subscriptions.put(dfdTemplate, s);
		}catch(Exception e){
                  if(logger.isLoggable(Logger.SEVERE))
                    logger.log(Logger.SEVERE,"Subscribe error: "+e.getMessage());
			throw new NotUnderstoodException(e.getMessage());
		}
	}


	public Enumeration getSubscriptionDfAgentDescriptions(){
		return subscriptions.keys();
	}


	private SubscriptionResponder.Subscription getSubscription(Object key){
		SubscriptionResponder.Subscription sub = (SubscriptionResponder.Subscription)subscriptions.get(key);
		return sub;
	}


	// RITORNA Lista delle sottoscrizioni
	public	Enumeration getSubscriptions(){
		return	subscriptions.elements();
	}

	int offSetForSubscriptionToReturn = 0;

	// RITORNA Lista delle sottoscrizioni
	public	Enumeration getSubscriptions(int offset){
		// FIXME to do if this operation return a lot of subscriptions
		return null;
	}


	public void unsubscribe(SubscriptionResponder.Subscription sub ){

		ACLMessage aclSub = sub.getMessage();
		String convID = aclSub.getConversationId();
		Enumeration e = getSubscriptionDfAgentDescriptions();

		if( e != null ){
			while(e.hasMoreElements()){
				DFAgentDescription dfd = (DFAgentDescription)e.nextElement();
				SubscriptionResponder.Subscription s = getSubscription((Object) dfd);
				if((s.getMessage().getConversationId()).equals(convID)){
					subscriptions.remove(dfd);
					break;
				}
			}
		}
	}

	/*public void setSubscriptionResponder(SubscriptionResponder sResp){
		sr = sResp;
	}

	public void setLeaseManager(LeaseManager leaseMng){
		lm = leaseMng;
	}*/


	// Helper method to match two Agent Identifiers
	public static final boolean matchAID(AID template, AID fact) {

	  // Match the GUID in the ':name' slot
	    String templateName = template.getName();
	    if(templateName != null) {
	        String factName = fact.getName();
	        if((factName == null) || (!templateName.equalsIgnoreCase(factName)))
		    	return false;
	    }

	    // Match the address sequence. See 'FIPA Agent Management Specification, Sect. 6.4.2.1'
	    Iterator itTemplate = template.getAllAddresses();
	    Iterator itFact = fact.getAllAddresses();

	    // All the elements in the template sequence must appear in the
	    // fact sequence, in the same order
	    while(itTemplate.hasNext()) {
	        String templateAddr = (String)itTemplate.next();

	        // Search 'templateAddr' into the remaining part of the fact sequence
	        boolean found = false;
	        while(!found && itFact.hasNext()) {
	  	        String factAddr = (String)itFact.next();
		        found = templateAddr.equalsIgnoreCase(factAddr);
	        }
	        if(!found) // An element of the template does not appear in the fact sequence
				return false;
	   	}

	    // Match the resolvers sequence. See 'FIPA Agent Management Specification, Sect. 6.4.2.1'
	    itTemplate = template.getAllResolvers();
	    itFact = fact.getAllResolvers();

	    while(itTemplate.hasNext()) {
	      AID templateRes = (AID)itTemplate.next();

	      // Search 'templateRes' into the remaining part of the fact sequence
	      boolean found = false;
	      while(!found && itFact.hasNext()) {
			AID factRes = (AID)itFact.next();
			found = matchAID(templateRes, factRes); // Recursive call
	      }
	      if(!found) // An element of the template does not appear in the fact sequence
			  return false;
	    }

	    return true;
	  }


}
