/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain;

//#MIDP_EXCLUDE_FILE

import java.util.Enumeration;

import rca.util.leap.ArrayList;
import rca.util.leap.List;

import rca.domain.FIPAAgentManagement.*;
import rca.domain.KBManagement.*;
import rca.lang.acl.ACLMessage;

import rca.proto.SubscriptionResponder;

import rca.content.*;
import rca.content.lang.sl.*;
import rca.content.onto.*;
import rca.content.abs.*;

/**
 * Original author Elisabetta Cortese - TILab
 *
 */
class KBSubscriptionManager implements SubscriptionResponder.SubscriptionManager{
	
	KB kBase;
	ContentManager cm;
	
	public KBSubscriptionManager(KB k){
		super();
		kBase = k;
	}
	
	public void setContentManager(ContentManager c){
		cm = c;
	}
	
	public boolean register(SubscriptionResponder.Subscription sub) throws RefuseException, NotUnderstoodException{
		
		DFAgentDescription dfdTemplate = null;
		SearchConstraints constraints = null;
		AbsIRE absIota = null;
		
		try{
			// Get DFD template and search constraints from the subscription message 
			ACLMessage subMessage = sub.getMessage();
			
			absIota = (AbsIRE) cm.extractAbsContent(subMessage);
			AbsPredicate absResult = absIota.getProposition();
			AbsAgentAction absAction = (AbsAgentAction) absResult.getAbsObject(BasicOntology.RESULT_ACTION);
			AbsAgentAction absSearch = (AbsAgentAction) absAction.getAbsObject(BasicOntology.ACTION_ACTION);
			Search search = (Search) FIPAManagementOntology.getInstance().toObject(absSearch);
			
			dfdTemplate = (DFAgentDescription) search.getDescription();
			constraints = search.getConstraints();
			
			// Register the Subscription
			kBase.subscribe(dfdTemplate, sub);
			
		}catch(Exception e){
			throw new NotUnderstoodException(e.getMessage());
		}
		// Search for DFDs that already match the specified template.
		// Note that we ignore the SearchConstraint.maxResult here
		// FIXME: Getting all DFDs matching a template may cause out-of-memory.
		// We should use an iterated search and send back possibly more than one
		// notification.
		List results = kBase.search(dfdTemplate, -1); 
		
		// If some DFD matches the template, notify the subscribed agent 
		if(results.size() > 0){
			notify(sub, results, absIota);
			return true;
		}
		return false;
	}
	
	
	//OK
	// degeregister the subscritpion from hashtable
	public boolean deregister( SubscriptionResponder.Subscription sub ) throws FailureException {
		kBase.unsubscribe(sub);
		return false;
	}
	
	
	private DFAgentDescription getDFAgentDescriptionFromACL(ACLMessage aclM){
		DFAgentDescription dfd = null;
		try{
			AbsIRE absIota = (AbsIRE) cm.extractAbsContent(aclM);
			AbsPredicate absResult = absIota.getProposition();
			AbsAgentAction absAction = (AbsAgentAction) absResult.getAbsObject(BasicOntology.RESULT_ACTION);
			AbsAgentAction absSearch = (AbsAgentAction) absAction.getAbsObject(BasicOntology.ACTION_ACTION);
			Search search = (Search) FIPAManagementOntology.getInstance().toObject(absSearch);
			
			dfd = (DFAgentDescription) search.getDescription();
		}catch(Exception e){
			e.printStackTrace();
		} 
		return dfd;
	}
	
	/**
	 Handle registrations/deregistrations/modifications by notifying 
	 subscribed agents if necessary
	 */
	void handleChange(DFAgentDescription dfd, DFAgentDescription oldDfd) {
		// Create a temporary MemKB just to re-use the match() method. 
		DFMemKB memTemp = new DFMemKB(0);
		
		Enumeration e = kBase.getSubscriptions();
		while (e.hasMoreElements()) {
			SubscriptionResponder.Subscription sub = (SubscriptionResponder.Subscription) e.nextElement();
			DFAgentDescription template = getDFAgentDescriptionFromACL(sub.getMessage());
			if ( memTemp.match(template, dfd) || ((oldDfd!=null) && memTemp.match(template, oldDfd))) {
				// This subscriber must be notified
				List results = new ArrayList();
				results.add(dfd);
				ACLMessage aclSub = sub.getMessage();
				AbsIRE absIota=null;
				try{
					absIota = (AbsIRE) cm.extractAbsContent(aclSub);
				}catch(Exception ex){
					ex.printStackTrace();
				}
				notify(sub, results, absIota);
			}
		}
	}
	
	private void notify(SubscriptionResponder.Subscription sub, List results, AbsIRE absIota) {
		try {
			ACLMessage notification = sub.getMessage().createReply();
			notification.setPerformative(ACLMessage.INFORM);
			AbsPredicate absEquals = new AbsPredicate(SLVocabulary.EQUALS);
			absEquals.set(SLVocabulary.EQUALS_LEFT, absIota);
			absEquals.set(SLVocabulary.EQUALS_RIGHT, FIPAManagementOntology.getInstance().fromObject(results));
			
			cm.fillContent(notification, absEquals);
			//pass to Subscription the message to send
			sub.notify(notification);
		}
		catch (Exception e) {
			e.printStackTrace();
			//FIXME: Check whether a FAILURE message should be sent back.       
		}
	}
}
