/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain;

//#MIDP_EXCLUDE_FILE

import rca.core.AID;

import rca.domain.FIPAAgentManagement.*;
import rca.domain.FIPAAgentManagement.InternalError;

import rca.lang.acl.ACLMessage;

import rca.proto.SimpleAchieveREInitiator;

import rca.content.*;
import rca.content.onto.basic.*;

/**
   This behaviour is used by the df agent only (it is not defined 
   as an Inner class due to compilation problems in PJAVA) and handles 
   a request to another DF. These requests can 
   be triggered by an action of the DF-Applet ontology (e.g. Federate) 
   or by a DF GUI event. The onEnd() method must be re-defined to manage
   the result properly.
   Original author Giovanni Caire - TILAB
 */
class RemoteDFRequester extends SimpleAchieveREInitiator {
	private AID remoteDF;
	private Concept myAction;
	private Object result = null;
	
	RemoteDFRequester(AID remoteDF, Concept myAction) {
		super(null, null);
		this.remoteDF = remoteDF;
		this.myAction = myAction;
	}
	
  protected ACLMessage prepareRequest(ACLMessage msg){
  	ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
  	request.addReceiver(remoteDF);
  	request.setOntology(FIPAManagementOntology.getInstance().getName());
  	request.setLanguage(FIPANames.ContentLanguage.FIPA_SL0);
  	request.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
  	
  	Action act = new Action(remoteDF, myAction);
  	try {
  		myAgent.getContentManager().fillContent(request, act);
  		return request;
  	}
  	catch (Exception e) {
  		// Should never happen
  		e.printStackTrace();
  		result = new InternalError("Error encoding request ["+e.getMessage()+"]");
  	}
  	return null;
  }
  
  protected void handleInform(ACLMessage inform) {
  	try {
  		Predicate p = (Predicate) myAgent.getContentManager().extractContent(inform);
  		if (p instanceof Result) {
  			result = ((Result) p).getValue();
  		}
  	}
  	catch (Exception e) {
  		result = new InternalError("Error decoding response ["+e.getMessage()+"]");
  	}
  }
  
  protected void handleRefuse(ACLMessage refuse) {
  	result = new InternalError("Agent "+remoteDF.getName()+" replyed with "+ACLMessage.getPerformative(refuse.getPerformative()));
  }
  
  protected void handleNotUnderstood(ACLMessage notUnderstood) {
  	result = new InternalError("Agent "+remoteDF.getName()+" replyed with "+ACLMessage.getPerformative(notUnderstood.getPerformative()));
  }
  
  protected void handleFailure(ACLMessage failure) {
  	result = new InternalError("Agent "+remoteDF.getName()+" replyed with "+ACLMessage.getPerformative(failure.getPerformative()));
  }
  
  protected Object getResult() {
  	return result;
  }
  
  protected AID getRemoteDF() {
  	return remoteDF;
  }
  
  protected Concept getAction() {
  	return myAction;
  }
}
    	
