/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.domain.FIPAAgentManagement.Envelope;
import rca.content.Concept;

/**

  This class represents an ACL message.

  Original author Giovanni Rimassa - Universita` di Parma
  @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $

*/
public class ACLMessage implements Concept {

    private Envelope env;
    private String payload;

    /**
       Default constructor. A default constructor is necessaty for
       rca ontological classes.
    */
    public ACLMessage() {
    }

    /**
       Set the message envelope.
       @param e The transport-level envelope to attach to this ACL
       message.
    */
    public void setEnvelope(Envelope e) {
	env = e;
    }

    /**
       Retrieve the message envelope.
       @return The transport-level envelope attached to this ACL
       message, or <code>null</code> if no envelope was set.
    */
    public Envelope getEnvelope() {
	return env;
    }

    /**
       Set the representation of the message payload.
       @param r The name of the representation expressing the message
       payload.
    */
    public void setAclRepresentation(String r) {
	if(env != null)
	    env.setAclRepresentation(r);
    }

    /**
       Retrieve the representation of the message payload.
       @return The ACL representation of the message.
    */
    public String getAclRepresentation() {
	if(env != null)
	    return env.getAclRepresentation();
	else
	    return null;
    }

    /**
       Set the payload (i.e. the speech act level part) of this ACL
       message.
       @param p A string containing the encoding of the payload
       according to a concrete ACL representaiton.
    */
    public void setPayload(String p) {
	payload = p;
    }

    /**
       Retrieve the payload (i.e. the speech act level part) of this
       ACL message.
       @return A string containing the encoding of the payload
       according to a concrete ACL representation.
    */
    public String getPayload() {
	return payload;
    }

}
