/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.core.AID;
import rca.core.BehaviourID;

/**
   An introspection event, recording that a behaviour was added to an
   agent.

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $
*/
public class AddedBehaviour implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Added-Behaviour";
  
    private AID agent;
    private BehaviourID behaviour;

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public AddedBehaviour() {
    }

    /**
       Set the <code>agent</code> of this event.
       @param id The agent identifier of the agent the behaviour was
       added to.
    */
    public void setAgent(AID id) {
	agent = id;
    }

    /**
       Retrieve the value of the <code>agent</code> slot of this
       event, containing the agent identifier of the agent the
       behaviour was added to.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgent() {
	return agent;
    }

    /**
       Set the <code>behaviour</code> of this event.
       @param id The behaviour identifier of the newly added
       behaviour.
    */
    public void setBehaviour(BehaviourID id) {
	behaviour = id;
    }

    /**
       Retrieve the value of the <code>behaviour</code> slot of this
       event, containing the behaviour identifier of the newly added
       behaviour.
       @return The value of the <code>behaviour</code> slot, or
       <code>null</code> if no value was set.
    */
    public BehaviourID getBehaviour() {
	return behaviour;
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

}
