/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.introspection;

import rca.core.ContainerID;

/**
   This class represents the <code>added-mtp</code> concept in the
   <code>rca-introspection</code> ontology.

   Original author Giovanni Rimassa - Universita' di Parma
   @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
 */
public class AddedMTP implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Added-MTP";

    private String address;
    private String proto;
    private ContainerID where;

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public AddedMTP() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>address</code> of this event.
       @param s The address URL exported by the newly added MTP.
    */
    public void setAddress(String s) {
	address = s;
    }

    /**
       Retrieve the value of the <code>address</code> slot of this
       event, containing the address URL of the newly added MTP.
       @return The value of the <code>address</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getAddress() {
	return address;
    }

    /**
       Set the <code>proto</code> slot of this event.
       @param p The protocol name for the newly added MTP.
    */
    public void setProto(String p) {
	proto = p;
    }

    /**
       Retrieve the value of the <code>proto</code> slot of this
       event, containing the protocol name for the newly added MTP.
       @return The value of the <code>proto</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getProto() {
	return proto;
    }

    /**
       Set the <code>where</code> slot of this event.
       @param id The container identifier of the container where the
       newly added MTP was deployed.
    */
    public void setWhere(ContainerID id) {
	where = id;
    }

    /**
       Retrieve the value of the <code>where</code> slot of this
       event, containing the container identifier of the container
       where the newly added MTP was deployed.
       @return The value of the <code>where</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getWhere() {
	return where;
    }

}
