/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

import rca.core.AID;
import rca.core.ContainerID;


/**
   An introspection event, recording the birth of a new agent within
   the platform.
   
   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class BornAgent implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Born-Agent";

    private AID agent;
    private ContainerID where;
    private String state;
    private String ownership;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public BornAgent() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>where</code> slot of this event.
       @param id The container identifier of the container where the
       newly added agent was born.
    */
    public void setWhere(ContainerID id) {
	where = id;
    }

    /**
       Retrieve the value of the <code>where</code> slot of this
       event, containing the container identifier of the container
       where the newly added agent was born.
       @return The value of the <code>where</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getWhere() {
	return where;
    }

    /**
       Set the <code>agent</code> slot of this event.
       @param id The agent identifier of the newly born agent.
    */
    public void setAgent(AID id) {
	agent = id;
    }

    /**
       Retrieve the value of the <code>agent</code> slot of this
       event, containing the agent identifier of the newly born agent.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgent() {
	return agent;
    }

    /**
       Set the <code>state</code> slot of this event.
       @param as The name of the initial state of the newly born
       agent.
    */
    public void setState(String as) {
	state = as;
    }

    /**
       Retrieve the value of the <code>state</code> slot of this
       event, containing the initial state of the newly born agent.
       @return The value of the <code>state</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getState() {
	return state;
    }

    /**
       Set the <code>ownership</code> slot of this event.
       @param o The name of the entity owning the newly born agent.
    */
    public void setOwnership(String o) {
	ownership = o;
    }

    /**
       Retrieve the value of the <code>ownership</code> slot of this
       event, containing the name of the entity owning the newly born
       agent.
       @return The value of the <code>where</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getOwnership() {
	return ownership;
    }

}
