/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.core.AID;
import rca.core.ContainerID;

/**
   An introspection event, recording the change of the ownership of an
   agent within the platform.

   Original author Michele Tomaiuolo -  Universita' di Parma
   @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $
*/
public class ChangedAgentOwnership implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Changed-Agent-Ownership";

    private AID agent;
    private String from;
    private String to;

    private ContainerID where;

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public ChangedAgentOwnership() {
    }

    /**
       Set the <code>where</code> slot of this event.
       @param id The container identifier of the container where the
       agent is deployed.
    */
    public void setWhere(ContainerID id) {
    where = id;
    }

    /**
       Retrieve the value of the <code>where</code> slot of this
       event, containing the container identifier of the container
       where the agent is deployed.
       @return The value of the <code>where</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getWhere() {
    return where;
    }

    /**
       Set the <code>agent</code> slot of this event.
       @param id The agent identifier of the agent whose ownership
       changed.
    */
    public void setAgent(AID id) {
    agent = id;
    }

    /**
       Retrieve the value of the <code>agent</code> slot of this
       event, containing the agent identifier of the agent whose
       ownership changed.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgent() {
    return agent;
    }

    /**
       Set the <code>from</code> slot of this event.
       @param o The name of the entity that owned the agent before
       this event occurred.
    */
    public void setFrom(String o) {
    from = o;
    }

    /**
       Retrieve the value of the <code>from</code> slot of this event,
       containing the name of the entity that owned the agent before
       this event occurred.
       @return The value of the <code>from</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getFrom() {
    return from;
    }

    /**
       Set the <code>to</code> slot of this event.
       @param o The name of the entity owning the agent after this
       event occurred.
    */
    public void setTo(String o) {
    to = o;
    }

    /**
       Retrieve the value of the <code>to</code> slot of this
       event, containing the name of the entity owning the agent after this
       event occurred.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getTo() {
    return to;
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
    return NAME;
    }

}
