/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

import rca.core.AID;
import rca.core.ContainerID;


/**
 An introspection event, recording the death of an agent within the
 platform.
 
 Original author Giovanni Rimassa -  Universita' di Parma
 @version $Date: 2006-02-21 08:56:26 +0100 (Tue, 21 Feb 2006) $ $Revision: 5860 $
 */

public class DeadAgent implements Event {
	
	/**
	 A string constant for the name of this event.
	 */
	public static final String NAME = "Dead-Agent";
	
	private AID agent;
	private ContainerID where;
	private Boolean containerRemoved;
	
	
	/**
	 Default constructor. A default constructor is necessary for
	 ontological classes.
	 */
	public DeadAgent() {
	}
	
	/**
	 Retrieve the name of this event.
	 @return A constant value for the event name.
	 */
	public String getName() {
		return NAME;
	}
	
	/**
	 Set the <code>where</code> slot of this event.
	 @param id The container identifier of the container where the
	 newly dead agent was deployed.
	 */
	public void setWhere(ContainerID id) {
		where = id;
	}
	
	/**
	 Retrieve the value of the <code>where</code> slot of this
	 event, containing the container identifier of the container
	 where the newly dead agent was deployed.
	 @return The value of the <code>where</code> slot, or
	 <code>null</code> if no value was set.
	 */
	public ContainerID getWhere() {
		return where;
	}
	
	/**
	 Set the <code>agent</code> slot of this event.
	 @param id The agent identifier of the newly dead agent.
	 */
	public void setAgent(AID id) {
		agent = id;
	}
	
	/**
	 Retrieve the value of the <code>agent</code> slot of this
	 event, containing the agent identifier of the newly dead agent.
	 @return The value of the <code>agent</code> slot, or
	 <code>null</code> if no value was set.
	 */
	public AID getAgent() {
		return agent;
	}
	
	public Boolean getContainerRemoved() {
		return containerRemoved;
	}
	
	public void setContainerRemoved(Boolean cr) {
		containerRemoved = cr;
	}
}
