/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import java.util.Date;

import rca.core.Location;
import rca.content.Concept;

/**
   This class represents the <code>event-record</code> concept in the
   <code>rca-introspection</code> ontology.

   Original author Giovanni Rimassa - Universita' di Parma
   @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $
 */
public class EventRecord implements Concept {

    private Event what;
    private Date when;
    private Location where;

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public EventRecord() {
    }

    /**
       Construct an event record, attaching a location and a timestamp
       to an event object.
       @param evt The occurred event.
       @param l The location where the event occurred.
    */
    public EventRecord(Event evt, Location l) {
	what = evt;
	when = new Date();
	where = l;
    }

    /**
       Set the <code>what</code> slot of this event.
       @param id The occurred event.
    */
    public void setWhat(Event evt) {
	what = evt;
    }

    /**
       Retrieve the value of the <code>what</code> slot of this
       event, containing the occurred event.
       @return The value of the <code>what</code> slot, or
       <code>null</code> if no value was set.
    */
    public Event getWhat() {
	return what;
    }

    /**
       Set the <code>when</code> slot of this event.
       @param d The time instant when the event occurred.
    */
    public void setWhen(Date d) {
	when = d;
    }

    /**
       Retrieve the value of the <code>when</code> slot of this event,
       containing the time instant when the event occurred.
       @return The value of the <code>when</code> slot, or
       <code>null</code> if no value was set.
    */
    public Date getWhen() {
	return when;
    }

    /**
       Set the <code>where</code> slot of this event.
       @param l The location where the event occurred.
    */
    public void setWhere(Location l) {
	where = l;
    }

    /**
       Retrieve the value of the <code>where</code> slot of this
       event, containing the location where the event occurred.
       @return The value of the <code>where</code> slot, or
       <code>null</code> if no value was set.
    */
    public Location getWhere() {
	return where;
    }

}
