/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.core.AID;
import rca.core.ContainerID;


/**
   An introspection event, recording the freezing of a formerly
   active agent within the platform.

   Original author Giovanni Rimassa -  FRAMeTech s.r.l.
*/
public class FrozenAgent implements Event {


    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Frozen-Agent";

    private AID agent;
    private ContainerID where;
    private ContainerID bufferContainer;

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public FrozenAgent() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>agent</code> slot of this event.
       @param id The agent identifier of the newly suspended agent.
    */
    public void setAgent(AID id) {
	agent = id;
    }

    /**
       Retrieve the value of the <code>agent</code> slot of this
       event, containing the agent identifier of the newly suspended
       agent.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgent() {
	return agent;
    }

    /**
       Set the <code>where</code> slot of this event.
       @param id The container identifier of the container where the
       newly frozen agent was deployed.
    */
    public void setWhere(ContainerID id) {
	where = id;
    }

    /**
       Retrieve the value of the <code>where</code> slot of this
       event, containing the container identifier of the container
       where the newly frozen agent was deployed.
       @return The value of the <code>where</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getWhere() {
	return where;
    }

    /**
       Set the <code>buffer-container</code> slot of this event.
       @param id The container identifier of the container where the
       frozen agent will appear to reside (and where ACL messages for
       that agent will be redirected and buffered).
    */
    public void setBufferContainer(ContainerID id) {
	bufferContainer = id;
    }

    /**
       Retrieve the value of the <code>buffer-container</code> slot of
       this event, containing the container identifier of the
       container where the frozen agent will appear to reside (and
       where ACL messages for that agent will be redirected and
       buffered).
       @return The value of the <code>buffer-container</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getBufferContainer() {
	return bufferContainer;
    }

}
