/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.introspection;

import rca.core.ContainerID;

/**
 * This class represents the <code>kill-container-requested</code> concept in the
 * <code>rca-introspection</code> ontology.
 * Original author Giovanni Caire - TILAB
 * @since rca 3.4
 */
public class KillContainerRequested implements Event {
	/**
	 A string constant for the name of this event.
	 */
	public static final String NAME = "kill-Container-Requested";

	private ContainerID container;

	/**
	 Default constructor. A default constructor is necessary for
	 ontological classes.
	 */
	public KillContainerRequested() {
	}

	/**
	 Retrieve the name of this event.
	 @return A constant value for the event name.
	 */
	public String getName() {
		return NAME;
	}

	/**
	 Set the <code>container</code> of this event.
	 @param id The container identifier of the container that is requested to be killed.
	 */
	public void setContainer(ContainerID id) {
		container = id;
	}

	/**
	 Retrieve the value of the <code>container</code> slot of this
	 event, containing the identifier of the container that is requested
	 to be killed.
	 @return The value of the <code>container</code> slot, or
	 <code>null</code> if no value was set.
	 */
	public ContainerID getContainer() {
		return container;
	}
}
