/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

import rca.core.AID;
import rca.core.ContainerID;

/**
   An introspection event, recording the migration of an agent within
   the platform.

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/

public class MovedAgent implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Moved-Agent";

    private AID agent;
    private ContainerID from;
    private ContainerID to;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public MovedAgent() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>agent</code> slot of this event.
       @param id The agent identifier of the newly moved agent.
    */
    public void setAgent(AID id) {
	agent = id;
    }

    /**
       Retrieve the value of the <code>agent</code> slot of this
       event, containing the agent identifier of the newly moved agent.
       @return The value of the <code>agent</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getAgent() {
	return agent;
    }


    /**
       Set the <code>from</code> slot of this event.
       @param id The container identifier of the container from where
       the agent migrated.
    */
    public void setFrom(ContainerID id) {
	from = id;
    }

    /**
       Retrieve the value of the <code>from</code> slot of this event,
       containing the container identifier of the container from where
       the agent migrated.
       @return The value of the <code>from</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getFrom() {
	return from;
    }

    /**
       Set the <code>to</code> slot of this event.
       @param id The container identifier of the container where the
       agent migrated.
    */
    public void setTo(ContainerID id) {
	to = id;
    }

    /**
       Retrieve the value of the <code>to</code> slot of this
       event, containing the container identifier of the container
       where the agent migrated.
       @return The value of the <code>to</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getTo() {
	return to;
    }


}
