/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.domain.FIPAAgentManagement.APDescription;
/**
   This class represents the <code>platform-description</code>
   predicate, whose argument is a platform description object from the
   FIPA Agent Management ontology.
   
   Original author Tiziana Trucco -  CSELT S.p.A.
   @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $
*/
public class PlatformDescription implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = IntrospectionVocabulary.PLATFORMDESCRIPTION;

    private APDescription platform;

    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public PlatformDescription() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>platform</code> slot of this event.
       @param p The <code>ap-description</code> this notification
       refers to.
    */
    public void setPlatform(APDescription p) {
	platform = p;
    }

    /**
       Retrieve the value of the <code>platform</code> slot of this
       event, containing the <code>ap-description</code> this
       notification refers to.
       @return The value of the <code>platform</code> slot, or
       <code>null</code> if no value was set.
    */
    public APDescription getPlatform() {
	return platform;
    }


}
