/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.core.AID;

/**
   An introspection event, recording the posting of an ACL message
   within the platform.

   Original author Giovanni Rimassa -  Universita' di Parma
   @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $
*/
public class PostedMessage implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Posted-Message";

    private ACLMessage message;
    private AID sender;
    private AID receiver;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public PostedMessage() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>sender</code> slot of this event.
       @param id The agent identifier of the agent that sent the
       message.
    */
    public void setSender(AID id) {
	sender = id;
    }

    /**
       Retrieve the value of the <code>sender</code> slot of this
       event, containing the agent identifier of the agent that sent
       the message.
       @return The value of the <code>sender</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getSender() {
	return sender;
    }
    
    /**
       Set the <code>message</code> slot of this event.
       @param msg The ACL message that was posted to the receiver
       agent message queue.
    */
    public void setMessage(ACLMessage msg) {
	message = msg;
    }

    /**
       Retrieve the value of the <code>message</code> slot of this
       event, containing the ACL message that was posted to the
       receiver agent message queue.
       @return The value of the <code>message</code> slot, or
       <code>null</code> if no value was set.
    */
    public ACLMessage getMessage() {
	return message;
    }

    /**
       Set the <code>receiver</code> slot of this event.
       @param id The agent identifier of the agent owning the message
       queue the message was posted to.
    */
    public void setReceiver(AID id) {
	receiver = id;
    }

    /**
       Retrieve the value of the <code>receiver</code> slot of this
       event, containing the agent identifier of the agent owning the
       message queue the message was posted to.
       @return The value of the <code>receiver</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getReceiver() {
	return receiver;
    }

}
