/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.introspection;


import rca.core.ContainerID;

/**

   An introspection event, recording the removal of an agent container
   within the platform.

   Original author Giovanni Rimassa - Universita' di Parma
   @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
 */
public class RemovedContainer implements Event {

    /**
       A string constant for the name of this event.
    */
    public static final String NAME = "Removed-Container";

    private ContainerID container;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public RemovedContainer() {
    }

    /**
       Retrieve the name of this event.
       @return A constant value for the event name.
    */
    public String getName() {
	return NAME;
    }

    /**
       Set the <code>container</code> slot of this event.
       @param id The container identifier of the newly removed
       container.
    */
    public void setContainer(ContainerID id) {
	container = id;
    }

    /**
       Retrieve the value of the <code>container</code> slot of this
       event, containing the container identifier of the newly removed
       container.
       @return The value of the <code>container</code> slot, or
       <code>null</code> if no value was set.
    */
    public ContainerID getContainer() {
	return container;
    }

}
