/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.introspection;

//#APIDOC_EXCLUDE_FILE

import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;

import rca.core.AID;

import rca.content.AgentAction;

/**

  This class represents the <code>stop-notify</code> action,
  requesting to end a continuous notification of some events via ACL
  messages.

  Original author Giovanni Rimassa - Universita' di Parma
  @version $Date: 2005-02-16 18:18:28 +0100 (Wed, 16 Feb 2005) $ $Revision: 5552 $

*/
public class StopNotify implements AgentAction {

    private AID observed;
    private List events = new ArrayList();


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public StopNotify() {
    }

    /**
       Set the <code>observed</code> slot of this action.
       @param id The agent identifier of the agent whose events were
       being notified so far.
    */
    public void setObserved(AID id) {
	observed = id;
    }

    /**
       Retrieve the value of the <code>observed</code> slot of this
       event, containing the agent identifier of the agent whose
       events were being notified so far.
       @return The value of the <code>observed</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getObserved() {
	return observed;
    }

    /**
       Add an event name to the <code>events</code> slot collection of
       this object.
       @param evName The event name to add to the collection.
    */
    public void addEvents(String evName) {
	events.add(evName);
    }

    /**
       Remove all event names from the <code>events</code> slot
       collection of this object.
    */
    public Iterator getAllEvents() {
	return events.iterator();
    }


}
