/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.mobility;

import rca.content.onto.*;
import rca.content.schema.*;
import rca.domain.FIPAAgentManagement.ExceptionOntology;

/**
   This class represents the BehaviourLoading ontology including
   the concepts and actions required to dynamically load and execute 
   rca behaviours whose code is not included in the JVM classpath.
   @see rca.core.behaviours.LoaderBehaviour
   Original author Giovanni Caire - TILAB
 */
public class BehaviourLoadingOntology  extends Ontology implements BehaviourLoadingVocabulary {

  private static Ontology theInstance = new BehaviourLoadingOntology();
  
  /**
     This method returns the singleton instance of the 
     Behaviour-Loading ontology.
     @return The singleton <code>Ontology</code> object, containing the 
     schemas for the elements of the Behaviour-Loading ontology.
  */
  public static Ontology getInstance() {
    return theInstance;
  }

  private BehaviourLoadingOntology() {
  	super(NAME, new Ontology[]{ExceptionOntology.getInstance(), SerializableOntology.getInstance()}, new ReflectiveIntrospector());
    	
		try {
	  	add(new ConceptSchema(PARAMETER), Parameter.class);	 	 	
	  	add(new AgentActionSchema(LOAD_BEHAVIOUR), LoadBehaviour.class);
	  	
	  	ConceptSchema cs = (ConceptSchema)getSchema(PARAMETER);
	  	cs.add(PARAMETER_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
	  	cs.add(PARAMETER_VALUE, (TermSchema)TermSchema.getBaseSchema(), ObjectSchema.OPTIONAL);
	  	cs.add(PARAMETER_MODE, (PrimitiveSchema)getSchema(BasicOntology.INTEGER));
  	
	  	AgentActionSchema as = (AgentActionSchema)getSchema(LOAD_BEHAVIOUR);
	  	as.add(LOAD_BEHAVIOUR_CLASS_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
	  	as.add(LOAD_BEHAVIOUR_CODE, (PrimitiveSchema)getSchema(BasicOntology.BYTE_SEQUENCE), ObjectSchema.OPTIONAL);
	  	as.add(LOAD_BEHAVIOUR_ZIP, (PrimitiveSchema)getSchema(BasicOntology.BYTE_SEQUENCE), ObjectSchema.OPTIONAL);
	  	as.add(LOAD_BEHAVIOUR_PARAMETERS, (ConceptSchema)getSchema(PARAMETER), 0, ObjectSchema.UNLIMITED);	  	
    } 
    catch(OntologyException oe) {
    	oe.printStackTrace();
    }
  } 
}
