/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

/**

  This class represents the <code>clone-agent</code> action,
  requesting to clone an agent within the platform.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class CloneAction extends MoveAction {

    private String newName;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public CloneAction() {
    }

    /**
       Set the <code>new-name</code> slot of this action.
       @param nn The new local name (i.e. without the platform ID) for the
       new, cloned agent.
    */
    public void setNewName(String nn) {
      newName = nn;
    }

    /**
       Retrieve the value of the <code>new-name</code> slot of this
       event, containing the new local name (i.e. without the platform ID)
       for the new, cloned agent.
       @return The value of the <code>new-name</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getNewName() {
      return newName;
    }

}
