/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

//#MIDP_EXCLUDE_FILE

import rca.content.AgentAction;
import rca.util.leap.List;

/**
   This action represents a request to load a <code>Behaviour</code> whose
   code is not included in the classpath of the JVM where the agent that is 
   going to execute the behaviour lives.
   @see rca.core.behaviours.LoaderBehaviour
   Original author Giovanni Caire - TILAB
 */
public class LoadBehaviour implements AgentAction {
	private String className;
	private byte[] code;
	private byte[] zip;
	private List parameters;
	
	public LoadBehaviour() {
	}
	
	/**
	   Sets the name of the class of the behaviour to load
	 */
	public void setClassName(String className) {
		this.className = className;
	}
	
	/**
	   @return the name of the class of the behaviour to load
	 */
	public String getClassName() {
		return className;
	}
	
	/**
	   Sets the code of the class of the behaviour to load.
	   <code>code</code> must be filled with the content of the class 
	   file of the behaviour to load. 
	   If the behaviour requires other classes, the <code>setZip()</code>
	   method must be used instead.
	 */
	public void setCode(byte[] code) {
		this.code = code;
	}
	
	/**
	   @return the code of the class of the behaviour to load.
	 */
	public byte[] getCode() {
		return code;
	}
	
	/**
	   Sets the code of the behaviour to load as the content of a zip
	   file.
	 */
	public void setZip(byte[] zip) {
		this.zip = zip;
	}
	
	/**
	   @return the code of the behaviour to load as the content of a zip
	   file.
	 */
	public byte[] getZip() {
		return zip;
	}
	
	/**
	   Set the list of parameters to be passed to the behaviour.
	   These parameters will be inserted into the behaviour 
	   <code>DataStore</code>
	 */
	public void setParameters(List parameters) {
		this.parameters = parameters;
	}
	
	/**
	   @return the list of parameters to be passed to the behaviour.
	 */
	public List getParameters() {
		return parameters;
	}
}