/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

import rca.core.*;
import rca.content.*;

/**

  This class represents the <code>mobile-agent-description</code>
  concept.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class MobileAgentDescription implements Concept {

    private AID name;
    private Location destination;
    private MobileAgentProfile agentProfile;
    private String agentVersion;
    private String signature;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public MobileAgentDescription() {
    }

    /**
       Set the <code>name</code> slot of this action.
       @param id The agent identifier of the described agent.
    */
    public void setName(AID id) {
      name = id;
    }

    /**
       Retrieve the value of the <code>name</code> slot of this event,
       containing the agent identifier of the described agent.
       @return The value of the <code>name</code> slot, or
       <code>null</code> if no value was set.
    */
    public AID getName() {
      return name;
    }

    /**
       Set the <code>destination</code> slot of this action.
       @param d The destination of the mobility operation performed by
       the described agent.
    */
    public void setDestination(Location d) {
      destination = d;
    }

    /**
       Retrieve the value of the <code>destination</code> slot of this
       event, containing the destination of the mobility operation
       performed by the described agent.
       @return The value of the <code>destination</code> slot, or
       <code>null</code> if no value was set.
    */
    public Location getDestination() {
      return destination;
    }

    /**
       Set the <code>agent-profile</code> slot of this action.
       @param ap The profile for the described agent.
    */
    public void setAgentProfile(MobileAgentProfile ap) {
      agentProfile = ap;
    }

    /**
       Retrieve the value of the <code>agent-profile</code> slot of
       this event, containing the profile for the descrbed agent.
       @return The value of the <code>agent-profile</code> slot, or
       <code>null</code> if no value was set.
    */
    public MobileAgentProfile getAgentProfile() {
      return agentProfile;
    }

    /**
       Set the <code>agent-version</code> slot of this action.
       @param v The version string for the described agent.
    */
    public void setAgentVersion(String v) {
      agentVersion = v;
    }

    /**
       Retrieve the value of the <code>agent-version</code> slot of
       this event, containing the version string for the described
       agent.
       @return The value of the <code>agent-version</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getAgentVersion() {
      return agentVersion;
    }

    /**
       Set the <code>signature</code> slot of this action.
       @param nn The signature string for the described agent.
    */
    public void setSignature(String s) {
      signature = s; 
    }

    /**
       Retrieve the value of the <code>signature</code> slot of this
       event, containing the signature string for the described agent.
       @return The value of the <code>signature</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getSignature() {
      return signature;
    }

}
