/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

import rca.content.*;

/**

  This class represents the <code>mobile-agent-os</code> concept.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class MobileAgentOS implements Concept {
    private String name;
    private Long majorVersion;
    private Long minorVersion;
    private String dependencies;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public MobileAgentOS() {
    }

    /**
       Set the <code>name</code> slot of this action.
       @param n The OS of the described agent.
    */
    public void setName(String n) {
      name = n;
    }

    /**
       Retrieve the value of the <code>name</code> slot of this
       action, containing the OS of the described agent.
       @return The value of the <code>name</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getName() {
      return name;
    }

    /**
       Set the <code>major-version</code> slot of this action.
       @param v The major version number of the OS of the described
       agent.
    */
    public void setMajorVersion(Long v) {
      majorVersion = v;
    }

    /**
       Retrieve the value of the <code>major-version</code> slot of
       this event, containing the major version number of the OS of
       the described agent.
       @return The value of the <code>major-version</code> slot, or
       <code>null</code> if no value was set.
    */
    public Long getMajorVersion() {
      return majorVersion;
    }

    /**
       Set the <code>minor-version</code> slot of this action.
       @param v The minor version number of the OS of the described
       agent.
    */
    public void setMinorVersion(Long v) {
      minorVersion = v;
    }

    /**
       Retrieve the value of the <code>minor-version</code> slot of
       this event, containing the minor version number of the OS of
       the described agent.
       @return The value of the <code>minor-version</code> slot, or
       <code>null</code> if no value was set.
    */
    public Long getMinorVersion() {
      return minorVersion;
    }

    /**
       Set the <code>dependencies</code> slot of this action.
       @param d The OS dependencies of the described agent.
    */
    public void setDependencies(String d) {
      dependencies = d;
    }

    /**
       Retrieve the value of the <code>dependencies</code> slot of
       this event, containing the OS dependencies of the described
       agent.
       @return The value of the <code>dependencies</code> slot, or
       <code>null</code> if no value was set.
    */
    public String getDependencies() {
      return dependencies;
    }

  } 
