/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

import rca.content.*;

/**

  This class represents the <code>mobile-agent-profile</code> concept.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class MobileAgentProfile implements Concept {

    private MobileAgentSystem system;
    private MobileAgentLanguage language;
    private MobileAgentOS os;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public MobileAgentProfile() {
    }

    /**
       Set the <code>system</code> slot of this action.
       @param s The runtime system of the described agent.
    */
    public void setSystem(MobileAgentSystem s) {
      system = s;
    }

    /**
       Retrieve the value of the <code>system</code> slot of this
       action, containing the runtime system of the described agent.
       @return The value of the <code>system</code> slot, or
       <code>null</code> if no value was set.
    */
    public MobileAgentSystem getSystem() {
      return system;
    }

    /**
       Set the <code>language</code> slot of this action.
       @param l The language of the described agent.
    */
    public void setLanguage(MobileAgentLanguage l) {
      language = l;
    }

    /**
       Retrieve the value of the <code>language</code> slot of this
       action, containing the language of the described agent.
       @return The value of the <code>language</code> slot, or
       <code>null</code> if no value was set.
    */
    public MobileAgentLanguage getLanguage() {
      return language;
    }

    /**
       Set the <code>os</code> slot of this action.
       @param o The OS of the described agent.
    */
    public void setOS(MobileAgentOS o) {
      os = o;
    }

    /**
       Retrieve the value of the <code>os</code> slot of this action,
       containing the OS of the described agent.
       @return The value of the <code>os</code> slot, or
       <code>null</code> if no value was set.
    */
    public MobileAgentOS getOS() {
      return os;
    }

  }
