/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.domain.mobility;

import rca.content.onto.*;
import rca.content.schema.*;
import rca.domain.JADEAgentManagement.*;

/**
Original author Giovanni Rimassa - Universita` di Parma
@version $Date: 2003-03-10 17:29:48 +0100 (Mon, 10 Mar 2003) $ $Revision: 3777 $
*/

/**
   This class represents the ontology used for rca mobility. There is
   only a single instance of this class.
   @see rca.domain.mobility.MobilityOntology#getInstance()
 */
public class MobilityOntology extends Ontology implements MobilityVocabulary {

  public static final String NAME = "rca-mobility-ontology";
  
  private static Ontology theInstance = new MobilityOntology();

  public static Ontology getInstance() {
    return theInstance;
  }

  private MobilityOntology() {
  
  	super(NAME, JADEManagementOntology.getInstance());
  
  	try{
			// Adds the roles of the basic ontology (ACTION, AID,...)
	add(new ConceptSchema(MOBILE_AGENT_DESCRIPTION), MobileAgentDescription.class);
	add(new ConceptSchema(MOBILE_AGENT_PROFILE), MobileAgentProfile.class);
	add(new ConceptSchema(MOBILE_AGENT_SYSTEM), MobileAgentSystem.class);
	add(new ConceptSchema(MOBILE_AGENT_LANGUAGE), MobileAgentLanguage.class);
	add(new ConceptSchema(MOBILE_AGENT_OS), MobileAgentOS.class);

	
	add(new AgentActionSchema(CLONE), CloneAction.class);
	add(new AgentActionSchema(MOVE), MoveAction.class);
	
	ConceptSchema cs = (ConceptSchema)getSchema(MOBILE_AGENT_DESCRIPTION);
	cs.add(MOBILE_AGENT_DESCRIPTION_NAME, (ConceptSchema)getSchema(BasicOntology.AID));
	cs.add(MOBILE_AGENT_DESCRIPTION_DESTINATION, (ConceptSchema)getSchema(LOCATION));
	cs.add(MOBILE_AGENT_DESCRIPTION_AGENT_PROFILE, (ConceptSchema)getSchema(MOBILE_AGENT_PROFILE), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_DESCRIPTION_AGENT_VERSION, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_DESCRIPTION_SIGNATURE, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);
	
	cs = (ConceptSchema)getSchema(MOBILE_AGENT_PROFILE);
	cs.add(MOBILE_AGENT_PROFILE_SYSTEM, (ConceptSchema)getSchema(MOBILE_AGENT_SYSTEM), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_PROFILE_LANGUAGE, (ConceptSchema)getSchema(MOBILE_AGENT_LANGUAGE), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_PROFILE_OS, (ConceptSchema)getSchema(MOBILE_AGENT_OS));
	
	cs = (ConceptSchema)getSchema(MOBILE_AGENT_SYSTEM);
	cs.add(MOBILE_AGENT_SYSTEM_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
	cs.add(MOBILE_AGENT_SYSTEM_MAJOR_VERSION, (PrimitiveSchema)getSchema(BasicOntology.INTEGER));
	cs.add(MOBILE_AGENT_SYSTEM_MINOR_VERSION, (PrimitiveSchema)getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_SYSTEM_DEPENDENCIES, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

	cs = (ConceptSchema)getSchema(MOBILE_AGENT_LANGUAGE);
	cs.add(MOBILE_AGENT_LANGUAGE_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
	cs.add(MOBILE_AGENT_LANGUAGE_MAJOR_VERSION, (PrimitiveSchema)getSchema(BasicOntology.INTEGER));
	cs.add(MOBILE_AGENT_LANGUAGE_MINOR_VERSION, (PrimitiveSchema)getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_LANGUAGE_DEPENDENCIES, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

	cs = (ConceptSchema)getSchema(MOBILE_AGENT_OS);
	cs.add(MOBILE_AGENT_OS_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
	cs.add(MOBILE_AGENT_OS_MAJOR_VERSION, (PrimitiveSchema)getSchema(BasicOntology.INTEGER));
	cs.add(MOBILE_AGENT_OS_MINOR_VERSION, (PrimitiveSchema)getSchema(BasicOntology.INTEGER), ObjectSchema.OPTIONAL);
	cs.add(MOBILE_AGENT_OS_DEPENDENCIES, (PrimitiveSchema)getSchema(BasicOntology.STRING), ObjectSchema.OPTIONAL);

	AgentActionSchema as = (AgentActionSchema)getSchema(MOVE);
	as.add(MOVE_MOBILE_AGENT_DESCRIPTION, (ConceptSchema)getSchema(MOBILE_AGENT_DESCRIPTION));
	
	as = (AgentActionSchema)getSchema(CLONE);
	as.addSuperSchema((AgentActionSchema)getSchema(MOVE));
	//as.add(CLONE_MOBILE_AGENT_DESCRIPTION, (ConceptSchema)getSchema(MOBILE_AGENT_DESCRIPTION));
	as.add(CLONE_NEW_NAME, (PrimitiveSchema)getSchema(BasicOntology.STRING));
	
	

    }
    catch(OntologyException oe) {
      oe.printStackTrace();
    }
  }

}
