/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

import rca.content.*;

/**

  This class represents the <code>move-agent</code> action, requesting
  to move an agent within the platform.

  Original author Giovanni Rimassa -  Universita' di Parma
  @version $Date: 2003-11-19 17:04:37 +0100 (Wed, 19 Nov 2003) $ $Revision: 4567 $
*/
public class MoveAction implements AgentAction {

    private MobileAgentDescription agentToMove;


    /**
       Default constructor. A default constructor is necessary for
       ontological classes.
    */
    public MoveAction() {
    }

    /**
       Set the <code>mobile-agent-description</code> slot of this
       action.
       @param desc The description of the agent to migrate.
    */
    public void setMobileAgentDescription(MobileAgentDescription desc) {
      agentToMove = desc;
    }

    /**
       Retrieve the value of the <code>mobile-agent-description</code>
       slot of this event, containing the description of the agent to
       migrate.
       @return The value of the <code>mobile-agent-description</code>
       slot, or <code>null</code> if no value was set.
    */
    public MobileAgentDescription getMobileAgentDescription() {
      return agentToMove;
    }

}
