/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.domain.mobility;

//#MIDP_EXCLUDE_FILE

import rca.content.Concept;

/**
   This concept represents a parameter to be passed to a <code>Behaviour</code>
   in the dynamic loading procedure.
   @see LoadBehaviour
   @see rca.core.behaviours.LoaderBehaviour
   Original author Giovanni Caire - TILAB
 */
public class Parameter implements Concept {
  // Parameter mode values
  public static final int IN_MODE = 0;
  public static final int OUT_MODE = 1;
  public static final int INOUT_MODE = 2;

    private String name;
    private Object value;
    private int mode = IN_MODE;

    public Parameter() {
    }

    public Parameter(String name, Object value) {
        this.name = name;
        this.value = value;
    }

    public Parameter(String name, Object value, int mode) {
        this.name = name;
        this.value = value;
        this.mode = mode;
    }

    /**
       Sets the name of this parameter. This will be used as
       the key in the dinamically loaded behaviour <code>DataStore</code>
       for the parameter value
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
       @return the name of this parameter.
     */
    public String getName() {
        return name;
    }

    /**
       Sets the value of this parameter. The BehaviourLoading
       ontology extends the SerializableOntology and therefore
       whatever <code>Serializable</code> object can be used.
     */
    public void setValue(Object value) {
        this.value = value;
    }

    /**
       @return the value of this parameter.
     */
    public Object getValue() {
        return value;
    }

    /**
       Sets the IO mode of this parameter.
       This must be one of <code>IN_MODE</code>, <code>OUT_MODE</code>,
       <code>INOUT_MODE</code>.
     */
    public void setMode(int mode) {
        this.mode = mode;
    }

    /**
       @return the IO mode of this parameter.
     */
    public int getMode() {
        return mode;
    }
}