/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.gui;

//#J2ME_EXCLUDE_FILE

// Import required Java classes 
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

// Import required rca classes
import rca.domain.FIPAAgentManagement.APService;

/**
To show an <code>APService</code> object within a Swing dialog.
Original author Tiziana Trucco - CSELT S.p.A.
@version $Date: 2005-04-15 17:45:02 +0200 (Fri, 15 Apr 2005) $ $Revision: 5669 $
*/

@SuppressWarnings({"deprecation"})
public class APServicePanel extends JPanel
{
    /** @serial*/

    JTextField name_Field;
    /** @serial*/
    JTextField type_Field;

    VisualStringList address_List;

    // CONSTRUCTORS
    /**
    Create a panel to show an APService.
    */
    APServicePanel(Dialog parent)
    {
        super();

         GridBagLayout gridBag = new GridBagLayout();
        GridBagConstraints  c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.SOUTH;
        setLayout(gridBag);

        JLabel label = new JLabel("Name: ");

      c.gridx = 0;
        c.gridy = 0;
      c.gridwidth = 1;
        gridBag.setConstraints(label,c);
        add(label);

        label = new JLabel("Type: ");
        c.ipady = 0;
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        gridBag.setConstraints(label,c);
        add(label);


    name_Field = new JTextField();
    name_Field.setEditable(false);
        name_Field.setBackground(java.awt.Color.white);
        c.gridx = 1;
        c.gridy = 0;
    gridBag.setConstraints(name_Field,c);
        add(name_Field);

        type_Field = new JTextField();
        type_Field.setEditable(false);
        type_Field.setBackground(java.awt.Color.white);
        c.gridx = 1;
        c.gridy = 1;
    gridBag.setConstraints(type_Field,c);
    add(type_Field);

    JPanel addressPanel = new JPanel();
    addressPanel.setLayout(new BoxLayout(addressPanel,BoxLayout.Y_AXIS));
    addressPanel.setBorder(BorderFactory.createTitledBorder("Addresses"));

    java.util.ArrayList a = new java.util.ArrayList();
    address_List = new VisualStringList(a.iterator(),parent);
      address_List.setEnabled(false); // to sets the popUpMenu to show the three choices Add Edit Remove
      address_List.setDimension(new Dimension(200,50));

      addressPanel.add(address_List);

      c.gridx = 0;
      c.gridy = 2;
      c.gridwidth = 2;

      gridBag.setConstraints(addressPanel,c);
      add(addressPanel);


    }

    /**
    Set the field of the gui.
    */
    private void setAPService(APService ap){

        if(ap != null)
        {
            name_Field.setText(ap.getName());
            type_Field.setText(ap.getType());
            //#DOTNET_EXCLUDE_BEGIN
            address_List.resetContent(ap.getAllAddresses());
            //#DOTNET_EXCLUDE_END
        }
    }

    /**
    To show an <code>APService</code> in A JDialog.
    */
    public static void viewAPServiceDialog(APService ap,Dialog parent,String title){

        final JDialog tempDlg = new JDialog(parent, title, true);

        APServicePanel MTP_Panel = new APServicePanel(parent);
        MTP_Panel.setAPService(ap);

        JButton okButton = new JButton("OK");
        JPanel buttonPanel = new JPanel();
        // Use default (FlowLayout) layout manager to dispose the OK button
        buttonPanel.add(okButton);

        tempDlg.getContentPane().setLayout(new BorderLayout());
        tempDlg.getContentPane().add("Center", MTP_Panel);
        tempDlg.getContentPane().add("South", buttonPanel);

        okButton.addActionListener(new ActionListener()
                                       {
                                            public void actionPerformed(ActionEvent e)
                                            {
                                                tempDlg.dispose();
                                            }
                                       } );

        tempDlg.pack();
        tempDlg.setResizable(false);
        if (parent != null) {
          int locx = parent.getX() + (parent.getWidth() - tempDlg.getWidth()) / 2;
          if (locx < 0)
            locx = 0;
          int locy = parent.getY() + (parent.getHeight() - tempDlg.getHeight()) / 2;
          if (locy < 0)
            locy = 0;
          tempDlg.setLocation(locx,locy);
        }
        tempDlg.show();

    }




}
