/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.gui;

//#APIDOC_EXCLUDE_FILE
//#J2ME_EXCLUDE_FILE

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JPopupMenu;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * This is a mouse listener attached to an AgentTree and is in charge of displaying
 * the proper popup menu when the user right-click on an item in the AgentTree.
 * 
 * Original author Giovanni Caire - TILAB
 */
class AgentTreePopupManager extends MouseAdapter {
	AgentTree agentTree;

	public AgentTreePopupManager(AgentTree agentTree) {
		this.agentTree = agentTree;
	}

	public void mouseReleased(MouseEvent e) {
		handleMouseEvent(e);
	}

	public void mousePressed(MouseEvent e) {
		handleMouseEvent(e);
	}

	private void handleMouseEvent(MouseEvent e) {
		if (e.isPopupTrigger()) {
			JPopupMenu popup = getPopup(e);
			if (popup != null) {
				popup.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}
	
	private JPopupMenu getPopup(MouseEvent e) {
		AgentTree.Node current;
		String typeNode;
		int selRow = agentTree.tree.getRowForLocation(e.getX(), e.getY());
		TreePath selPath = agentTree.tree.getPathForLocation(e.getX(), e.getY());

		if (selRow != -1) {
			TreePath[] paths = agentTree.tree.getSelectionPaths();
			current = (AgentTree.Node) selPath.getLastPathComponent();
			typeNode = current.getType();
			TreeSelectionModel model = agentTree.tree.getSelectionModel();
			if (!agentTree.tree.isRowSelected(selRow))
				model.setSelectionPath(selPath);
			else {
				model.setSelectionPaths(paths);
				//sameTypeNode(typeNode, paths, model);
			}
			return agentTree.getPopupMenu(typeNode);
		} 
		else {
			return null;
		}
	}

	/*private void sameTypeNode(String typeNode, TreePath[] paths, TreeSelectionModel model) {
		AgentTree.Node current;

		for (int i = 0; i < paths.length; i++) {
			current = (AgentTree.Node) paths[i].getLastPathComponent();
			if (!typeNode.equals(current.getType()))
				model.removeSelectionPath(paths[i]);
		}
	}*/
}