/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.gui;

// Import required rca classes
import rca.util.leap.List;
import rca.util.leap.ArrayList;
import rca.util.leap.Iterator;

/**
This class defines the object type <code>GuiEvent</code> used to notify 
an event to a GuiAgent. It has two mandatory attributes:the source of the event 
and an integer identifying the type of event and an optional list of parameters 
than can be added to the event object.The type of each parameter must extends
<code>java.lang.Object</code>; therefore primitive object (e.g.int) should be wrapped
into appropriate objects(e.g <code>java.lang.Integer</code>).

@see rca.gui.GuiAgent
Original author Giovanni Caire - CSELT S.p.A.
@version $Date: 2003-11-20 11:55:37 +0100 (Thu, 20 Nov 2003) $ $Revision: 4572 $
*/
public class GuiEvent 
{
    //#APIDOC_EXCLUDE_BEGIN
    protected Object source;
    protected int type;
    //#APIDOC_EXCLUDE_END

  private List parameters;

    /**
       Create a GUI event.
       @param eventSource The logical source of this event.
       @param eventType An integer value, identifying the kind of this
       event.
    */ 
  public GuiEvent(Object eventSource, int eventType)
  {
    source = eventSource;
    type = eventType;	
    parameters = new ArrayList();
  }

    /**
       Retrieve the kind of this GUI event.
       @return The kind of this event.
    */
    public int getType()
    {
	return(type);
    }

    /**
       Retrieve the logical source of this GUI event.
       @return The event source, or <code>null</code> if no source was
       set.
    */
    public Object getSource()
    {
	return(source);
    }

  /** Add a new parameter to this event.
  * @param param is the parameter 
  **/
  public void addParameter(Object param) {
    parameters.add(param);
  }     

  /**
   * Get the parameter in the given position.
   * @return the Object with the parameter.
   **/
  public Object getParameter(int number) {
    return parameters.get(number);
  }

  /**
   * Get an Iterator over all the parameters.
   * @return An iterator, scanning the event parameter list.
   **/
  public Iterator getAllParameter() {
    return parameters.iterator();
  }
}
