/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/


package rca.gui;

//#J2ME_EXCLUDE_FILE

// Import required Java classes 
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This class implements the gui to manipulate a string.

   Original author Giovanni Caire - CSELT S.p.A.
   @version $Date: 2003-11-20 11:55:37 +0100 (Thu, 20 Nov 2003) $ $Revision: 4572 $
*/
public class StringDlg extends JDialog 
{
	String     hint;
	JTextField txtString;
  String out;
  Component parentGUI;
	
  // CONSTRUCTORS
	/*StringDlg(Frame parent, String hint) 
	{
		super(parent);
		this.hint = new String(hint);
		this.out = null;
	}

	StringDlg(Dialog parent, String hint) 
	{
		super(parent);
		this.hint = new String(hint);
		this.out = null;
	}*/
	
    /**
       Create a dialog with the given parent and initial string value.
       @param parent The parent component for this dialog.
       @param hint The initial value for the manipulated string.
    */
    public StringDlg(Component parent,String hint) 
    {
	super();
	parentGUI = parent;
	this.hint = new String(hint);
	this.out = null;
    }
	
    /**
       Allow the user to manipulate a string value through this
       dialog.
       @param value The string value to edit.
       @return A new string, containing the edited value.
    */
    public String editString(String value)
    {
	setTitle("Edit");

	JPanel p = new JPanel();
	//p.setLayout(new GridLayout(2, 1));
	p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
	JLabel l = new JLabel(hint);
	p.add(Box.createRigidArea(new Dimension(0,6)));
	p.add(l);
	txtString = new JTextField();
	txtString.setText(value);
	txtString.setPreferredSize(new Dimension(300, txtString.getPreferredSize().height));
	p.add(txtString);
	p.add(Box.createRigidArea(new Dimension(0,15)));
	getContentPane().add(p, BorderLayout.CENTER);

	p = new JPanel();
	JButton bOK = new JButton("OK");
	JButton bCancel = new JButton("Cancel");
	bOK.setPreferredSize(bCancel.getPreferredSize());
	p.add(bOK);
	p.add(bCancel);
	bOK.addActionListener( new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{    
		    String param = (String) e.getActionCommand();
		    if (param.equals("OK"))
			{
			    String insertedValue = (txtString.getText()).trim();
			    if(insertedValue.length() == 0)
                  		{
				    JOptionPane.showMessageDialog(null,"Must have non-empty fields !","Error Message",JOptionPane.ERROR_MESSAGE);
				    return;
                  		}
			    else 
				{
				    out = insertedValue;
				    dispose();
				}
			}
		} 
	    } );
	bCancel.addActionListener( new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{    
		    String param = (String) e.getActionCommand();
		    if (param.equals("Cancel"))
			dispose();
		    
		} 
	    } );
	getContentPane().add(p, BorderLayout.SOUTH);

	setModal(true);
	setResizable(false);

	ShowCorrect();

	
	return out;
    }

    /**
       Allows the user to view a string value through a read-only
       version of this dialog.
       @param value The string value to display.
    */
    public void viewString(String value)
    {
		
	setTitle("View");

	JPanel p = new JPanel();
	//p.setLayout(new GridLayout(2, 1));
	p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
	JLabel l = new JLabel(hint);
	p.add(Box.createRigidArea(new Dimension(0,6)));
	p.add(l);
	txtString = new JTextField();
	txtString.setEditable(false);
	txtString.setText(value);
	txtString.setPreferredSize(new Dimension(300, txtString.getPreferredSize().height));
	p.add(txtString);
	p.add(Box.createRigidArea(new Dimension(0,15)));
	getContentPane().add(p, BorderLayout.CENTER);

	p = new JPanel();
	JButton bOK = new JButton("OK");
		
	
	p.add(bOK);
		
	bOK.addActionListener( new ActionListener()
	    {
		public void actionPerformed(ActionEvent e)
		{    
		    String param = (String) e.getActionCommand();
		    if (param.equals("OK"))
			{
			    dispose();
			}
		} 
	    } );

	getContentPane().add(p, BorderLayout.SOUTH);

	setModal(true);
	setResizable(false);
	
	ShowCorrect();
	
    }

    private void ShowCorrect() 
    {
	pack();
    
	setLocation(parentGUI.getX() + (parentGUI.getWidth() - getWidth()) / 2, parentGUI.getY() + (parentGUI.getHeight() - getHeight()) / 2);
	setVisible(true);
	toFront();
    }

}
