/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.gui;

//#APIDOC_EXCLUDE_FILE
//#J2ME_EXCLUDE_FILE

import java.awt.Component;
import java.util.Iterator;
import rca.core.AID;
/**
* This class extends the VisualStringList in order to show a list of AID (Agent-Identifer).
* To show these items correctly, have been overridden the methods:
* <code>getElementName</code> to return the name of the agent (shown in the list).
* <code>editElement</code> to show the ADIGui when required.
*
* @see rca.gui.VisualStringList
* @see rca.gui.AIDGui
* Original author Tiziana Trucco - CSELT S.p.A
* @version $Date: 2003-11-20 11:55:37 +0100 (Thu, 20 Nov 2003) $ $Revision: 4572 $
*/
public class VisualAIDList extends VisualStringList
{
    /**
  @serial
  */
  boolean checkSlots;

    VisualAIDList(Iterator content,Component owner)
    {
        super(content,owner);
        checkSlots = true;//the default behaviour is that all the mandatory slots are checked.
    }

    protected String getElementName(Object el)
    {
        return (((AID)el).getName());
    }

    /**
       Allow the user to edit the chosen agent identifier.
       @param el The chosen agent identifier.
       @param isEditable A boolean flag telling whether the user is
       allowed to modify che chosen agent identifier or not.
    */
    protected Object editElement(Object el, boolean isEditable)
    {

        AIDGui gui = new AIDGui(owner);
         return gui.ShowAIDGui((AID)el,isEditable,checkSlots);

    }
    /**
    This method is used to ensure that the mandatory fields would be corrected inserted.
    */
    public void setCheckMandatorySlots(boolean checkMandatorySlots)
    {
        this.checkSlots = checkMandatorySlots;
    }

}
