/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.gui;

//#APIDOC_EXCLUDE_FILE
//#J2ME_EXCLUDE_FILE

import java.awt.Component;
import java.util.Properties;
import java.util.Enumeration;
import java.util.ArrayList;

/**
* This class extends the VisualStringList in order to show a list of user defined property.
* To show these items correctly, have been overridden the methods:
* 
* <code>editElement</code> to show the UserPropertyGui when required.
* <code>removeElements</code> since the superclass only maintain the key values is necessary 
* to update properties.
* @see rca.gui.VisualStringList
* @see rca.gui.UserPropertyGui

Original author Tiziana Trucco - CSELT S.p.A
@version $Date: 2003-11-20 11:55:37 +0100 (Thu, 20 Nov 2003) $ $Revision: 4572 $
*/

public class VisualPropertiesList extends VisualStringList
{
    /**
  @serial
  */
  private Properties userDefinedSlots;

    VisualPropertiesList(Properties content, Component owner)
    {
        super(new ArrayList().iterator(),owner);

        this.userDefinedSlots = content;

        Enumeration e = content.propertyNames();

        ArrayList list = new ArrayList();
        while(e.hasMoreElements())
            list.add(e.nextElement());

        resetContent(list.iterator());
    }



    protected Object editElement(Object el, boolean isEditable)
    {
        SingleProperty p;
    String out = null;

        if (el == null)
            p = new SingleProperty("", "");
        else
            p = new SingleProperty((String)el,userDefinedSlots.getProperty((String)el));
        UserPropertyGui gui = new UserPropertyGui(owner);
        p = gui.ShowProperty(p,isEditable);

        if(p != null)
        {
            out = p.getKey();
            userDefinedSlots.setProperty(out,p.getValue());
         }

        return out;

    }

    // This method have been overridden because in this case
    //is necessary to update the properties
    protected void removeElement(Object el)
    {
        super.removeElement(el);
        userDefinedSlots.remove(el);
    }

    /**
    * This method must be used to retry the actual properties.
    * The method getContent of the super class only returns the keys.
    */

    public Properties getContentProperties()
    {
        return userDefinedSlots;
    }

    /**
    * This method must be used after a resetContent on this list to update the properties.
    */

    public void setContentProperties(Properties p)
    {
        this.userDefinedSlots = p;
    }

}
