/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.imtp.rmi;


import java.rmi.RemoteException;

import rca.core.BaseNode;
import rca.core.HorizontalCommand;
import rca.core.IMTPException;


/**
   This class wraps an RMI endpoint representing the local platform
   node. When this node is sent over the network, the RMI stub is
   transferred, too.

   Original author Giovanni Rimassa - FRAMeTech s.r.l

 */
class NodeAdapter extends BaseNode {

    public NodeAdapter(String name, boolean hasSM, int port, RMIIMTPManager mgr) throws RemoteException {
    super(name, hasSM);
    try {
        adaptee = new NodeRMIImpl(this, port, mgr);
    }
    catch(Exception e) {
        adaptee = new NodeRMIImpl(this, 0, mgr);
    }
    }

    public Object accept(HorizontalCommand cmd) throws IMTPException {
    try {
        return adaptee.accept(cmd);
    }
    catch(RemoteException re) {
        throw new IMTPException("An RMI error occurred", re);
    }
    }

    public NodeRMI getRMIStub() {
    return adaptee;
    }

    public boolean ping(boolean hang) throws IMTPException {
    try {
        return adaptee.ping(hang);
    }
    catch(RemoteException re) {
        throw new IMTPException("RMI exception", re);
    }
    }

    public void exit() throws IMTPException {
    try {
        adaptee.exit();
    }
    catch(RemoteException re) {
        throw new IMTPException("RMI exception", re);
    }
    }

    public void interrupt() throws IMTPException {
    try {
        adaptee.interrupt();
    }
    catch(RemoteException re) {
        throw new IMTPException("RMI exception", re);
    }
    }

    public void platformManagerDead(String deadPmAddress, String notifyingPmAddr) throws IMTPException {
        if (myServiceManager != null) {
            // Local call
            super.platformManagerDead(deadPmAddress, notifyingPmAddr);
        }
        else {
            // Remote call
                try {
                    adaptee.platformManagerDead(deadPmAddress, notifyingPmAddr);
                }
                catch(RemoteException re) {
                    throw new IMTPException("RMI exception", re);
                }
        }
    }

    private NodeRMI adaptee;

}
