/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.imtp.rmi;


import java.rmi.*;

import rca.core.HorizontalCommand;
import rca.core.IMTPException;

/**

  This RMI interface allows to dispatch rca kernel-level commands
  across the network.

  Original author Giovanni Rimassa - FRAMeTech s.r.l.
 */
interface NodeRMI extends Remote {

    /**
       Accepts a command to be processed (from the implementor point
       of view).

       @param cmd The command to be processed.
       @return The return value of the remote operation represented by
       this horizontal command.
       @throws RemoteException If a network problem occurs.
    */
    Object accept(HorizontalCommand cmd) throws RemoteException, IMTPException;

    /**
       Check whether this node is reachable.

       @param hang A boolean flag. When <code>false</code>, the method
       returns immediately; when <code>true</code>, the method blocks
       and will return only when unblocked from the remote end
       (through regular return or exception.
       @return If the node is exiting, <code>true</code> is returned,
       and <code>false</code> otherwise.
       @throws RemoteException If a network problem occurs.
    */
    boolean ping(boolean hang) throws RemoteException;

    /**
       Shut down this node.
       @throws RemoteException If a network problem occurs.
    */
    void exit() throws RemoteException;

    /**
       Release blocked ping() calls on this node.
       @throws RemoteException If a network problem occurs.       
    */
    void interrupt() throws RemoteException;

    void platformManagerDead(String deadPmAddress, String notifyingPmAddr) throws RemoteException, IMTPException;
}
