/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.imtp.rmi;


import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import rca.core.HorizontalCommand;
import rca.core.ServiceException;
import rca.core.IMTPException;


/**
   Original author Giovanni Rimassa - FRAMeTech s.r.l.
 */
class NodeRMIImpl extends UnicastRemoteObject implements NodeRMI {

    public NodeRMIImpl(NodeAdapter impl, int port, RMIIMTPManager mgr) throws RemoteException {
    super(port, mgr.getClientSocketFactory(), mgr.getServerSocketFactory());
    myNode = impl;
    }

    public Object accept(HorizontalCommand cmd) throws RemoteException, IMTPException {

    try {
        if(terminating) {
        throw new IMTPException("Dead node");
        }
        return myNode.serveHorizontalCommand(cmd);
    }
    catch(ServiceException se) {
        throw new IMTPException("Service Error", se);
    }
    }

    public void platformManagerDead(String deadPmAddress, String notifyingPmAddr) throws RemoteException, IMTPException {
        if(terminating) {
                throw new IMTPException("Dead node");
        }
        myNode.platformManagerDead(deadPmAddress, notifyingPmAddr);
    }

    public boolean ping(boolean hang) throws RemoteException {
      if(hang) {
      waitTermination();
      }
      return terminating;
    }

    public void exit() throws RemoteException {
    // Unblock threads hung in ping() method (this will deregister the container)
    terminating = true;
    notifyTermination();
    }

    public void interrupt() throws RemoteException {
    notifyTermination();
    }

    private void waitTermination() {
    synchronized(terminationLock) {
        try {
        terminationLock.wait();
        }
        catch(InterruptedException ie) {
        System.out.println("PING wait interrupted");
        // Do nothing
        }
    }
    }

    private void notifyTermination() {
      synchronized(terminationLock) {
      terminationLock.notifyAll();
      }
    }


    // This monitor is used to hang a remote ping() call in order to
    // detect node failures.
    private Object terminationLock = new Object();
    private boolean terminating = false;

    private NodeAdapter myNode;

}
