/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.imtp.rmi;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import rca.core.IMTPException;
import rca.core.Node;
import rca.core.NodeDescriptor;
import rca.core.Service;
import rca.core.ServiceDescriptor;
import rca.core.ServiceException;
import rca.core.PlatformManager;

import rca.security.JADESecurityException;

import java.util.Vector;

/**
   Original author Giovanni Rimassa - FRAMeTech s. r. l.
 */
class ServiceManagerRMIImpl extends UnicastRemoteObject implements ServiceManagerRMI {
  private PlatformManager impl;


  /** Creates new ServiceManagerRMIImpl */
  public ServiceManagerRMIImpl(PlatformManager pm, RMIIMTPManager mgr, int port) throws RemoteException {
		super(port, mgr.getClientSocketFactory(), mgr.getServerSocketFactory());
		impl = pm;
  }
  
  PlatformManager getPlatformManager() {
  	return impl;
  }

  public String getPlatformName() throws RemoteException {
		try {
		    return impl.getPlatformName();
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }

  public String addNode(NodeDescriptor dsc, Vector nodeServices, boolean propagated) throws RemoteException, ServiceException, JADESecurityException {
		try {
		    return impl.addNode(dsc, nodeServices, propagated);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public void removeNode(NodeDescriptor dsc, boolean propagated) throws RemoteException, ServiceException {
		try {
		    impl.removeNode(dsc, propagated);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public void addSlice(ServiceDescriptor service, NodeDescriptor dsc, boolean propagated)  throws RemoteException, ServiceException {
		try {
		    impl.addSlice(service, dsc, propagated);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public void removeSlice(String serviceKey, String sliceKey, boolean propagated)  throws RemoteException, ServiceException {
		try {
		    impl.removeSlice(serviceKey, sliceKey, propagated);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public void addReplica(String newAddr, boolean propagated)  throws RemoteException, ServiceException {
		try {
		    impl.addReplica(newAddr, propagated);
		}
		catch(IMTPException imtpe) {
		    // Note that addReplica() may activate remote calls --> An IMTPException in this case must be properly handled 
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call. "+imtpe);
		}
  }
  
  public void removeReplica(String address, boolean propagated)  throws RemoteException, ServiceException {
		try {
		    impl.removeReplica(address, propagated);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }

  public Service.Slice findSlice(String serviceKey, String sliceKey) throws RemoteException, ServiceException {
		try {
		    return impl.findSlice(serviceKey, sliceKey);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public Vector findAllSlices(String serviceKey) throws RemoteException, ServiceException {
		try {
		    return impl.findAllSlices(serviceKey);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public void adopt(Node n, Node[] children) throws RemoteException {
		try {
		    impl.adopt(n, children);
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
  
  public void ping() throws RemoteException {
		try {
		    impl.ping();
		}
		catch(IMTPException imtpe) {
		    // It should never happen, since this is a local call
			imtpe.printStackTrace();
		    throw new RemoteException("IMTPException in local call");
		}
  }
}

