/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.mtp;

import rca.domain.FIPAAgentManagement.Envelope;
import rca.core.Profile;

/**
   This interface is the receiver's view of an MTP.
 */
public interface InChannel {

  /**
     Callback interface to be notified of message arrivals over this
     Message Transport Protocol.
   */
  public static interface Dispatcher {
    void dispatchMessage(Envelope env, byte[] payload);
  }

  /**
     Activates an MTP handler for incoming messages on a default
     address.
		 @parameter p is the Profile from which the configuration parameters
		 for this instance of rca container can be retrieved
     @return A <code>TransportAddress</code>, corresponding to the
     chosen default address.
     @exception MTPException Thrown if some MTP initialization error
     occurs.
  */
  TransportAddress activate(Dispatcher disp, Profile p) throws MTPException;

  /**
     Activates an MTP handler for incoming messages on a specific
     address.
     @param ta A <code>TransportAddress</code> object, representing
     the transport address to listen to.
		 @parameter p is the Profile from which the configuration parameters
		 for this instance of rca container can be retrieved
     @exception MTPException Thrown if some MTP initialization error
     occurs.
   */
  void activate(Dispatcher disp, TransportAddress ta, Profile p) throws MTPException;

  /**
     Deactivates the MTP handler listening at a given transport
     address.
     @param ta The <code>TransportAddress</code> object the handle to
     close is listening to.
     @exception MTPException Thrown if some MTP cleanup error occurs.
   */
  void deactivate(TransportAddress ta) throws MTPException;

  /**
     Deactivates all the MTP handlers.
     @exception MTPException Thrown if some MTP cleanup error occurs.
   */
  void deactivate() throws MTPException;

}
