/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.mtp;


/**
   Abstract interface for Message Transport Protocols

   Original author Giovanni Rimassa - Universita` di Parma
   @version $Date: 2001-10-05 15:36:54 +0200 (Fri, 05 Oct 2001) $ $Revision: 2757 $

*/
public interface MTP extends InChannel, OutChannel {

  /**
     Converts a string representing a valid address in this MTP to a
     <code>TransportAddress</code> object.
     @param rep The string representation of the address.
     @return A <code>TransportAddress</code> object, created from the
     given string.
     @exception MTPException If the given string is not a valid
     address according to this MTP.
   */
  TransportAddress strToAddr(String rep) throws MTPException;

  /**
     Converts a <code>TransportAddress</code> object into a string
     representation.
     @param ta The <code>TransportAddress</code> object.
     @return A string representing the given address.
     @exception MTPException If the given
     <code>TransportAddress</code> is not a valid address for this
     MTP.
   */
  String addrToStr(TransportAddress ta) throws MTPException;

  /**
     Reads the name of the message transport protocol managed by this
     MTP. The FIPA standard message transport protocols have a name
     starting with <code><b>"fipa.mts.mtp"</b></code>.
     @return A string, that is the name of this MTP.
   */
  String getName();

  String[] getSupportedProtocols();

}
