/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.mtp;

import rca.core.CaseInsensitiveString;
import rca.util.leap.Serializable;

public class MTPDescriptor implements Serializable {
  private String name;
  private String className;
  private String[] addresses;
  private String[] protoNames;

  public MTPDescriptor(String n, String cn, String[] a, String[] pn) {
    name = n;
    className = cn;
    addresses = a;
    protoNames = pn;
  }

  public String getName() {
    return name;
  }

  public String getClassName() {
      return className;
  }

  public String[] getAddresses() {
    return addresses;
  }

  public String[] getSupportedProtocols() {
    return protoNames;
  }
  
  public boolean equals(Object obj) {
  	try {
  		MTPDescriptor toBeCompared = (MTPDescriptor) obj;
  		if (!CaseInsensitiveString.equalsIgnoreCase(name, toBeCompared.getName())) {
  			return false;
  		}
  		String[] comparedAddresses = toBeCompared.getAddresses();
  		if (addresses == null && comparedAddresses == null) {
  			// No need to compare addresses
  			return true;
  		}
  		if (addresses.length != comparedAddresses.length) {
  			return false;
  		}
  		for (int i = 0; i < addresses.length; ++i) {
  			int j;
  			for (j = 0; j < comparedAddresses.length; ++j) {
		  		if (CaseInsensitiveString.equalsIgnoreCase(addresses[i], comparedAddresses[j])) {
		  			break;
		  		}
  			}
  			if (j >= comparedAddresses.length) {
  				// Address not found
  				return false;
  			}
  		}
  		return true;
  	}
  	catch (Exception e) {
  		return false;
  	}
  }


    //#MIDP_EXCLUDE_BEGIN

    // For persistence service
    private MTPDescriptor() {
    }

    // For persistence service
    private void setName(String n) {
	name = n;
    }

    // For persistence service
    private void setClassName(String cn) {
	className = cn;
    }

    // For persistence service
    private void setAddresses(String[] a) {
	addresses = a;
    }

    // For persistence service
    private void setSupportedProtocols(String[] sp) {
	protoNames = sp;
    }


    // For persistence service
    private Long persistentID;

    // For persistence service
    private Long getPersistentID() {
	return persistentID;
    }

    // For persistence service
    private void setPersistentID(Long l) {
	persistentID = l;
    }


    //#MIDP_EXCLUDE_END

}



