/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.mtp.http;

import rca.mtp.TransportAddress;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.InetAddress;

public class HTTPAddress implements TransportAddress {
  
  private URL url;
  
  HTTPAddress(URL addr) throws MalformedURLException {
    url = addr;
  }  
  
  HTTPAddress(String addr) throws MalformedURLException {
    url = new URL(addr);
  }

  /*#DOTNET_INCLUDE_BEGIN
  HTTPAddress(String addr, int port) throws MalformedURLException
  {
 	this(addr, port, false);
  }
  #DOTNET_INCLUDE_END*/
  
  HTTPAddress(String addr, int port, boolean https) 
    throws MalformedURLException {
        if(https){
        url = new URL("https",addr,port,"/acc");
      }else{
        url = new URL("http",addr,port,"/acc");
      }
  }
  
  /** Check if HTTP addresses are equivalent (same port, same host, same proto) */
  public boolean equals(HTTPAddress a) {
    try {
      if ((getPort().equals(a.getPort())) &&
          (InetAddress.getByName(url.getHost()).equals(InetAddress.getByName(a.getHost()))) &&
          (getProto().equals(a.getProto()))) {
        return true;
      }
    }
    catch(Exception e) {
      // Probably unknown host exception
    }
    return false;
  }
  
  /** Get the value of protocol */    
  public String getProto() {
    return url.getProtocol();
  }
  
  /** Get the value of host */
  public String getHost() {
    return url.getHost();
  }
   
  /** Get the value of port */
  public String getPort() {
    return Integer.toString(url.getPort());
  }

  /** Get the value of port */
  public int getPortNo() {
    return url.getPort();
  }
   
  /** Get the value of file */
  public String getFile() {
    return url.getFile();
  }
   
  /** Get the value of anchor */
  public String getAnchor() {
    return url.getRef();
  }   
  
  /** convert to String */
  public String toString() {
    return url.toString();
  }
  
} //End of HTTPAddress class
