/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.mtp.http;

//#PJAVA_EXCLUDE_FILE

import rca.core.Profile;
import rca.mtp.MTPException;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;

import javax.net.ServerSocketFactory;
import javax.net.SocketFactory;

//#DOTNET_EXCLUDE_BEGIN
import javax.net.ssl.KeyManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.TrustManager;

import rca.mtp.http.https.*;
//#DOTNET_EXCLUDE_END

/**
 * Singleton class for obtaining sockets. HTTP MTP calls methods use this class
 * every time that a socket is needed. 
 * 
 * Original author <a href="mailto:Joan.Ametller@uab.es">Joan Ametller Esquerra</a>
 * 
 */
public class HTTPSocketFactory {

  public static HTTPSocketFactory getInstance() {
    if (_instance == null)
      _instance = new HTTPSocketFactory();
    return _instance;
  }

	public void configure(Profile profile, HTTPAddress hta) throws Exception {
		//#DOTNET_EXCLUDE_BEGIN
		if (hta.getProto().equals("https")) {
			_usingHttps = true;
			try {
				String trustManagerClass =
					profile.getParameter(
					PREFIX + "trustManagerClass",
					"rca.mtp.http.https.NoAuthentication");

				String keyManagerClass =
					profile.getParameter(
					PREFIX + "keyManagerClass",
					"rca.mtp.http.https.KeyStoreKeyManager");

				HTTPSTrustManager tm =
					(HTTPSTrustManager)Class.forName(trustManagerClass).newInstance();
				tm.init(profile);

				HTTPSKeyManager km =
					(HTTPSKeyManager)Class.forName(keyManagerClass).newInstance();
				km.init(profile);

				if (profile
					.getParameter(PREFIX + "needClientAuth", "no")
					.equals("yes"))
					_needClientAuth = true;

				SSLContext sctx = SSLContext.getInstance("TLS");
				sctx.init(new KeyManager[] { km }, new TrustManager[] { tm }, null);

				_socketFactory = sctx.getSocketFactory();
				_serverSocketFactory = sctx.getServerSocketFactory();
      } catch (Exception e) {
				throw new MTPException("Error initializing secure conection", e);
			}
    } else {
			_socketFactory = SocketFactory.getDefault();
			_serverSocketFactory = ServerSocketFactory.getDefault();
		}
		//#DOTNET_EXCLUDE_END

		/*#DOTNET_INCLUDE_BEGIN
		 _socketFactory = SocketFactory.getDefault();
		 _serverSocketFactory = ServerSocketFactory.getDefault();
		 #DOTNET_INCLUDE_END*/
	}

  public Socket createSocket(String host, int port) throws IOException {
    return _socketFactory.createSocket(host, port);
  }

  public Socket createSocket(
    String host,
    int port,
    InetAddress dest,
    int outport)
    throws IOException {
    return _socketFactory.createSocket(host, port, dest, outport);
  }

  public ServerSocket createServerSocket(int port) throws IOException {
    ServerSocket ss = _serverSocketFactory.createServerSocket(port);
    //#DOTNET_EXCLUDE_BEGIN
	if (_usingHttps)
       ((SSLServerSocket)ss).setNeedClientAuth(_needClientAuth);
	//#DOTNET_EXCLUDE_END
    return ss;
  }

  private HTTPSocketFactory() {
  }

  private static HTTPSocketFactory _instance;
  private static final String PREFIX = "jade_mtp_http_https_";
  private SocketFactory _socketFactory;
  private ServerSocketFactory _serverSocketFactory;
  private boolean _needClientAuth = false;
  private boolean _usingHttps = false;
}
