/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
 
package rca.mtp.http.https;

//#PJAVA_EXCLUDE_FILE


import rca.core.Profile;

import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;

import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;

/**
 * This class implements a concrete behaviour for HTTPS Trust decisions. If 
 * we configure the platform to use this TrustManager, we must also specify
 * two configuration properties:
 * 
 * <b>jade_mtp_http_https_firendListFile</b>: This property must indicate
 * the location of a java KeyStore in the disc.
 * 
 * <b>jade_mtp_http_https_friendListFilePass</b> Password for this KeyStore.
 * 
 * The KeyStore must contain a list of Trusted Certificates. When a new HTTPS
 * connection is stablished, trust decisions will be made in terms of
 * the certificates inside the KeyStore. If the Certificate sent by the remote
 * peer is inside of the KeyStore or signed by one of the issuers of the 
 * inside the KeyStore, the connection will be stablished. Otherwise the 
 * connection will not be possible.
 * 
 * Original author <a href="mailto:Joan.Ametller@uab.es">Joan Ametller Esquerra</a>
 * 
 */
public class FriendListAuthentication implements HTTPSTrustManager {

  public X509Certificate[] getAcceptedIssuers() {
    return _tm.getAcceptedIssuers();
  }

  public void checkClientTrusted(X509Certificate[] cert, String authType)
    throws CertificateException {  
    _tm.checkClientTrusted(cert, authType);   
  }

  public void checkServerTrusted(X509Certificate[] cert, String authType)
    throws CertificateException {
    _tm.checkServerTrusted(cert, authType);
  }
  
  public void init(Profile profile) throws Exception{
    KeyStore ks = KeyStore.getInstance("JKS");
    String filename = profile.getParameter("jade_mtp_http_https_friendListFile", "");
    char[] pass = profile.getParameter("jade_mtp_http_https_friendListFilePass", "").toCharArray();
    ks.load(new FileInputStream(filename), null);
    TrustManagerFactory tmf = 
      TrustManagerFactory.getInstance("SunX509");
    tmf.init(ks);
    _tm = (X509TrustManager)tmf.getTrustManagers()[0];
  }
  
  private X509TrustManager _tm;

}
