/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
 
package rca.mtp.http.https;

//#PJAVA_EXCLUDE_FILE

import rca.core.Profile;

import java.net.Socket;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.security.KeyStore;
import java.io.FileInputStream;
/**
 * HTTPSKeyManager that extracts its trust material from a given KeyStore.
 * This key manager needs some configuration parameters:
 * 
 * <b>jade_mtp_http_https_keyStoreFile</b>: The File Containing the KeyStore. This
 * keystore can only contain a key entry (Private Key + X509 Certificate). The
 * password encrypting this key must be the same password of the KeyStore. This
 * key material will be used to authenticate local platform when remote 
 * platforms will attempt to connect with.
 * 
 * <b>jade_mtp_http_https_keyStorePass</b>: The password for the key store 
 * indicated with the parameter described above.
 * 
 * Original author <a href="mailto:Joan.Ametller@uab.es">Joan Ametller Esquerra</a>
 * 
 */
public class KeyStoreKeyManager implements HTTPSKeyManager {

  public void init(Profile profile) throws Exception {
    
    String pass = profile.getParameter(PREFIX+"keyStorePass", "");
    String keyfile = profile.getParameter(PREFIX + "keyStoreFile", "");
    KeyStore ks = KeyStore.getInstance("JKS");
    ks.load(new FileInputStream(keyfile),pass.toCharArray());
    alias = ((String)ks.aliases().nextElement());
    privateKey = (PrivateKey)ks.getKey(alias, pass.toCharArray());
    Certificate[] certs = ks.getCertificateChain(alias);
    cert = new X509Certificate[certs.length];
    for(int i=0;i<certs.length;i++) cert[i] = (X509Certificate)certs[i];
  }

  public PrivateKey getPrivateKey(String arg0) {
    return privateKey;
  }

  public X509Certificate[] getCertificateChain(String arg0) {
    return cert;
  }

  public String[] getClientAliases(String arg0, Principal[] arg1) {
    return new String[]{alias};
  }

  public String[] getServerAliases(String arg0, Principal[] arg1) {
    return getClientAliases(arg0,arg1);
  }

  public String chooseServerAlias(String arg0, Principal[] arg1, Socket arg2) {
    return alias;
  }

  public String chooseClientAlias(
    String[] arg0,
    Principal[] arg1,
    Socket arg2) {
    return alias;
  }
  
  private PrivateKey privateKey;
  private X509Certificate[] cert;
  private String alias;
  private static final String PREFIX     = "jade_mtp_http_https_";
}
