/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.mtp.http.https;

//#PJAVA_EXCLUDE_FILE

import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import rca.core.Profile;
import javax.net.ssl.X509TrustManager;
import javax.net.ssl.TrustManagerFactory;
import java.security.KeyStore;
import java.io.FileInputStream;
import java.io.File;
/**
 * This Trust manager authenticates certificates issued by official Top level
 * Certification Authorities. The certificate list containing Certification Authorities
 * certificates is located at JAVA_HOME/jre/lib/security/cacerts. This file is a java
 * KeyStore file. Certificate list can be listed using the following command:
 * 
 * keytool -list -keystore JAVA_HOME/jre/lib/security/cacerts
 * 
 * where JAVA_HOME is the path where java SDK is installed.
 * 
 * Original author <a href="mailto:Joan.Ametller@uab.es">Joan Ametller Esquerra</a>
 * 
 */
public class StrongAuthentication implements HTTPSTrustManager {

  public X509Certificate[] getAcceptedIssuers() {
    return _tm.getAcceptedIssuers();
  }

  public void checkClientTrusted(X509Certificate[] cert, String authType)
    throws CertificateException {  
    _tm.checkClientTrusted(cert, authType);   
  }

  public void checkServerTrusted(X509Certificate[] cert, String authType)
    throws CertificateException {
    _tm.checkServerTrusted(cert, authType);
  }
  
  public void init(Profile profile) throws Exception{
    KeyStore ks = KeyStore.getInstance("JKS");
    String s = File.separator;
    String javaHome = System.getProperty("java.home");
    String defaultTs = javaHome + s + "lib" + s + "security"+ s +"cacerts";
    ks.load(new FileInputStream(defaultTs), null);
    TrustManagerFactory tmf = 
      TrustManagerFactory.getInstance("SunX509");
    tmf.init(ks);
    _tm = (X509TrustManager)tmf.getTrustManagers()[0];
  }
  
  private X509TrustManager _tm;
}
