/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.proto.states;

//#CUSTOM_EXCLUDE_FILE

import rca.core.*;
import rca.core.behaviours.*;
import rca.lang.acl.*;

import rca.util.leap.Iterator;


/**
 * This behaviour sends a reply to a given message adjusting all 
 * protocol fields and receivers.
 * It reads in DataStore the message and the reply at the keys passed 
 * in the Constructor.  
 * Original author Fabio Bellifemine - TILab
 * Original author Giovanni Caire - TILab
 * Original author Marco Monticone
 * @version $Date: 2005-09-16 15:54:46 +0200 (Fri, 16 Sep 2005) $ $Revision: 5780 $
 **/
public class ReplySender extends OneShotBehaviour {
	
		public static final int NO_REPLY_SENT = -1;
		private int ret;
		private String replyKey, msgKey;

 /**
  * Constructor.
  * @param a The Agent executing this behaviour
  * @param replyKey DataStore's key where to read the reply message
  * @param msgKey DataStore's key where to read the message to reply to.
  * @param ds the dataStore for this bheaviour
 **/ 
public ReplySender(Agent a, String replyKey, String msgKey, DataStore ds) {
		this(a, replyKey, msgKey);
		setDataStore(ds);
}
		
 /**
  * Constructor.
  * @param a The Agent executing this behaviour
  * @param replyKey DataStore's key where to read the reply message
  * @param msgKey DataStore's key where to read the message to reply to.
 **/ 
public ReplySender(Agent a, String replyKey, String msgKey) {
		super(a);
	  this.replyKey = replyKey;
	  this.msgKey = msgKey;	
}
	
public void action(){
	ret=NO_REPLY_SENT;
	DataStore ds = getDataStore();
	ACLMessage reply = (ACLMessage) ds.get(replyKey);
	if (reply != null) {
			ACLMessage msg = (ACLMessage) ds.get(msgKey);
			if (msg != null) {
			  adjustReply(myAgent, reply, msg);
			  myAgent.send(reply);
				ret = reply.getPerformative();
			}		
	}
}

 public int onEnd() {
	 return ret;
 }
 
	public void setMsgKey(String msgKey) {
		this.msgKey = msgKey;
	}
 
	public void setReplyKey(String replyKey) {
	  this.replyKey = replyKey;
	}
 
	/**
     Adjust all protocol fields and receivers in a reply to a given
     message.
 	 */
	public static void adjustReply(Agent myAgent, ACLMessage reply, ACLMessage msg) {
		// Set the conversationId
		reply.setConversationId(msg.getConversationId());
		// Set the inReplyTo
		reply.setInReplyTo(msg.getReplyWith());
		// Set the Protocol.
		reply.setProtocol(msg.getProtocol());
		// Set ReplyWith if not yet set
		if (reply.getReplyWith() == null)
			reply.setReplyWith(myAgent.getName() + java.lang.System.currentTimeMillis()); 
	    
		// Set the receivers if not yet set
		if (!reply.getAllReceiver().hasNext()) {
			boolean no_reply_to = true;
			Iterator it = msg.getAllReplyTo();
			while(it.hasNext()){
				no_reply_to=false;
				reply.addReceiver((AID)it.next());
			}
			if(no_reply_to) {
				reply.addReceiver(msg.getSender());
			}	
		}
	}

    //#APIDOC_EXCLUDE_BEGIN

    // For persistence service
    protected ReplySender() {
    }

    //#APIDOC_EXCLUDE_END

	
}
