/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.proto.states;

//#CUSTOM_EXCLUDE_FILE

import rca.core.behaviours.*;

/**
 * This behaviour is used as a state of a FSMbehavior in order to reset 
 * some of the FSM's states or the whole FSM
 *
 * Original author Giovanni Caire - TILab
 * Original author Marco Monticone
 */
public class StateResetter extends OneShotBehaviour{
  
	String[] sname;
	
  /**
   *  Constructor.
   * @param states  Represent the names of FSM's to reset. 
   * If null the parent Behaviour is resetted
   * 
   */
	public StateResetter(String[] states) {
		sname=states;
	}
	
	/**
   *  Constructor.
   *  equivalent to StateResetter(null)
   */
	public StateResetter(){
		this(null);
	}
	
	public void action() {
		Behaviour st;
		FSMBehaviour p = (FSMBehaviour)parent;
		if(sname == null){
			p.reset();
		}
		else{
			for(int i=0;i<sname.length;i++){
					st=p.getState(sname[i]);
					st.reset();
			}
		}	
	}
}
	
	
	
	

  
	

	