/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.security;

import java.util.Enumeration;
import java.io.IOException;

/**
 * This is a collection of Objects that can be used in order 
 * to get privileges to perform an action or access a service.
 * 
   Original author Giosue Vitaglione - TILAB
   @version $Date: 2004-05-26 09:35:55 +0200 (Wed, 26 May 2004) $ $Revision: 5086 $
 */
public interface Credentials extends rca.util.leap.Serializable {
  
  /**
   *  Used to retrieve all the contained credentials. 
   */
  Enumeration elements();
  
  /** 
   * Look into the various credentials, if there is one that
   * certify the ownership for this credentials, return the JADEPrincipal
   * of the owner. Otherwise return null.
   */
  JADEPrincipal getOwner();
   
  /**
   * Encode the Credentials into a byte array, so that it can be transported in an ACLMessage.
   * The encoding format is:
   * Credential class | Encoding size | Encoding 
   */
  byte[] encode() throws IOException;
  
  /**
   * Returns a new Credentials from the encoded information. 
   */
  Credentials decode(byte[] enc) throws IOException;
  

}
