/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.security;


/**
	The <code>JADEPrincipal</code> interface represents all principals
	acting on the platform. They will be held responsible for their
	own agents and containers.

	Original author Michele Tomaiuolo - Universita` di Parma
	Original author Giosue Vitaglione - Telecom Italia LAB

	@version $Date: 2004-05-10 17:36:04 +0200 (Mon, 10 May 2004) $ $Revision: 5069 $
*/
public interface JADEPrincipal
//#ALL_EXCLUDE_BEGIN
		extends java.security.Principal
//#ALL_EXCLUDE_END
{
  
	/**
   * The name which marks an unidentified principal.
   */
	public static final String NONE = "none";
  
	/**
   * Returns the name of this principal as known to its platform.
   * @return The name.
   */
	public String getName();

	/**
   * Returns the SDSI name of this principal (if any)
   * @return The SDSIName
   */
	public  SDSIName getSDSIName();
  
  
	/**
   *	Checks the hierarchical relationship between two principals, i.e.
   *tells if this principal belongs to group <code>p</code>.
   *If this holds, all permissions granted to <code>p</code> will be
   *automatically granted to this principal, too.
   *@param p The principal to check.
   *@return True if this principal is a member of group <code>p</code>.
   */
  //public boolean implies(JADEPrincipal p);
  
  public byte[] getEncoded();
  public String toString();
  public int hashCode ();
  public boolean equals(Object p);
}
