/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.security;

/**
 * The <code>SDSIName</code> interface represents a wrapper for
 *  public key used in asymmetric cryptographic operations.
 *
 * It is also used as unique identifier for rca principals.
 *
 * Original author Giosue Vitaglione - Telecom Italia LAB
 * @version $Date: 2004-05-10 17:36:04 +0200 (Mon, 10 May 2004) $ $Revision: 5069 $
 */
public interface SDSIName {
	
  /**
   * Returns the standard algorithm name for this key.
   * @return the name of the algorithm associated with this key.
   */
	public String getAlgorithm();

	/**
   * Returns the key in its primary encoding format. 
   * @return The encoded form of the key.
   */
	public byte[] getEncoded();
  
	/**
   * Returns the name of the primary encoding format of this key.
   * @return the primary encoding format of the key.
   */
	public String getFormat();

	/**
   * Returns the sequence of the names local to the Public Key
   * in a SDSI fashion. Last item is always ".".
   *
   * Example:  "." => Simply the Public Key, no local names here involved.
   * Example:  "bob", "."  => The principal defined as 'bob'
   * by the owner of the given public key
   * Example:  "bob", "alice", "."  => The principal defined as 'alice' 
   * by the principal defined by 'bob' by the owner of the given public key
   */
	public String[] getLocalNames();
  
  /**
   * @return the right most local name
   */
  public String getLastLocalName();
   
  /**
   * Returns the java.security.PublicKey object.
   * Only for The J2SE version
   * TODO: with pre-compiler directives
   */
  /*
    public PublicKey getPublicKey();
   */

}
