/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.util;
 
//#J2ME_EXCLUDE_FILE
//#APIDOC_EXCLUDE_FILE

import java.io.*;

/**
   This class is an OutputStream whose output is duplicated and
   forwarded to two different output streams.
   Original author Giovanni Caire - TILab
 */
public class PrintStreamSplitter extends PrintStream {
	private PrintStream s1, s2;
	
    /**
       Create a new stream that feeds the output to the two given
       streams.
       @param s1 The first stream to write to.
       @param s2 The second stream to write to.
       @throws IOException If some stream operation fails.
    */
	public PrintStreamSplitter(PrintStream s1, PrintStream s2) throws IOException {
		super(new OutputStream() {
			public void write(int b) throws IOException {
			}
		} );

		this.s1 = s1;
		this.s2 = s2;
	}

    /**
       Print a Java object to the two streams.
       @param obj The Java object to print.
    */	
	public void print(Object obj) {
		s1.print(obj);
		s2.print(obj);
	}

    /**
       Print a string to the two streams.
       @param str The string to print.
    */	
	public void print(String str) {
		s1.print(str);
		s2.print(str);
	}

    /**
       Print a new line to the two streams.
    */	
	public void println() {
		s1.println();
		s2.println();
	}

    /**
       Print a Java object and a new line to the two streams.
       @param obj The Java object to print.
    */	
	public void println(Object obj) {
		s1.println(obj);
		s2.println(obj);
	}

    /**
       Print a string and a new line to the two streams.
       @param str The string to print.
    */	
	public void println(String str) {
		s1.println(str);
		s2.println(str);
	}
}
