/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.util;

/**

  This class acts as a base class for all the exceptions that wrap
  another (nested) exception. The typical usage for descendents of
  this class is to throw them within the <code>catch</code> block for
  their nested exception.

  Original author Giovanni Rimassa - Universita' di Parma
  @version $Date: 2005-04-29 18:18:18 +0200 (Fri, 29 Apr 2005) $ $Revision: 5686 $

 */
public abstract class WrapperException extends Exception {
  private Throwable nested = null;

    /**
       Create a wrapper exception with the given message and nested
       exception.
       @param m The exception message text.
       @param t The nested exception that caused this one.
    */
    protected WrapperException(String m, Throwable t) {
  	super(m);
	nested = t;
    }

    /**
       Create a wrapper exception with the given message.
       @param m The exception message text.
    */
    protected WrapperException(String m) {
  	super(m);
    }

    /**
       Retrieve the exception message text.
       @return The exception message, including the nested exception
       text if present.
    */
    public String getMessage() {
	if((nested != null)) {
	    return super.getMessage() + " [nested "+nested.toString() + "]";
	}
	return super.getMessage();
    }

    /**
       Prints the stack trace of this exception on the standard output
       stream. If a nested exception is present, its stack trace is
       also printed.
    */
    public void printStackTrace() {
	super.printStackTrace();
	if(nested != null) {
	    System.err.println("Nested Exception:");
	    nested.printStackTrace();
	}
    }

    /**
       Reads the exception wrapped by this object.
       @return the <code>Throwable</code> object that is the exception 
       that was originally thrown.
    */
    public Throwable getNested() {
	return nested;
    }

}
