/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.util.leap;
import java.util.NoSuchElementException;

/**
 * Singleton Iterator for an empty collection. 
 * The usage of the singleton-pattern for this simple object allows just 
 * to save some memory. 
 * @version $Date: 2003-11-20 17:04:51 +0100 (Thu, 20 Nov 2003) $ $Revision: 4573 $
 * Original author Fabio Bellifemine, TILAB
 */
public class EmptyIterator implements Iterator {

		private final static Iterator it = new EmptyIterator();

    /**
     * Constructor declaration
     * 
     */
    private EmptyIterator() {
    } 

		/**
		 * Returns the singleton EmptyIterator object.
		 * Most of the methods of this class are instance methods and must be 
		 * invoked with respect to the singleton object. 
		 **/
		public static Iterator getInstance() {
				return it;
		}

    /**
       Checks whether the iterator can scan further.
       @return This method always returns <code>false</code>.
     */
    public boolean hasNext() {
      return false;
    } 

    /**
       Retrieves the next element in the collection scanned by this
       iterator.
       @return This method always throws an exception.
       @throws NoSuchElementException Always, because the underlying
       collection is always empty.
     */
    public Object next() {
      throw new NoSuchElementException();
    } 

    /**
       Remove the element pointed to by this iterator from the
       collection.
       @throws RuntimeException Always, because the underlying
       collection is always empty.
     */
    public void remove() {
      throw new RuntimeException();
    } 

  }
