/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.util.leap;
import java.util.Enumeration;
import rca.util.Logger;

/**
 * Implementation of an Iterator constructed by an
 * <code>Enumeration</code>.
 * @version $Date: 2004-12-02 15:39:36 +0100 (Thu, 02 Dec 2004) $ $Revision: 5473 $
 * Original author Fabio Bellifemine, TILAB
 */
public class EnumIterator implements Iterator {

		private Enumeration e;
    /**
     * Constructor declaration
     * 
     */
    public EnumIterator(Enumeration enumeration) {
				e = enumeration;
    } 

    /**
       Checks whether the iterator can scan further by looking at the
       underlying <code>Enumeration</code>.
       @return The return value of the <code>hasMoreElements()</code>
       method of the underlying <code>Enumeration</code>..
     */
    public boolean hasNext() {
      return e.hasMoreElements();
    } 

    /**
       Retrieves the next element in the collection scanned by this
       iterator, forwarding the request to the underlying
       <code>Enumeration</code>.
       @return The return value of the <code>nextElement()</code> of
       the underlying <code>Enumeration</code>.
     */
    public Object next() {
      return e.nextElement();
    } 

    /**
       Remove the element pointed to by this iterator. <b>This
       operation is not supported and this method will always throw a
       runtime exception</b>
     */
    public void remove() {
	RuntimeException e = new RuntimeException("Unsupported Operation");
	Logger logger = Logger.getMyLogger(this.getClass().getName());
	if(logger.isLoggable(Logger.WARNING))
		logger.log(Logger.WARNING,e.getMessage());
	throw e;
    }

  }
