/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.util.leap;

/**
   The LEAP (environment-dependent) implementation of the &qote;SortedSet&qote; interface.
   This class appears to be exactly the same in J2SE, PJAVA and MIDP.
   The internal implementation is different in the three cases however.
   
   Original author  Nicolas Lhuillier
   @version 1.0, 20/10/00
   
   @see java.util.SortedSet
   @see java.util.TreeSet
 */
public class SortedSetImpl implements SortedSet {
    private java.util.TreeSet hiddenSet = null;

    /**
     * Default Constructor, creates an empty Set,
     * according to the elements' natural order.
     */
    public SortedSetImpl() {
        hiddenSet = new java.util.TreeSet();
    }

    /**
     * @see rca.util.leap.Collection interface
     */
    public boolean add(Object o) {
        return hiddenSet.add(o);
    } 

    /**
     * @see rca.util.leap.Collection interface
     */
    public boolean isEmpty() {
        return hiddenSet.isEmpty();
    } 

    /**
     * @see rca.util.leap.Collection interface
     */
    public boolean remove(Object o) {
        return hiddenSet.remove(o);
    } 

    /**
     * @see rca.util.leap.Collection interface
     */
    public Iterator iterator() {
        return new Iterator() {
            java.util.Iterator it = SortedSetImpl.this.hiddenSet.iterator();

            /**
             * Method declaration
             * 
             * @return
             * 
             * @see
             */
            public boolean hasNext() {
                return it.hasNext();
            } 

            /**
             * Method declaration
             * 
             * @return
             * 
             * @see
             */
            public Object next() {
                return it.next();
            } 

            /**
             * Method declaration
             * 
             * @see
             */
            public void remove() {
                it.remove();
            } 

        };
    } 

    /**
     * @see rca.util.leap.Collection interface
     */
    public Object[] toArray() {
        return hiddenSet.toArray();
    } 

    /**
     * @see rca.util.leap.Collection interface
     */
    public int size() {
        return hiddenSet.size();
    } 

    /**
     * @see rca.util.leap.SortedSet interface
     */
    public Object first() {
        return hiddenSet.first();
    } 

}

