/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/

package rca.wrapper;

/**
   This class is maintained for backward compatibility only. 
   It is not depracated since, for backward compatibility 
   reasons, it is used internally by the framework.
   Use <code>rca.wrapper.ContainerController</code> instead.

   Original author Giovanni Caire - TILAB
 */
public class AgentContainer extends ContainerController implements PlatformController {
  
	public AgentContainer(ContainerProxy cp, rca.core.AgentContainer impl, String platformName) {
  	super(cp, impl, platformName);
  }
  
  public String getName() {
  	return getPlatformName();
  }
  
  public void start() throws ControllerException {
  }

  public void suspend() throws ControllerException {
  	initPlatformController();
  	myPlatformController.suspend();
  }

  public void resume() throws ControllerException {
  	initPlatformController();
  	myPlatformController.resume();
  }
  
  public State getState() {
  	try {
	  	initPlatformController();
	  	return myPlatformController.getState();
  	}
  	catch (Exception e) {
  		return PlatformState.PLATFORM_STATE_KILLED;
  	}
  }

  public void addPlatformListener(Listener aListener) throws ControllerException {
  	initPlatformController();
  	myPlatformController.addPlatformListener(aListener);
  }

  public void removePlatformListener(Listener aListener) throws ControllerException {
  	initPlatformController();
  	myPlatformController.removePlatformListener(aListener);
  }
  
}
