/*---------------------------------------------------------------------------*
*  Copyright (c) 2006        Southeastern Universities Research Association, *
*                            Thomas Jefferson National Accelerator Facility  *
*                                                                            *
*    This software was developed under a United States Government license    *
*    described in the NOTICE file included as part of this distribution.     *
*                                                                            *
*    Author:  Vardan Gyurjyan                                                *
*             gurjyan@jlab.org                  Jefferson Lab, MS-6A         *
*             Phone: (757) 269-5879             12000 Jefferson Ave.         *
*             Fax:   (757) 269-5518             Newport News, VA 23606       *
*                                                                            *
*----------------------------------------------------------------------------*/
package rca.wrapper;

/**
 * Provides a concrete implementation of the State interface for agents.
   <br>
   <b>NOT available in MIDP</b>
   <br>
 * Original author David Bell, Dick Cowan: Hewlett-Packard
 */
public class AgentState extends StateBase {
    static final int LOWEST_STATE = 1;

    /**
     * An array of string names, one for each of the valid state this object
     * can represent. This array MUST be initialized before the other static
     * constructors.
     */
    static final String[] stateNames = {
        "Initiated",    // 1
        "Active",       // 2
        "Idle",         // 3
        "Suspended",    // 4
        "Waiting",      // 5
        "Deleted",      // 6
        "Transit"       // 7
    };

    public static final int cAGENT_STATE_INITIATED = 1;
    public static final int cAGENT_STATE_ACTIVE    = 2;
    public static final int cAGENT_STATE_IDLE      = 3;
    public static final int cAGENT_STATE_SUSPENDED = 4;
    public static final int cAGENT_STATE_WAITING   = 5;
    public static final int cAGENT_STATE_DELETED   = 6;
    public static final int cAGENT_STATE_TRANSIT   = 7;

    static final State AGENT_STATE_INITIATED = new AgentState(cAGENT_STATE_INITIATED);
    static final State AGENT_STATE_ACTIVE = new AgentState(cAGENT_STATE_ACTIVE);
    static final State AGENT_STATE_IDLE = new AgentState(cAGENT_STATE_IDLE);
    static final State AGENT_STATE_SUSPENDED = new AgentState(cAGENT_STATE_SUSPENDED);
    static final State AGENT_STATE_WAITING = new AgentState(cAGENT_STATE_WAITING);
    static final State AGENT_STATE_DELETED = new AgentState(cAGENT_STATE_DELETED);
    static final State AGENT_STATE_INTRANSIT = new AgentState(cAGENT_STATE_TRANSIT);

    /**
     * Creates a state object for a given code.  To avoid coding errors,
     * the integer codes used to create instances of this class should
     * come from the {@link StateConstants StateConstants} class.
     * <p>
     * If the code is "out of range" (i.e. not a valid code), an
     * {@link java.lang.IllegalArgumentException IllegalArgumentException}
     * will be thrown.
     *
     * @param code The integer code identifier for a particular state.
     */
    AgentState(int code) {
        super(code, LOWEST_STATE, stateNames);
    }

    /**
     * For testing, simply list the valid state descriptions and numbers.
     * @param args Command line arguments, currently not used.
     */
    public static void main(String[] args) {
        AgentState aState = new AgentState(LOWEST_STATE);

        System.out.println(aState.legalRange());
    }
}
